/*-
 * Copyright (c) 1998 Dag-Erling Codan Smrgrav
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *	$FreeBSD: libh/bin/setup/setup/fetch.c,v 1.1 2000/06/23 09:41:53 alex Exp $
 */

#include <sys/param.h>
#include <sys/stat.h>
#include <sys/socket.h>

#include <ctype.h>
#include <err.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sysexits.h>
#include <unistd.h>

#include <fetch.h>

#define MINBUFSIZE	4096

int
fetch(char *URL, char *path)
{
	struct url     *url;
	struct url_stat us;
	FILE           *f, *of;
	size_t          size;
	off_t           count;
	char            flags[8];
	int             ch, n;
	char           *buf;	/* transfer buffer */

	if ((url = fetchParseURL(URL)) == NULL)
		errx(1, "%s: parse error", URL);

	*flags = 0;

	/* FTP specific flags */
	if (strcmp(url->scheme, "ftp") == 0) {
		strcat(flags, "p");	/* passive ftp */
		strcat(flags, "d");	/* no proxy */
	}
	/* HTTP specific flags */
	if (strcmp(url->scheme, "http") == 0) {
		strcat(flags, "d");	/* no proxy */
	}
	if ((of = fopen(path, "w")) == NULL) {
		warn("%s", path);
		return (-1);
	}
	count = url->offset;

	if (fetchStat(url, &us, flags) == -1)
		warnx("%s: size not known", path);

	if ((f = fetchGet(url, flags)) == NULL) {
		/* libcom_err has already printed an error message */
		fclose(of);
		return (-1);
	}
	fetchFreeURL(url);

	n = 0;
	if ((buf = malloc(MINBUFSIZE)) == NULL)
		errx(1, strerror(ENOMEM));

	if (us.size == -1) {
		/*
		 * We have no idea how much data to expect, so do it byte by
		 * byte. This is incredibly inefficient, but there's not much
		 * we can do about it... :(
		 */
		while (1) {
			if ((ch = fgetc(f)) == EOF || fputc(ch, of) == EOF)
				break;
			n++;
		}
	} else {
		/* we know exactly how much to transfer, so do it efficiently */
		for (size = MINBUFSIZE; count != us.size; n++) {
			if (us.size - count < MINBUFSIZE)
				size = us.size - count;
			if (fread(buf, size, 1, f) != 1 || fwrite(buf, size, 1, of) != 1)
				break;
		}
	}

	free(buf);
	/* check the status of our files */
	if (ferror(f))
		warn("%s", URL);
	if (ferror(of))
		warn("%s", path);
	if (ferror(f) || ferror(of)) {
		fclose(f);
		fclose(of);
		unlink(path);
		return (-1);
	}
	fclose(f);
	fclose(of);

	return (0);
}
