#! /usr/bin/perl
# Eugene Skepner <eu@qub.com>
# $Id: find_classes_descriptions.pl,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $
# 
# $FreeBSD: libh/find_classes_descriptions.pl,v 1.3 2000/05/23 20:48:04 alex Exp $

if ( scalar( @ARGV ) < 1 ) {
	die "Usage: $0 -s <system> <file-to-scan> ...\n";
}

if ( $ARGV[0] eq "-s" ) {
	shift;
	$the_only_system = shift;
} else {
	undef $the_only_system;
}

# ----------------------------------------------------------------------

%classes = ();
%systems = ();
@files = @ARGV;

while ( <> ) {

	s://.*::o; # strip C++ comments
	if ( /LanguageInterface::Object::ClassDescription ([^\s]+)::sClassDescription[^A-Za-z0-9]/ ) {
		$classes{$ARGV} = "$classes{$ARGV} $1";
	}
}

# ----------------------------------------------------------------------

print "/*\n * This file is automatically generated by $0,\n * do not edit!\n */\n\n";
print "#ifndef HSystem_hh\n";
print "#include \"HSystem.hh\"\n";
print "#endif\n\n";

print "#define H_BUILD_SYSTEMS 1\n\n";

foreach $filename (keys %classes) {
	if ( $classes{$filename} ne "" ) {
		($system,$file) = $filename =~ m:^(.+)/([^/]+)\.cd\.cc$:;
		if ( defined $the_only_system ) {
			$systems{$the_only_system} = $the_only_system;
		} else {
			($sys) = $system =~ m:/([^/]+)$:;
			$systems{$system} = $sys;
		}

		print "#ifndef ${file}_hh\n";
		print "#include \"${file}.hh\"\n";
		print "#endif\n\n";
     }
}

print "void\nbuild_systems(Systems& systems)\n{\n";

foreach $system (keys %systems) {

	print "	pair<Systems::iterator,bool> s = systems.insert(HSystem(\"$systems{$system}\"));\n";
	print "	if (!s.second)\n";
	print "		throw runtime_error(string(\"System \") + s.first->name + \" already exists\");\n";
	print "	HSystem& system = const_cast<HSystem&>(*s.first);\n";

	foreach $filename (@files) {
		if ( defined( $the_only_system ) || $filename =~ m:^$system/: ) {
			@classes = split( / /, $classes{$filename} );
			foreach $class (@classes) {
				if ( $class ne "" ) {
					print "	system.add_class(${class}::thisClassDescription(),\n";
					print "			${class}::fileToInclude());\n";
				}
			}
		}
	}
}

print "} /* build systems */ \n\n";

exit 0;
