/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/common/Configuration.cd.cc,v 1.5 2001/06/20 10:06:26 alex Exp $
 */

/* -*- C++ -*-   Wed Dec 23 1998
 * Eugene Skepner <eu@qub.com>
 * $Id: Configuration.cd.cc,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $
 */

#ifndef Configuration_hh
#include "Configuration.hh"
#endif

//======================================================================

LanguageInterface::Object::MethodDescription Configuration::sMethods[] =
{
	MethodDescription("Configuration",
			  "instance",
			  LanguageInterface::Object::MethodDescription::fStatic,
				"Returns configuration object",
				MethodDescription::Value(MethodDescription::vtObject,
							 "Configuration"),
			  MethodDescription::vtVoid),
	MethodDescription("usePackagePrefix",
			  "usePackagePrefix",
			  LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fSafeExecution | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
			  "",
			  MethodDescription::vtBool,
			  MethodDescription::Value(MethodDescription::vtBool,
						   0,
						   "set-or-get",
						   "use_package_prefix")),
	MethodDescription("prefix",
			  "prefix",
			  LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fSafeExecution | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
			  "Prefix to install package (if usePackagePrefix is false)",
			  MethodDescription::vtString,
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "set-or-get",
						   "new_prefix")),
	MethodDescription( "path_to_patch", "path_to_patch",
			   LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fSafeExecution | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
			   "Pathname of the patch program",
			   MethodDescription::vtString, MethodDescription::Value(MethodDescription::vtString,
										 0,
										 "set-or-get",
										 "new_path_to_patch")),
	MethodDescription("no_action",
			  "no_action",
			  LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fSafeExecution | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
			  "Should the installation/deistallation program does real actions (writing to disk, updating database) or just report what should be done.",
			  MethodDescription::vtBool,
			  MethodDescription::Value(MethodDescription::vtBool,
						   0,
						   "set-or-get",
						   "no_action")),
	MethodDescription("tempDir",
			  "tempDir",
			  LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fSafeExecution | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
			  "Sets or gets directory for temporary files",
			  MethodDescription::vtString, MethodDescription::Value(MethodDescription::vtString,
										0,
										"set-or-get",
										"new_temp_dir")),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Configuration::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Configuration::sClassDescription(
	"Configuration", "Configuration",
	"Sysinstall configuration",
	sMethods,
	sCallbacks
);

//======================================================================
