/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/common/Configuration.hh,v 1.3 2001/06/20 10:06:26 alex Exp $
 */

// -*- C++ -*-  Wed Dec 23 1998
// Eugene Skepner <eu@qub.com>
// $Id: Configuration.hh,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef Configuration_hh
#define Configuration_hh

#ifndef LanguageInterface_hh
#include "LanguageInterface.hh"
#endif

//======================================================================

class Configuration : virtual public LanguageInterface::Object
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     Configuration();
     virtual ~Configuration();

     bool usePackagePrefix() const { return mUsePackagePrefix; }
     void usePackagePrefix( bool use ) { mUsePackagePrefix = use; }

     const string& prefix() const { return mPrefix; }
     void prefix( const string& aPrefix ) { mPrefix = aPrefix; }

     const string& path_to_patch() const { return mPathToPatch; }
     void path_to_patch( const string& aPathToPatch ) { mPathToPatch = aPathToPatch; }

     bool no_action() const { return mNoAction; }
     void no_action( bool aNoAction ) { mNoAction = aNoAction; }

     const string& tempDir() const { return mTempDir; }
     void tempDir( const string& aTempDir ) { mTempDir = aTempDir; }

     static Pointer<Configuration> instance()
          {
               if ( !sConfiguration )
                    sConfiguration = new Configuration();
               return sConfiguration;
          }

  private:
     bool mUsePackagePrefix;
     string mPrefix;
     string mPathToPatch;
     bool mNoAction;
     string mTempDir;

     static Pointer<Configuration> sConfiguration;

}; // class Configuration

//======================================================================

#endif // Configuration_hh
