/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/common/H.cc,v 1.4 2001/06/20 10:06:26 alex Exp $
 */

// -*- C++ -*-   Wed Nov 25 1998
// Eugene Skepner <eu@qub.com>
// $Id: H.cc,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef H_hh
#include "H.hh"
#endif

#include <algorithm>
#include <ctime>

//======================================================================

void H::dereference( const list< Pointer<LanguageInterface::Object> >& aObjects1, const list< Pointer<LanguageInterface::Object> >& aObjects2, const list< Pointer<LanguageInterface::Object> >& aObjects3, const list< Pointer<LanguageInterface::Object> >& aObjects4, const list< Pointer<LanguageInterface::Object> >& aObjects5, const list< Pointer<LanguageInterface::Object> >& aObjects6, const list< Pointer<LanguageInterface::Object> >& aObjects7, const list< Pointer<LanguageInterface::Object> >& aObjects8, const list< Pointer<LanguageInterface::Object> >& aObjects9, LanguageInterface* aLanguageInterface )
{
	if ( !aLanguageInterface->gc() ) { // ignore if automatic garbage collection is used
//		  cerr << "H::dereference(): ";
//		  copy( aObjects1.begin(), aObjects1.end(), ostream_iterator<const void*>( cerr, " : " ) );
//		  cerr << endl;

		for ( list<Pointer<LanguageInterface::Object> >::const_iterator i = aObjects1.begin(); i != aObjects1.end(); ++i )
			aLanguageInterface->unregisterObject( *i );
		for ( list<Pointer<LanguageInterface::Object> >::const_iterator i = aObjects2.begin(); i != aObjects2.end(); ++i )
			aLanguageInterface->unregisterObject( *i );
		for ( list<Pointer<LanguageInterface::Object> >::const_iterator i = aObjects3.begin(); i != aObjects3.end(); ++i )
			aLanguageInterface->unregisterObject( *i );
		for ( list<Pointer<LanguageInterface::Object> >::const_iterator i = aObjects4.begin(); i != aObjects4.end(); ++i )
			aLanguageInterface->unregisterObject( *i );
		for ( list<Pointer<LanguageInterface::Object> >::const_iterator i = aObjects5.begin(); i != aObjects5.end(); ++i )
			aLanguageInterface->unregisterObject( *i );
		for ( list<Pointer<LanguageInterface::Object> >::const_iterator i = aObjects6.begin(); i != aObjects6.end(); ++i )
			aLanguageInterface->unregisterObject( *i );
		for ( list<Pointer<LanguageInterface::Object> >::const_iterator i = aObjects7.begin(); i != aObjects7.end(); ++i )
			aLanguageInterface->unregisterObject( *i );
		for ( list<Pointer<LanguageInterface::Object> >::const_iterator i = aObjects8.begin(); i != aObjects8.end(); ++i )
			aLanguageInterface->unregisterObject( *i );
		for ( list<Pointer<LanguageInterface::Object> >::const_iterator i = aObjects9.begin(); i != aObjects9.end(); ++i )
			aLanguageInterface->unregisterObject( *i );
	}

} // H::dereference

//======================================================================

Pointer<HObjectsReferences> H::objects_references( LanguageInterface* aLanguageInterface )
{
	HObjectsReferences* hor = new HObjectsReferences( aLanguageInterface->gc() ? 0 : aLanguageInterface );
	if ( !aLanguageInterface->gc() )
		aLanguageInterface->mHObjectsReferences.push_back( hor );
	return(hor);

} // H::objects_references

//======================================================================

HObjectsReferences::~HObjectsReferences()
{
	if ( mLanguageInterface && !mLanguageInterface->gc() ) {
//		  cerr << "HObjectsReferences::~HObjectsReferences() " << mReferencedObjects.size() << endl;
//		  for( LanguageInterface::ReferencedObjects::const_iterator i = mReferencedObjects.begin(); i != mReferencedObjects.end(); ++i )
//			  cerr << "~HObjectsReferences(): " << LanguageInterface::object2string( *i ) << endl;
		while ( mReferencedObjects.begin() != mReferencedObjects.end() ) {
			mLanguageInterface->unregisterObject( *mReferencedObjects.rbegin() );
		}
		for ( list<HObjectsReferences*>::iterator i = mLanguageInterface->mHObjectsReferences.begin(); i != mLanguageInterface->mHObjectsReferences.end(); ++i ) {
			if ( *i == this ) {
				mLanguageInterface->mHObjectsReferences.erase( i );
				break;
			}
		}
	}

} // HObjectsReferences::~HObjectsReferences

//======================================================================

void HObjectsReferences::release()
{
	if ( mLanguageInterface && !mLanguageInterface->gc() ) {
		while ( mReferencedObjects.begin() != mReferencedObjects.end() ) {
			mLanguageInterface->unregisterObject( *mReferencedObjects.begin() );
		}
	}

} // HObjectsReferences::release

//======================================================================

string H::string_time_t( time_t t )
{
	string result = ctime( &t );
	if ( *result.rbegin() == '\n' )
		result.erase( result.end() - 1 );
	return(result);

} // H::string_time_t

//======================================================================
