/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/common/H.hh,v 1.4 2001/06/20 10:06:26 alex Exp $
 */

// -*- C++ -*-  Wed Nov 25 1998
// Eugene Skepner <eu@qub.com>
// $Id: H.hh,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef H_hh
#define H_hh

//======================================================================

#ifndef LanguageInterface_hh
#include "LanguageInterface.hh"
#endif

#include <ctime>

//======================================================================

class HObjectsReferences : virtual public LanguageInterface::Object
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

     friend class H;

  public:
     virtual ~HObjectsReferences();

     virtual void referenceObject( const Object* object )
          {
               if ( object && object != this && find( mReferencedObjects.begin(), mReferencedObjects.end(), object ) == mReferencedObjects.end() )
                    mReferencedObjects.push_back( object );
          }

     virtual void dereferenceObject( const Object* object )
          {
                 // Do not combine find and erase into one statement
               LanguageInterface::ReferencedObjects::iterator toErase = find( mReferencedObjects.begin(), mReferencedObjects.end(), object );
               if ( toErase != mReferencedObjects.end() )
                    mReferencedObjects.erase( toErase );
          }

     virtual void release();

  protected:
     HObjectsReferences( LanguageInterface* aLanguageInterface ) : mLanguageInterface( aLanguageInterface ) {}

  private:
     LanguageInterface* mLanguageInterface;
     LanguageInterface::ReferencedObjects mReferencedObjects;

}; // class HObjectsReferences

//======================================================================

class H : virtual public LanguageInterface::Object
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     static void dereference( const list< Pointer<LanguageInterface::Object> >& aObjects1, const list< Pointer<LanguageInterface::Object> >& aObjects2, const list< Pointer<LanguageInterface::Object> >& aObjects3, const list< Pointer<LanguageInterface::Object> >& aObjects4, const list< Pointer<LanguageInterface::Object> >& aObjects5, const list< Pointer<LanguageInterface::Object> >& aObjects6, const list< Pointer<LanguageInterface::Object> >& aObjects7, const list< Pointer<LanguageInterface::Object> >& aObjects8, const list< Pointer<LanguageInterface::Object> >& aObjects9, LanguageInterface* aLanguageInterface );

     static bool is_null( const Pointer<LanguageInterface::Object> aObject )
          {
               return !aObject;
          }

     static bool not_null( const Pointer<LanguageInterface::Object> aObject )
          {
               return aObject;
          }

     static string as_string( const Pointer<LanguageInterface::Object> aObject )
          {
               return LanguageInterface::object2string( aObject );
          }

     static Pointer<LanguageInterface::Object> null()
          {
               return 0;
          }

     static string string_time_t( time_t t );

     static Pointer<HObjectsReferences> objects_references( LanguageInterface* aLanguageInterface );

}; // class H

//======================================================================

#endif // H_hh
