/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/common/HSystem.hh,v 1.3 2001/06/20 10:06:26 alex Exp $
 */

// -*- C++ -*-  Sun Nov 29 1998
// Eugene Skepner <eu@qub.com>
// $Id: HSystem.hh,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef HSystem_hh
#define HSystem_hh

#ifndef LanguageInterface_hh
#include "LanguageInterface.hh"
#endif

#include <iostream.h>

#include <set>
#include <list>

//======================================================================

class HSystem
{
  public:
     typedef list<const LanguageInterface::Object::ClassDescription*> Descriptions;
     typedef set<string> Includes;
     typedef list<const char*> CommandAccessToRegister;
     typedef list<pair<string,string> > StaticMethods;

     HSystem( const string& aName )
          : name( aName ) {}
     HSystem( const HSystem& aHSystem )
          : name( aHSystem.name ) {}

     void add_class( const LanguageInterface::Object::ClassDescription* class_desc, const string& include_file )
          {
               descriptions.push_back( class_desc );
               includes.insert( include_file );
          }

     void gen_header( ostream& out );
     void gen_classes_interfaces( ostream& out );
     void gen_register_command_access( ostream& out );
     void gen_footer( ostream& out );

     Descriptions descriptions;
     Includes includes;
     CommandAccessToRegister commandAccessToRegister;
     StaticMethods staticMethods;

     string name;
     bool operator < ( const HSystem& toCompare ) const { return name < toCompare.name; }
     bool operator == ( const HSystem& toCompare ) const { return name == toCompare.name; }

  private:
     void gen_class_header( ostream& out, const LanguageInterface::Object::ClassDescription* aClassDescription );
     void gen_callback_interfaces( ostream& out, const LanguageInterface::Object::ClassDescription* aClassDescription );
     void gen_dynamic_methods_interfaces( ostream& out, const LanguageInterface::Object::ClassDescription* aClassDescription );
     void gen_static_method_interfaces( ostream& out, const LanguageInterface::Object::ClassDescription* aClassDescription );
     string gen_result_to_obj( LanguageInterface::Object::MethodDescription::Value t );
     string gen_method_argument( LanguageInterface::Object::MethodDescription::Value t, int argno, int method_argno, bool last_arg_in_set_method );

}; // class HSystem

//======================================================================

typedef set<HSystem> Systems;

//======================================================================

void build_systems( Systems& systems );

//======================================================================

#endif // HSystem_hh
