/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/common/Log.hh,v 1.3 2001/06/20 10:06:27 alex Exp $
 */

// -*- C++ -*-  Thu Dec 24 1998
// Eugene Skepner <eu@qub.com>
// $Id: Log.hh,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef Log_hh
#define Log_hh

#ifndef LanguageInterface_hh
#include "LanguageInterface.hh"
#endif

#include <strstream>

//======================================================================

class Log : virtual public LanguageInterface::Object
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     enum Type { WARNING, NO_ACTION, ERROR, FATAL };

     Log() {}

     virtual void clear();
     virtual ostream& stream() = 0;
     virtual string text() { return string(); }

     static Pointer<Log> instance();
     static void type( const string& aType );
     static string type() { return sLogType; }

  private:
     static string sLogType;

}; // class Log

//======================================================================

class LogStderr : public Log
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     LogStderr() {}

     virtual ostream& stream() { return cerr; }

}; // class LogStderr

//======================================================================

class LogString : public Log
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

     class Ostrstream : public ostrstream
     {
       public:
          virtual ~Ostrstream() { rdbuf()->freeze( 0 ); }
     };

  public:
     LogString() : mStream( new Ostrstream() ) {}

     virtual ostream& stream() { return *mStream; }
     virtual string text() { stream() << ends; return mStream->str(); }

  private:
     Ostrstream* mStream;

}; // class LogString

//======================================================================

inline ostream& operator << ( ostream& os, Log::Type )
{
     return os;
}

//======================================================================

#define LOG(Type,Inserts) Log::instance()->stream() << #Type << ": " << Inserts << endl

//======================================================================

#endif // Log_hh
