/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/common/Pointer.cc,v 1.5 2001/06/20 10:06:27 alex Exp $
 */

// -*- C++ -*-   Tue Mar 24 1998
// Eugene Skepner <eu@qub.com>
// $Id: Pointer.cc,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef Pointer_hh
#include "Pointer.hh"
#endif

#include <slist>
#include <algorithm>

//======================================================================

typedef slist<const void*> HeapPointers;

HeapPointers heapPointers;

//======================================================================

/*
* Search for the passed in pointer in the list of heap pointers.  If it is
* found, remove it from the list of pointers and return 0.  If it is not found,
* assume it is a reference to an automatic variable and return -1.  This is
* used to initialize the reference count the first time an object is
* referenced.
*/
int Shared::isOnHeap( const Shared* ps )
{
	const void* p = dynamic_cast<const void*>( ps );

	HeapPointers::iterator i = find( heapPointers.begin(), heapPointers.end(), p );
	if ( i == heapPointers.end() ) {
		return -1; // not allocated using operator new
	}
	else {
		heapPointers.erase( i );
		return 0;
	}
}

//======================================================================

/*
* Put the given pointer onto the heap.  This is called when a new pointer is
* first allocated but is not yet referenced.
*/
void Shared::onHeap( const void* p )
{
	heapPointers.insert( heapPointers.end(), p );

} // Shared::onHeap

//======================================================================
