/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/common/Type.hh,v 1.3 2001/06/20 10:06:28 alex Exp $
 */

// -*- C++ -*-  Sun Nov 22 1998
// Eugene Skepner <eu@qub.com>
// $Id: Type.hh,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

/*
 * This header provides an inline function for converting a short-hand
 * type-name (g++ name mangling perhaps?) into a full, intelligble type
 * name.
 */

#ifndef Type_hh
#define Type_hh

#include <cctype>
#include <string>

//======================================================================

inline string type_to_name( const char* type )
{
     if ( *type == 'P' )
          ++type;

     if ( !strcmp( "i", type ) )
          return "int";
     else if ( !strcmp( "Ui", type ) )
          return "unsigned int";
     else if ( !strcmp( "s", type ) )
          return "short";
     else if ( !strcmp( "Us", type ) )
          return "unsigned short";
     else if ( !strcmp( "l", type ) )
          return "long";
     else if ( !strcmp( "Ul", type ) )
          return "unsigned long";
     else if ( !strcmp( "x", type ) )
          return "long long";
     else if ( !strcmp( "Ux", type ) )
          return "unsigned long long";
     else if ( !strcmp( "c", type ) )
          return "char";
     else if ( !strcmp( "Uc", type ) )
          return "unsigned char";
     else if ( !strcmp( "f", type ) )
          return "float";
     else if ( !strcmp( "d", type ) )
          return "double";
     else if ( !strcmp( "r", type ) )
          return "long double";
     else if ( !strncmp( "t12basic_string", type, 15 ) )
          return "string";
     else {
          if ( !strncmp( "t7Pointer1Z", type, 11 ) )
               type += 11;
          while ( isdigit( *type ) )
               ++type;
          return type;
     }
}

//======================================================================

#endif // Type_hh
