/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/database/DatabaseEngine.cc,v 1.4 2001/06/20 10:08:11 alex Exp $
 */

// -*- C++ -*-   Wed Aug 12 1998
// $Id: DatabaseEngine.cc,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef DatabaseEngine_hh
#include "DatabaseEngine.hh"
#endif

#include <sys/file.h>
#include <cerrno>
#include <unistd.h>

//======================================================================

void DatabaseEngine::lock()
{
	if ( mLockFile < 0 ) {
		const string lockFile = baseDir() + "/lock";
		if ( ( mLockFile = open( lockFile.c_str(), O_CREAT, 0600 ) ) < 0 )
			throw runtime_error( string( "Database lock failed: unable to open/create lock file " ) + lockFile + ": " + ::strerror( errno ) );
		if ( flock( mLockFile, LOCK_EX|LOCK_NB ) < 0 )
			throw runtime_error( string( "Database lock failed: unable to flock file " ) + lockFile + ": " + ::strerror( errno ) );
	}

} // DatabaseEngine::lock

//======================================================================

void DatabaseEngine::unlock()
{
	if ( mLockFile >= 0 ) {
		flock( mLockFile, LOCK_UN );
		close( mLockFile );
		mLockFile = -1;
	}

} // DatabaseEngine::unlock

//======================================================================

size_t DatabaseEngine::TableDescription::numberOfFields() const
{
	size_t numFields = 0;
	for ( ; fields[numFields].name; ++numFields );
	return(numFields);

} // DatabaseEngine::TableDescription::numberOfFields

//======================================================================

size_t DatabaseEngine::TableDescription::fieldByName( const char* aFieldName ) const
{
	for ( size_t i = 0; fields[i].name; ++i )
		if ( !strcmp( fields[i].name, aFieldName ) )
			return i;

	throw runtime_error( string( "Internal: DatabaseEngine::TableDescription::fieldByName: cannot find field " ) + aFieldName + " in table " + name );

} // DatabaseEngine::TableDescription::fieldByName

//======================================================================
