/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/disk/Disk.cd.cc,v 1.5 2001/06/20 10:07:00 alex Exp $
 */

// -*- C++ -*-   Tue Feb  9 1999
// Eugene Skepner <eu@qub.com>
// $Id: Disk.cd.cc,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef Disk_hh
#include "Disk.hh"
#endif

//======================================================================

LanguageInterface::Object::MethodDescription Disk::sMethods[] =
{
	MethodDescription("Disk", "open",
			  LanguageInterface::Object::MethodDescription::fStatic
			  | LanguageInterface::Object::MethodDescription::fSafeExecution,
			  "Opens the named disk",
			  MethodDescription::Value(MethodDescription::vtObject,
						   "Disk"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   0,
						   "devname")),
	MethodDescription("open", "open",
			  LanguageInterface::Object::MethodDescription::fStatic 
			  | LanguageInterface::Object::MethodDescription::fSafeExecution,
			  "Opens the named disk",
			  MethodDescription::Value(MethodDescription::vtObject,
						   "Disk"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   0,
						   "devname")),
	MethodDescription("clone", "clone",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Clone a copy of a tree.  Useful for \"Undo\" functionality",
			  MethodDescription::Value(MethodDescription::vtObject,
						   "Disk"),
			  MethodDescription::vtVoid ),
	MethodDescription("chunks", "chunks",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns chunks head object",
			  MethodDescription::Value(MethodDescription::vtObject,
						   "DiskChunk"),
			  MethodDescription::vtVoid),
	MethodDescription("debug", "debug",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Print the content of the tree to stdout",
			  MethodDescription::vtVoid, MethodDescription::vtVoid),
	MethodDescription("name", "name",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "",
			  MethodDescription::vtString, MethodDescription::vtVoid),
	MethodDescription("flags", "flags_names",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "The only flag value by now is DISK_ON_TRACK, meaning that this disk is handled by the On-Track Disk Manager.",
			  MethodDescription::Value(MethodDescription::vtListPtr,
						   "Container<list<string> >"),
			  MethodDescription::vtVoid),
	MethodDescription("bios_cyl", "bios_cyl",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "",
			  MethodDescription::vtULong,
			  MethodDescription::vtVoid),
	MethodDescription("bios_hd", "bios_hd",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "",
			  MethodDescription::vtULong, MethodDescription::vtVoid),
	MethodDescription("bios_sect", "bios_sect",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "",
			  MethodDescription::vtULong, MethodDescription::vtVoid),
	MethodDescription("modified", "modified",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns if memory disk struture differs from written on disk",
			  MethodDescription::vtBool, MethodDescription::vtVoid),
	MethodDescription("Disk_Names", "Disk_Names",
			  LanguageInterface::Object::MethodDescription::fStatic
			  | LanguageInterface::Object::MethodDescription::fSafeExecution,
			  "Returns all disk's names (ad0, ad1, da0 ...)",
			  MethodDescription::Value(MethodDescription::vtListPtr,
						   "Container<list<string> >"),
			  MethodDescription::vtVoid),
	MethodDescription("standard_slice_types",
			  "standard_slice_types",
			  LanguageInterface::Object::MethodDescription::fStatic,
			  "Returns list of known slice types names",
			  MethodDescription::Value(MethodDescription::vtListPtr,
						   "Container<list<string> >"),
			  MethodDescription::vtVoid),
	MethodDescription("standard_slice_type_id", "standard_slice_type_id",
			  LanguageInterface::Object::MethodDescription::fStatic,
			  "Returns id of known slice type (by name) or 0",
			  MethodDescription::vtInt,
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   0,
						   "name")),
	MethodDescription("slice_type_name", "slice_type_name",
			  LanguageInterface::Object::MethodDescription::fStatic,
			  "Returns slice type name by id",
			  MethodDescription::vtString,
			  MethodDescription::Value(MethodDescription::vtInt,
						   0,
						   0,
						   "slice_id")),
	MethodDescription("Delete_Chunk", "Delete_Chunk",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns 0 on success, 1 on failure",
			  MethodDescription::vtInt,
			  MethodDescription::Value(MethodDescription::vtObject,
						   "DiskChunk",
						   0,
						   "chunk")),
	MethodDescription("Create_Chunk", "Create_Chunk",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Creates a chunk with the specified paramters. Returns 0 on success, 1 on failure",
			  MethodDescription::vtInt,
			  MethodDescription::Value(MethodDescription::vtULong,
						   0,
						   0,
						   "offset"),
			  MethodDescription::Value(MethodDescription::vtULong,
						   0,
						   0,
						   "size"),
			  MethodDescription::Value(MethodDescription::vtULong,
						   0,
						   0,
						   "subtype"),
			  MethodDescription::Value(MethodDescription::vtList,
						   "list<string>",
						   "list<string>()",
						   "flags")),
	MethodDescription("Create_Chunk_DWIM", "Create_Chunk_DWIM",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Creates a partition inside the given parent of the given size, and returns it. The first unused chunk big enough is used.",
			  MethodDescription::Value(MethodDescription::vtObject,
						   "DiskChunk"),
			  MethodDescription::Value(MethodDescription::vtObject,
						   "DiskChunk",
						   0,
						   "parent"),
			  MethodDescription::Value(MethodDescription::vtULong,
						   0,
						   0,
						   "size"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   0,
						   "type"),
			  MethodDescription::Value(MethodDescription::vtList,
						   "list<string>",
						   "list<string>()",
						   "flags")),
	MethodDescription( "All_FreeBSD", "All_FreeBSD",
			   LanguageInterface::Object::MethodDescription::fDynamic,
			   "Make one FreeBSD chunk covering the entire disk; if force_all is set, bypass all BIOS geometry considerations.",
			   MethodDescription::vtVoid, MethodDescription::Value(MethodDescription::vtBool,
									       0,
									       0,
									       "force_all")),
	MethodDescription("Set_Bios_Geom", "Set_Bios_Geom",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Set the geometry the bios uses.",
			  MethodDescription::vtVoid,
			  MethodDescription::Value(MethodDescription::vtULong,
						   0,
						   0,
						   "cylinders"),
			  MethodDescription::Value(MethodDescription::vtULong,
						   0,
						   0,
						   "heads"),
			  MethodDescription::Value(MethodDescription::vtULong,
						   0,
						   0,
						   "sectors")),
	MethodDescription("undo",
			  "undo",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Undo changes.",
			  MethodDescription::vtVoid, MethodDescription::vtVoid),
	MethodDescription("Write_Disk", "Write_Disk",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Write data to disk.",
			  MethodDescription::vtInt, MethodDescription::vtVoid),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Disk::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Disk::sClassDescription(
	"Disk",
	"Disk",
	"Interface to libdisk",
	sMethods,
	sCallbacks
);

//======================================================================

LanguageInterface::Object::MethodDescription DiskChunk::sMethods[] =
{
	MethodDescription("offset", "offset",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "",
			  MethodDescription::vtLong,
			  MethodDescription::vtVoid),
	MethodDescription("size", "size",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "",
			  MethodDescription::vtULong,
			  MethodDescription::vtVoid),
	MethodDescription("end", "end",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "",
			  MethodDescription::vtULong,
			  MethodDescription::vtVoid),
	MethodDescription("name", "name",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "",
			  MethodDescription::vtString,
			  MethodDescription::vtVoid),
	MethodDescription("type", "type",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "0 - whole, 1 - unknown, 2 - fat, 3 - freebsd, 4 - extended, 5 - part, 6 - unused",
			  MethodDescription::vtInt,
			  MethodDescription::vtVoid),
	MethodDescription("type_text", "type_text",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "",
			  MethodDescription::vtString,
			  MethodDescription::vtVoid),
	MethodDescription("subtype", "subtype",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "",
			  MethodDescription::vtInt,
			  MethodDescription::vtVoid),
	MethodDescription("slice_type_name", "slice_type_name",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "",
			  MethodDescription::vtString,
			  MethodDescription::vtVoid),
	MethodDescription("flags", "flags_names",
			  LanguageInterface::Object::MethodDescription::fDynamic 
			  | LanguageInterface::Object::MethodDescription::fGetOrSet,
			  "CHUNK_BSD_COMPAT (in the BSD-compatibility, and has a short name too, ie ad0s4f -> ad0f), CHUNK_ALIGN (should be aligned), CHUNK_IS_ROOT (This 'part' is a rootfs, allocate 'a'), CHUNK_ACTIVE (active slice in the MBR), CHUNK_FORCE_ALL (Force a dedicated disk for FreeBSD, bypassing all BIOS geometry considerations)",
			  MethodDescription::Value(MethodDescription::vtListPtr,
						   "Container<list<string> >"),
			  MethodDescription::Value(MethodDescription::vtList,
						   "list<string>",
						   "get-or-set",
						   "flags")),
	MethodDescription("mountpoint", "mountpoint",
			  LanguageInterface::Object::MethodDescription::fDynamic
			  | LanguageInterface::Object::MethodDescription::fGetOrSet,
			  "Returns or sets partition mount point",
			  MethodDescription::vtString,
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "get-or-set",
						   "mountpoint")),
	MethodDescription("newfs", "newfs",
			  LanguageInterface::Object::MethodDescription::fDynamic 
			  | LanguageInterface::Object::MethodDescription::fGetOrSet,
			  "Returns or sets partition newfs flag: 0-no, 1-yes, 2-unknown",
			  MethodDescription::vtInt,
			  MethodDescription::Value(MethodDescription::vtInt,
						   0,
						   "get-or-set",
						   "newfs")),
	MethodDescription("newfs_cmd", "newfs_cmd",
			  LanguageInterface::Object::MethodDescription::fDynamic
			  | LanguageInterface::Object::MethodDescription::fGetOrSet,
			  "Returns or sets newfs command for the partition",
			  MethodDescription::vtString,
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "get-or-set",
						   "mountpoint")),
	MethodDescription("is_fs", "is_fs",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns if partition is allocated for FreeBSD filesystem",
			  MethodDescription::vtBool,
			  MethodDescription::vtVoid),
	MethodDescription("is_swap", "is_swap",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns if partition is allocated for swap",
			  MethodDescription::vtBool,
			  MethodDescription::vtVoid),
	MethodDescription("part", "part",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Partition chunk",
			  MethodDescription::Value(MethodDescription::vtObject,
						   "DiskChunk"),
			  MethodDescription::vtVoid),
	MethodDescription("next", "next",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Next chunk",
			  MethodDescription::Value(MethodDescription::vtObject,
						   "DiskChunk"),
			  MethodDescription::vtVoid),
	MethodDescription("set_type", "set_type",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Sets the slice subtype and type",
			  MethodDescription::vtVoid,
			  MethodDescription::Value(MethodDescription::vtInt,
						   0,
						   0,
						   "subtype")),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription DiskChunk::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription DiskChunk::sClassDescription(
	"DiskChunk",
	"DiskChunk",
	"Interface to disk chunk",
	sMethods,
	sCallbacks
);

//======================================================================
