#! /usr/local/bin/tclsh8.2

set hui_type "auto"

if {[info exists argc]} {
    if {$argc > 0} {
	set hui_type [lindex $argv 0]
    }
}

if [info exists env(H_SHARED_LIB_VERSION)] {
    set H_SHARED_LIB_VERSION $env(H_SHARED_LIB_VERSION)
} else {
    set H_SHARED_LIB_VERSION "0"
}

# Order is important!
load ../common/libh.so.$H_SHARED_LIB_VERSION
load ../tcl/libhtcl.so.$H_SHARED_LIB_VERSION

load ../file/libhfile.so.$H_SHARED_LIB_VERSION
load ../tcl/libhtclfile.so.$H_SHARED_LIB_VERSION

load ../hui/libhui.so.$H_SHARED_LIB_VERSION
load ../tcl/libhtclhui.so.$H_SHARED_LIB_VERSION

load ./libhdisk.so.$H_SHARED_LIB_VERSION
load ../tcl/libhtcldisk.so.$H_SHARED_LIB_VERSION

# ----------------------------------------------------------------------

set ONE_MEG 2048
set FS_MIN_SIZE $ONE_MEG
set ROOT_MIN_SIZE 20
set ROOT_PARTITION_SIZE [expr 32 * $ONE_MEG]
set PHYSICAL_MEMORY 32
set SWAP_PARTITION_SIZE [expr 16 * $ONE_MEG + $PHYSICAL_MEMORY * $ONE_MEG * 2]
set VAR_PARTITION_MIN_SIZE [expr 30 * $ONE_MEG]
set USR_PARTITION_MIN_SIZE [expr 80 * $ONE_MEG]

# ----------------------------------------------------------------------

set about_msg "   FreeBSD Disk Slicer   \n\nv. 1"

set disk_window_desc {
    horizontal box {
	spacing 1
	vertical box1 1 {
	    spacing 1
	    label {disk_geometry_label "Disk Geometry: $disk_geometry"} 0 left
	    spacing 1
	    listBox {disk_slices $disk_slices_items 0 $disk_slices_label} 2 center
	    spacing 1
	    horizontal box11 0 {
		stretch 1
		vertical box111 0 {
		    button {create_button "&Create Slice" [list slice_create [list $disk [$dialog byName disk_slices]]] "normal"} 0 center
		    button {delete_button "&Delete Slice" [list slice_delete [list $disk [$dialog byName disk_slices]]] "normal"} 0 center
		    button {change_type_button "Change &Type" [list slice_type [list $disk [$dialog byName disk_slices]]] "normal"} 0 center
		    button {bootable_button "Toggle &Bootable" [list slice_bootable [list $disk [$dialog byName disk_slices]]] "normal"} 0 center
		}
		stretch 1
		vertical box112 0 {
		    button {entire_button "Use &Entire Disk" [list disk_entire [list $disk [$dialog byName disk_slices]]] "normal"} 0 center
		    spacing 1
		    button {geometry_button "Set Drive &Geometry" [list disk_geometry [list $disk [$dialog byName disk_slices] [$dialog byName disk_geometry_label]]] "normal"} 0 center
		    spacing 1
		    button {bad_block_button "Toggle B&ad Blocks Scan" [list disk_scan [list $disk [$dialog byName disk_slices]]] "normal"} 0 center
		}
		stretch 1
		vertical box113 0 {
		    button {undo_button "&Undo All Changes" [list disk_undo [list $disk [$dialog byName disk_slices] [$dialog byName disk_geometry_label]]] "normal"} 0 center
		    spacing 1
		    button {write_button "&Write Changes" [list disk_write $disk] "normal"} 0 center
		    spacing 1
		    button {partitions_button "Slice &Partitions" [list slice_partitions [list $disk [$dialog byName disk_slices]]] "normal"} 0 center
		}
		stretch 1
	    }
	}
	spacing 1
    }
}

set slice_type_dialog_desc {
    horizontal box {
	spacing 1
	listBox {slice_types $slice_types 0 "&Standard Types"} 2 center
	spacing 1
	vertical box1 1 {
	    inputLine {slice_type $initial_slice_type 5 "integer_range 0 99999" "&Type"} 0 center
	    spacing 1
	    label {note_label1 "Note:  If you choose a non-FreeBSD"} 0 left
	    label {note_label2 "partition type, it will not be formatted"} 0 left
	    label {note_label3 "or otherwise prepared, it will simply"} 0 left
	    label {note_label4 "reserve space for you to use another tool,"} 0 left
	    label {note_label5 "such as DOS format, to later format and"} 0 left
	    label {note_label6 "actually use the partition."} 0 left
	    stretch 1
	    button {ok_button "&OK" [list button_click_result "OK"] "default"} 0 center
	    spacing 1
	    button {cancel_button "&Cancel" [list button_click_result "CANCEL"] "normal"} 0 center
	}
	spacing 1
    }
}

set slice_create_dialog_desc {
    horizontal box {
	spacing 1
	vertical box1 1 {
	    inputLine {slice_size $initial_slice_size 10 "none" "&Size (in blocks or megabytes if M is appended)"} 0 center
	    spacing 1
	    horizontal box11 0 {
		label {type_label1 "Type:"} 0 left
		spacing 1
		label {type_name "                      "} 0 left
		spacing 1
		label {l "("} 0 left
		label {type_id $type_id} 0 left
		label {l ")"} 0 left
		stretch 1
		button {ok_button "&Choose Type" [list slice_create_choose_type $dialog] "normal"} 0 center
	    }
	    stretch 1
	    horizontal box12 0 {
		stretch 1
		button {ok_button "&OK" [list button_click_result "OK"] "default"} 0 center
		stretch 1
		button {cancel_button "&Cancel" [list button_click_result "CANCEL"] "normal"} 0 center
		stretch 1
	    }
	}
	spacing 1
    }
}

set desk_geometry_dialog_desc {
    horizontal box {
	spacing 1
	vertical box1 1 {
	    inputLine {cylinders $cylinders 6 "integer_range 1 999999" "&Cylinders:"} 0 center
	    spacing 1
	    inputLine {heads $heads 6 "integer_range 1 999999" "&Heads:"} 0 center
	    spacing 1
	    inputLine {sectors $sectors 6 "integer_range 1 999999" "&Sectors:"} 0 center
	    stretch 1
	    horizontal box12 0 {
		stretch 1
		button {ok_button "&OK" [list button_click_result "OK"] "default"} 0 center
		stretch 1
		button {cancel_button "&Cancel" [list button_click_result "CANCEL"] "normal"} 0 center
		stretch 1
	    }
	}
	spacing 1
    }
}

set slice_partitions_dialog_desc {
    horizontal box {
	spacing 1
	vertical box1 1 {
	    horizontal box11 0 {
		label {disk_slice_label "Disk: [$disk name]   Partition: [$slice name]   Free: "} 0 left
		label {free_space_label "                                        "} 0 left
		stretch 1
	    }
	    listBox {slice_partitions "<none>" 0 $slice_partitions_label} 2 center
	    spacing 1
	    horizontal box12 0 {
		stretch 1
		vertical box121 0 {
		    button {create_button "&Create" [list partition_create [list $disk $slice $dialog]] "normal"} 0 center
		    button {delete_button "&Delete" [list partition_delete [list $disk $slice $dialog]] "normal"} 0 center
		}
		stretch 1
		vertical box122 0 {
		    button {mount_point_button "&Mount Point" [list partition_mount_point [list $disk $slice $dialog]] "normal"} 0 center
		    button {auto_defaults_button "&Auto Defaults for all" [list partition_auto_defaults [list $disk $slice $dialog]] "normal"} 0 center
		}
		stretch 1
		vertical box123 0 {
		    button {toggle_newfs_button "Toggle New &Filesystem" [list partition_toggle_newfs [list $disk $slice $dialog]] "normal"} 0 center
		    button {newfs_options_button "New Filesystem &Options" [list partition_newfs_options [list $disk $slice $dialog]] "normal"} 0 center
		}
		stretch 1
	    }
	    spacing 1
	    horizontal box13 0 {
		stretch 1
		button {done_button "Do&ne" [list partition_done [list $disk $slice $dialog]] "default"} 1 center
		stretch 1
		button {done_button "&Help" [list partition_help [list $disk $slice $dialog]] "normal"} 1 center
		stretch 1
	    }
	}
	spacing 1
    }
}

set partition_help_dialog_desc {
    vertical box {
	textEdit {help $help_text 0 "&Help:"} 1 center
  	spacing 1
	horizontal box11 0 {
	    stretch 1
	    button {ok_button "&Dismiss" {button_click_result OK} "default"} 0 center
	    stretch 1
	}
    }
}

set partition_create_dialog_desc {
    horizontal box {
	spacing 1
	vertical box1 1 {
	    label {free_space_label "Free space: $free_space"} 0 left
	    spacing 1
	    inputLine {partition_size $partition_size 10 "none" "Si&ze (trailing M for megabytes, C for cylinders)"} 0 center
	    spacing 1
	    horizontal box11 0 {
		radioMenu {partition_type {"&File System" "&Swap partition."} 0 "&Partition type"} 1 center
		spacing 1
		vertical box111 1 {
		    inputLine {mount_point $mount_point 255 "none" "&Mount Point"} 0 center
		    stretch 1
		}
	    }
	    stretch 1
	    horizontal box12 0 {
		stretch 1
		button {ok_button "&OK" [list button_click_result "OK"] "default"} 0 center
		stretch 1
		button {cancel_button "&Cancel" [list button_click_result "CANCEL"] "normal"} 0 center
		stretch 1
	    }
	}
	spacing 1
    }
}

set partition_mount_point_dialog_desc {
    horizontal box {
	spacing 1
	vertical box1 1 {
	    inputLine {mount_point $mount_point 255 "none" "&Mount point for the $partition partition"} 0 center
	    stretch 1
	    horizontal box12 0 {
		stretch 1
		button {ok_button "&OK" [list button_click_result "OK"] "default"} 0 center
		stretch 1
		button {cancel_button "&Cancel" [list button_click_result "CANCEL"] "normal"} 0 center
		stretch 1
	    }
	}
	spacing 1
    }
}

set partition_newfs_cmd_dialog_desc {
    horizontal box {
	spacing 1
	vertical box1 1 {
	    inputLine {cmd $cmd 255 "none" "&Newfs command and options for $partition"} 0 center
	    stretch 1
	    horizontal box12 0 {
		stretch 1
		button {ok_button "&OK" [list button_click_result "OK"] "default"} 0 center
		stretch 1
		button {cancel_button "&Cancel" [list button_click_result "CANCEL"] "normal"} 0 center
		stretch 1
	    }
	}
	spacing 1
    }
}

# ----------------------------------------------------------------------

proc make_dialog_layout_interior {dialog box desc preset} {

    foreach arg $preset {set [lindex $arg 0] [lindex $arg 1]}

    while {[llength $desc] > 0} {
	switch -exact [lindex $desc 0] {
	    "stretch" {
		$box addStretch [lindex $desc 1]
		set desc [lreplace $desc 0 1]
	    }
	    "spacing" {
		$box addSpacing [lindex $desc 1]
		set desc [lreplace $desc 0 1]
	    }
	    "strut" {
		$box addStrut [lindex $desc 1]
		set desc [lreplace $desc 0 1]
	    }
	    "vertical" {
		set subbox [$box addVerticalBox [lindex $desc 1] [lindex $desc 2]]
		make_dialog_layout_interior $dialog $subbox [lindex $desc 3] $preset
		set desc [lreplace $desc 0 3]
		H::delete $subbox
	    }
	    "horizontal" {
		set subbox [$box addHorizontalBox [lindex $desc 1] [lindex $desc 2]]
		make_dialog_layout_interior $dialog $subbox [lindex $desc 3] $preset
		set desc [lreplace $desc 0 3]
		H::delete $subbox
	    }
	    default {
		set widget [eval $dialog [lindex $desc 0] [lindex $desc 1]]
		$box addWidget $widget [lindex $desc 2] [lindex $desc 3]
		set desc [lreplace $desc 0 3]
	    }
	}
    }
}

proc make_dialog {dialog desc preset current_name} {

    if {[llength $desc] != 3} {
	error "Invalid dialog description, list of 3 elements expected: layout_type name interior_description"
    }

    switch -exact [lindex $desc 0] {
	"horizontal" {
	    set box [$dialog horizontalBox [lindex $desc 1]]
	}
	"vertical" {
	    set box [$dialog verticalBox [lindex $desc 1]]
	}
	default {
	    error "`vertical' or `horizontal' expected as top-level layout for dialog"
	}
    }

    make_dialog_layout_interior $dialog $box [lindex $desc 2] $preset
    $dialog current [$dialog byName $current_name]
    $box activate
    H::delete $box
    return $dialog
}

# ----------------------------------------------------------------------

proc about_dialog {hui {button {}} {user_data {}}} {

    global about_msg
    message_dialog $hui About $about_msg center {{"~T~ake It" {button_click_result OK} "default"} {"~L~eave It" {button_click_result OK} "normal"}}
}

# ----------------------------------------------------------------------

proc yes_no_dialog {hui title message} {

    message_dialog $hui $title $message left {{"~Y~es" {button_click_result YES} "default"} {"~N~o" {button_click_result NO} "normal"}}
}

# ----------------------------------------------------------------------

proc format_message_for_dialog {hui message} {

    set max_line_length [expr [lindex [$hui size] 0] - 6]

    # Split text into array by "\n"
    set msg {}
    while [string length $message] {
	set index [string first "\n" $message]
	set to_append ""
	if {$index == -1} {
	    set to_append $message
	    set message ""
	} else {
	    set to_append [string range $message 0 [expr $index - 1]]
	    set message [string range $message [expr $index + 1] end]
	}

	while {[string length $to_append] > $max_line_length} {
	    set index [string wordstart $to_append $max_line_length]
	    if {$index == 0} {break}
	    lappend msg [string range $to_append 0 [expr $index - 1]]
	    set to_append [string range $to_append $index end]
	}
	lappend msg $to_append
    }

    return $msg
}

# ----------------------------------------------------------------------

proc message_dialog {hui title message alignment buttons} {

    set objects_references [H::objects_references]

    set msg [format_message_for_dialog $hui $message]

    set minimum_dialog_width [expr [llength $buttons] * 4]
    if {$minimum_dialog_width < 20} {set minimum_dialog_width 20}

    set maximum_dialog_width [expr [lindex [$hui size] 0] - 2]

    set dialog_height [expr [llength $msg] + 5]
    set dialog_width 0
    foreach text $msg {
	if {[string length $text] > $dialog_width} {
	    set dialog_width [string length $text]
	}
    }

    incr dialog_width 2
    if {$dialog_width < $minimum_dialog_width} {set dialog_width $minimum_dialog_width}
    if {$dialog_width > $maximum_dialog_width} {set dialog_width $maximum_dialog_width}

    set dialog_x [expr [expr [lindex [$hui size] 0] - $dialog_width] / 2]
    set dialog_y [expr [expr [lindex [$hui size] 1] - $dialog_height] / 2]

    set main_window [$hui byName main_window]
    set dialog [$hui dialog $main_window message_dialog $title "$dialog_x $dialog_y" "$dialog_width $dialog_height"]

    set dialog_main_box [$dialog horizontalBox dialog_box]
    $dialog_main_box addSpacing 1
    set dialog_box [$dialog_main_box addVerticalBox dialog_box 0]
    $dialog_main_box addSpacing 1

    $dialog_box addStretch 1

    foreach text $msg {
	set label [$dialog label label $text]
	$dialog_box addWidget $label 0 $alignment
    }

    $dialog_box addStretch 1

    set buttons_box [$dialog_box addHorizontalBox buttons_box 0]

    set button_no 1
    set current_button [H::null]
    foreach button $buttons {
	set button_widget [$dialog button button [lindex $button 0] [lindex $button 1] [lindex $button 2]]
	$buttons_box addWidget $button_widget 1
	if {$button_no < [llength $buttons]} {
	    $buttons_box addStretch 2
	}
	if {$button_no == 1} {set current_button $button_widget}
	if {![string compare "default" [lindex $button 2]]} {set current_button $button_widget}
	incr button_no
    }

    $dialog_box addSpacing 1

    if {![H::is_null $current_button]} {
	$dialog current $current_button
    }

    $dialog_main_box activate
    $dialog exec
    set status [$dialog result]

    H::delete $objects_references

    return $status
}

# ----------------------------------------------------------------------

proc make_manubar {hui disks} {

    set menuBar [$hui menuBar]

    set FileMenu [$menuBar addSubmenu b1 &File "File Comment"]
    $FileMenu addButton qfm &Quit {exit_hui 0} "Alt-X" "Quit"

    set DiskMenu [$menuBar addSubmenu options &Disk ""]
    foreach disk $disks {
	$DiskMenu addButton "[$disk name]_button" [$disk name] "browse_disk $disk" "" ""
    }

    $menuBar addSeparator sep

    set HelpMenu [$menuBar addSubmenu help &Help ""]
    $HelpMenu addButton about &About {about_dialog {}} "F1" "Show about message"

}

# ----------------------------------------------------------------------

proc exit_hui {hui button exit_code} {

    $hui exit $exit_code
}

# ----------------------------------------------------------------------

proc button_click_result {hui button user_data} {
    return $user_data;
}

# ----------------------------------------------------------------------

proc browse_disk {hui button disk} {

    make_main_window $hui $disk

}

# ----------------------------------------------------------------------

proc disk_geometry_text {disk} {

    format "%d cyls/%d heads/%d sectors = %d sectors" [$disk bios_cyl] [$disk bios_hd] [$disk bios_sect] [expr [$disk bios_cyl] * [$disk bios_hd] * [$disk bios_sect]]
}

# ----------------------------------------------------------------------

proc make_main_window {hui disk} {

    set main_window [$hui byName main_window]
    if [H::is_null $main_window] {
		if  [H::is_null $disk] {
			set main_window [$hui window main_window "no-disk" "0 0" "80 22"]
		} else {
			set main_window [$hui window main_window [$disk name] "0 0" "80 22"]
		}
    } else {
	$main_window erase
	$main_window title [$disk name]
    }

    if {![H::is_null $disk]} {
	lappend preset [list disk_geometry [disk_geometry_text $disk]]
	set disk_slices_info {}
	for {set slice [[$disk chunks] part]} {![H::is_null $slice]} {set slice [$slice next]} {
	    lappend disk_slices_info [slice_info $slice]
	}
	lappend preset [list disk_slices_items $disk_slices_info]
	set disk_slices_label_format [eval [concat format {{ %%%ds%%%ds%%%ds%%%ds%%%ds%%%ds%%%ds%%%ds}} [disk_slices_formats]]]
	lappend preset [list disk_slices_label [format $disk_slices_label_format "Offset" "Size" "End" "Name" "PType" "Desc" "Subtype" "Flags"]]
	lappend preset [list disk $disk]

	global disk_window_desc
	make_dialog $main_window $disk_window_desc $preset disk_slices
    } else {
    }
    $main_window show
}

# ----------------------------------------------------------------------

proc disk_slices_formats {} {

    return [list 9 10 10 9 7 13 8 6]
}

# ----------------------------------------------------------------------

proc slice_info {slice} {

    set slice_info_format [eval [concat format {{%%%dd%%%dd%%%dd%%%ds%%%dd%%%ds%%%dd%%%ds}} [disk_slices_formats]]]

    return [format $slice_info_format [$slice offset] [$slice size] [$slice end] [$slice name] [$slice type] [$slice slice_type_name] [$slice subtype] [slice_chunk_flags_letters $slice]]
}

# ----------------------------------------------------------------------

proc slice_chunk_flags_letters {slice} {

    set result ""
    set flags [$slice flags]
    if {[lsearch $flags "CHUNK_BSD_COMPAT"] > 0} { set result "${result}C" }
    if {[lsearch $flags "CHUNK_ACTIVE"] > 0} { set result "${result}A" }
    if {[lsearch $flags "CHUNK_ALIGN"] > 0} { set result "${result}=" }
    if {[lsearch $flags "CHUNK_PAST_1024"] > 0} { set result "${result}>" }
    if {[lsearch $flags "CHUNK_IS_ROOT"] > 0} { set result "${result}R" }
    if {[lsearch $flags "CHUNK_BAD144"] > 0} { set result "${result}B" }
    return $result;
}

# ----------------------------------------------------------------------

proc slice_of_disk {disk sliceno} {

    set slice [[$disk chunks] part]
    for {set i 0} {$i < $sliceno && ![H::is_null $slice]} {incr i} {
	set next_slice [$slice next]
	H::delete $slice
	set slice $next_slice
    }
    return $slice
}

# ----------------------------------------------------------------------

proc slice_bootable {hui button disk_and_listbox} {

    set disk [lindex $disk_and_listbox 0]
    set listbox [lindex $disk_and_listbox 1]
    set slice [slice_of_disk $disk [$listbox select]]
    slice_toggle_bootable $hui $slice
    display_disk_slices $hui $listbox $disk
}

# ----------------------------------------------------------------------

proc slice_toggle_bootable {hui slice} {

    set flags [$slice flags]
    set bootable_flag_index [lsearch -exact $flags "CHUNK_ACTIVE"]
    if {$bootable_flag_index >= 0} {
	set flags [lreplace $flags $bootable_flag_index $bootable_flag_index]
    } else {
	lappend flags "CHUNK_ACTIVE"
    }
    $slice flags $flags

}

# ----------------------------------------------------------------------

proc display_disk_slices {hui listbox disk} {

    set selected [$listbox select]
    $listbox removeAll
    set i 0
    for {set slice [[$disk chunks] part]} {![H::is_null $slice]} {set slice [$slice next]; incr i} {
	$listbox add $i [slice_info $slice]
    }
    if {$selected < $i} {
	$listbox select $selected
    }
}

# ----------------------------------------------------------------------

proc slice_delete {hui button disk_and_listbox} {

    set disk [lindex $disk_and_listbox 0]
    set listbox [lindex $disk_and_listbox 1]
    set slice [slice_of_disk $disk [$listbox select]]
    if {![string compare [$slice type_text] "unused"]} {
	message_dialog $hui "" "Slice is already unused!" center {{"~O~K" {button_click_result OK} "default"}}
    } else {
	$disk Delete_Chunk $slice
	display_disk_slices $hui $listbox $disk
    }
}

# ----------------------------------------------------------------------

proc slice_type {hui button disk_and_listbox} {

    set disk [lindex $disk_and_listbox 0]
    set listbox [lindex $disk_and_listbox 1]
    set slice [slice_of_disk $disk [$listbox select]]
    if {![string compare [$slice type_text] "unused"]} {
	message_dialog $hui "" "Slice is currently unused (use create instead)" center {{"~O~K" {button_click_result OK} "default"}}
    } else {
	set type [slice_type_dialog $hui [$slice slice_type_name] [$slice subtype]]
	if [string compare $type "CANCEL"] {
	    $slice set_type $type
	    display_disk_slices $hui $listbox $disk
	}
    }
}

# ----------------------------------------------------------------------

proc slice_type_dialog {hui initial_type_name initial_subtype} {

    set main_window [$hui byName main_window]

    set dialog_objects_references [H::objects_references]

    set standard_types [Disk::standard_slice_types]
    set dialog [$hui dialog $main_window slice_type_dialog "Slice Type" "2 2" "74 15"]
    lappend preset [list slice_types $standard_types] [list initial_slice_type $initial_subtype]
    global slice_type_dialog_desc
    make_dialog $dialog $slice_type_dialog_desc $preset slice_types

    set initial [lsearch -exact $standard_types $initial_type_name]
    if {$initial >= 0 } {
	[$dialog byName slice_types] select $initial
    }
    [$dialog byName slice_types] onSelect [list slice_types_listbox_value_changed [$dialog byName slice_type]]

    $dialog exec
    set result [$dialog result]
    if {[string compare $result "OK"] == 0} {
	set result [[$dialog byName slice_type] text]
    }

    H::delete $dialog_objects_references

    return $result
}

# ----------------------------------------------------------------------

proc slice_types_listbox_value_changed {hui listbox id inputline} {

    $inputline text [Disk::standard_slice_type_id [$listbox text $id]]
}

# ----------------------------------------------------------------------

proc slice_create {hui button disk_and_listbox} {

    set disk [lindex $disk_and_listbox 0]
    set listbox [lindex $disk_and_listbox 1]
    set slice [slice_of_disk $disk [$listbox select]]
    if [string compare [$slice type_text] "unused"] {
	message_dialog $hui "" "Slice in use, delete it first or move to an unused one." center {{"~O~K" {button_click_result OK} "default"}}
    } else {
	set type [slice_create_dialog $hui $slice]
	if [string compare $type "CANCEL"] {
	    $disk Create_Chunk [$slice offset] [lindex $type 1] [lindex $type 0] [concat [$slice flags] "CHUNK_ALIGN"]
	    display_disk_slices $hui $listbox $disk
	}
    }
}

# ----------------------------------------------------------------------

proc slice_create_dialog {hui slice} {

    set main_window [$hui byName main_window]
    set dialog_objects_references [H::objects_references]

    set dialog [$hui dialog $main_window slice_type_dialog "Create Slice" "10 6" "60 8"]
    lappend preset [list initial_slice_size [$slice size]] [list type_id [Disk::standard_slice_type_id "FreeBSD"]]
    global slice_create_dialog_desc
    make_dialog $dialog $slice_create_dialog_desc $preset slice_size
    [$dialog byName type_name] text [Disk::slice_type_name [[$dialog byName type_id] text]]

    $dialog exec
    set result [$dialog result]
    if {[string compare $result "OK"] == 0} {
	set size [[$dialog byName slice_size] text]
	if {[string match "*\[Mm\]" $size]} {
	    global ONE_MEG
	    set size [expr $ONE_MEG * [string trim $size "Mm"]]
	}
	set result [list [[$dialog byName type_id] text] $size]
    }

    H::delete $dialog_objects_references

    return $result
}

# ----------------------------------------------------------------------

proc slice_create_choose_type {hui button dialog} {

    set initial_type_name [[$dialog byName type_name] text]
    set initial_subtype [[$dialog byName type_id] text]
    set type [slice_type_dialog $hui $initial_type_name $initial_subtype]
    if [string compare $type "CANCEL"] {
	[$dialog byName type_id] text $type
	[$dialog byName type_name] text [Disk::slice_type_name $type]
    }
}

# ----------------------------------------------------------------------

proc disk_entire {hui button disk_and_listbox} {

    set disk [lindex $disk_and_listbox 0]
    set listbox [lindex $disk_and_listbox 1]

    set result [yes_no_dialog $hui "" "Do you want to allocate all the disk to use FreeBSD\nwith a true partition entry\nso as to remain cooperative with any future possible\noperating systems on the drive(s)?\n?\n(See also the section about ``dangerously dedicated''\ndisks in the FreeBSD FAQ.)"]
    if {![string compare $result "YES"]} {
	set force_all 1
    } else {
	set force_all 0
    }

    $disk All_FreeBSD $force_all
    display_disk_slices $hui $listbox $disk
}

# ----------------------------------------------------------------------

proc disk_scan {hui button disk_and_listbox} {

    set disk [lindex $disk_and_listbox 0]
    set listbox [lindex $disk_and_listbox 1]
    set slice [slice_of_disk $disk [$listbox select]]

    if [string compare [$slice type_text] "freebsd"] {
	message_dialog $hui "" "Can only scan for bad blocks in FreeBSD slice." center {{"~O~K" {button_click_result OK} "default"}}
    } else {
	if {![string match "sd*" [$disk name]] || ![string match "da*" [$disk name]] || ![string compare "YES" [yes_no_dialog $hui "" "This typically makes sense only for ESDI, IDE or MFM drives.\nAre you sure you want to do this on a SCSI disk?"]]} {
	    disk_toggle_scan_for_bad_blocks $hui $slice
	    display_disk_slices $hui $listbox $disk
	}
    }
}

# ----------------------------------------------------------------------

proc disk_toggle_scan_for_bad_blocks {hui slice} {

    if [string compare [$slice type_text] "freebsd"] {
	message_dialog $hui "" "Can only scan for bad blocks in FreeBSD slice." center {{"~O~K" {button_click_result OK} "default"}}
    } else {
	set flags [$slice flags]
	set scan_flag_index [lsearch -exact $flags "CHUNK_BAD144"]
	if {$scan_flag_index >= 0} {
	    set flags [lreplace $flags $scan_flag_index $scan_flag_index]
	} else {
	    lappend flags "CHUNK_BAD144"
	}
	$slice flags $flags
    }
}

# ----------------------------------------------------------------------

proc disk_geometry {hui button disk_and_listbox} {

    set disk [lindex $disk_and_listbox 0]
    set listbox [lindex $disk_and_listbox 1]
    set label [lindex $disk_and_listbox 2]

    set main_window [$hui byName main_window]
    set dialog_objects_references [H::objects_references]

    set dialog [$hui dialog $main_window disk_geometry_dialog "Disk Geometry" "20 4" "40 11"]
    lappend preset [list cylinders [$disk bios_cyl]] [list heads [$disk bios_hd]] [list sectors [$disk bios_sect]]
    global desk_geometry_dialog_desc
    make_dialog $dialog $desk_geometry_dialog_desc $preset cylinders

    $dialog exec
    set result [$dialog result]
    if {[string compare $result "OK"] == 0} {
	$disk Set_Bios_Geom [[$dialog byName cylinders] text] [[$dialog byName heads] text] [[$dialog byName sectors] text]
	$label text [disk_geometry_text $disk]
	display_disk_slices $hui $listbox $disk
    }

    H::delete $dialog_objects_references
}

# ----------------------------------------------------------------------

proc disk_undo {hui button disk_and_listbox} {

    set disk [lindex $disk_and_listbox 0]
    set listbox [lindex $disk_and_listbox 1]
    set label [lindex $disk_and_listbox 2]

    if {![$disk modified]} {
	message_dialog $hui "" "Cannot undo because information was not modified since read or written to disk" center {{"~O~K" {button_click_result OK} "default"}}
    } else {
	if {![string compare "YES" [yes_no_dialog $hui "" "Are you SURE you want to Undo everything?"]]} {
	    $disk undo
	    $label text [disk_geometry_text $disk]
	    display_disk_slices $hui $listbox $disk
	}
    }
}

# ----------------------------------------------------------------------

proc disk_write {hui button disk} {

    if {![string compare "YES" [message_dialog $hui "Write" "Are you absolutely sure you want to write data on disk?\nThis operation cannot be undone!" center {{"~Y~es" {button_click_result YES} "normal"} {"~N~o" {button_click_result NO} "default"}}]]} {
	if [$disk Write_Disk] {
	    message_dialog $hui "Error" "Unable to write data to disk [$disk name]" center {{"~O~K" {button_click_result OK} "default"}}
	} else {
	    message_dialog $hui "Success" "Wrote FDISK partition information out successfully." center {{"~O~K" {button_click_result OK} "default"}}
	    # scan for bad blocks
	    for {set slice [[$disk chunks] part]} {![H::is_null $slice]} {set slice [$slice next]} {
		if {![string compare [$slice flags] "CHUNK_BAD144"]} {
		    #[$disk scan $slice]
		}
	    }
	}
    }
}

# ----------------------------------------------------------------------

proc slice_partitions {hui button disk_and_listbox} {

    set disk [lindex $disk_and_listbox 0]
    set listbox [lindex $disk_and_listbox 1]
    set slice [slice_of_disk $disk [$listbox select]]

    if [string compare [$slice type_text] "freebsd"] {
	message_dialog $hui "" "Can only partition FreeBSD slices." center {{"~O~K" {button_click_result OK} "default"}}
    } else {
	set main_window [$hui byName main_window]
	set dialog_objects_references [H::objects_references]

	set dialog [$hui dialog $main_window slice_partitions_dialog "Disklabel Editor" "0 0" "78 20"]
	lappend preset [list disk $disk] [list slice $slice]
	lappend preset [list slice_partitions_label [format [eval [concat format {{  %%%ds%%%ds%%%ds%%%ds}} [partition_formats]]] "&Part" "Mount" "Size" "Newfs"]]
	global slice_partitions_dialog_desc
	make_dialog $dialog $slice_partitions_dialog_desc $preset slice_partitions
	display_slice_partitions $hui [$dialog byName slice_partitions] [$dialog byName free_space_label] $slice

	$dialog exec

	display_disk_slices $hui $listbox $disk

	H::delete $dialog_objects_references
    }
}

# ----------------------------------------------------------------------

proc partition_formats {} {

    return [list -10 -25 25 10]
}

# ----------------------------------------------------------------------

proc partition_info {partition} {

    set partition_info_format [eval [concat format {{%%%ds%%%ds%%%ds%%%ds}} [partition_formats]]]
    global ONE_MEG
    if {![string compare [$partition type_text] "part"] && [$partition subtype] == 7} {
	switch -exact [$partition newfs] 1 {set newfs "UFS Yes"} 0 {set newfs "UFS No"} 2 {set newfs "*"}
    } else {
	set newfs ""
    }
    return [format $partition_info_format [$partition name] [$partition mountpoint] "[expr [$partition size] / $ONE_MEG]MB" $newfs]
}

# ----------------------------------------------------------------------

# Returns list of slice's partitions of the specified type: used or
# unused
proc slice_partitions_enumerate {slice type} {

    set used_partitions {}
    set unused_partitions {}
    for {set partition [$slice part]} {![H::is_null $partition]} {set partition [$partition next]} {
	if {![string compare "unused" [$partition type_text]]} {
	    lappend unused_partitions $partition
	} else {
	    lappend used_partitions $partition
	}
    }
    if {$type == "unused"} {
	H::delete $used_partitions
	return $unused_partitions
    } else {
	H::delete $unused_partitions
	return $used_partitions
    }
}

# ----------------------------------------------------------------------

# Erases listbox and inserts used partitions info into it
proc display_slice_partitions {hui listbox free_space_label slice} {

    set partitions [slice_partitions_enumerate $slice used]

    global ONE_MEG
    set selected [$listbox select]
    $listbox removeAll
    for {set i 0} {$i < [llength $partitions]} {incr i} {
	set partition [lindex $partitions $i]
	#puts stderr "([$partition name]) [expr [$partition size] / $ONE_MEG]MB ([$partition type_text]) (mount: [$partition mountpoint]) (flags: [$partition flags]) (is_swap: [$partition is_swap]) (is_fs: [$partition is_fs])"
	$listbox add $i [partition_info $partition]
    }
    H::delete $partitions
    if {$selected >= 0 && $selected < $i} {
	$listbox select $selected
    }

    set free_space [slice_free_space $slice]
    $free_space_label text [format "%d blocks (%dMB)" $free_space [expr $free_space / $ONE_MEG]]
}

# ----------------------------------------------------------------------

# How much space is in this FreeBSD slice?
proc slice_free_space {slice} {

    set size [$slice size]
    set partitions [slice_partitions_enumerate $slice used]
    foreach partition $partitions {
	incr size -[$partition size]
    }
    H::delete $partitions
    return $size
}

# ----------------------------------------------------------------------

# Returns partition of slice by number skiping unused partitions.
# Returns null if number of used partitions less or equal to partno
proc partition_of_slice {slice partno} {

    set partitions [slice_partitions_enumerate $slice used]
    if {[llength $partitions] > $partno} {
	set result [lindex $partitions $partno]
	set partitions [lreplace $partitions $partno $partno]
    } else {
	set result [H::null]
    }
    H::delete $partitions

    return $result
}

# ----------------------------------------------------------------------

proc partition_delete {hui button disk_slice_dialog} {

    set disk [lindex $disk_slice_dialog 0]
    set slice [lindex $disk_slice_dialog 1]
    set main_dialog [lindex $disk_slice_dialog 2]
    set listbox [$main_dialog byName slice_partitions]
    if {[$listbox select] >= 0} {
	set partition [partition_of_slice $slice [$listbox select]]

	if {![string compare "fat" [$partition type]]} {
	    message_dialog $hui "Error" "Use the Disk Partition Editor to delete DOS partitions" center {{"~O~K" {button_click_result OK} "default"}}
	} else {
	    global ONE_MEG
	    #puts stderr "Delete: ([$partition name]) [expr [$partition size] / $ONE_MEG]MB ([$partition type_text]) (mount: [$partition mountpoint]) (flags: [$partition flags]) (is_swap: [$partition is_swap]) (is_fs: [$partition is_fs])\n"
	    $disk Delete_Chunk $partition
	    display_slice_partitions $hui $listbox [$main_dialog byName free_space_label] $slice
	}
    } else {
	message_dialog $hui "Error" "Nothing to delete" center {{"~O~K" {button_click_result OK} "default"}}
    }
    return ""
}

# ----------------------------------------------------------------------

proc partition_create {hui button disk_slice_dialog} {

    set disk [lindex $disk_slice_dialog 0]
    set slice [lindex $disk_slice_dialog 1]
    set main_dialog [lindex $disk_slice_dialog 2]
    set listbox [$main_dialog byName slice_partitions]

    set sz [slice_free_space $slice]
    global ONE_MEG
    if {$sz < $ONE_MEG} {
	message_dialog $hui "Error" "Not enough space to create an additional FreeBSD partition" center {{"~O~K" {button_click_result OK} "default"}}
    } else {
	set partition_parameters [partition_create_dialog $hui $slice $sz $sz ""]
	if {[llength $partition_parameters] == 3} {
	    set new_partition [partition_create_do $hui $disk $slice [lindex $partition_parameters 0] [lindex $partition_parameters 1] [lindex $partition_parameters 2]]
	    H::delete $new_partition
	    display_slice_partitions $hui $listbox [$main_dialog byName free_space_label] $slice
	}
    }

    return ""
}

# ----------------------------------------------------------------------

# Returns created partition or null if problems encountered
proc partition_create_do {hui disk slice size type mountpoint} {

    set flags {}
    set root [expr ![string compare $type "fs"] && ![string compare $mountpoint "/"]]
    if $root {lappend flags "CHUNK_IS_ROOT"}
    set new_partition [$disk Create_Chunk_DWIM $slice $size $type $flags]
    if [partition_create_check_created $hui $new_partition $disk $root] {
	$new_partition mountpoint $mountpoint
    }
    return $new_partition
}

# ----------------------------------------------------------------------

# Returns list of partition parameters (or empty list if user
# cancelled operation): size (in blocks), type ("fs" or "swap"), mount
# point
proc partition_create_dialog {hui slice free_space size mount_point} {

    global ONE_MEG
    set main_window [$hui byName main_window]
    set dialog_objects_references [H::objects_references]

    set dialog [$hui dialog $main_window partition_create_dialog "Create Partition" "5 7" "70 11"]
    lappend preset [list free_space [format "%d blocks (%dMB)" $free_space [expr $free_space / $ONE_MEG]]] [list partition_size $size] [list mount_point $mount_point]
    global partition_create_dialog_desc
    make_dialog $dialog $partition_create_dialog_desc $preset partition_size

    set done 0
    set result_list {}
    while {!$done} {
	$dialog exec

	set result [$dialog result]
	if {[string compare $result "OK"] == 0} {
	    set partition_size [partition_size_convert [[$dialog byName partition_size] text] $slice]
	    if {[[$dialog byName partition_type] set] == 0} {set partition_type "fs"} else {set partition_type "swap"}
	    set mount_point [[$dialog byName mount_point] text]
	    set done [partition_create_check_specified_parameters $hui $slice $partition_size $partition_type $mount_point]
	    if $done { lappend result_list $partition_size $partition_type $mount_point }
	} else {
	    set done 1
	}
    }

    H::delete $dialog_objects_references
    return $result_list
}

# ----------------------------------------------------------------------

proc partition_size_convert {to_convert slice} {

    if {[string match "*\[Mm\]" $to_convert]} {
	global ONE_MEG
	set size [expr $ONE_MEG * [string trim $to_convert "Mm"]]
    } elseif {[string match "*\[Cc\]" $to_convert]} {
	set size [expr [$slice bios_hd] * [$slice bios_sec] * [string trim $to_convert "Cc"]]
    } else {
	set size $to_convert
    }
    return $size
}

# ----------------------------------------------------------------------

# Returns 0 if invalid or 1 if valid
proc partition_create_check_specified_parameters {hui slice partition_size partition_type mount_point} {

    global ONE_MEG ROOT_MIN_SIZE
    if {$partition_size <= $ONE_MEG} {
	message_dialog $hui "Error" [format "The minimum filesystem size is %dMB" 1] center {{"~O~K" {button_click_result OK} "default"}}
	return 0
    }
    set root [expr ![string compare $partition_type "fs"] && ![string compare $mount_point "/"]]
    if $root {
	if {[lsearch [$slice flags] "CHUNK_BSD_COMPAT"] < 0} {
	    message_dialog $hui "Error" "This region cannot be used for your root partition as the\nFreeBSD boot code cannot deal with a root partition created\nin that location. Please choose another location or smaller\nsize for your root partition!" center {{"~O~K" {button_click_result OK} "default"}}
	    return 0
	}
	if {$partition_size < [expr $ROOT_MIN_SIZE * $ONE_MEG]} {
	    message_dialog $hui "Error" "This is smaller than the recommended size for a\nroot partition.  For a variety of reasons, root\npartitions should usually be at least ${ROOT_MIN_SIZE}MB in size" center {{"~O~K" {button_click_result OK} "default"}}
	    return 0
	}
    }
    return 1
}

# ----------------------------------------------------------------------

# Returns 0 if invalid or 1 if valid
proc partition_create_check_created {hui new_partition disk root} {

    if [H::is_null $new_partition] {
	message_dialog $hui "Error" "Unable to create the partition. Too big?" center {{"~O~K" {button_click_result OK} "default"}}
	return 0
    } elseif {$root && [lsearch [$new_partition flags] "CHUNK_PAST_1024"] >= 0} {
	message_dialog $hui "Error" "This region cannot be used for your root partition as it starts\nor extends past the 1024'th cylinder mark and is thus a\npoor location to boot from.  Please choose another\nlocation (or smaller size) for your root partition and try again!" center {{"~O~K" {button_click_result OK} "default"}}
	$disk Delete_Chunk $new_partition
	return 0
    }
    return 1
}

# ----------------------------------------------------------------------

proc partition_help {hui button disk_slice_dialog} {

    set main_window [$hui byName main_window]
    set dialog_objects_references [H::objects_references]

    set dialog [$hui dialog $main_window help_dialog "Help" "0 0" "78 20"]
    lappend preset [list help_text "This is the FreeBSD DiskLabel Editor."]
    global partition_help_dialog_desc
    make_dialog $dialog $partition_help_dialog_desc $preset help

    $dialog exec

    H::delete $dialog_objects_references

    return ""
}

# ----------------------------------------------------------------------

proc partition_done {hui button disk_slice_dialog} {

    return "DONE"
}

# ----------------------------------------------------------------------

proc partition_mount_point {hui button disk_slice_dialog} {

    set disk [lindex $disk_slice_dialog 0]
    set slice [lindex $disk_slice_dialog 1]
    set main_dialog [lindex $disk_slice_dialog 2]
    set listbox [$main_dialog byName slice_partitions]

    if {[$listbox select] >= 0} {
	set partition [partition_of_slice $slice [$listbox select]]

	if [$partition is_swap] {
	    message_dialog $hui "Error" "You don't need to specify a mountpoint for a swap partition." center {{"~O~K" {button_click_result OK} "default"}}
	} elseif {![$partition is_fs]} {
	    message_dialog $hui "Error" "Bogus partition under cursor???" center {{"~O~K" {button_click_result OK} "default"}}
	} else {
	    set main_window [$hui byName main_window]
	    set dialog_objects_references [H::objects_references]

	    set dialog [$hui dialog $main_window help_dialog "Mount Point" "20 8" "40 5"]
	    lappend preset [list mount_point [$partition mountpoint]] [list partition [$partition name]]
	    global partition_mount_point_dialog_desc
	    make_dialog $dialog $partition_mount_point_dialog_desc $preset mount_point

	    set done 0
	    set result_list {}
	    while {!$done} {
		$dialog exec
		if {[string compare [$dialog result] "OK"] == 0} {
		    set mountpoint [[$dialog byName mount_point] text]
		    set done [expr ![string compare $mountpoint [$partition mountpoint]]]
		    if {!$done} {set done [expr ![partition_mount_point_check_conflict $hui $disk $partition $mountpoint]]}
		    if $done {
			if {![string compare "/" $mountpoint]} {
			    set flags [$partition flags]
			    if {[lsearch $flags "CHUNK_IS_ROOT"] < 0} {
				lappend flags "CHUNK_IS_ROOT"
				$partition flags $flags
			    }
			}
			$partition mountpoint $mountpoint
		    }
		} else {
		    set done 1
		}
	    }
	    display_slice_partitions $hui $listbox [$main_dialog byName free_space_label] $slice

	    H::delete $dialog_objects_references
	}
    } else {
	message_dialog $hui "Error" "Nothing to set mount point for" center {{"~O~K" {button_click_result OK} "default"}}
    }
    return ""
}

# ----------------------------------------------------------------------

# returns 1 if there is conflict (other partition uses this mount point)
proc partition_mount_point_check_conflict {hui disk partition mountpoint} {

    set result 0
    set objects_references [H::objects_references]
    global disks
    foreach xdisk $disks {
	for {set slice [[$xdisk chunks] part]} {![H::is_null $slice] && !$result} {set slice [$slice next]} {
	    for {set spartition [$slice part]} {![H::is_null $spartition] && !$result} {set spartition [$spartition next]} {
		set result [expr ![string compare [$spartition mountpoint] $mountpoint]]
	    }
	}
	if $result {break}
    }
    H::delete $objects_references
    if $result {
	message_dialog $hui "Error" "You already have a mount point for $mountpoint assigned!" center {{"~O~K" {button_click_result OK} "default"}}
	return $result
    }

    if {![string match "/*" $mountpoint]} {
	message_dialog $hui "Error" "Mount point must start with a / character" center {{"~O~K" {button_click_result OK} "default"}}
	return 1
    }

    if {![string compare [$partition type_text] "fat"] && [expr ![string compare $mountpoint "/"] ||![string compare $mountpoint "/usr"] || ![string compare $mountpoint "/var"]]} {
	message_dialog $hui "Error" "$mountpoint is an invalid mount point for a DOS partition!" center {{"~O~K" {button_click_result OK} "default"}}
	return 1
    }
    return 0
}

# ----------------------------------------------------------------------

proc partition_toggle_newfs {hui button disk_slice_dialog} {

    set disk [lindex $disk_slice_dialog 0]
    set slice [lindex $disk_slice_dialog 1]
    set main_dialog [lindex $disk_slice_dialog 2]
    set listbox [$main_dialog byName slice_partitions]
    if {[$listbox select] >= 0} {
	set partition [partition_of_slice $slice [$listbox select]]
	$partition newfs [switch [$partition newfs] 0 {format 1} 1 {format 0} 2 {format 1}]
	display_slice_partitions $hui $listbox [$main_dialog byName free_space_label] $slice
    }
    return ""
}

# ----------------------------------------------------------------------

proc partition_newfs_options {hui button disk_slice_dialog} {

    set disk [lindex $disk_slice_dialog 0]
    set slice [lindex $disk_slice_dialog 1]
    set main_dialog [lindex $disk_slice_dialog 2]
    set listbox [$main_dialog byName slice_partitions]
    if {[$listbox select] >= 0} {
	set partition [partition_of_slice $slice [$listbox select]]

	set main_window [$hui byName main_window]
	set dialog_objects_references [H::objects_references]

	set dialog [$hui dialog $main_window help_dialog "Newfs Command" "20 8" "40 5"]
	lappend preset [list cmd [$partition newfs_cmd]] [list partition [$partition name]]
	global partition_newfs_cmd_dialog_desc
	make_dialog $dialog $partition_newfs_cmd_dialog_desc $preset cmd

	$dialog exec
	if {[string compare [$dialog result] "OK"] == 0} {
	    $partition newfs_cmd [[$dialog byName cmd] text]
	}

	display_slice_partitions $hui $listbox [$main_dialog byName free_space_label] $slice
	H::delete $dialog_objects_references
    }
    return ""
}

# ----------------------------------------------------------------------

# Returns list of four partition objects expected to be used as root,
# swap, usr, var. If any partition does not exists (on disks objects)
# returns null element
proc find_standard_mount_partitions {hui} {

    set result [list [H::null] [H::null] [H::null] [H::null]]
    global disks
    foreach disk $disks {
	for {set slice [[$disk chunks] part]} {![H::is_null $slice]} {set next_slice [$slice next]; H::delete $slice; set slice $next_slice} {
	    set partitions [slice_partitions_enumerate $slice used]
	    foreach partition $partitions {
		if [$partition is_swap] {
		    if [H::is_null [lindex $result 1]] {set result [lreplace $result 1 1 $partition]} else {H::delete $partition}
		} elseif [$partition is_fs] {
		    switch [$partition mountpoint] {
			"/"    { if [H::is_null [lindex $result 0]] {set result [lreplace $result 0 0 $partition]} else {H::delete $partition} }
			"/usr" { if [H::is_null [lindex $result 2]] {set result [lreplace $result 2 2 $partition]} else {H::delete $partition} }
			"/var" { if [H::is_null [lindex $result 3]] {set result [lreplace $result 3 3 $partition]} else {H::delete $partition} }
			default {H::delete $partition}
		    }
		} else {
		    H::delete $partition
		}
	    }
	}
    }
    return $result
}

# ----------------------------------------------------------------------

proc partition_auto_defaults {hui button disk_slice_dialog} {

    set disk [lindex $disk_slice_dialog 0]
    set slice [lindex $disk_slice_dialog 1]
    set main_dialog [lindex $disk_slice_dialog 2]
    set listbox [$main_dialog byName slice_partitions]

    set size [slice_free_space $slice]
    global FS_MIN_SIZE
    if {$size < $FS_MIN_SIZE} {
	message_dialog $hui "Error" "Not enough free space to create a new partition in the slice" center {{"~O~K" {button_click_result OK} "default"}}
    } else {
	set objects_references [H::objects_references]
	set allocated [find_standard_mount_partitions $hui]
	if [H::is_null [lindex $allocated 0]] {
	    global ROOT_PARTITION_SIZE
	    set allocated [lreplace $allocated 0 0 [partition_create_do $hui $disk $slice $ROOT_PARTITION_SIZE "fs" "/"]]
	}
	if [H::is_null [lindex $allocated 1]] {
	    global SWAP_PARTITION_SIZE
	    set allocated [lreplace $allocated 1 1 [partition_create_do $hui $disk $slice $SWAP_PARTITION_SIZE "swap" ""]]
	}
	if [H::is_null [lindex $allocated 3]] {
	    global VAR_PARTITION_MIN_SIZE
	    set allocated [lreplace $allocated 3 3 [partition_create_do $hui $disk $slice $VAR_PARTITION_MIN_SIZE "fs" "/var"]]
	}
	if [H::is_null [lindex $allocated 2]] {
	    set allocated [lreplace $allocated 2 2 [partition_create_do $hui $disk $slice [slice_free_space $slice] "fs" "/usr"]]
	}
	display_slice_partitions $hui $listbox [$main_dialog byName free_space_label] $slice
	H::delete $objects_references
    }
    return ""
}

# ----------------------------------------------------------------------

#  proc partition_undo {hui button disk_slice_dialog} {

#      set disk [lindex $disk_slice_dialog 0]
#      set slice [lindex $disk_slice_dialog 1]
#      set main_dialog [lindex $disk_slice_dialog 2]
#      set listbox [$main_dialog byName slice_partitions]

#      if {![string compare "YES" [yes_no_dialog $hui "" "Are you SURE you want to Undo all the modifications made in this slice partitioning?"]]} {
#  	global slice_partitions_undo_info
#  	if {[llength $slice_partitions_undo_info] == 2} {

#  	    # delete partitions of the slice
#  	    set partitions [slice_partitions_enumerate used]
#  	    foreach partition $partitions {$disk Delete_Chunk $partition}
#  	    H::delete $partitions

#  	    # create new partitions out of undo information
#  	    # ???

#  	    display_slice_partitions $hui $listbox [$main_dialog byName free_space_label] $slice
#  	} else {
#  	    message_dialog $hui "Error" "Undo information is not available" center {{"~O~K" {button_click_result OK} "default"}}
#  	}
#      }
#      return ""
#  }

# ----------------------------------------------------------------------

set hui [Hui::open $hui_type]

set disks {}

foreach disk_name [Disk::Disk_Names] {
    lappend disks [Disk $disk_name]
}

make_manubar $hui $disks
if {[llength $disks] > 0} {
    make_main_window $hui [lindex $disks 0]
} else {
    make_main_window $hui [H::null]
}

$hui run
H::delete $disks

# ----------------------------------------------------------------------
