/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/file/Archive.cd.cc,v 1.4 2001/06/20 10:10:29 alex Exp $
 */

// -*- C++ -*-   Wed Dec 23 1998
// Eugene Skepner <eu@qub.com>
// $Id: Archive.cd.cc,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef Archive_hh
#include "Archive.hh"
#endif

//======================================================================

LanguageInterface::Object::MethodDescription Archive::sMethods[] =
{
	MethodDescription( "open", "open",
				LanguageInterface::Object::MethodDescription::fStatic,
				"Opens archive for reading",
				MethodDescription::Value( MethodDescription::vtObject, "Archive" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "url" ) ),
	MethodDescription( "create", "create",
				LanguageInterface::Object::MethodDescription::fStatic | LanguageInterface::Object::MethodDescription::fSafeExecution,
				"Creates new archive",
				MethodDescription::Value( MethodDescription::vtObject, "Archive" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "url" ) ),
	MethodDescription( "url", "url",
				LanguageInterface::Object::MethodDescription::fDynamic,
				"Returns URL of the archive.",
				MethodDescription::vtString, MethodDescription::vtVoid ),
	MethodDescription( "directory", "directory",
				LanguageInterface::Object::MethodDescription::fDynamic,
				"Returns list of files currently in the archive.",
				MethodDescription::Value( MethodDescription::vtList, "list<string>" ), MethodDescription::vtVoid ),
	MethodDescription( "numberOfFiles", "numberOfFiles",
				LanguageInterface::Object::MethodDescription::fDynamic,
				"Returns number of files currently in the archive.",
				MethodDescription::vtUInt, MethodDescription::vtVoid ),
	MethodDescription( "add", "add",
				LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fSafeExecution,
				"Adds file to the archive.",
				MethodDescription::vtVoid,
				MethodDescription::Value( MethodDescription::vtString, 0, 0, "pathname" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "pathname_to_record" ) ),
	MethodDescription( "size", "size",
				LanguageInterface::Object::MethodDescription::fDynamic,
				"Returns size of the file stored in the archive.",
				MethodDescription::vtULong, MethodDescription::Value( MethodDescription::vtString, 0, 0, "file" ) ),
	MethodDescription( "extract", "extract",
				LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fSafeExecution,
				"Extracts the file and puts it on disk, overwrite existing file.",
				MethodDescription::vtVoid,
				MethodDescription::Value( MethodDescription::vtString, 0, 0, "file_to_extract" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "pathname_to_store" ) ),
	MethodDescription( "extract_append", "extract_append",
				LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fSafeExecution,
				"Extracts the file and puts it on disk, appends if file already exists.",
				MethodDescription::vtVoid,
				MethodDescription::Value( MethodDescription::vtString, 0, 0, "file_to_extract" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "pathname_to_store" ), MethodDescription::Value( MethodDescription::vtBool, 0, 0, "create" ) ),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Archive::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Archive::sClassDescription(
	"Archive", "Archive",
	"Interface to ziplib",
	sMethods,
	sCallbacks
);

//======================================================================
