/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/file/Archive.hh,v 1.3 2001/06/20 10:10:29 alex Exp $
 */

// -*- C++ -*-  Wed Dec 23 1998
// Eugene Skepner <eu@qub.com>
// $Id: Archive.hh,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef Archive_hh
#define Archive_hh

#ifndef LanguageInterface_hh
#include "LanguageInterface.hh"
#endif

#ifndef File_hh
#include "File.hh"
#endif

#ifndef Container_hh
#include "Container.hh"
#endif

#ifndef ziplib_hh
#include "ziplib.hh"
#endif

#include <list>

//======================================================================

class Archive : virtual public LanguageInterface::Object
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     Archive();
     Archive( const string& aUrl, bool aCreate );
     virtual ~Archive();

       // Returns list of files currently in the archive
     virtual Pointer<Container<list<string> > > directory();

       // Returns number of files currently in the archive
     virtual size_t numberOfFiles();

       // Adds file to the archive (if it's writable)
     virtual void add( const string& pathname, const string& pathname_to_record );

       // Returns size of the file stored in the archive
     virtual unsigned long size( const string& file );

       // Extracts the file and puts it on disk, overwrite existing file
     virtual void extract( const string& file_to_extract, const string& pathname_to_store );
     virtual void extract_append( const string& file_to_extract, const string& pathname_to_append, bool create );

       // Extracts the file into memory
     virtual void* extract( const string& file_to_extract, unsigned long& length );

     virtual const string& url() const { return mStreamFile.url(); }

     static Pointer<Archive> open( const string& aUrl );
     static Pointer<Archive> create( const string& aUrl );

  private:
     File mStreamFile;
     Zip mZip;

}; // class Archive

//======================================================================

#endif // Archive_hh
