/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/file/DirectoriesAccess.hh,v 1.3 2001/06/20 10:10:30 alex Exp $
 */

// -*- C++ -*-  Wed Dec 23 1998
// Eugene Skepner <eu@qub.com>
// $Id: DirectoriesAccess.hh,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef DirectoriesAccess_hh
#define DirectoriesAccess_hh

#ifndef LanguageInterface_hh
#include "LanguageInterface.hh"
#endif

#include <list>

//======================================================================

class DirectoriesAccess : virtual public LanguageInterface::Object
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     DirectoriesAccess();
     virtual ~DirectoriesAccess();

     typedef list<string> List;

     virtual const List& deny() const { return mDeny; }
     virtual void deny( const List& aDeny );
     virtual const List& allow() const { return mAllow; }
     virtual void allow( const List& aAllow );

     virtual bool accessAllowed( const string& aDir ) const;

     static Pointer<DirectoriesAccess> instance();

  protected:
     virtual void normalize( List& aDirs );

  private:
     List mDeny;
     List mAllow;

     enum DirComparison { dcNoRelated, dcEqual, dcSubdirectory /* aAsked is a subdirectory of aMain */ };

     virtual DirComparison compare( const string& aAsked, const string& aMain ) const;

     static Pointer<DirectoriesAccess> sDirectoriesAccess;

}; // class DirectoriesAccess

//======================================================================

#endif // DirectoriesAccess_hh
