/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/file/Dirname.cc,v 1.5 2001/06/20 10:10:30 alex Exp $
 */

// -*- C++ -*-   Mon Dec 21 1998
// Eugene Skepner <eu@qub.com>
// $Id: Dirname.cc,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef File_hh
#include "File.hh"
#endif

//======================================================================

string File::dirname( const string& path )
{
	char* name;
	name = strdup( path.c_str() );

	char* p = name;

	/*
	* (1) If string is //, skip steps (2) through (5).
	* (2) If string consists entirely of slash characters, string
	*	shall be set to a single slash character.  In this case,
	*	skip steps (3) through (8).
	*/
	for ( ;; ++p ) {
		if (!*p) {
			if ( p > name )
				return("/");
			else
				return(".");
		}
		if ( *p != '/' )
			break;
	}

	/*
	* (3) If there are any trailing slash characters in string, they
	*	shall be removed.
	*/
	for ( ; *p; ++p );
	while ( *--p == '/' )
		continue;
	*++p = '\0';

	/*
	* (4) If there are no slash characters remaining in string,
	*	string shall be set to a single period character.  In this
	*	case skip steps (5) through (8).
	*
	* (5) If there are any trailing nonslash characters in string,
	*	they shall be removed.
	*/
	while ( --p >= name )
		if ( *p == '/' )
			break;
	++p;
	if ( p == name )
		return(".");

	/*
	* (6) If the remaining string is //, it is implementation defined
	*	whether steps (7) and (8) are skipped or processed.
	*
	* This case has already been handled, as part of steps (1) and (2).
	*/

	/*
	* (7) If there are any trailing slash characters in string, they
	*	shall be removed.
	*/
	while ( --p >= name )
		if ( *p != '/' )
			break;
	++p;

	/*
	* (8) If the remaining string is empty, string shall be set to
	*	a single slash character.
	*/
	*p = '\0';
	if( p == name )
		return("/");
	else {
	string s( name );
	free( name );
	return s;
	}
} // dirname

//======================================================================

string File::basename( const string& path, const string& suffix )
{
	char* name;
	name = strdup( path.c_str() );

	char* p = name;

	/*
	* (1) If string is // it is implementation defined whether steps (2)
	*	through (5) are skipped or processed.
	*
	* (2) If string consists entirely of slash characters, string shall
	*	be set to a single slash character.  In this case, skip steps
	*	(3) through (5).
	*/
	for ( ;; ++p ) {
		if (!*p) {
			if ( p > name )
				return("/");
			else
				return "";
		}
		if ( *p != '/' )
			break;
	}

	/*
	* (3) If there are any trailing slash characters in string, they
	*	shall be removed.
	*/
	for ( ; *p; ++p )
		continue;
	while ( *--p == '/' )
		continue;
	*++p = '\0';

	/*
	* (4) If there are any slash characters remaining in string, the
	*	prefix of string up to an including the last slash character
	*	in string shall be removed.
	*/
	while ( --p >= name )
		if (*p == '/')
			break;
	++p;

	/*
	* (5) If the suffix operand is present, is not identical to the
	*	characters remaining in string, and is identical to a suffix
	*	of the characters remaining in string, the suffix suffix
	*	shall be removed from string.
	*/
	if ( suffix.length() > 0 ) {
		const int suffixlen = suffix.length();
		const int stringlen = strlen( p );
		if ( suffixlen < stringlen ) {
			const int off = stringlen - suffixlen;
			if ( !strcmp( p + off, suffix.c_str() ) )
				p[off] = '\0';
		}
	}
	string s( p );
	free( name );
	return s;

} // basename

//======================================================================
