/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/file/Fetch.hh,v 1.3 2001/06/20 10:10:30 alex Exp $
 */

// -*- C++ -*-  Wed Dec 23 1998
// Eugene Skepner <eu@qub.com>
// $Id: Fetch.hh,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef Fetch_hh
#define Fetch_hh

#ifndef LanguageInterface_hh
#include "LanguageInterface.hh"
#endif

#ifndef Container_hh
#include "Container.hh"
#endif

#include <list>
#include <sys/param.h>
#include <cstdarg>

//======================================================================

class Fetch : virtual public LanguageInterface::Object
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     Fetch();
     virtual ~Fetch();

     struct Stat
     {
          off_t size;
          time_t time;
          enum Type { Regular, Directory, Other } type;
     };

     virtual bool isRemote( const string& aUrl );
     virtual void stat( const string& aUrl, Stat& aStat );
     virtual Pointer<Container<list<string> > > directory( const string& aUrl );
     virtual void download( const string& aUrl, const string& aLocalName );

     static Pointer<Fetch> instance();

  private:
     virtual bool hasSchemeSpecifier( const string& aUrl );
     virtual string addSchemeSpecifier( const string& aUrl );

     static void com_err_hook( const char* whoami, long code, const char* format, va_list args );
     static bool sComErrHookInstalled;
     static string sComErr;

     static Pointer<Fetch> sFetch;

}; // class Fetch

//======================================================================

#endif // Fetch_hh
