/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/file/File.hh,v 1.4 2001/07/17 20:35:06 alex Exp $
 */

// -*- C++ -*-  Wed Dec 23 1998
// Eugene Skepner <eu@qub.com>
// $Id: File.hh,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef File_hh
#define File_hh

#ifndef LanguageInterface_hh
#include "LanguageInterface.hh"
#endif

#ifndef Container_hh
#include "Container.hh"
#endif

//======================================================================

class RealFile;

//======================================================================

class File : virtual public LanguageInterface::Object
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     File();
     File( const string& aUrl );
     virtual ~File();

       // If file is remote it is downloaded into directory for
       // temporary file. Returns local file name. If
       // remove_on_close and file was downloaded it is removed upon
       // closing.
     virtual string open_local( bool remove_on_close );
     virtual void close();
     virtual FILE* open_for_reading( bool remove_local_copy_on_close );
     virtual FILE* create();

     const string& url() const { return mUrl; }
     bool isLocal() const;
     string localName() const; // returns an empty string if file is remote

     static string md5( const string& pathname );
     static string dirname( const string& path );
     static string basename( const string& path, const string& suffix );
     static void patch( const string& file_to_patch, const void* patch_data, size_t patch_length, bool reverse = false );
     static int exec( const string& aCommand );
     static void make_path( const string& path );
     static bool exist( const string& aPathname );

       // Returns list of files in the specified directory and all
       // subdirectories (if aRecurseIntoSubdirs), includes entries
       // for subdirectories if aIncludeDirs
     static Pointer<Container<list<Pointer<RealFile> > > > scan( const string& aDir, bool aRecurseIntoSubdirs, bool aIncludeDirs, Pointer<Container<list<string> > > aIgnoreNames );

  private:
     string mUrl;
     mutable string mLocalName;
     bool mRemoveOnClose;
     FILE *mFile;

     static void read_directory( const string& aDir, bool aRecurseIntoSubdirs, bool aIncludeDirs, Pointer<Container<list<string> > > aIgnoreNames, list<Pointer<RealFile> >& files );
     static void add_file( const string& aName, const string& aDir, bool aRecurseIntoSubdirs, bool aIncludeDirs, Pointer<Container<list<string> > > aIgnoreNames, list<Pointer<RealFile> >& files );

}; // class File

//======================================================================

#endif // File_hh
