/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/file/FileInfo.cc,v 1.4 2001/06/20 10:10:31 alex Exp $
 */

// -*- C++ -*-   Mon Dec 21 1998
// Eugene Skepner <eu@qub.com>
// $Id: FileInfo.cc,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef FileInfo_hh
#include "FileInfo.hh"
#endif

#ifndef RealFile_hh
#include "RealFile.hh"
#endif

//======================================================================

FileInfo::FileInfo( const RealFile& aRealFile )
	: mPathname( aRealFile.pathname() ), mLength( aRealFile.length() ), mMd5( aRealFile.md5() ),
	mMtime( aRealFile.mtime() ), mUser( aRealFile.user() ), mGroup( aRealFile.group() ),
	mMode( aRealFile.mode() ), mSymlink( aRealFile.symlink() ), mDevice( aRealFile.device() )
{
} // FileInfo::FileInfo

//======================================================================

FileInfo::FileInfo( const string& aPathname, const RealFile& aRealFile )
	: mPathname( aPathname ), mLength( aRealFile.length() ), mMd5( aRealFile.md5() ),
	mMtime( aRealFile.mtime() ), mUser( aRealFile.user() ), mGroup( aRealFile.group() ),
	mMode( aRealFile.mode() ), mSymlink( aRealFile.symlink() ), mDevice( aRealFile.device() )
{
} // FileInfo::FileInfo

//======================================================================

FileInfo::FileInfo( const string& aPathname, unsigned long aLength, time_t aMtime, const string& aUser, const string& aGroup, mode_t aMode, const string& aSymlink, dev_t aDevice, string aMd5 )
	: mPathname( aPathname ), mLength( aLength ), mMd5( aMd5 ), mMtime( aMtime ), mUser( aUser ),
	mGroup( aGroup ), mMode( aMode ), mSymlink( aSymlink ), mDevice( aDevice )
{
} // FileInfo::FileInfo

//======================================================================

void FileInfo::device( const vector<long>& aDevice )
{
	if ( aDevice.size() != 2 )
		mDevice = makedev( 0, 0 );
	else
		mDevice = makedev( aDevice[0], aDevice[1] );

} // FileInfo::device

//======================================================================

Pointer<Container<vector<long> > > FileInfo::device() const
{
	Pointer<Container<vector<long> > > result;
	if ( major( mDevice ) == 0 ) {
		result = new Container<vector<long> >;
	}
	else {
		result = new Container<vector<long> >( 2 );
		(*result)[0] = major( mDevice );
		(*result)[1] = minor( mDevice );
	}
	return(result);

} // FileInfo::device

//======================================================================
