/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/file/FileInfo.cd.cc,v 1.4 2001/06/20 10:10:31 alex Exp $
 */

// -*- C++ -*-   Mon Dec 21 1998
// Eugene Skepner <eu@qub.com>
// $Id: FileInfo.cd.cc,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef FileInfo_hh
#include "FileInfo.hh"
#endif

//======================================================================

LanguageInterface::Object::MethodDescription FileInfo::sMethods[] =
{
	MethodDescription( "FileInfo", "make",
				LanguageInterface::Object::MethodDescription::fStatic,
				"Returns newly created FileInfo object",
				MethodDescription::Value( MethodDescription::vtObject, "FileInfo" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "pathname" ), MethodDescription::Value( MethodDescription::vtULong, 0, 0, "length" ), MethodDescription::Value( MethodDescription::vtULong, 0, 0, "mtime" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "user" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "group" ), MethodDescription::Value( MethodDescription::vtUInt, 0, 0, "mode" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "symlink" ), MethodDescription::Value( MethodDescription::vtList, "vector<long>", 0, "device" ), MethodDescription::Value( MethodDescription::vtString, 0, "string()", "md5" ) ),
	MethodDescription( "pathname", "pathname",
				LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
				"Sets or gets pathname of the file",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "optional-get-or-set" ) ),
	MethodDescription( "length", "length",
				LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
				"Sets or gets",
				MethodDescription::vtULong, MethodDescription::Value( MethodDescription::vtULong, 0, "optional-get-or-set" ) ),
	MethodDescription( "md5", "md5",
				LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
				"Sets or gets",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "optional-get-or-set" ) ),
	MethodDescription( "mtime", "mtime",
				LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
				"Sets or gets",
				MethodDescription::vtLong, MethodDescription::Value( MethodDescription::vtLong, 0, "optional-get-or-set" ) ),
	MethodDescription( "user", "user",
				LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
				"Sets or gets",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "optional-get-or-set" ) ),
	MethodDescription( "group", "group",
				LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
				"Sets or gets",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "optional-get-or-set" ) ),
	MethodDescription( "mode", "mode",
				LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
				"Sets or gets file permissions",
				MethodDescription::vtUInt, MethodDescription::Value( MethodDescription::vtUInt, 0, "optional-get-or-set" ) ),
	MethodDescription( "symlink", "symlink",
				LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
				"Sets or gets",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "optional-get-or-set" ) ),
	MethodDescription( "dev", "dev",
				LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
				"Gets or sets dev_t",
				MethodDescription::vtULongLong, MethodDescription::Value( MethodDescription::vtULongLong, 0, "optional-get-or-set" ) ),
	MethodDescription( "device", "device",
				LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
				"Sets or gets",
				MethodDescription::Value( MethodDescription::vtList, "vector<long>" ), MethodDescription::Value( MethodDescription::vtList, "vector<long>", "optional-get-or-set", "device_major_minor" ) ),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription FileInfo::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription FileInfo::sClassDescription(
	"FileInfo", "FileInfo",
	"File stats",
	sMethods,
	sCallbacks
);

//======================================================================
