/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/file/FileInfo.hh,v 1.3 2001/06/20 10:10:31 alex Exp $
 */

// -*- C++ -*-  Mon Dec 21 1998
// Eugene Skepner <eu@qub.com>
// $Id: FileInfo.hh,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef FileInfo_hh
#define FileInfo_hh

#ifndef LanguageInterface_hh
#include "LanguageInterface.hh"
#endif

#ifndef Container_hh
#include "Container.hh"
#endif

#include <sys/types.h>
#include <vector>

//======================================================================

class RealFile;

//======================================================================

class FileInfo : virtual public LanguageInterface::Object
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     FileInfo( const RealFile& aRealFile );
     FileInfo( const string& aPathname, const RealFile& aRealFile );
     FileInfo( const string& aPathname, unsigned long aLength, time_t aMtime, const string& aUser, const string& aGroup, mode_t aMode, const string& aSymlink, dev_t aDevice, string aMd5 = string() );

     static Pointer<FileInfo> make( const string& aPathname, unsigned long aLength, time_t aMtime, const string& aUser, const string& aGroup, mode_t aMode, const string& aSymlink, const vector<long>& aDevice, string aMd5 )
          { FileInfo* fi = new FileInfo( aPathname, aLength, aMtime, aUser, aGroup, aMode, aSymlink, 0, aMd5 ); fi->device( aDevice ); return fi; }

     const string& pathname() const { return mPathname; }
     void pathname( const string& aPathname ) { mPathname = aPathname; }
     unsigned long length() const { return mLength; }
     void length( unsigned long aLength ) { mLength = aLength; }
     const string& md5() const { return mMd5; }
     void md5( const string& aMd5 ) { mMd5 = aMd5; }
     time_t mtime() const { return mMtime; }
     void mtime( time_t aMtime ) { mMtime = aMtime; }
     const string& user() const { return mUser; }
     void user(const string&  aUser ) { mUser = aUser; }
     const string& group() const { return mGroup; }
     void group( const string& aGroup ) { mGroup = aGroup; }
     mode_t mode() const { return mMode; }
     void mode( mode_t aMode ) { mMode = aMode; }
     const string& symlink() const { return mSymlink; }
     void symlink( const string& aSymlink ) { mSymlink = aSymlink; }
     dev_t dev() const { return mDevice; }
     void dev( dev_t aDevice ) { mDevice = aDevice; }
     Pointer<Container<vector<long> > > device() const;
     void device( const vector<long>& aDevice );

  private:
     string mPathname;
     unsigned long mLength;
     string mMd5;
     time_t mMtime;
     string mUser;
     string mGroup;
     mode_t mMode;
     string mSymlink;
     dev_t mDevice;

}; // class FileInfo

//======================================================================

#endif // FileInfo_hh
