/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/file/RealFile.cd.cc,v 1.4 2001/06/20 10:10:31 alex Exp $
 */

// -*- C++ -*-   Mon Dec 21 1998
// Eugene Skepner <eu@qub.com>
// $Id: RealFile.cd.cc,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef RealFile_hh
#include "RealFile.hh"
#endif

//======================================================================

LanguageInterface::Object::MethodDescription RealFile::sMethods[] =
{
	MethodDescription( "pathname", "pathname",
				LanguageInterface::Object::MethodDescription::fDynamic,
				"Gets pathname of the existing file",
				MethodDescription::vtString, MethodDescription::vtVoid ),
	MethodDescription( "length", "length",
				LanguageInterface::Object::MethodDescription::fDynamic,
				"Returns length of the file",
				MethodDescription::vtULong, MethodDescription::vtVoid ),
	MethodDescription( "md5", "md5",
				LanguageInterface::Object::MethodDescription::fDynamic,
				"Returns MD5 checksum of the file",
				MethodDescription::vtString, MethodDescription::vtVoid ),
	MethodDescription( "mtime", "mtime",
				LanguageInterface::Object::MethodDescription::fDynamic,
				"Returns time of the last modification of the file",
				MethodDescription::vtLong, MethodDescription::vtVoid ),
	MethodDescription( "user", "user",
				LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fSafeExecution | LanguageInterface::Object::MethodDescription::fGetOrSet,
				"Sets or gets",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "optional-get-or-set" ) ),
	MethodDescription( "group", "group",
				LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fSafeExecution | LanguageInterface::Object::MethodDescription::fGetOrSet,
				"Sets or gets",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "optional-get-or-set" ) ),
	MethodDescription( "mode", "mode",
				LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fSafeExecution | LanguageInterface::Object::MethodDescription::fGetOrSet,
				"Sets or gets",
				MethodDescription::vtUInt, MethodDescription::Value( MethodDescription::vtUInt, 0, "optional-get-or-set" ) ),
	MethodDescription( "symlink", "symlink",
				LanguageInterface::Object::MethodDescription::fDynamic,
				"Returns the contents of symlink if file is a symbolic link",
				MethodDescription::vtString, MethodDescription::vtVoid ),
	MethodDescription( "dev", "dev",
				LanguageInterface::Object::MethodDescription::fDynamic,
				"Returns dev_t",
				MethodDescription::vtULongLong, MethodDescription::vtVoid ),
	MethodDescription( "device", "device",
				LanguageInterface::Object::MethodDescription::fDynamic,
				"Returns device major and minor",
				MethodDescription::Value( MethodDescription::vtList, "vector<long>" ), MethodDescription::vtVoid ),
	MethodDescription( "filesystem", "filesystem",
				LanguageInterface::Object::MethodDescription::fDynamic,
				"Returns the mount point of a filesystem the file resides on",
				MethodDescription::vtString, MethodDescription::vtVoid ),
	MethodDescription( "inode", "inode",
				LanguageInterface::Object::MethodDescription::fDynamic,
				"Returns inode number of the file",
				MethodDescription::vtString, MethodDescription::vtVoid ),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription RealFile::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription RealFile::sClassDescription(
	"RealFile", "RealFile",
	"Existing file stats",
	sMethods,
	sCallbacks
);

//======================================================================
