/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/file/RealFile.hh,v 1.3 2001/06/20 10:10:31 alex Exp $
 */

// -*- C++ -*-  Mon Dec 21 1998
// Eugene Skepner <eu@qub.com>
// $Id: RealFile.hh,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef RealFile_hh
#define RealFile_hh

#ifndef LanguageInterface_hh
#include "LanguageInterface.hh"
#endif

#ifndef Container_hh
#include "Container.hh"
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <vector>

//======================================================================

class RealFile : virtual public LanguageInterface::Object
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     RealFile( const string& aPathname );
     RealFile( const string& aPathname, struct stat aStat );

     const string& pathname() const { return mPathname; }
     unsigned long length() const { return mStat.st_size; }
     string md5() const;
     time_t mtime() const { return mStat.st_mtime; }
     void mtime( time_t aTime );
     string user() const;
     void user( const string& aUser );
     string group() const;
     void group( const string& aGroup );
     mode_t mode() const { return mStat.st_mode; }
     void mode( mode_t aMode );
     string symlink() const;
     dev_t dev() const { return ( S_ISCHR( mStat.st_mode ) || S_ISBLK( mStat.st_mode ) ) ? mStat.st_rdev : 0; }
     Pointer<Container<vector<long> > > device() const;

     string filesystem() const;
     string inode() const;

  private:
     string mPathname;
     struct stat mStat;

}; // class RealFile

//======================================================================

#endif // RealFile_hh
