/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/Hui.cc,v 1.6 2001/06/20 10:24:44 alex Exp $
 */

// -*- C++ -*-   Tue Dec  1 1998
// Eugene Skepner <eu@qub.com>
// $Id: Hui.cc,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef Hui_hh
#include "Hui.hh"
#endif

#ifdef USE_Qt
#ifndef HuiQtNew_hh
#include "HuiQtNew.hh"
#endif
#endif

#ifdef TVision
#ifndef HuiTVisionNew_hh
#include "HuiTVisionNew.hh"
#endif
#endif

#include <cstdlib>

//======================================================================

Hui::Hui()
{

} // Hui::Hui

//======================================================================

Hui::~Hui()
{

} // Hui::~Hui

//======================================================================

Pointer<Hui> Hui::open( const string& aType )
{
	Type t;
	if ( aType == "auto" )
		t = Hui::tAuto;
	else if ( aType == "text" )
		t = Hui::tText;
	else if ( aType == "graphics" )
		t = Hui::tGraphics;
	else
		throw Error( string( "Wrong Hui type `" ) + aType + "' (possible values: auto, text, graphics)" );

	return(open( t ));

} // Hui::open

//======================================================================

Pointer<Hui> Hui::open( Type type )
{
#ifdef USE_Qt
	if ( type == tAuto ) {
		if ( getenv( "DISPLAY" ) )
			type = tGraphics;
		else
			type = tText;
	}
#else
	type = tText;
#endif

	Pointer<Hui> hui;
	switch ( type ) {
	case tAuto:
		break;
		case tText:
#ifdef TVision
		hui = newHuiTVision();
#else
		throw Error( "Hui Text type is not supported by this build of the library" );
#endif
		break;
	case tGraphics:
#ifdef USE_Qt
		{
		char* argv[] = { (char*)"Error:", (char*)"-geometry", (char*)"+0+0" /*"500x300"*/ };
		hui = newHuiQt( sizeof( argv ) / sizeof( argv[0] ), argv );
		}
#else
		throw Error( "Hui Graphics type is not supported by this build of the library" );
#endif
		break;
	}

#if 0
	if ( hui && !strcmp( progressIndicatorStack->objecttypename(), "HuiProgressIndicatorStack" ) )
	dynamic_cast<HuiProgressIndicatorStack*>( progressIndicatorStack )->hui( hui );
#endif
	return(hui);

} // Hui::open

//======================================================================

Hui::Size& Hui::Size::operator += ( const Hui::Position& aPosition )
{
	if ( width() != maxWidth() )
		width( width() + aPosition.x() );
	if ( height() != maxHeight() )
		height( height() + aPosition.y() );

	return(*this);
} // Hui::Size::operator +=

//======================================================================

Hui::Size& Hui::Size::operator -= ( const Hui::Position& aPosition )
{
	if ( width() != maxWidth() )
		width( width() - aPosition.x() );
	if ( height() != maxHeight() )
		height( height() - aPosition.y() );

	return(*this);
} // Hui::Size::operator +=

//======================================================================

Hui::Widget::Widget( Hui& aHui, const string& aName )
: mName( aName ), mOnEnter( aHui, *this, 0 ), mOnLeave( aHui, *this, 0 )
{

} // Hui::Widget::Widget

//======================================================================

string Hui::Widget::name( const string& aName )
{
	if ( aName.length() == 1 && aName[0] == '\0' ) {
		return(mName);
	} else {
		string oldName = mName;
		mName = aName;
		return(oldName);
	}
} // Hui::Widget::name

//======================================================================

void Hui::Widget::enter()
{
	mOnEnter.activate( false );
} // Hui::Widget::enter

//======================================================================

void Hui::Widget::leave()
{
	mOnLeave.activate( false );
} // Hui::Widget::leave

//======================================================================

const string& Hui::WidgetCallbackData::activate( bool call_callback_completed )
{
	try {
		if ( callbackData() ) {
			callbackData()->activate( *this );
			if ( call_callback_completed )
				widget()->callback_completed( *this );
		}
	} catch ( exception& err ) {
		LOG( ERROR, "Hui::WidgetCallbackData::activate: " << err.what() );
	}
	return(mResult);
} // Hui::WidgetCallbackData::activate

//======================================================================

Pointer<Hui::Widget> Hui::MenuSubmenuWidget::byName( const string& aName ) const
{
	Pointer<Hui::Widget> result = 0;
	for ( Items::const_iterator i = mItems.begin(); i != mItems.end() && !result; ++i ) {
		result = (*i)->byName( aName );
	}
	return(result);
} // Hui::MenuSubmenuWidget::byName

//======================================================================

void Hui::MenuSubmenuWidget::addItem( Pointer<MenuItemWidget> aBefore, Pointer<Hui::MenuItemWidget> aItem )
{
	mItems.insert( find( mItems.begin(), mItems.end(), aBefore ), aItem );
} // Hui::MenuSubmenuWidget::addItem

//======================================================================

bool Hui::MenuSubmenuWidget::removeItem( Pointer<Hui::MenuItemWidget> aItem )
{
	bool result = false;
	Items::iterator i = find( mItems.begin(), mItems.end(), aItem );
	if ( i != mItems.end() ) {
		mItems.erase( i );
		result = true;
	} else {
		for ( Items::iterator i = mItems.begin(); i != mItems.end() && !result; ++i ) {
			result = (*i)->removeItem( aItem );
		}
	}
	return(result);
} // Hui::MenuSubmenuWidget::removeItem

//======================================================================
