/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/Hui.cd.cc,v 1.19 2001/06/20 10:24:44 alex Exp $
 */

// -*- C++ -*-   Tue Dec  8 1998
// Eugene Skepner <eu@qub.com>
// $Id: Hui.cd.cc,v 1.1.1.1 2000/02/15 16:30:34 alex Exp $

#ifndef Hui_hh
#include "Hui.hh"
#endif

LanguageInterface::Object::MethodDescription Hui::sMethods[] = {
	MethodDescription("run",
		      "run",
		      LanguageInterface::Object::MethodDescription::fDynamic,
		      "Run the application, application is running until exit is called by one of the application callbacks.",
		      MethodDescription::vtVoid,
		      MethodDescription::vtVoid),
	MethodDescription("exit",
			  "exit",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Exit the application",
			  MethodDescription::vtVoid,
			  MethodDescription::Value(MethodDescription::vtInt,
						   0,
						   "0",
						   "status")),
	MethodDescription("processPendingEvents",
			  "processPendingEvents",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Processes pending events.",
			  MethodDescription::vtVoid,
			  MethodDescription::vtVoid),
	MethodDescription("open",
			  "open",
			  LanguageInterface::Object::MethodDescription::fStatic,
			  "Opens Hui, type: auto (default), text, graphics",
			  MethodDescription::vtObject,
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "\"auto\"")),
	MethodDescription("size",
			  "size",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns list of two elements: display width and height",
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<Hui::Dimension>"),
			  MethodDescription::vtVoid),
	MethodDescription("byName",
			  "byName",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns first widget shown matching the name specified",
			  MethodDescription::vtObject,
			  MethodDescription::vtString),
	MethodDescription("dialog",
			  "dialog",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Creates modal dialog widget",
			  MethodDescription::vtObject,
			  MethodDescription::Value(MethodDescription::vtObject,
						   "Hui::Widget"),
			  MethodDescription::vtString,
			  MethodDescription::vtString,
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<Hui::Dimension>",
						   "vector<Hui::Dimension>(0)"),
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<Hui::Dimension>",
						   "vector<Hui::Dimension>(0)"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "\"free\"",
						   "alignment")),
	
	MethodDescription( "window",
			   "window",
			   LanguageInterface::Object::MethodDescription::fDynamic,
			   "Creates window widget",
			   MethodDescription::vtObject,
			   MethodDescription::vtString,
			   MethodDescription::vtString,
			   MethodDescription::Value(MethodDescription::vtList,
						    "vector<Hui::Dimension>",
						    "vector<Hui::Dimension>(0)" ),
			   MethodDescription::Value(MethodDescription::vtList,
						    "vector<Hui::Dimension>",
						    "vector<Hui::Dimension>(0)" ),
			   MethodDescription::Value(MethodDescription::vtString,
						    0,
						    "\"free\"",
						    "alignment")),
	MethodDescription("menuBar",
			  "menuBar",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns menu bar widget",
			  MethodDescription::Value(MethodDescription::vtObject,
						   "Hui::MenuSubmenuWidget"),
			  MethodDescription::vtVoid),
	MethodDescription("fileSelectionDialog",
			  "fileSelectionDialog",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Brings file selection dialog and returns selected file name or an empty string if user pressed cancel.",
			  MethodDescription::vtString,
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   0,
						   "title"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   0,
						   "wildcard")),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::sCallbacks[] =
{
	CallbackDescription("Hui::WidgetCallbackData",
			"hui()",
			"widget()" ),
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::sClassDescription(
	"Hui",
	"Hui",
	"Common user interface",
	sMethods,
	sCallbacks
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::Widget::sMethods[] =
{
	MethodDescription("name",
		      "name",
		      LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
		      "Sets or gets widget's name",
		      MethodDescription::vtString,
		      MethodDescription::Value(MethodDescription::vtString,
					       0,
					       "get-requested")),
	MethodDescription("show",
			  "show",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Shows the widget",
			  MethodDescription::vtVoid,
			  MethodDescription::vtVoid),
	MethodDescription("hide",
			  "hide",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Hides the widget",
			  MethodDescription::vtVoid,
			  MethodDescription::vtVoid),
	MethodDescription("hidden",
			  "hidden",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns if the widget is hidden",
			  MethodDescription::vtBool, MethodDescription::vtVoid),
	MethodDescription("disable",
			  "disable",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Disables the widget",
			  MethodDescription::vtVoid,
			  MethodDescription::vtVoid),
	MethodDescription("enable",
			  "enable",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Enables the widget",
			  MethodDescription::vtVoid, MethodDescription::vtVoid ),
	MethodDescription("disabled",
			  "disabled",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns if the widget is disabled",
			  MethodDescription::vtBool,
			  MethodDescription::vtVoid),
	MethodDescription("onEnter",
			  "onEnter",
			  LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
			  "Installs/deinstalls/returns callback to be called if the widget is entered. If callback_data is not specified, returns current callback data. If new callback_data is specified (could be an empty list to just remove old callback), old callback data is returned.",
			  MethodDescription::vtCallbackData,
			  MethodDescription::Value(MethodDescription::vtCallbackData,
						   0,
						   "LanguageInterface::sNoCallbackData",
						   "callback_data")),
	MethodDescription("onLeave",
			  "onLeave",
			  LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
			  "Installs/deinstalls/returns callback to be called if the widget is left. If callback_data is not specified, returns current callback data. If new callback_data is specified (could be an empty list to just remove old callback), old callback data is returned.",
			  MethodDescription::vtCallbackData,
			  MethodDescription::Value(MethodDescription::vtCallbackData,
						   0,
						   "LanguageInterface::sNoCallbackData",
						   "callback_data")),
	MethodDescription("move",
			  "move",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Moves the widget to the specified position",
			  MethodDescription::vtVoid,
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<Hui::Dimension>",
						   0,
						   "position")),
	MethodDescription("position",
			  "position",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns the widget position relative to the parent widget",
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<Hui::Dimension>"),
			  MethodDescription::vtVoid),
	MethodDescription("resize",
			  "resize",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Resizes the widget",
			  MethodDescription::vtVoid,
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<Hui::Dimension>",
						   0,
						   "new_size")),
	MethodDescription("size",
			  "size",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns the widget size",
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<Hui::Dimension>"),
			  MethodDescription::vtVoid),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::Widget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::Widget::sClassDescription(
	"Hui::Widget",
	"Hui::Widget",
	"Base class for widgets",
	sMethods,
	sCallbacks
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::EmbeddedWidget::sMethods[] =
{
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::EmbeddedWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::EmbeddedWidget::sClassDescription(
	"Hui::EmbeddedWidget",
	"Hui::EmbeddedWidget",
	"Base class for embedded widgets",
	sMethods,
	sCallbacks,
	Hui::Widget::thisClassDescription()
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::GroupWidget::sMethods[] =
{
    MethodDescription("label",
		      "label",
		      LanguageInterface::Object::MethodDescription::fDynamic,
		      "Returns label widget embedded into this group widget",
		      MethodDescription::Value(MethodDescription::vtObject,
					       "Hui::LabelWidget"),
		      MethodDescription::Value(MethodDescription::vtString,
					       0,
					       0,
					       "widget_name"),
		      MethodDescription::Value(MethodDescription::vtString,
					       0,
					       0,
					       "label_text"),
		      MethodDescription::Value(MethodDescription::vtList,
					       "vector<Hui::Dimension>",
					       "vector<Hui::Dimension>(0)",
					       "position")),
	MethodDescription("button",
			  "button",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns button widget embedded into this group widget\nbutton_type: normal or default",
			  MethodDescription::Value(MethodDescription::vtObject,
						   "Hui::ButtonWidget"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   0,
						   "widget_name"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   0,
						   "button_text"),
			  MethodDescription::Value(MethodDescription::vtCallbackData,
						   0,
						   "Pointer<LanguageInterface::CallbackData>()",
						   "callback_data"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "\"normal\"",
						   "button_type"),
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<Hui::Dimension>",
						   "vector<Hui::Dimension>(0)",
						   "position"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "string()",
						   "comment")),
	MethodDescription("inputLine",
			  "inputLine",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns InputLine widget embedded into this group widget\nvalidator: `none', `picture', `valid_chars', `integer_range min max', `real_range min max'",
			  MethodDescription::Value(MethodDescription::vtObject,
						   "Hui::InputLineWidget"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   0,
						   "widget_name"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   0,
						   "initial_text"),
			  MethodDescription::Value(MethodDescription::vtInt,
						   0,
						   0,
						   "maximum_length"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "string()",
						   "validator"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "string()",
						   "title"),
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<Hui::Dimension>",
						   "vector<Hui::Dimension>(0)",
						   "position"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "string()",
						   "comment")),
	MethodDescription("checkBoxes",
			  "checkBoxes",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns CheckBoxes widget embedded into this group widget",
			  MethodDescription::Value(MethodDescription::vtObject,
						   "Hui::CheckBoxesWidget"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   0,
						   "widget_name"),
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<string>",
						   0,
						   "labels"),
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<bool>",
						   0,
						   "initial_values"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "string()",
						   "title"),
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<Hui::Dimension>",
						   "vector<Hui::Dimension>(0)",
						   "position"),
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<string>",
						   "vector<string>(0)",
						   "comments")),
	MethodDescription("radioMenu",
			  "radioMenu",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns RadioMenu widget embedded into this group widget",
			  MethodDescription::Value(MethodDescription::vtObject,
						   "Hui::RadioMenuWidget"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   0,
						   "widget_name"),
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<string>",
						   0,
						   "labels"),
			  MethodDescription::Value(MethodDescription::vtInt,
						   0,
						   0,
						   "initial_value"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "string()",
						   "title"),
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<Hui::Dimension>",
						   "vector<Hui::Dimension>(0)",
						   "position"),
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<string>",
						   "vector<string>(0)",
						   "comments")),
	MethodDescription("listBox",
			  "listBox",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns ListBox widget embedded into this group widget",
			  MethodDescription::Value(MethodDescription::vtObject,
						   "Hui::ListBoxWidget"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   0,
						   "widget_name"),
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<string>",
						   0,
						   "labels"),
			  MethodDescription::Value(MethodDescription::vtInt,
						   0,
						   0,
						   "initially_selected"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "string()",
						   "title"),
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<Hui::Dimension>",
						   "vector<Hui::Dimension>(0)",
						   "position"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "string()",
						   "comment")),
	MethodDescription("textEdit",
			  "textEdit",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns TextEdit widget embedded into this group widget\nvalidator: `none', `picture', `valid_chars', `integer_range min max', `real_range min max'", /*XXX validator??*/
			  MethodDescription::Value(MethodDescription::vtObject,
						   "Hui::TextEditWidget"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   0,
						   "widget_name"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   0,
						   "initial_text"),
			  MethodDescription::Value(MethodDescription::vtBool,
						   0,
						   0,
						   "allow_edit"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "string()",
						   "title"),
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<Hui::Dimension>",
						   "vector<Hui::Dimension>(0)",
						   "position"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "string()",
						   "comment")),
	MethodDescription("progressIndicator",
			  "progressIndicator",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns progressIndicator widget embedded into this group widget",
			  MethodDescription::Value(MethodDescription::vtObject,
						   "Hui::ProgressIndicatorWidget"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   0,
						   "widget_name"),
			  MethodDescription::Value(MethodDescription::vtFloat,
						   0,
						   0,
						   "min"),
			  MethodDescription::Value(MethodDescription::vtFloat,
						   0,
						   0,
						   "max"),
			  MethodDescription::Value(MethodDescription::vtFloat,
						   0,
						   0,
						   "initial"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "string()",
						   "title"),
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<Hui::Dimension>",
						   "vector<Hui::Dimension>(0)",
						   "position"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "string()",
						   "comment")),
	
	MethodDescription("container",
			  "container",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns container widget embedded into this group widget",
			  MethodDescription::Value(MethodDescription::vtObject,
						   "Hui::ContainerWidget"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   0,
						   "widget_name"),
			  MethodDescription::Value(MethodDescription::vtList,
						   "vector<Hui::Dimension>",
						   "vector<Hui::Dimension>(0)",
						   "position"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "\"horizontal\"",
						   "alignment"),
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "string()",
						   "comment")),
	MethodDescription("erase",
			  "erase",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Removes all subwidgets from the group widget.",
			  MethodDescription::vtVoid, MethodDescription::vtVoid),
	MethodDescription("arrange", "arrange",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Arranges the subwidgets",
			  MethodDescription::vtVoid,
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   "\"\"",
						   "aAlignment"),
			  MethodDescription::Value(MethodDescription::vtBool,
						   0,
						   "1",
						   "arrange_new")),
	MethodDescription("byName",
			  "byName",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Returns first embedded widget matching the name specified",
			  MethodDescription::vtObject,
			  MethodDescription::Value(MethodDescription::vtString,
						   0,
						   0,
						   "name")),
	MethodDescription("current",
			  "current",
			  LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
			  "Sets or gets current embedded widget",
			  MethodDescription::vtObject,
			  MethodDescription::Value(MethodDescription::vtObject,
						   "Hui::EmbeddedWidget",
						   "Pointer<Hui::EmbeddedWidget>()",
						   "new_current") ),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::GroupWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::GroupWidget::sClassDescription(
	"Hui::GroupWidget",
	"Hui::GroupWidget",
	"Base class for widgets having subwidgets",
	sMethods,
	sCallbacks,
	Hui::Widget::thisClassDescription()
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::ToplevelWidget::sMethods[] =
{
	MethodDescription( "title", "title", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
				"Sets or gets widget's title",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "get-requested" ) ),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::ToplevelWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::ToplevelWidget::sClassDescription(
	"Hui::ToplevelWidget",
	"Hui::ToplevelWidget",
	"Base class for top level widgets",
	sMethods,
	sCallbacks,
	Hui::GroupWidget::thisClassDescription()
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::WindowWidget::sMethods[] =
{
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::WindowWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::WindowWidget::sClassDescription(
	"Hui::WindowWidget",
	"Hui::WindowWidget",
	"Base class for Window widgets",
	sMethods,
	sCallbacks,
	Hui::ToplevelWidget::thisClassDescription()
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::DialogWidget::sMethods[] =
{
	MethodDescription( "result", "result", LanguageInterface::Object::MethodDescription::fDynamic,
				"Returns result of the dialog execution",
				MethodDescription::vtString, MethodDescription::vtVoid ),
	MethodDescription( "exec", "exec", LanguageInterface::Object::MethodDescription::fDynamic,
				"Executes modal dialog",
				MethodDescription::vtVoid, MethodDescription::vtVoid ),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::DialogWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::DialogWidget::sClassDescription(
	"Hui::DialogWidget",
	"Hui::DialogWidget",
	"Base class for Dialog widgets",
	sMethods,
	sCallbacks,
	Hui::ToplevelWidget::thisClassDescription()
);

//=====================================================================

LanguageInterface::Object::MethodDescription Hui::LabelWidget::sMethods[] =
{
	MethodDescription( "text", "text", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
				"Sets or gets label text",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "get-requested" ) ),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::LabelWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::LabelWidget::sClassDescription(
	"Hui::LabelWidget",
	"Hui::LabelWidget",
	"Base class for Label widgets",
	sMethods,
	sCallbacks,
	Hui::EmbeddedWidget::thisClassDescription()
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::ButtonWidget::sMethods[] =
{
	MethodDescription( "text", "text", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
				"Sets or gets button text",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "get-requested" ) ),
	MethodDescription( "onClick", "onClick", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
				"Installs/deinstalls/returns callback to be called if the button is clicked. If callback_data is not specified, returns current callback data. If new callback_data is specified (could be an empty list to just remove old callback), old callback data is returned.",
				MethodDescription::vtCallbackData, MethodDescription::Value( MethodDescription::vtCallbackData, 0, "LanguageInterface::sNoCallbackData", "callback_data" ) ),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::ButtonWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::ButtonWidget::sClassDescription(
	"Hui::ButtonWidget",
	"Hui::ButtonWidget",
	"Base class for Button widgets",
	sMethods,
	sCallbacks,
	Hui::EmbeddedWidget::thisClassDescription()
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::InputLineWidget::sMethods[] =
{
	MethodDescription( "text", "text", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
				"Sets or gets InputLine text",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "get-requested" ) ),
	MethodDescription( "title", "title", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
				"Sets or gets List title",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "get-requested", "new_title" ) ),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::InputLineWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::InputLineWidget::sClassDescription(
	"Hui::InputLineWidget",
	"Hui::InputLineWidget",
	"Base class for InputLine widgets",
	sMethods,
	sCallbacks,
	Hui::EmbeddedWidget::thisClassDescription()
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::ButtonsGroupWidget::sMethods[] =
{
	MethodDescription( "text", "text", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
				"Sets or gets ButtonsGroup item text",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtInt, 0, 0, "item_no" ), MethodDescription::Value( MethodDescription::vtString, 0, "get-requested", "new_text" ) ),
	MethodDescription( "numberOfItems", "numberOfItems", LanguageInterface::Object::MethodDescription::fDynamic,
				"Returns number of items in ButtonsGroup",
				MethodDescription::vtInt, MethodDescription::vtVoid ),
	MethodDescription( "title", "title", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
				"Sets or gets ButtonsGroup title",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "get-requested", "new_title" ) ),
	MethodDescription( "disable_item", "disable_item", LanguageInterface::Object::MethodDescription::fDynamic,
				"Disables ButtonsGroup item",
				MethodDescription::vtVoid, MethodDescription::Value( MethodDescription::vtInt, 0, 0, "item_no" ) ),
	MethodDescription( "enable_item", "enable_item", LanguageInterface::Object::MethodDescription::fDynamic,
				"Enables ButtonsGroup item",
				MethodDescription::vtVoid, MethodDescription::Value( MethodDescription::vtInt, 0, 0, "item_no" ) ),
	MethodDescription( "item_disabled", "item_disabled", LanguageInterface::Object::MethodDescription::fDynamic,
				"Returns if ButtonsGroup item disabled",
				MethodDescription::vtBool, MethodDescription::Value( MethodDescription::vtInt, 0, 0, "item_no" ) ),
	MethodDescription( "onClick", "onClick", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
				"Installs/deinstalls/returns callback to be called if one of the ButtonsGroup is clicked. If callback_data is not specified, returns current callback data. If new callback_data is specified (could be an empty list to just remove old callback), old callback data is returned.",
				MethodDescription::vtCallbackData, MethodDescription::Value( MethodDescription::vtCallbackData, 0, "LanguageInterface::sNoCallbackData", "callback_data" ) ),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::ButtonsGroupWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::ButtonsGroupWidget::sClassDescription(
	"Hui::ButtonsGroupWidget",
	"Hui::ButtonsGroupWidget",
	"Base class for ButtonsGroup widgets",
	sMethods,
	sCallbacks,
	Hui::EmbeddedWidget::thisClassDescription()
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::CheckBoxesWidget::sMethods[] =
{
	MethodDescription( "state", "state", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
				"Sets or gets CheckBoxes item state",
				MethodDescription::vtBool, MethodDescription::Value( MethodDescription::vtInt, 0, 0, "item_no" ), MethodDescription::Value( MethodDescription::vtInt, 0, "get-requested", "new_state" ) ),
	MethodDescription( "toggle", "toggle", LanguageInterface::Object::MethodDescription::fDynamic,
				"Toggles CheckBoxes item, returns it's new state",
				MethodDescription::vtBool, MethodDescription::Value( MethodDescription::vtInt, 0, 0, "item_no" ) ),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::CheckBoxesWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::CheckBoxesWidget::sClassDescription(
	"Hui::CheckBoxesWidget",
	"Hui::CheckBoxesWidget",
	"Base class for CheckBoxes widgets",
	sMethods,
	sCallbacks,
	Hui::ButtonsGroupWidget::thisClassDescription()
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::RadioMenuWidget::sMethods[] =
{
	MethodDescription( "set", "set", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
				"Sets or gets RadioMenu set item",
				MethodDescription::vtInt, MethodDescription::Value( MethodDescription::vtInt, 0, "get-requested", "item_no" ) ),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::RadioMenuWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::RadioMenuWidget::sClassDescription(
	"Hui::RadioMenuWidget",
	"Hui::RadioMenuWidget",
	"Base class for RadioMenu widgets",
	sMethods,
	sCallbacks,
	Hui::ButtonsGroupWidget::thisClassDescription()
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::ListBoxWidget::sMethods[] =
{
	MethodDescription( "text", "text", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
				"Sets or gets List item text",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtInt, 0, 0, "item_no" ), MethodDescription::Value( MethodDescription::vtString, 0, "get-requested", "new_text" ) ),
	MethodDescription( "numberOfItems", "numberOfItems", LanguageInterface::Object::MethodDescription::fDynamic,
				"Returns number of items in List",
				MethodDescription::vtInt, MethodDescription::vtVoid ),
	MethodDescription( "title", "title", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
				"Sets or gets List title",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "get-requested", "new_title" ) ),
	MethodDescription( "select", "select", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
				"Sets or gets selected item",
				MethodDescription::vtInt, MethodDescription::Value( MethodDescription::vtInt, 0, "get-requested", "item_no" ) ),
	MethodDescription( "add", "add", LanguageInterface::Object::MethodDescription::fDynamic,
				"Adds new item to the list",
				MethodDescription::vtVoid, MethodDescription::Value( MethodDescription::vtInt, 0, 0, "before" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "text_to_add" ) ),
	MethodDescription( "remove", "remove", LanguageInterface::Object::MethodDescription::fDynamic,
				"Removes item from the list",
				MethodDescription::vtVoid, MethodDescription::Value( MethodDescription::vtInt, 0, 0, "itemno" ) ),
	MethodDescription("removeAll",
			  "removeAll",
			  LanguageInterface::Object::MethodDescription::fDynamic,
			  "Removes all items from the list",
			  MethodDescription::vtVoid, MethodDescription::vtVoid ),
	MethodDescription("onSelect",
			  "onSelect",
			  LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
			  "Installs/deinstalls/returns callback to be called if one of the list items is selected by user. If callback_data is not specified, returns current callback data. If new callback_data is specified (could be an empty list to just remove old callback), old callback data is returned.",
			  MethodDescription::vtCallbackData,
			  MethodDescription::Value(MethodDescription::vtCallbackData,
						   0,
						   "LanguageInterface::sNoCallbackData",
						   "callback_data")),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::ListBoxWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::ListBoxWidget::sClassDescription(
	"Hui::ListBoxWidget",
	"Hui::ListBoxWidget",
	"Base class for ListBox widgets",
	sMethods,
	sCallbacks,
	Hui::EmbeddedWidget::thisClassDescription()
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::TextEditWidget::sMethods[] =
{
	MethodDescription( "text", "text", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
				"Sets or gets TextEdit text",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "get-requested" ) ),
	MethodDescription( "title", "title", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
				"Sets or gets List title",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "get-requested", "new_title" ) ),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::TextEditWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::TextEditWidget::sClassDescription(
	"Hui::TextEditWidget",
	"Hui::TextEditWidget",
	"Base class for TextEdit widgets",
	sMethods,
	sCallbacks,
	Hui::EmbeddedWidget::thisClassDescription()
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::ProgressIndicatorWidget::sMethods[] =
{
	MethodDescription( "set", "set", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
				"Sets or gets ProgressIndicator value",
				MethodDescription::vtFloat, MethodDescription::Value( MethodDescription::vtFloat, 0, "get-requested", "new_value" ) ),
	MethodDescription( "title", "title", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
				"Sets or gets List title",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "get-requested", "new_title" ) ),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::ProgressIndicatorWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::ProgressIndicatorWidget::sClassDescription(
	"Hui::ProgressIndicatorWidget",
	"Hui::ProgressIndicatorWidget",
	"Base class for ProgressIndicator widgets",
	sMethods,
	sCallbacks,
	Hui::EmbeddedWidget::thisClassDescription()
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::ContainerWidget::sMethods[] =
{
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::ContainerWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::ContainerWidget::sClassDescription(
	"Hui::ContainerWidget",
	"Hui::ContainerWidget",
	"Base class for Container widgets",
	sMethods,
	sCallbacks,
	Hui::GroupWidget::thisClassDescription()
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::MenuItemWidget::sMethods[] =
{
	MethodDescription( "text", "text", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
				"Sets or gets menu button text",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "get-requested" ) ),
	MethodDescription( "onClick", "onClick", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSetReturnsOld,
				"Installs/deinstalls/returns callback to be called if the menu button is clicked. If callback_data is not specified, returns current callback data. If new callback_data is specified (could be an empty list to just remove old callback), old callback data is returned.",
				MethodDescription::vtCallbackData, MethodDescription::Value( MethodDescription::vtCallbackData, 0, "LanguageInterface::sNoCallbackData", "callback_data" ) ),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::MenuItemWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::MenuItemWidget::sClassDescription(
	"Hui::MenuItemWidget",
	"Hui::MenuItemWidget",
	"Base class for MenuItem widgets",
	sMethods,
	sCallbacks,
	Hui::EmbeddedWidget::thisClassDescription()
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::MenuSeparatorWidget::sMethods[] =
{
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::MenuSeparatorWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::MenuSeparatorWidget::sClassDescription(
	"Hui::MenuSeparatorWidget",
	"Hui::MenuSeparatorWidget",
	"Base class for MenuSeparator widgets",
	sMethods,
	sCallbacks,
	Hui::MenuItemWidget::thisClassDescription()
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::MenuButtonWidget::sMethods[] =
{
	MethodDescription( "key", "key", LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
				"Sets or gets menu button key",
				MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "get-requested" ) ),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::MenuButtonWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::MenuButtonWidget::sClassDescription(
	"Hui::MenuButtonWidget",
	"Hui::MenuButtonWidget",
	"Base class for MenuButton widgets",
	sMethods,
	sCallbacks,
	Hui::MenuItemWidget::thisClassDescription()
);

//======================================================================

LanguageInterface::Object::MethodDescription Hui::MenuSubmenuWidget::sMethods[] =
{
	MethodDescription( "addButton", "addButton", LanguageInterface::Object::MethodDescription::fDynamic,
				"Adds a menu Button",
				MethodDescription::Value( MethodDescription::vtObject, "Hui::MenuItemWidget" ),
				MethodDescription::Value( MethodDescription::vtString, 0, 0, "name" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "text" ), MethodDescription::Value( MethodDescription::vtCallbackData, 0, "LanguageInterface::sNoCallbackData", "callback_data" ), MethodDescription::Value( MethodDescription::vtString, 0, "string()", "key" ), MethodDescription::Value( MethodDescription::vtString, 0, "string()", "comment" ), MethodDescription::Value( MethodDescription::vtObject, "Hui::MenuItemWidget", "Pointer<Hui::MenuItemWidget>()", "before" ) ),
	MethodDescription( "addSeparator", "addSeparator", LanguageInterface::Object::MethodDescription::fDynamic,
				"Adds a menu separator",
				MethodDescription::Value( MethodDescription::vtObject, "Hui::MenuItemWidget" ),
				MethodDescription::Value( MethodDescription::vtString, 0, 0, "name" ), MethodDescription::Value( MethodDescription::vtObject, "Hui::MenuItemWidget", "Pointer<Hui::MenuItemWidget>()", "before" ) ),
	MethodDescription( "addSubmenu", "addSubmenu", LanguageInterface::Object::MethodDescription::fDynamic,
				"Adds a submenu",
				MethodDescription::Value( MethodDescription::vtObject, "Hui::MenuItemWidget" ),
				MethodDescription::Value( MethodDescription::vtString, 0, 0, "name" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "text" ), MethodDescription::Value( MethodDescription::vtString, 0, "string()", "comment" ), MethodDescription::Value( MethodDescription::vtObject, "Hui::MenuItemWidget", "Pointer<Hui::MenuItemWidget>()", "before" ) ),
	MethodDescription( "remove", "remove", LanguageInterface::Object::MethodDescription::fDynamic,
				"Removes a subitem",
				MethodDescription::vtVoid, MethodDescription::Value( MethodDescription::vtObject, "Hui::MenuItemWidget", 0, "item" ) ),
	MethodDescription( "byName", "byName", LanguageInterface::Object::MethodDescription::fDynamic,
				"Finds a subitem by it's name",
				MethodDescription::Value( MethodDescription::vtObject, "Hui::Widget" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "item_name" ) ),
	MethodDescription()
};

LanguageInterface::Object::CallbackDescription Hui::MenuSubmenuWidget::sCallbacks[] =
{
	CallbackDescription()
};

LanguageInterface::Object::ClassDescription Hui::MenuSubmenuWidget::sClassDescription(
	"Hui::MenuSubmenuWidget",
	"Hui::MenuSubmenuWidget",
	"Base class for MenuSubmenu widgets",
	sMethods,
	sCallbacks,
	Hui::MenuItemWidget::thisClassDescription()
);

//======================================================================
