/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiQt.cc,v 1.28 2001/06/20 10:24:44 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiQt.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiQt_hh
#include "HuiQt.hh"
#endif

#ifndef File_hh
#include "File.hh"
#endif

#ifndef QFILEDLG_H
#include <qfiledlg.h>
#endif

//======================================================================

QResizableDialog::QResizableDialog( QWidget* aParent, const string& aName, bool aModal )
	: QDialog( aParent, aName.c_str(), aModal )
{
//	  XSelectInput( dpy, winId(), KeyPressMask | KeyReleaseMask |
//				 ButtonPressMask | ButtonReleaseMask |
//				 KeymapStateMask |
//				 ButtonMotionMask |
//				 EnterWindowMask | LeaveWindowMask |
//				 FocusChangeMask |
//				 ExposureMask |
//				 StructureNotifyMask | SubstructureRedirectMask );

} // QResizableDialog::QResizableDialog

//======================================================================

QResizableMainWindow::QResizableMainWindow( const string& aName )
	: QMainWindow( 0, aName.c_str() )
{
} // QResizableMainWindow::QResizableMainWindow

//======================================================================

HuiQt::HuiQt( int argc, char** argv )
	: QApplication( argc, argv )
{

} // HuiQt::HuiQt

//======================================================================

HuiQt::~HuiQt()
{

} // HuiQt::~HuiQt

//======================================================================

void HuiQt::run() // Run the application
{
	QApplication::exec();

} // HuiQt::run

//======================================================================

void HuiQt::exit( int status ) // exit the application
{
	QApplication::exit( status );

} // HuiQt::exit

//======================================================================

void HuiQt::processPendingEvents()
{
	QApplication::processEvents();

} // HuiQt::processPendingEvents

//======================================================================

Hui::Size HuiQt::size() const
{
	if ( subwidgets().size() > 0 ) {
		return dynamic_cast<const HuiQt::Widget&>( **subwidgets().begin() ).translate( QApplication::desktop()->size() );
	}
	else {
		static Hui::Size size( 80, 25 );
		QFontMetrics fm = fontMetrics();
		const int char_width = fm.width( 'O' );
		const int char_height = fm.lineSpacing(); // + fm.descent();
		size.width( desktop()->width() / char_width );
		size.height( desktop()->height() / char_height );
		return(size);
	}

} // HuiQt::size

//======================================================================

Pointer<Hui::Widget> HuiQt::byName( const string& aName )
{
	Pointer<Hui::Widget> result;
	for ( Subwidgets::const_iterator w = mWidgets.begin(); !result && w != mWidgets.end(); ++w )
		result = (*w)->byName( aName );
	return(result);

} // HuiQt::byName

//======================================================================

Pointer<Hui::DialogWidget>
HuiQt::dialog(Pointer<Hui::Widget> aParent,
	      const string& aName,
	      const string& aTitle,
	      const Position& aPosition,
	      const Size& aSize,
	      const string& aAlignment)
{
	Pointer<DialogWidget> dialog = new DialogWidget(*this,
							aParent,
							aName,
							aTitle,
							aPosition,
							aSize);
	dialog->arrange(aAlignment, FALSE);

	return(dialog);

} // HuiQt::dialog

//======================================================================

Pointer<Hui::WindowWidget> HuiQt::window( const string& aName, const string& aTitle, const Position& aPosition, const Size& aSize, const string& aAlignment, bool no_menu )
{
	Pointer<WindowWidget> window = new WindowWidget( *this, aName, aTitle, aPosition, aSize );
	if ( !no_menu && mMenuBar )
		window->recreateMenu( *mMenuBar );

	window->arrange(aAlignment, FALSE);

	return(window);

} // HuiQt::window

//======================================================================

string HuiQt::fileSelectionDialog( const string& aTitle, const string& aWildCard )
{
#ifndef LANGUAGE_INTERFACE
	QFileDialog* fd = new QFileDialog( File::dirname( aWildCard ).c_str(), File::basename( aWildCard, "" ).c_str(), 0, aTitle.c_str(), TRUE );
	fd->setCaption( aTitle.c_str() );
	QString result = "";
	if ( fd->exec() == QDialog::Accepted ) {
		result = fd->selectedFile();
	}
	delete fd;
	return (const char*)result;
#endif
} // HuiQt::fileSelectionDialog

//======================================================================

Pointer<Hui::MenuSubmenuWidget> HuiQt::menuBar()
{
	if ( !mMenuBar ) {
		mMenuBar = new MenuBarWidget( *this );
	}
	return(mMenuBar);

} // HuiQt::menuBar

//======================================================================

string HuiQt::translateAccel( const string& aText )
{
	string result = aText;
	const string::size_type p = result.find( '~' );
	if ( p != string::npos ) {
		result.replace( p, 1, 1, '&' );
		const string::size_type p1 = result.find( '~', p + 1 );
		if ( p1 != string::npos )
			result.erase( p1, 1 );
	}
	return(result);

} // HuiQt::translateAccel

//======================================================================

string HuiQt::stripAccel( const string& aText )
{
	string result = translateAccel(aText);
	const string::size_type p = result.find('&');
	if (p != string::npos)
		result.erase(result.find( '&' ), 0);
	return(result);

} // HuiQt::stripAccel

//======================================================================

Pointer<Hui::Widget> HuiQt::Subwidgets::find( QWidget* p ) const
{
	Pointer<Hui::Widget> result;
	for ( const_iterator w = begin(); !result && w != end(); ++w ) {
		const HuiQt::Widget* qw = dynamic_cast<const HuiQt::Widget*>( (const Hui::Widget*)*w );
		if ( qw && qw->qwidget() == p ) {
			result = *w;
		}
		else {
			Pointer<const HuiQt::GroupWidget> gw = dynamic_cast<const HuiQt::GroupWidget*>( (const Hui::Widget*)*w );
			if ( gw )
				result = gw->subwidgets().find( p );
		}
	}
	return(result);

} // HuiQt::Subwidgets::find

//======================================================================

void HuiQt::Subwidgets::add( Pointer<Hui::Widget> w )
{
	/* only add, if it's not already in the list */
	iterator i = ::find( begin(), end(), w );

	if ( i == end() )
		push_back( w );

} // HuiQt::Subwidgets::add

//======================================================================

void HuiQt::Subwidgets::remove( Pointer<Hui::Widget> w )
{
	iterator i = ::find( begin(), end(), w );
	if ( i != end() )
		erase( i );

} // HuiQt::Subwidgets::remove

//======================================================================
