/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiQt.hh,v 1.24 2001/07/07 14:47:55 alex Exp $
 */

// -*- C++ -*-  Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiQt.hh,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiQt_hh
#define HuiQt_hh

#ifndef Hui_hh
#include "Hui.hh"
#endif

#ifndef QAPP_H
#include <qapp.h>
#endif

#ifndef QLAYOUT_H
#include <qlayout.h>
#endif

//======================================================================

class QLabel;
class QResizableDialog;
class QResizableMainWindow;
class QPushButtonEnterLeave;
class QLineEditEnterLeave;
class QValidator;
class QButtonGroup;
class QListBoxEnterLeave;
class QMultiLineEditEnterLeave;
class QProgressBar;
class QMenuData;

//======================================================================

class HuiQResizable : public QObject
{
	Q_OBJECT;

public slots:
	virtual void resized(const QSize& aSize)
	{
		widget_resized(aSize);
	}

	virtual void enteredLeft( bool aEntered )
	{
		widget_entered_left(aEntered);
	}

protected:
	virtual void widget_resized(const QSize& aSize) = 0;
	virtual void widget_entered_left(bool aEntered) = 0;
	
}; // class HuiQResizable

//======================================================================

class HuiQtEnterLeave : public QObject
{
	Q_OBJECT;

public:
	HuiQtEnterLeave(Hui::Widget& aWidget)
		: mWidget(aWidget) {}
	
public slots:
	virtual void enter() { mWidget.enter(); }
	virtual void leave() { mWidget.leave(); }

private:
	Hui::Widget& mWidget;

}; // class HuiQtEnterLeave

//======================================================================

class HuiQtButtonClick : public QObject
{
	Q_OBJECT;

public:
	HuiQtButtonClick(Hui::ButtonWidget& aWidget)
		: mWidget(aWidget) {}

public slots:
	virtual void click() { mWidget.click(); }

private:
	Hui::ButtonWidget& mWidget;

}; // class HuiQtButtonClick

//======================================================================

class HuiQtButtonGroupClick : public QObject
{
	Q_OBJECT;
	
public:
	HuiQtButtonGroupClick(Hui::ButtonsGroupWidget& aWidget)
		: mWidget(aWidget) {}
	
public slots:
	virtual void click(int id) { mWidget.click(id); }

private:
	Hui::ButtonsGroupWidget& mWidget;

}; // class HuiQtButtonGroupClick

//======================================================================

class HuiQtMenuClick : public QObject,
		       virtual public Shared
{
	Q_OBJECT;

public:
	HuiQtMenuClick(Hui::MenuItemWidget& aWidget)
		: mWidget(aWidget) {}

public slots:
	virtual void click() { mWidget.click(); }

private:
	Hui::MenuItemWidget& mWidget;

}; // class HuiQtMenuClick

//======================================================================

class HuiQtListBoxHighlight : public QObject
{
	Q_OBJECT;

public:
	HuiQtListBoxHighlight(Hui::ListBoxWidget& aWidget)
		: mWidget(aWidget) {}

public slots:
	virtual void highlighted(int id) { mWidget.highlighted(id); }

private:
	Hui::ListBoxWidget& mWidget;

}; // class HuiQtListBoxHighlight

//======================================================================

class HuiQt : public QApplication, public Hui
{
	CLASS_METHODS_DESCRIPTIONS_DECLARATION;

public:
	HuiQt(int argc, char** argv);
	virtual ~HuiQt();
	
	virtual void run(); // Run the application
	virtual void exit(int status); // exit the application
	virtual void processPendingEvents();
	virtual const char* type() const { return "graphics"; }

	virtual Size size() const;
	
	// --------------------------------------------------
	
	virtual Pointer<Hui::Widget> byName(const string& aName);

	virtual Pointer<Hui::DialogWidget> dialog(Pointer<Hui::Widget> aParent,
						  const string& aName,
						  const string& aTitle,
						  const Position& aPosition,
						  const Size& aSize,
						  const string& aAlignment);
	virtual Pointer<Hui::WindowWidget> window(const string& aName,
						  const string& aTitle,
						  const Position& aPosition,
						  const Size& aSize,
						  const string& aAlignment,
						  bool no_menu = false);
	virtual Pointer<Hui::MenuSubmenuWidget> menuBar();
	virtual string fileSelectionDialog(const string& aTitle,
					   const string& aWildCard);

	// --------------------------------------------------

protected:

	class Subwidgets : public list< Pointer<Hui::Widget> >
	{
	public:
		Subwidgets() {}
		virtual ~Subwidgets() {}

		virtual Pointer<Hui::Widget> find(QWidget* p) const;
		virtual void add(Pointer<Hui::Widget> w);
		virtual void remove(Pointer<Hui::Widget> w);
		
	}; // class Subwidgets
	
	// --------------------------------------------------

	class GroupWidget;

	class Widget : virtual public Hui::Widget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

		friend class HuiQt;

	public:
		virtual ~Widget();

		virtual void show();
		virtual void hide();
		virtual bool hidden() const;

		virtual void disable();
		virtual void enable();
		virtual bool disabled() const;

		virtual void move( const Position& aPosition );
		virtual void move( const QPoint& to );
		virtual Position position() const;

		virtual void resize( const Size& aSize );
		virtual void resize(const QSize& to, int is_arranging = 0);
		virtual Size size() const;

		virtual QWidget* qwidget() = 0;
		virtual const QWidget* qwidget() const = 0;

		void parent(Pointer<GroupWidget> aParent) { mParent = aParent; };
		Pointer<GroupWidget> parent() { return mParent ; }
		const Pointer<GroupWidget> parent() const { return mParent ; }

		QPoint real_position() const {
			return (mPosition);
		}

		QSize real_size() const {
			return (mSize);
		}

		int getFixedSize() { return(mFixedSize); }
		void setFixedSize(int to) { mFixedSize = to; }

	protected:
		Widget(Hui& aHui,
		       const string& aName)
			: Hui::Widget(aHui, aName),
			  mHuiQtEnterLeave(*this) {
			mFixedSize = -1;
		}

		virtual QRect translate(const Hui::Position& aPosition,
					const Hui::Size& aSize) const;
		virtual QPoint translate(const Hui::Position& aPosition) const {
			return translate(aPosition, Hui::Size()).topLeft();
		}
		virtual QSize translate(const Hui::Size& aSize ) const {
			return translate(Hui::Position(), aSize).size();
		}
		virtual Hui::Size translate(const QSize& aSize) const;
		virtual Hui::Position translate(const QPoint& aPosition) const;

		HuiQtEnterLeave& huiQtEnterLeave() {
			return mHuiQtEnterLeave;
		}

		void setposition(const QPoint& aPosition) {
			mPosition = aPosition;
		}
		void setsize(const QSize& aSize) {
			mSize = aSize;
		}

	private:
		HuiQtEnterLeave mHuiQtEnterLeave;
		Pointer<GroupWidget> mParent;
		QPoint mPosition;
		QSize mSize;
		int mFixedSize;

	}; // class Widget
	
	class GroupWidget : virtual public Hui::GroupWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;
	
	public:
		virtual ~GroupWidget();
	
		virtual Pointer<Hui::LabelWidget> label(const string& aName,
							const string& aText,
							const Hui::Position& aPosition);
		virtual Pointer<Hui::ButtonWidget> button(const string& aName,
							  const string& aText,
							  Pointer<LanguageInterface::CallbackData> aCallbackData,
							  const string& aButtonType,
							  const Hui::Position& aPosition,
							  const string& aComment);
		virtual Pointer<Hui::InputLineWidget> inputLine(const string& aName,
								const string& aInitialText,
								int aMaxLength,
								const string& aValidator,
								const string& aTitle,
								const Hui::Position& aPosition,
								const string& aComment);
		virtual Pointer<Hui::CheckBoxesWidget> checkBoxes(const string& aName,
								  const vector<string>& aTexts,
								  const vector<bool>& aInitialValues,
								  const string& aTitle,
								  const Hui::Position& aPosition,
								  const vector<string>& aComments);
		virtual Pointer<Hui::RadioMenuWidget> radioMenu(const string& aName,
								const vector<string>& aTexts,
								const int aInitialValue,
								const string& aTitle,
								const Hui::Position& aPosition,
								const vector<string>& aComments);
		virtual Pointer<Hui::ListBoxWidget> listBox(const string& aName,
							    const vector<string>& aTexts,
							    const int aInitiallySelected,
							    const string& aTitle,
							    const Hui::Position& aPosition,
							    const string& aComment);
		virtual Pointer<Hui::TextEditWidget> textEdit(const string& aName,
							      const string& aInitialText,
							      bool aAllowEdit,
							      const string& aTitle,
							      const Hui::Position& aPosition,
							      const string& aComment);
		virtual Pointer<Hui::ProgressIndicatorWidget> progressIndicator(const string& aName,
										float aMin,
										float aMax,
										float aInitial,
										const string& aTitle,
										const Hui::Position& aPosition,
										const string& aComment);
		virtual Pointer<Hui::ContainerWidget> container(const string& aName,
								const Hui::Position& aPosition,
								const string& aAlignment,
								const string& aComment);

		virtual void arrange();
		virtual void arrange(const string& aAlignment,
				     bool arrange_new);

		virtual void current(Pointer<Hui::EmbeddedWidget> aWidget);
		virtual Pointer<Hui::Widget> current() const;
	
		virtual QWidget* qwidget() = 0;
		virtual const QWidget* qwidget() const = 0;
	
		virtual Pointer<Hui::Widget> byName(const string& aName) const;
		virtual void erase();
	
		virtual HuiQt& hui() const { return mHui; };
	
		const Subwidgets& subwidgets() const { return mWidgets; }
		Subwidgets& subwidgets() { return mWidgets; }

		virtual QPoint real_position() = 0;
		virtual QSize real_size() = 0;

	protected:
		GroupWidget(HuiQt& aHui,
			    const string& aName,
			    const Position& aPosition,
			    const Size& aSize );

	private:
		HuiQt& mHui;
		Subwidgets mWidgets;
		int mArrangement;

	}; // class GroupWidget
	
	class ContainerWidget : public Hui::ContainerWidget,
				public GroupWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;
	
	public:
		ContainerWidget(HuiQt& aHui,
				Pointer<GroupWidget> aParent,
				const string& aName);
		virtual ~ContainerWidget();
	
		// Functions, that need to be virtualized
		void show();
		void hide();
		bool hidden() const;

		void disable();
		void enable();
		bool disabled() const;

		void move(const Hui::Position &);
		void move(const QPoint &);
		class Hui::Position position() const;

		void resize(const Hui::Size &);
		void resize(const QSize &, int is_arranging = 0);
		class Hui::Size size() const;
		QSize real_size() { return (mSize); }
		QPoint real_position() { return (mPosition); }

		void parent(Pointer<GroupWidget> aParent) { mParent = aParent; };
		const Pointer<GroupWidget> parent() const { return mParent ; }
		Pointer<GroupWidget> parent() { return mParent ; }
    
		// Refer to parent's qwidget()
		QWidget* qwidget();
		const QWidget* qwidget() const;

		int getFixedSize() { return(mFixedSize); }
		void setFixedSize(int to) { mFixedSize = to; }

	protected:
		QPoint translate(const Hui::Position& aPosition) const;
		QSize translate(const Hui::Size& aSize) const;
		Hui::Position translate(const QPoint& aPosition) const;
		Hui::Size translate(const QSize& aSize) const;

	private:
		QPoint mPosition;
		QSize mSize;
		Pointer<GroupWidget> mParent;
		bool mHidden;
		bool mDisabled;
		int mFixedSize;
	
	}; // ContainerWidget;
    
	// --------------------------------------------------

	class ToplevelWidget
		: virtual public Hui::ToplevelWidget,
		  public GroupWidget, public HuiQResizable
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		virtual ~ToplevelWidget();

		virtual void title( const string& aTitle );
		virtual string title() const;

	protected:
		ToplevelWidget(HuiQt& aHui,
			       const string& aName,
			       const Position& aPosition,
			       const Size& aSize);

		virtual void widget_resized(const QSize& aSize);
		virtual void widget_entered_left(bool aEntered);

	}; // class ToplevelWidget

	// --------------------------------------------------

	class WindowWidget : public Widget,
			     public ToplevelWidget,
			     public Hui::WindowWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		WindowWidget(HuiQt& aHui,
			     const string& aName,
			     const string& aTitle,
			     const Position& aPosition,
			     const Size& aSize);
		virtual ~WindowWidget();

		virtual void show();
		virtual void hide();
		virtual void resize(const Size& aSize);
		virtual void resize(const QSize& to);

		virtual QWidget* qwidget();
		virtual const QWidget* qwidget() const;

		virtual void recreateMenu(Hui::MenuSubmenuWidget& aMenuData);

		QSize real_size() { return qwidget()->size(); }
		QPoint real_position() { return QPoint(0, 0); }

	protected:
		virtual QRect translate(const Hui::Position& aPosition,
					const Hui::Size& aSize) const {
			return HuiQt::Widget::translate(aPosition, aSize);
		}
		virtual Hui::Size translate(const QSize& aSize) const {
			return HuiQt::Widget::translate(aSize);
		}

	private:
		HuiQt& mHui;
		QResizableMainWindow* mMainWindow;
		list< Pointer<HuiQtMenuClick> > mMenuClicks;

		virtual void recreateMenu(QMenuData* popup,
					  Hui::MenuSubmenuWidget& aMenuData);

	}; // class WindowWidget

	// --------------------------------------------------

	class DialogWidget : public Widget,
			     public ToplevelWidget,
			     public Hui::DialogWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		DialogWidget(HuiQt& aHui,
			     Pointer<Hui::Widget> aParent,
			     const string& aName,
			     const string& aTitle,
			     const Position& aPosition,
			     const Size& aSize);
		virtual ~DialogWidget();

		virtual const string& result() const;
		virtual const string& exec();

		virtual bool running() const { return mRunning; }
		virtual void endModal(const string& aResult);

		virtual QWidget* qwidget();
		virtual const QWidget* qwidget() const;

		QSize real_size() { return qwidget()->size(); }
		QPoint real_position() { return QPoint(0, 0); }

	protected:
		virtual QRect translate(const Hui::Position& aPosition,
					const Hui::Size& aSize) const {
			return HuiQt::Widget::translate(aPosition, aSize);
		}
		virtual Hui::Size translate(const QSize& aSize) const {
			return HuiQt::Widget::translate(aSize);
		}

		QResizableDialog* qtwindow() { return mWindow; }

	private:
		HuiQt& mHui;
		QResizableDialog* mWindow;
		string mResult;
		bool mRunning;

	}; // class DialogWidget

	// --------------------------------------------------

	class LabelWidget : public Widget, public Hui::LabelWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		LabelWidget(Hui& aHui,
			    Pointer<GroupWidget> aParent,
			    const string& aName,
			    const string& aText);
		virtual ~LabelWidget();
		
		virtual void text(const string& aText);
		virtual string text() const;

		virtual QWidget* qwidget();
		virtual const QWidget* qwidget() const;

	private:
		QLabel* mLabel;

	}; // class LabelWidget

	// --------------------------------------------------

	class ButtonWidget : public Widget,
			     public Hui::ButtonWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		ButtonWidget(HuiQt& aHui,
			     Pointer<GroupWidget> aParent,
			     const string& aName,
			     const string& aText,
			     Pointer<LanguageInterface::CallbackData> aCallbackData,
			     const string& aButtonType,
			     const string& aComment);
		virtual ~ButtonWidget();

		virtual void text(const string& aText);
		virtual string text() const;
		
		virtual QWidget* qwidget();
		virtual const QWidget* qwidget() const;

		virtual void callback_completed(const WidgetCallbackData& aCallbackData);

	private:
		QPushButtonEnterLeave* mButton;
		HuiQtButtonClick mButtonClick;
		string mComment;

	}; // class ButtonWidget

	// --------------------------------------------------

	class TitledWidget : public Widget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		TitledWidget(Hui& aHui,
			     Pointer<GroupWidget> aParent,
			     const string& aName,
			     const string& aTitle);

		virtual void title(const string& aTitle);
		virtual string title() const;
		virtual void move(const QPoint& to);
		virtual void resize(const QSize& to, int is_arranging = 0);
		virtual void move(const Hui::Position& to) { Widget::move(to); }
		virtual void resize(const Hui::Size& to) { Widget::resize(to); }

		QLabel* qtitle() { return mTitle; }
		const QLabel* qtitle() const { return mTitle; }

	private:
		QLabel* mTitle;

	}; // class TitledWidget

	// --------------------------------------------------

	class InputLineWidget : public TitledWidget,
				public Hui::InputLineWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		InputLineWidget(Hui& aHui,
				Pointer<GroupWidget> aParent,
				const string& aName,
				const string& aInitialText,
				int aMaxLength,
				const string& aValidator,
				const string& aTitle,
				const string& aComment);

		virtual void text(const string& aText);
		virtual string text() const;

		virtual void title(const string& aTitle) { TitledWidget::title(aTitle); }
		virtual string title() const { return TitledWidget::title(); }

		virtual QWidget* qwidget();
		virtual const QWidget* qwidget() const;

	private:
		QLineEditEnterLeave* mInputLine;
		QValidator* mValidator;
		string mComment;

	}; // class InputLineWidget

	friend class InputLineWidget;

	// --------------------------------------------------

	class ButtonsGroupWidget : public Widget,
				   virtual public Hui::ButtonsGroupWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		ButtonsGroupWidget(Hui& aHui,
				   Pointer<GroupWidget> aParent,
				   const string& aName,
				   const string& aTitle,
				   const vector<string>& aComments);

		virtual void text(int boxno,
				  const string& aText);
		virtual string text(int boxno) const;
		virtual void title(const string& aTitle);
		virtual string title() const;
		virtual void disable();
		virtual void enable();
		virtual void disable_item(int boxno);
		virtual void enable_item(int boxno);
		virtual bool item_disabled(int boxno) const;
		virtual int numberOfItems() const;


		virtual QWidget* qwidget();
		virtual const QWidget* qwidget() const;

	protected:
		QButtonGroup* boxes() { return mBoxes; }
		const QButtonGroup* boxes() const { return mBoxes; }
		HuiQtButtonGroupClick& click() { return mButtonGroupClick; }

	private:
		QButtonGroup* mBoxes;
		HuiQtButtonGroupClick mButtonGroupClick;
		vector<string> mComments;

	}; // class ButtonsGroupWidget

	// --------------------------------------------------

	class CheckBoxesWidget : public ButtonsGroupWidget,
				 public Hui::CheckBoxesWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		CheckBoxesWidget(Hui& aHui,
				 Pointer<GroupWidget> aParent,
				 const string& aName,
				 const vector<string>& aTexts,
				 const vector<bool>& aInitialValues,
				 const string& aTitle,
				 const vector<string>& aComments);

		virtual bool state(int boxno,
				   int aNewState);
		virtual bool state(int boxno) const;
		virtual bool toggle(int boxno);

	}; // class CheckBoxesWidget

	// --------------------------------------------------

	class RadioMenuWidget : public ButtonsGroupWidget,
				public Hui::RadioMenuWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		RadioMenuWidget(Hui& aHui,
				Pointer<GroupWidget> aParent,
				const string& aName,
				const vector<string>& aTexts,
				int aInitialValue,
				const string& aTitle,
				const vector<string>& aComments);

		virtual void set(int itemno);
		virtual int set() const;

	}; // class RadioMenuWidget

	// --------------------------------------------------

	class ListBoxWidget : public TitledWidget,
			      public Hui::ListBoxWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		ListBoxWidget(Hui& aHui,
			      Pointer<GroupWidget> aParent,
			      const string& aName,
			      const vector<string>& aTexts,
			      int aInitiallySelected,
			      const string& aTitle,
			      const string& aComment);

		virtual int numberOfItems() const;
		virtual string text(int aItemNo) const;
		virtual void text(int aItemNo, const string& aText);
		virtual int select() const;
		virtual void select(int aItemNo);
		virtual void add(int aBefore, const string& aText);
		virtual void remove(int aItemNo);
		virtual void removeAll();

		virtual void title(const string& aTitle) {
			mTitleText = aTitle;
			updatetexts();
		}
		virtual string title() const {
			return mTitleText;
		}
		
		virtual QWidget* qwidget();
		virtual const QWidget* qwidget() const;

	protected:
		virtual void updatetexts();
		
		virtual int widest_column(const vector<string>& aTexts,
					  const string& aTitle);
		virtual string columns_titletext_magic(const vector<string>& aTexts,
							      const string& aTitle);
		virtual vector<string> columns_text_magic(const vector<string>& aTexts,
						  const string& aTitle);
		
	private:
		vector<string> mTexts;
		string mTitleText;
		QListBoxEnterLeave* mListBox;
		HuiQtListBoxHighlight mListBoxHighlight;
		string mComment;

	}; // class ListBoxWidget

	// --------------------------------------------------

	class TextEditWidget : public TitledWidget,
			       public Hui::TextEditWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		TextEditWidget(Hui& aHui,
			       Pointer<GroupWidget> aParent,
			       const string& aName,
			       const string& aInitialText,
			       bool aAllowEdit,
			       const string& aTitle,
			       const string& aComment);

		virtual void text(const string& aText);
		virtual string text() const;

		virtual void title(const string& aTitle) { TitledWidget::title(aTitle); }
		virtual string title() const { return TitledWidget::title(); }

		virtual QWidget* qwidget();
		virtual const QWidget* qwidget() const;
		
	private:
		QMultiLineEditEnterLeave* mTextEdit;
		string mComment;
		
	}; // class TextEditWidget
	
	// --------------------------------------------------

	class ProgressIndicatorWidget : public TitledWidget,
					public Hui::ProgressIndicatorWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		ProgressIndicatorWidget(Hui& aHui,
					Pointer<GroupWidget> aParent,
					const string& aName,
					float aMin,
					float aMax,
					float aInitial,
					const string& aTitle,
					const string& aComment);

		virtual void set(float aValue);
		virtual float set() const;

		virtual void title(const string& aTitle) { TitledWidget::title(aTitle); }
		virtual string title() const { return TitledWidget::title(); }

		virtual QWidget* qwidget();
		virtual const QWidget* qwidget() const;

	private:
		Hui& mHui;
		QProgressBar* mProgressIndicator;
		float mMin, mMax;
		string mComment;

	}; // class ProgressIndicatorWidget

	// --------------------------------------------------

	class MenuItemWidget : public Widget,
			       virtual public Hui::MenuItemWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		MenuItemWidget(Hui& aHui,
			       const string& aName,
			       Pointer<LanguageInterface::CallbackData> aCallbackData,
			       MenuItemWidget* aParent);

		virtual void move(const Position& aPosition) {
			Hui::MenuItemWidget::move(aPosition);
		}
		virtual Position position() const {
			return Hui::MenuItemWidget::position();
		}
		virtual void resize(const Size& aSize) {
			Hui::MenuItemWidget::resize(aSize);
		}
		virtual Size size() const {
			return Hui::MenuItemWidget::size();
		}
		virtual Size minSize() const { return Size(); }
		virtual Size maxSize() const { return Size(); }
		virtual QWidget* qwidget() { return 0; }
		virtual const QWidget* qwidget() const { return 0; }
		virtual void disable();
		virtual void enable();
		virtual bool disabled() const;
		virtual void refresh();

		HuiQt& hui() { return mHui; }

	protected:
		MenuItemWidget* parent() { return mParent; }

	private:
		HuiQt& mHui;
		MenuItemWidget* mParent;
		bool mDisabled;

	}; // class MenuItemWidget

	// --------------------------------------------------

	class MenuSeparatorWidget : public MenuItemWidget,
				    public Hui::MenuSeparatorWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		MenuSeparatorWidget(Hui& aHui,
				    const string& aName,
				    MenuItemWidget* aParent);

	private:

	}; // class MenuSeparatorWidget

	// --------------------------------------------------

	class MenuButtonWidget : public MenuItemWidget,
				 public Hui::MenuButtonWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		MenuButtonWidget(Hui& aHui,
				 const string& aName,
				 const string& aText,
				 Pointer<LanguageInterface::CallbackData> aCallbackData,
				 const string& aKey,
				 const string& aComment,
				 MenuItemWidget* aParent);

		virtual void key(const string& aKey);
		virtual void text(const string& aText);

		virtual void click() { Hui::MenuButtonWidget::click(); }

	private:

	}; // class MenuButtonWidget

	// --------------------------------------------------

	class MenuSubmenuWidget : public MenuItemWidget, public Hui::MenuSubmenuWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		MenuSubmenuWidget(Hui& aHui,
				  const string& aName,
				  const string& aText,
				  const string& aComment,
				  MenuItemWidget* aParent);
		virtual Pointer<Hui::MenuItemWidget> addButton(const string& aName,
							       const string& aText,
							       Pointer<LanguageInterface::CallbackData> aCallbackData,
							       const string& aKey,
							       const string& aComment,
							       Pointer<Hui::MenuItemWidget> aBefore);
		virtual Pointer<Hui::MenuItemWidget> addSeparator(const string& aName,
								  Pointer<Hui::MenuItemWidget> aBefore);
		virtual Pointer<Hui::MenuItemWidget> addSubmenu(const string& aName,
								const string& aText,
								const string& aComment,
								Pointer<Hui::MenuItemWidget> aBefore);
		virtual void add(Pointer<Hui::MenuItemWidget> aBefore,
				 Pointer<Hui::MenuItemWidget> aItem);
		virtual void remove(Pointer<Hui::MenuItemWidget> aItem);

		virtual void text(const string& aText);

	private:

	}; // class MenuSubmenuWidget

	// --------------------------------------------------

	class MenuBarWidget : public MenuSubmenuWidget
	{
		CLASS_METHODS_DESCRIPTIONS_DECLARATION;

	public:
		MenuBarWidget(Hui& aHui);
		virtual Pointer<Hui::MenuItemWidget> addSubmenu(const string& aName,
								const string& aText,
								const string& aComment,
								Pointer<Hui::MenuItemWidget> aBefore);
		virtual void refresh();

	private:

	}; // class MenuBarWidget

	// --------------------------------------------------

public:
	const Subwidgets& subwidgets() const { return mWidgets; }
	Subwidgets& subwidgets() { return mWidgets; }

	static string translateAccel( const string& aText );
	static string stripAccel( const string& aText );

private:
	Subwidgets mWidgets;
	Pointer<MenuSubmenuWidget> mMenuBar;

	static QValidator* make_lineedit_validator(const string& aValidatorType);

}; // class HuiQt

//======================================================================

#ifndef QDIALOG_H
#include <qdialog.h>
#endif

#ifndef QPUSHBT_H
#include <qpushbt.h>
#endif

#ifndef QLINED_H
#include <qlined.h>
#endif

#ifndef QCHKBOX_H
#include <qchkbox.h>
#endif

#ifndef QRADIOBT_H
#include <qradiobt.h>
#endif

#ifndef QLISTBOX_H
#include <qlistbox.h>
#endif

#ifndef QMLINED_H
#include <qmlined.h>
#endif

#ifndef QMAINWINDOW_H
#include <qmainwindow.h>
#endif

//======================================================================

class QResizableDialog : public QDialog
{
	Q_OBJECT;

public:
	QResizableDialog(QWidget* aParent,
			 const string& aName, bool aModal);
	//virtual ~QResizableDialog() { cerr << "~QResizableDialog()" << endl; }

	virtual void endModal() { accept(); }

signals:
	void resized(const QSize& aSize);
	void enteredLeft(bool aEntered);

protected:
	virtual void resizeEvent(QResizeEvent* event)
	{
		emit resized(event->size());
	}

	virtual void enterEvent(QResizeEvent* event)
	{
		emit enteredLeft(true);
		QDialog::enterEvent(event);
	}

	virtual void leaveEvent(QResizeEvent* event)
	{
		emit enteredLeft(false);
		QDialog::leaveEvent(event);
	}

}; // class QResizableDialog

//======================================================================

class QResizableMainWindow : public QMainWindow
{
	Q_OBJECT;

public:
	QResizableMainWindow(const string& aName);
	//virtual ~QResizableMainWindow() { cerr << "~QResizableMainWindow()" << endl; }

	signals:
	void resized(const QSize& aSize);
	void enteredLeft(bool aEntered);

protected:
	virtual void resizeEvent(QResizeEvent* event)
	{
		emit resized(event->size());
	}

	virtual void enterEvent(QResizeEvent* event)
	{
		emit enteredLeft(true);
		QMainWindow::enterEvent(event);
	}

	virtual void leaveEvent(QResizeEvent* event)
	{
		emit enteredLeft(false);
		QMainWindow::leaveEvent(event);
	}

}; // class QResizableMainWindow

//======================================================================

class QPushButtonEnterLeave : public QPushButton
{
	Q_OBJECT;

public:
	QPushButtonEnterLeave(const char* text,
			      QWidget* parent,
			      const char* name)
		: QPushButton(text, parent, name ) {}

protected:
	virtual void focusInEvent(QFocusEvent* e)
	{
		emit enter_signal();
		QPushButton::focusInEvent(e);
	}

	virtual void focusOutEvent(QFocusEvent* e)
	{
		emit leave_signal();
		QPushButton::focusOutEvent(e);
	}

signals:
	void enter_signal();
	void leave_signal();

}; // class QPushButtonEnterLeave

//======================================================================

class QLineEditEnterLeave : public QLineEdit
{
	Q_OBJECT;

public:
	QLineEditEnterLeave(QWidget* parent,
			    const char* name)
		: QLineEdit(parent, name) {}

protected:
	virtual void focusInEvent(QFocusEvent* e)
	{
		emit enter_signal();
		QLineEdit::focusInEvent(e);
	}

	virtual void focusOutEvent(QFocusEvent* e)
	{
		emit leave_signal();
		QLineEdit::focusOutEvent(e);
	}

signals:
	void enter_signal();
	void leave_signal();

}; // class QLineEditEnterLeave

//======================================================================

class QCheckBoxEnterLeave : public QCheckBox
{
	Q_OBJECT;

public:
	QCheckBoxEnterLeave(const char* title,
			    QWidget* parent,
			    const char* name = 0)
		: QCheckBox(title, parent, name) {}

protected:
	virtual void focusInEvent(QFocusEvent* e)
	{
		emit enter_signal();
		QCheckBox::focusInEvent(e);
	}

	virtual void focusOutEvent(QFocusEvent* e)
	{
		emit leave_signal();
		QCheckBox::focusOutEvent(e);
	}

signals:
	void enter_signal();
	void leave_signal();

}; // class QCheckBoxEnterLeave

//======================================================================

class QRadioButtonEnterLeave : public QRadioButton
{
	Q_OBJECT;

public:
	QRadioButtonEnterLeave(const char* title,
			       QWidget* parent,
			       const char* name = 0)
		: QRadioButton(title, parent, name) {}

protected:
	virtual void focusInEvent(QFocusEvent* e)
	{
		emit enter_signal();
		QRadioButton::focusInEvent(e);
	}

	virtual void focusOutEvent(QFocusEvent* e)
	{
		emit leave_signal();
		QRadioButton::focusOutEvent(e);
	}

signals:
	void enter_signal();
	void leave_signal();

}; // class QRadioButtonEnterLeave

//======================================================================

class QListBoxEnterLeave : public QListBox
{
	Q_OBJECT;

public:
	QListBoxEnterLeave(QWidget* parent,
			   const char* name)
		: QListBox(parent, name) {}

protected:
	virtual void focusInEvent(QFocusEvent* e)
	{
		emit enter_signal();
		QListBox::focusInEvent(e);
	}

	virtual void focusOutEvent(QFocusEvent* e)
	{
		emit leave_signal();
		QListBox::focusOutEvent(e);
	}

signals:
	void enter_signal();
	void leave_signal();

}; // class QListBoxEnterLeave

//======================================================================

class QMultiLineEditEnterLeave : public QMultiLineEdit
{
	Q_OBJECT;

public:
	QMultiLineEditEnterLeave(QWidget* parent,
				 const char* name)
		: QMultiLineEdit(parent, name) {}

protected:
	virtual void focusInEvent(QFocusEvent* e)
	{
		emit enter_signal();
		QMultiLineEdit::focusInEvent(e);
	}

	virtual void focusOutEvent(QFocusEvent* e)
	{
		emit leave_signal();
		QMultiLineEdit::focusOutEvent(e);
	}

signals:
	void enter_signal();
	void leave_signal();

}; // class QMultiLineEditEnterLeave

//======================================================================

#endif // HuiQt_hh
