/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiQt_ButtonWidget.cc,v 1.29 2001/06/20 10:24:45 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiQt.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiQt_hh
#include "HuiQt.hh"
#endif

//======================================================================

HuiQt::ButtonWidget::ButtonWidget( HuiQt& aHui, Pointer<GroupWidget> aParent, const string& aName, const string& aText, Pointer<LanguageInterface::CallbackData> aCallbackData, const string& aButtonType, const string& aComment )
	: Hui::Widget( aHui, aName ), Hui::EmbeddedWidget( aHui, aName ), HuiQt::Widget( aHui, aName ), Hui::ButtonWidget( aHui, aName, aCallbackData ),
	mButton( NULL), mButtonClick( *this ), mComment( aComment )
{
	ContainerWidget *cp;
	GroupWidget *gp;

	parent( aParent );

	if ((cp = dynamic_cast<ContainerWidget*> ( (Hui::Widget*) parent() )) != NULL )
		mButton = new QPushButtonEnterLeave(translateAccel(aText).c_str(),
			const_cast<QWidget*> (cp->qwidget()), aName.c_str() );
	else if ((gp = dynamic_cast<GroupWidget*> ( (Hui::Widget*) parent() )) != NULL )
		mButton = new QPushButtonEnterLeave(translateAccel(aText).c_str(),
			gp->qwidget(), aName.c_str() );
	else
		printf("argl, panic!\n");

	mButton->resize( mButton->sizeHint() );
	mButton->setMinimumSize( mButton->sizeHint() );
	setsize(mButton->size());

	if ( aButtonType == "default" )
		mButton->setDefault( true );
	else if ( aButtonType != "normal" )
		throw Error( string( "Invalid button type `" ) + aButtonType + "' (`normal' or `default' expected)" );

	mButton->connect( mButton, SIGNAL( enter_signal() ), &huiQtEnterLeave(), SLOT( enter() ) );
	mButton->connect( mButton, SIGNAL( leave_signal() ), &huiQtEnterLeave(), SLOT( leave() ) );
	mButton->connect( mButton, SIGNAL( clicked() ), &mButtonClick, SLOT( click() ) );

} // HuiQt::ButtonWidget::ButtonWidget

//======================================================================

HuiQt::ButtonWidget::~ButtonWidget()
{
	// Parent toplevel widget will delete mButton on it's deletion

} // HuiQt::ButtonWidget::~ButtonWidget

//======================================================================

void HuiQt::ButtonWidget::text( const string& aText )
{
	if ( mButton )
		mButton->setText( aText.c_str() );

} // HuiQt::ButtonWidget::text

//======================================================================

string HuiQt::ButtonWidget::text() const
{
	if ( mButton )
		return static_cast<const char*>(mButton->text());
	else
		return string();

} // HuiQt::ButtonWidget::text

//======================================================================

void
HuiQt::ButtonWidget::callback_completed(const WidgetCallbackData& aCallbackData)
{
	const string& result = aCallbackData.result();

	if (!result.empty()) {
		HuiQt::DialogWidget *dp = dynamic_cast<DialogWidget*>((Hui::Widget *) parent());
		HuiQt::ContainerWidget *cp = dynamic_cast<ContainerWidget*>((Hui::Widget *) parent());
		if (cp != NULL) {
			/* we need to find the TopLevelWidget */
			HuiQt::ContainerWidget *next;
			do {
				next = dynamic_cast<ContainerWidget*>((Hui::Widget *) cp->parent());
				if (next != NULL)
					cp = next;
			} while (next != NULL);

			/* cp now contains toplevel container */
			dp = dynamic_cast<DialogWidget*>((Hui::Widget *) cp->parent());
		}
		if (dp != NULL)
			dp->endModal(result);
	}

} // HuiQt::ButtonWidget::callback_completed

//======================================================================

QWidget* HuiQt::ButtonWidget::qwidget()
{
	return(mButton);

} // HuiQt::ButtonWidget::qwidget

//======================================================================

const QWidget* HuiQt::ButtonWidget::qwidget() const
{
	return(mButton);

} // HuiQt::ButtonWidget::qwidget

//======================================================================
