/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiQt_ButtonsGroupWidget.cc,v 1.27 2001/06/20 10:24:45 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiQt.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiQt_hh
#include "HuiQt.hh"
#endif

#ifndef QBTTNGRP_H
#include <qbttngrp.h>
#endif

//======================================================================

HuiQt::ButtonsGroupWidget::ButtonsGroupWidget(Hui& aHui,
					      Pointer<GroupWidget> aParent,
					      const string& aName,
					      const string& aTitle,
					      const vector<string>& aComments)
	: Hui::Widget(aHui, aName),
	  Hui::EmbeddedWidget(aHui, aName),
	  HuiQt::Widget(aHui, aName),
	  Hui::ButtonsGroupWidget(aHui, aName),
	  mButtonGroupClick(*this),
	  mComments(aComments)
{
	parent(aParent);

	ContainerWidget *cp = dynamic_cast<ContainerWidget*>((Hui::Widget*) parent());
	GroupWidget *gp = dynamic_cast<GroupWidget*>((Hui::Widget*) parent());
	
	if (cp != NULL)
		mBoxes = new QButtonGroup(stripAccel(aTitle).c_str(),
					  const_cast<QWidget*>(cp->qwidget()),
					  aName.c_str());
	else if (gp != NULL)
		mBoxes = new QButtonGroup(stripAccel(aTitle).c_str(),
					  const_cast<QWidget*>(gp->qwidget()),
					  aName.c_str());

} // HuiQt::ButtonsGroupWidget::ButtonsGroupWidget

//======================================================================

void HuiQt::ButtonsGroupWidget::text( int boxno, const string& aText )
{
	QButton* box = 0;
	if ( boxes() && ( box = boxes()->find( boxno ) ) != 0 )
		box->setText( aText.c_str() );

} // HuiQt::ButtonsGroupWidget::text

//======================================================================

string HuiQt::ButtonsGroupWidget::text( int boxno ) const
{
	QButton* box = 0;
	if ( boxes() && ( box = boxes()->find( boxno ) ) != 0 )
		return static_cast<const char*>(box->text());
	else
		return string();

} // HuiQt::ButtonsGroupWidget::text

//======================================================================

void HuiQt::ButtonsGroupWidget::title( const string& aTitle )
{
	if ( boxes() )
		boxes()->setTitle( aTitle.c_str() );

} // HuiQt::ButtonsGroupWidget::title

//======================================================================

string HuiQt::ButtonsGroupWidget::title() const
{
	if ( boxes() )
		return static_cast<const char*>(boxes()->title());
	else
		return string();

} // HuiQt::ButtonsGroupWidget::title

//======================================================================

QWidget* HuiQt::ButtonsGroupWidget::qwidget()
{
	return boxes();

} // HuiQt::ButtonsGroupWidget::qwidget

//======================================================================

const QWidget* HuiQt::ButtonsGroupWidget::qwidget() const
{
	return boxes();

} // HuiQt::ButtonsGroupWidget::qwidget

//======================================================================

void HuiQt::ButtonsGroupWidget::disable()
{
	QButton* box = 0;
	for ( int boxno = 0; ( box = boxes()->find( boxno ) ) != 0; ++boxno )
		box->setEnabled( false );

	HuiQt::Widget::disable();

} // HuiQt::ButtonsGroupWidget::disable

//======================================================================

void HuiQt::ButtonsGroupWidget::enable()
{
	QButton* box = 0;
	for ( int boxno = 0; ( box = boxes()->find( boxno ) ) != 0; ++boxno )
		box->setEnabled( true );

	HuiQt::Widget::enable();

} // HuiQt::ButtonsGroupWidget::enable

//======================================================================

void HuiQt::ButtonsGroupWidget::disable_item( int boxno )
{
	QButton* box = boxes()->find( boxno );
	if ( box )
		box->setEnabled( false );

} // HuiQt::ButtonsGroupWidget::disable_item

//======================================================================

void HuiQt::ButtonsGroupWidget::enable_item( int boxno )
{
	QButton* box = boxes()->find( boxno );
	if ( box )
		box->setEnabled( true );

} // HuiQt::ButtonsGroupWidget::enable_item

//======================================================================

bool HuiQt::ButtonsGroupWidget::item_disabled( int boxno ) const
{
	QButton* box = boxes()->find( boxno );
	if ( box )
		return !box->isEnabled();
	else
		return(true);

} // HuiQt::ButtonsGroupWidget::item_disabled

//======================================================================

int HuiQt::ButtonsGroupWidget::numberOfItems() const
{
	int result  = 0;
	for ( ; boxes()->find( result ); ++result );
	return(result);

} // HuiQt::ButtonsGroupWidget::numberOfItems

//======================================================================
