/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiQt_ContainerWidget.cc,v 1.28 2001/06/20 10:24:46 alex Exp $
 */

// -*- C++ -*-   09 September 2000
// Alexander Langer <alex@FreeBSD.org>

#ifndef HuiQt_hh
#include "HuiQt.hh"
#endif

//======================================================================

HuiQt::ContainerWidget::ContainerWidget(HuiQt& aHui,
					Pointer<GroupWidget> aParent,
					const string& aName)
    : Hui::Widget(aHui, aName),
      Hui::ContainerWidget(aHui, aName),
      Hui::GroupWidget(aHui, aName),
      HuiQt::GroupWidget(aHui, aName, Position(0, 0), Size(0, 0))
{
	parent(aParent);
	mFixedSize = -1;
	
} // HuiQt::ContainerWidget::ContainerWidget

//======================================================================

HuiQt::ContainerWidget::~ContainerWidget()
{

} // HuiQt::ContainerWidget::~ContainerWidget

//======================================================================

void HuiQt::ContainerWidget::show()
{
	Subwidgets::iterator sw;

	for (sw = subwidgets().begin();
	    sw != subwidgets().end();
	    sw++)
		(*sw)->show();

	mHidden = FALSE;

} // HuiQt::ContainerWidget::show

//======================================================================

void HuiQt::ContainerWidget::hide()
{
	Subwidgets::iterator sw;

	for (sw = subwidgets().begin();
	    sw != subwidgets().end();
	    sw++)
		(*sw)->hide();

	mHidden = TRUE;

} // HuiQt::ContainerWidget::hide

//======================================================================

bool HuiQt::ContainerWidget::hidden() const
{
	return(mHidden);

} // HuiQt::ContainerWidget::hidden()

//======================================================================

void HuiQt::ContainerWidget::disable()
{
	Subwidgets::iterator sw;

	for (sw = subwidgets().begin();
	    sw != subwidgets().end();
	    sw++)
		(*sw)->disable();

	mDisabled = TRUE;

} // HuiQt::ContainerWidget::disable

//======================================================================

void HuiQt::ContainerWidget::enable()
{
	Subwidgets::iterator sw;

	for (sw = subwidgets().begin();
	    sw != subwidgets().end();
	    sw++)
		(*sw)->enable();

	mDisabled = FALSE;

} // HuiQt::ContainerWidget::enable

//======================================================================

bool HuiQt::ContainerWidget::disabled() const
{
	return(mDisabled);

} // HuiQt::ContainerWidget::disabled

//======================================================================

void HuiQt::ContainerWidget::move(const Hui::Position &aPosition)
{
	/* Move Container */
	move(translate(aPosition));

} // HuiQt::ContainerWidget::move

//======================================================================

void HuiQt::ContainerWidget::move(const QPoint &aPosition)
{
	Subwidgets::iterator sw;
	QPoint offset;
	ContainerWidget *cp;

	offset = QPoint(0, 0);

	/* If parent is a ContainerWidget, too, we must add its position
	   This is, because we move() on the container's parent's widget
	*/
	if ((cp = dynamic_cast<HuiQt::ContainerWidget*>((Hui::Widget*) parent())) != NULL) {
		offset = cp->real_position();
	}

	/* Move Container */
	mPosition = aPosition + offset;

	/* Move all childs to its position */
	for (sw = subwidgets().begin();
	     sw != subwidgets().end();
	     sw++)
		(*sw)->move((*sw)->position());

} // HuiQt::ContainerWidget::move

//======================================================================

class Hui::Position HuiQt::ContainerWidget::position() const
{
	return(translate(mPosition));

} // HuiQt::ContainerWidget::position

//======================================================================

void HuiQt::ContainerWidget::resize(const Hui::Size & aSize)
{
	resize(translate(aSize));

} // HuiQt::ContainerWidget::resize

//======================================================================

void
HuiQt::ContainerWidget::resize(const QSize & aSize, int  is_arranging = 0)
{
	if (aSize.width() == -1 && aSize.height() == -1)
		return;
	if (aSize.width() >= 0 && aSize.height() >= 0) {
		mSize = aSize;
		arrange();
		return;
	}
	if (!is_arranging && (aSize.width() >= 0 || aSize.height() >= 0)) {
		ContainerWidget *cp = dynamic_cast<ContainerWidget*>( (Hui::Widget*) parent());
		GroupWidget *gp = dynamic_cast<GroupWidget*>( (Hui::Widget*) parent());

		setFixedSize((aSize.width() < 0) ? aSize.height() : aSize.width());

		/* arrange parent again since we changed the fixed size */
		if (cp != NULL)
			cp->arrange();
		if (gp != NULL)
			gp->arrange();
		return;
	}

} // HuiQt::ContainerWidget::resize

//======================================================================

class Hui::Size HuiQt::ContainerWidget::size() const
{
	return(translate(mSize));

} // HuiQt::ContainerWidget::size

//======================================================================

QWidget*
HuiQt::ContainerWidget::qwidget() {

	return parent()->qwidget();

} // HuiQt::ContainerWidget::qwidget

//======================================================================

const QWidget*
HuiQt::ContainerWidget::qwidget() const {
	return parent()->qwidget();
}

//======================================================================

Hui::Size
HuiQt::ContainerWidget::translate(const QSize& aSize) const
{
    const QFontMetrics fm = qwidget()->fontMetrics();
    const int char_width = fm.width( 'O' );
    const int char_height = fm.lineSpacing(); // + fm.descent();
    
    return Hui::Size( aSize.width() / char_width, aSize.height() / char_height );
    
} // HuiQt::ContainerWidget::translate

//======================================================================

Hui::Position
HuiQt::ContainerWidget::translate(const QPoint& aPosition) const
{
	const QFontMetrics fm = qwidget()->fontMetrics();
	const int char_width = fm.width( 'O' );
	const int char_height = fm.lineSpacing(); // + fm.descent();
	return Hui::Position( aPosition.x() / char_width, aPosition.y() / char_height);

} // HuiQt::ContainerWidget::translate

//======================================================================

QSize
HuiQt::ContainerWidget::translate(const Hui::Size& aSize) const
{
    const QFontMetrics fm = qwidget()->fontMetrics();
    const int char_width = fm.width( 'O' );
    const int char_height = fm.lineSpacing(); // + fm.descent();
    
    return QSize( aSize.width() * char_width, aSize.height() * char_height );
    
} // HuiQt::ContainerWidget::translate

//======================================================================

QPoint
HuiQt::ContainerWidget::translate(const Hui::Position& aPosition) const
{
	const QFontMetrics fm = qwidget()->fontMetrics();
	const int char_width = fm.width( 'O' );
	const int char_height = fm.lineSpacing(); // + fm.descent();

	return QPoint(aPosition.x() * char_width, aPosition.y() * char_height);

} // HuiQt::ContainerWidget::translate

//======================================================================
