/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiQt_DialogWidget.cc,v 1.27 2001/06/20 10:24:46 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiQt.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiQt_hh
#include "HuiQt.hh"
#endif

//======================================================================

HuiQt::DialogWidget::DialogWidget( HuiQt& aHui, Pointer<Hui::Widget> aParent, const string& aName, const string& aTitle, const Position& aPosition, const Size& aSize )
	: Hui::Widget( aHui, aName ), Hui::GroupWidget( aHui, aName ), HuiQt::Widget( aHui, aName ), HuiQt::ToplevelWidget( aHui, aName, aPosition, aSize ),
	Hui::DialogWidget( aHui, aName ), Hui::ToplevelWidget( aHui, aName ), mHui( aHui ),
	mWindow( new QResizableDialog( aParent ? dynamic_cast<HuiQt::Widget&>( *aParent ).qwidget() : 0, aName, true ) ),
	mRunning( false )
{
	mWindow->setCaption( aTitle.c_str() );
	mWindow->resize( translate( aPosition, aSize ).size() );
	QObject::connect( mWindow, SIGNAL( resized( const QSize& ) ), this, SLOT( resized( const QSize& ) ) );
	QObject::connect( mWindow, SIGNAL( enteredLeft( bool ) ), this, SLOT( enteredLeft( bool ) ) );

} // HuiQt::DialogWidget::DialogWidget

//======================================================================

HuiQt::DialogWidget::~DialogWidget()
{
	delete mWindow;

} // HuiQt::DialogWidget::~DialogWidget

//======================================================================

const string& HuiQt::DialogWidget::result() const
{
	return(mResult);

} // HuiQt::DialogWidget::result

//======================================================================

const string& HuiQt::DialogWidget::exec()
{
	if ( qtwindow() ) {
		try {
			mRunning = true;
			mHui.subwidgets().add( this );
		/*int status =*/ qtwindow()->exec();
			mHui.subwidgets().remove( this );
			mRunning = false;
		}
		catch ( ... ) {
			mHui.subwidgets().remove( this );
			mRunning = false;
			throw;
		}
	}

	return(mResult);

} // HuiQt::DialogWidget::exec

//======================================================================

void HuiQt::DialogWidget::endModal( const string& aResult )
{
	mResult = aResult;
	if ( qtwindow() )
		qtwindow()->endModal();

} // HuiQt::DialogWidget::endModal

//======================================================================

QWidget* HuiQt::DialogWidget::qwidget()
{
	return(mWindow);

} // HuiQt::DialogWidget::qwidget

//======================================================================

const QWidget* HuiQt::DialogWidget::qwidget() const
{
	return(mWindow);

} // HuiQt::DialogWidget::qwidget

//======================================================================
