/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiQt_GroupWidget.cc,v 1.29 2001/06/20 10:24:46 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiQt.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiQt_hh
#include "HuiQt.hh"
#endif

#ifndef QLABEL_H
#include <qlabel.h>
#endif

//======================================================================

HuiQt::GroupWidget::GroupWidget( HuiQt& aHui, const string& aName, const Position& /*aPosition*/, const Size& /*aSize*/ )
	: Hui::Widget( aHui, aName ), Hui::GroupWidget( aHui, aName ), mHui( aHui )
{

} // HuiQt::GroupWidget::GroupWidget

//======================================================================

HuiQt::GroupWidget::~GroupWidget()
{

} // HuiQt::GroupWidget::~GroupWidget

//======================================================================

Pointer<Hui::LabelWidget>
HuiQt::GroupWidget::label(const string& aName,
			  const string& aText,
			  const Hui::Position& aPosition)
{
	Pointer<LabelWidget> label = new LabelWidget(mHui,
						     this,
						     aName,
						     aText);
	subwidgets().add(label);
	label->move(aPosition);
	arrange();
	label->show();

	return(label);

} // HuiQt::GroupWidget::label

//======================================================================

Pointer<Hui::ButtonWidget>
HuiQt::GroupWidget::button(const string& aName,
			   const string& aText,
			   Pointer<LanguageInterface::CallbackData> aCallbackData,
			   const string& aButtonType,
			   const Hui::Position& aPosition,
			   const string& aComment)
{
	Pointer<ButtonWidget> button = new ButtonWidget(mHui,
							this,
							aName,
							aText,
							aCallbackData,
							aButtonType,
							aComment);
	subwidgets().add(button);
	button->move(aPosition);
	arrange();
	button->show();

	return(button);

} // HuiQt::GroupWidget::button

//======================================================================

Pointer<Hui::InputLineWidget>
HuiQt::GroupWidget::inputLine(const string& aName,
			      const string& aInitialText,
			      int aMaxLength,
			      const string& aValidator,
			      const string& aTitle,
			      const Hui::Position& aPosition,
			      const string& aComment)
{
	Pointer<InputLineWidget> inputLine = new InputLineWidget(mHui,
								 this,
								 aName,
								 aInitialText,
								 aMaxLength,
								 aValidator,
								 aTitle,
								 aComment);
	subwidgets().add(inputLine);
	inputLine->move(aPosition);
	arrange();
	inputLine->show();

	return(inputLine);

} // HuiQt::GroupWidget::inputLine

//======================================================================

Pointer<Hui::CheckBoxesWidget>
HuiQt::GroupWidget::checkBoxes(const string& aName,
			       const vector<string>& aTexts,
			       const vector<bool>& aInitialValues,
			       const string& aTitle,
			       const Hui::Position& aPosition,
			       const vector<string>& aComments)
{
	Pointer<CheckBoxesWidget> checkBoxes = new CheckBoxesWidget(mHui,
								    this,
								    aName,
								    aTexts,
								    aInitialValues,
								    aTitle,
								    aComments);
	subwidgets().add(checkBoxes);
	checkBoxes->move(aPosition);
	arrange();
	checkBoxes->show();

	return(checkBoxes);

} // HuiQt::GroupWidget::checkBoxes

//======================================================================

Pointer<Hui::RadioMenuWidget>
HuiQt::GroupWidget::radioMenu(const string& aName,
			      const vector<string>& aTexts,
			      const int aInitialValue,
			      const string& aTitle,
			      const Hui::Position& aPosition,
			      const vector<string>& aComments)
{
	Pointer<RadioMenuWidget> radioMenu = new RadioMenuWidget(mHui,
								 this,
								 aName,
								 aTexts,
								 aInitialValue,
								 aTitle,
								 aComments);
	subwidgets().add(radioMenu);
	radioMenu->move(aPosition);
	arrange();
	radioMenu->show();

	return(radioMenu);

} // HuiQt::GroupWidget::radioMenu

//======================================================================

Pointer<Hui::ListBoxWidget>
HuiQt::GroupWidget::listBox(const string& aName,
			    const vector<string>& aTexts,
			    const int aInitiallySelected,
			    const string& aTitle,
			    const Hui::Position& aPosition,
			    const string& aComment)
{
	Pointer<ListBoxWidget> listBox = new ListBoxWidget(mHui,
							   this,
							   aName,
							   aTexts,
							   aInitiallySelected,
							   aTitle,
							   aComment);
	subwidgets().add(listBox);
	listBox->move(aPosition);
	arrange();
	listBox->show();

	return(listBox);

} // HuiQt::GroupWidget::listBox

//======================================================================

Pointer<Hui::TextEditWidget>
HuiQt::GroupWidget::textEdit(const string& aName,
			     const string& aInitialText,
			     bool aAllowEdit,
			     const string& aTitle,
			     const Hui::Position& aPosition,
			     const string& aComment)
{
	Pointer<TextEditWidget> textEdit = new TextEditWidget(mHui,
							      this,
							      aName,
							      aInitialText,
							      aAllowEdit,
							      aTitle,
							      aComment);
	subwidgets().add(textEdit);
	textEdit->move(aPosition);
	arrange();
	textEdit->show();

	return(textEdit);

} // HuiQt::GroupWidget::textEdit

//======================================================================

Pointer<Hui::ProgressIndicatorWidget>
HuiQt::GroupWidget::progressIndicator(const string& aName,
				      float aMin,
				      float aMax,
				      float aInitial,
				      const string& aTitle,
				      const Hui::Position& aPosition,
				      const string& aComment)
{
	Pointer<ProgressIndicatorWidget> progressIndicator = new ProgressIndicatorWidget(mHui,
											 this,
											 aName,
											 aMin,
											 aMax,
											 aInitial,
											 aTitle,
											 aComment);
	subwidgets().add(progressIndicator);
	progressIndicator->move(aPosition);
	arrange();
	progressIndicator->show();

	return(progressIndicator);

} // HuiQt::GroupWidget::progressIndicator

//======================================================================

Pointer<Hui::ContainerWidget>
HuiQt::GroupWidget::container(const string& aName,
			      const Hui::Position& aPosition,
			      const string& aAlignment,
			      const string& aComment)
{
	Pointer<ContainerWidget> container = new ContainerWidget(mHui,
								 this,
								 aName);

	subwidgets().add(container);

	container->move(aPosition);
	container->arrange(aAlignment, FALSE);	/* keep this order */
	arrange();				/* keep this order */
	container->show();

	return(container);

} // HuiQt::GroupWidget::container

//======================================================================

void HuiQt::GroupWidget::arrange()
{
	Subwidgets::iterator sw;
	QSize newsize, mysize;
	QPoint newpos;

	if (mArrangement == ARRANGE_FREE)
		return;

	mysize = real_size();
	/* First, calculate how much of the given space has
	   to be shared by widgets w/o fixed size */
	int sharedsize = (mArrangement == ARRANGE_VERTICAL) ? mysize.height() : mysize.width();
	int numwidgets = 0;
	
	for (sw = subwidgets().begin();
	     sw != subwidgets().end();
	     sw++) {
		ContainerWidget *cp = dynamic_cast<HuiQt::ContainerWidget*>((Hui::Widget*)*sw);
		HuiQt::Widget *gp = dynamic_cast<HuiQt::Widget*>((Hui::Widget*)*sw);
		int widgetfixedsize = -1;
		
		if (cp != NULL)
			widgetfixedsize = cp->getFixedSize();
		if (gp != NULL)
			widgetfixedsize = gp->getFixedSize();
		if (widgetfixedsize != -1) {
			sharedsize -= widgetfixedsize;
		} else
			numwidgets++;
	}
	
	if (mArrangement == ARRANGE_HORIZONTAL) {
		int ipos = 0, n = 0;
		
		for (sw = subwidgets().begin();
		     sw != subwidgets().end();
		     sw++) {
			int widgetfixedsize = -1;
			ContainerWidget *cp = dynamic_cast<HuiQt::ContainerWidget*>((Hui::Widget*) *sw);
			HuiQt::Widget *gp = dynamic_cast<HuiQt::Widget*>((Hui::Widget*) *sw);
			
			/* Check, if the current widget wants a fixed size */
			if (cp != NULL)
				widgetfixedsize = cp->getFixedSize();
			if (gp != NULL)
				widgetfixedsize = gp->getFixedSize();
			
			newpos = QPoint(ipos, 0);

			if (widgetfixedsize != -1) {
				/* If so, just move and resize it */
				newsize = QSize(widgetfixedsize, mysize.height());
				if (cp != NULL) {
					cp->move(newpos);
					cp->resize(newsize, TRUE);
				}
				if (gp != NULL) {
					gp->move(newpos);
					gp->resize(newsize, TRUE);
				}
				ipos += widgetfixedsize;
			} else {
				/* Calculate new size */
				int newwidth = sharedsize / numwidgets;
				newsize = QSize(newwidth, mysize.height());
				if (cp != NULL) {
					cp->move(newpos);
					cp->resize(newsize, TRUE);
				}
				if (gp != NULL) {
					gp->move(newpos);
					gp->resize(newsize, TRUE);
				}
				ipos += newwidth;
				numwidgets--;
				sharedsize -= newwidth;
			}
			
		}
	}
	if (mArrangement == ARRANGE_VERTICAL) {
		int ipos = 0, n = 0;

		for (sw = subwidgets().begin();
		    sw != subwidgets().end();
		    sw++) {
			int widgetfixedsize = -1;
			ContainerWidget *cp = dynamic_cast<HuiQt::ContainerWidget*>((Hui::Widget*)*sw);
			HuiQt::Widget *gp = dynamic_cast<HuiQt::Widget*>((Hui::Widget*)*sw);

			/* Check, if the current widget wants a fixed size */
			if (cp != NULL)
				widgetfixedsize = cp->getFixedSize();
			if (gp != NULL)
				widgetfixedsize = gp->getFixedSize();
			
			newpos = QPoint(0, ipos);

			if (widgetfixedsize != -1) {
				/* If so, just move and resize it */
				newsize = QSize(mysize.width(), widgetfixedsize);
				if (cp != NULL) {
					cp->move(newpos);
					cp->resize(newsize, TRUE);
				}
				if (gp != NULL) {
					gp->move(newpos);
					gp->resize(newsize, TRUE);
				}
				ipos += widgetfixedsize;
			} else {
				/* Calculate new size */
				int newheight = sharedsize / numwidgets;
				newsize = QSize(mysize.width(), newheight);
				if (cp != NULL) {
					cp->move(newpos);
					cp->resize(newsize, TRUE);
				}
				if (gp != NULL) {
					gp->move(newpos);
					gp->resize(newsize, TRUE);
				}
				ipos += newheight;
				numwidgets--;
				sharedsize -= newheight;
			}
			
		}
	}

} // HuiQt::GroupWidget::arrange

//======================================================================

void HuiQt::GroupWidget::arrange(const string& aAlignment, bool arrange_new = TRUE)
{
	if ( aAlignment == "horizontal" )
		mArrangement = ARRANGE_HORIZONTAL;
	else if ( aAlignment == "vertical" )
		mArrangement = ARRANGE_VERTICAL;
	else if ( aAlignment == "free" )
		mArrangement = ARRANGE_FREE;
	else if ( aAlignment != "" )
		throw Error( string( "Invalid alignment  `" ) + aAlignment + "' (`horizontal', `vertical' or `free' expected)" );

	if (arrange_new)
		arrange();

} // HuiQt::GroupWidget::arrange

//======================================================================

Pointer<Hui::Widget> HuiQt::GroupWidget::byName( const string& aName ) const
{
	Pointer<Hui::Widget> result = Hui::Widget::byName( aName );
	for ( Subwidgets::const_iterator w = mWidgets.begin(); !result && w != mWidgets.end(); ++w )
		result = (*w)->byName( aName );
	return(result);

} // HuiQt::GroupWidget::byName

//======================================================================

void HuiQt::GroupWidget::current( Pointer<Hui::EmbeddedWidget> aWidget )
{
	Pointer<HuiQt::Widget> w = dynamic_cast<HuiQt::Widget*>( (Hui::EmbeddedWidget*)aWidget );
	if ( w && w->qwidget() )
		w->qwidget()->setFocus();

} // HuiQt::GroupWidget::current

//======================================================================

Pointer<Hui::Widget> HuiQt::GroupWidget::current() const
{
	return subwidgets().find( qwidget()->focusWidget() );

} // HuiQt::GroupWidget::current

//======================================================================

void HuiQt::GroupWidget::erase()
{
	Subwidgets::iterator sw;

	for (sw = subwidgets().begin();
	     sw != subwidgets().end();
	     sw++) {
		ContainerWidget *cp = dynamic_cast<HuiQt::ContainerWidget*>((Hui::Widget*)*sw);
		HuiQt::Widget *gp = dynamic_cast<HuiQt::Widget*>((Hui::Widget*)*sw);
		HuiQt::TitledWidget *tp = dynamic_cast<HuiQt::TitledWidget*>((Hui::Widget*)*sw);

		if (cp != NULL) {
			cp->erase();
		} else if (tp != NULL) {
			qwidget()->removeChild(tp->qwidget());
			if (tp->qtitle()) {
				qwidget()->removeChild(tp->qtitle());
				delete tp->qtitle();
			}
			delete tp->qwidget();
		} else if (gp != NULL) {
			qwidget()->removeChild(gp->qwidget());
			delete gp->qwidget();
		}
	}
	subwidgets().clear();

} // HuiQt::GroupWidget::erase

//======================================================================
