/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiQt_ListBoxWidget.cc,v 1.32 2001/06/20 10:24:46 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiQt.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiQt_hh
#include "HuiQt.hh"
#endif

#ifndef QLABEL_H
#include <qlabel.h>
#endif

#ifndef String_hh
#include "String.hh"
#endif

#define MAX(a,b) ((a) > (b) ? (a) : (b))

//======================================================================

int
HuiQt::ListBoxWidget::widest_column(const vector<string>& aTexts,
				    const string& aTitle)
{
	int max = 0;
	bool found = 0;
	string::size_type tmp;
	size_t item;

	for (item = 0; item < aTexts.size(); item++) {
		tmp = aTexts[item].find("\t", 0);
		if (tmp != aTexts[item].npos) {
			max = MAX(max, tmp);
			found = 1;
		}
	}

	tmp = aTitle.find("\t", 0);
	if (tmp != aTitle.npos) {
		max = MAX(max, tmp);
		found = 1;
		}

	if (!found)
		return -1;
	else
		return max;
}

//======================================================================

vector<string>
HuiQt::ListBoxWidget::columns_text_magic(const vector<string>& aTexts,
					 const string& aTitle)
{
	vector<string> newTexts = aTexts;
	string newTitle = aTitle;
	size_t item;
	int max;
	
	do {
		max = widest_column(newTexts, newTitle);
		if (max != -1) {
			/* Replace each \t with 2 at least one tab up to widest column */
			string::size_type pos;
			for (item = 0; item < newTexts.size(); item++) {
				pos = newTexts[item].find("\t", 0);
				
				if (pos != newTexts[item].npos) {
					while (pos != max + 1)
						newTexts[item].insert(pos++, 1, ' ');
					newTexts[item].replace(pos, 1, ' ');
				}
			}
			pos = newTitle.find("\t", 0);
			if (pos != newTitle.npos) {
				while(pos != max + 1)
					newTitle.insert(pos++, 1, ' ');
				newTitle.replace(pos, 1, ' ');
			}
		}
	} while (max != -1);

	return(newTexts);
}

//======================================================================

string
HuiQt::ListBoxWidget::columns_titletext_magic(const vector<string>& aTexts,
					  const string& aTitle)
{
	vector<string> newTexts = aTexts;
	string newTitle = aTitle;
	size_t item;
	int max;
	
	do {
		max = widest_column(newTexts, newTitle);
		if (max != -1) {
			/* Replace each \t with 2 at least one tab up to widest column */
			string::size_type pos;
			for (item = 0; item < newTexts.size(); item++) {
				pos = newTexts[item].find("\t", 0);
				
				if (pos != newTexts[item].npos) {
					while (pos != max + 1)
						newTexts[item].insert(pos++, 1, ' ');
					newTexts[item].replace(pos, 1, ' ');
				}
			}
			pos = newTitle.find("\t", 0);
			if (pos != newTitle.npos) {
				while(pos != max + 1)
					newTitle.insert(pos++, 1, ' ');
				newTitle.replace(pos, 1, ' ');
			}
		}
	} while (max != -1);

	return(newTitle);
}

//======================================================================

HuiQt::ListBoxWidget::ListBoxWidget(Hui& aHui,
				    Pointer<GroupWidget> aParent,
				    const string& aName,
				    const vector<string>& aTexts,
				    int aInitiallySelected,
				    const string& aTitle,
				    const string& aComment)
	: Hui::Widget(aHui, aName),
	  Hui::EmbeddedWidget(aHui, aName),
	  TitledWidget(aHui, aParent, aName, aTitle),
	  Hui::ListBoxWidget(aHui, aName),
	  mListBoxHighlight(*this),
	  mComment(aComment),
	  mTexts(aTexts),
	  mTitleText(aTitle)
{
	parent(aParent);
	
	ContainerWidget *cp = dynamic_cast<ContainerWidget*>((Hui::Widget*) parent());
	GroupWidget *gp = dynamic_cast<GroupWidget*>((Hui::Widget*) parent());
	
	if (cp != NULL)
		mListBox = new QListBoxEnterLeave(const_cast<QWidget*>(cp->qwidget()),
						  aName.c_str());
	else if (gp != NULL)
		mListBox = new QListBoxEnterLeave(const_cast<QWidget*>(gp->qwidget()),
						  aName.c_str());
	
	mListBox->setMultiSelection(FALSE);
	
	if (qtitle())
		qtitle()->setBuddy(mListBox);

	mListBox->setAutoUpdate(FALSE);

	updatetexts();

	mListBox->setFixedVisibleLines(2);
	mListBox->resize(mListBox->sizeHint());
	mListBox->setMinimumSize(mListBox->sizeHint());
	mListBox->setColumnMode(1);
	setsize(mListBox->size());

	mListBox->setAutoUpdate(TRUE);
	select(aInitiallySelected);
	mListBox->repaint();

	mListBox->connect(mListBox,
			  SIGNAL(enter_signal()),
			  &huiQtEnterLeave(),
			  SLOT(enter()));
	mListBox->connect(mListBox,
			  SIGNAL(leave_signal()),
			  &huiQtEnterLeave(),
			  SLOT(leave()));
	mListBox->connect(mListBox,
			  SIGNAL(highlighted(int)),
			  &mListBoxHighlight,
			  SLOT(highlighted(int)));

} // HuiQt::ListBoxWidget::ListBoxWidget

//======================================================================

void
HuiQt::ListBoxWidget::updatetexts()
{
	int selected = select();

	if (mListBox != NULL) {
		vector<string> listTexts = columns_text_magic(mTexts, mTitleText);

		mListBox->clear();
		for (vector<string>::const_iterator s = listTexts.begin();
		     s != listTexts.end();
		     s++)
			mListBox->insertItem(s->c_str());
	}

	select(selected);
	mListBox->setColumnMode(1);

	/* Update Title */
	string newtitle = columns_titletext_magic(mTexts, mTitleText);
	TitledWidget::title(newtitle);
}

//======================================================================

int
HuiQt::ListBoxWidget::numberOfItems() const
{
	if ( mListBox )
		return mListBox->count();
	else
		return 0;

} // HuiQt::ListBoxWidget::numberOfItems

//======================================================================

string
HuiQt::ListBoxWidget::text(int aItemNo) const
{
	if (mListBox) {
		if (size_t(aItemNo) >= mListBox->count())
			throw runtime_error(string("ListBox itemno ")
					    + to_string(aItemNo)
					    + " is out of range 0.."
					    + to_string(mListBox->count() - 1));
		return mTexts[aItemNo];
	}
	else
		return string();
	
} // HuiQt::ListBoxWidget::text

//======================================================================

void
HuiQt::ListBoxWidget::text(int aItemNo, const string& aText)
{
	if (mListBox) {
		if (size_t(aItemNo) >= mListBox->count())
			throw runtime_error(string("ListBox itemno " )
					    + to_string(aItemNo)
					    + " is out of range 0.."
					    + to_string(mListBox->count() - 1));
		mTexts[aItemNo] = aText;
		updatetexts();
	}
	
} // HuiQt::ListBoxWidget::text

//======================================================================

int
HuiQt::ListBoxWidget::select() const
{
	if (mListBox)
		return mListBox->currentItem();
	else
		return 0;

} // HuiQt::ListBoxWidget::select

//======================================================================

void
HuiQt::ListBoxWidget::select(int aItemNo)
{
	if (aItemNo == -1)
		return;

	if (numberOfItems() && mListBox) {
		if (size_t(aItemNo) >= mListBox->count())
			throw runtime_error(string("ListBox itemno ")
					    + to_string(aItemNo)
					    + " is out of range 0.."
					    + to_string(mListBox->count() - 1));
		mListBox->setCurrentItem(aItemNo);
	}
	
} // HuiQt::ListBoxWidget::select

//======================================================================

void
HuiQt::ListBoxWidget::add(int aBefore, const string& aText)
{
	if (aBefore < 0 || aBefore > numberOfItems())
		throw runtime_error(string("ListBox beforeItem ")
				    + to_string(aBefore)
				    + " is out of range 0.."
				    + to_string(mListBox->count()));
	
	mTexts.insert(mTexts.begin() + aBefore, aText);
		
	if (mListBox) {
		/* Insert first to keep correct selection! */
		mListBox->insertItem(aText.c_str(), aBefore);
		if (numberOfItems() == 1)
			select(0);
		updatetexts();
	}

} // HuiQt::ListBoxWidget::add

//======================================================================

void
HuiQt::ListBoxWidget::remove(int aItemNo)
{
	mTexts.erase(mTexts.begin() + aItemNo);
	
	if (mListBox) {
		if (aItemNo > 0 && select() >= aItemNo)
			select(select() - 1);
		mListBox->removeItem(aItemNo);
		updatetexts();
	}

} // HuiQt::ListBoxWidget::remove

//======================================================================

void
HuiQt::ListBoxWidget::removeAll()
{
	if (mListBox)
		mListBox->clear();
	mTexts.clear();
	
} // HuiQt::ListBoxWidget::removeAll

//======================================================================

QWidget*
HuiQt::ListBoxWidget::qwidget()
{
	return(mListBox);

} // HuiQt::ListBoxWidget::qwidget

//======================================================================

const QWidget*
HuiQt::ListBoxWidget::qwidget() const
{
	return(mListBox);
	
} // HuiQt::ListBoxWidget::qwidget

//======================================================================
