/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiQt_MenuSubmenuWidget.cc,v 1.27 2001/06/20 10:24:47 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiQt.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiQt_hh
#include "HuiQt.hh"
#endif

//======================================================================

HuiQt::MenuSubmenuWidget::MenuSubmenuWidget( Hui& aHui, const string& aName, const string& aText, const string& aComment, HuiQt::MenuItemWidget* aParent )
	: Hui::Widget( aHui, aName ), Hui::EmbeddedWidget( aHui, aName ), Hui::MenuItemWidget( aHui, aName, 0 ), HuiQt::MenuItemWidget( aHui, aName, 0, aParent ),
	Hui::MenuSubmenuWidget( aHui, aName, aText, aComment )
{

} // HuiQt::MenuSubmenuWidget::MenuSubmenuWidget

//======================================================================

Pointer<Hui::MenuItemWidget> HuiQt::MenuSubmenuWidget::addButton( const string& aName, const string& aText, Pointer<LanguageInterface::CallbackData> aCallbackData, const string& aKey, const string& aComment, Pointer<Hui::MenuItemWidget> aBefore )
{
	if ( parent() ) {
		Pointer<HuiQt::MenuItemWidget> item = new MenuButtonWidget( hui(), aName, aText, aCallbackData, aKey, aComment, this );
		add( aBefore, item );
		return(item);
	}
	else
		return 0;

} // HuiQt::MenuSubmenuWidget::addButton

//======================================================================

Pointer<Hui::MenuItemWidget> HuiQt::MenuSubmenuWidget::addSeparator( const string& aName, Pointer<Hui::MenuItemWidget> aBefore )
{
	if ( parent() ) {
		Pointer<HuiQt::MenuItemWidget> item = new MenuSeparatorWidget( hui(), aName, this );
		add( aBefore, item );
		return(item);
	}
	else
		return 0;

} // HuiQt::MenuSubmenuWidget::addSeparator

//======================================================================

Pointer<Hui::MenuItemWidget> HuiQt::MenuSubmenuWidget::addSubmenu( const string& aName, const string& aText, const string& aComment, Pointer<Hui::MenuItemWidget> aBefore )
{
	Pointer<HuiQt::MenuItemWidget> item = new MenuSubmenuWidget( hui(), aName, aText, aComment, this );
	add( aBefore, item );
	return(item);

} // HuiQt::MenuSubmenuWidget::addSubmenu

//======================================================================

void HuiQt::MenuSubmenuWidget::add( Pointer<Hui::MenuItemWidget> aBefore, Pointer<Hui::MenuItemWidget> aItem )
{
	addItem( aBefore, aItem );
	refresh();

} // HuiQt::MenuSubmenuWidget::add

//======================================================================

void HuiQt::MenuSubmenuWidget::remove( Pointer<Hui::MenuItemWidget> aItem )
{
	removeItem( aItem );
	refresh();

} // HuiQt::MenuSubmenuWidget::remove

//======================================================================

void HuiQt::MenuSubmenuWidget::text( const string& aText )
{
	setText( aText );

} // HuiQt::MenuSubmenuWidget::text

//======================================================================
