/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiQt_ProgressIndicatorWidget.cc,v 1.28 2001/06/20 10:24:47 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiQt.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiQt_hh
#include "HuiQt.hh"
#endif

#ifndef QLABEL_H
#include <qlabel.h>
#endif

#ifndef QPROGBAR_H
#include <qprogbar.h>
#endif

//======================================================================

HuiQt::ProgressIndicatorWidget::ProgressIndicatorWidget(Hui& aHui,
							Pointer<GroupWidget> aParent,
							const string& aName,
							float aMin,
							float aMax,
							float aInitial,
							const string& aTitle,
							const string& aComment)
	: Hui::Widget(aHui, aName),
	  Hui::EmbeddedWidget(aHui, aName),
	  TitledWidget(aHui, aParent, aName, aTitle),
	  Hui::ProgressIndicatorWidget(aHui, aName),
	  mHui(aHui),
	  mMin(aMin),
	  mMax(aMax),
	  mComment(aComment)
{
	parent(aParent);
	
	ContainerWidget *cp = dynamic_cast<ContainerWidget*>((Hui::Widget*) parent());
	GroupWidget *gp = dynamic_cast<GroupWidget*>((Hui::Widget*) parent());
	
	if (cp != NULL)
		mProgressIndicator = new QProgressBar(1000,
						      const_cast<QWidget*>(cp->qwidget()),
						      aName.c_str());
	else if (gp != NULL)
		mProgressIndicator = new QProgressBar(1000,
						      const_cast<QWidget*>(gp->qwidget()),
						      aName.c_str());

	if ( qtitle() ) {
		qtitle()->setBuddy( mProgressIndicator );
		mProgressIndicator->resize( qtitle()->sizeHint() + QSize( 0, 10 ) );
	}
	else {
		mProgressIndicator->resize( QSize( 20, 10 ) );
	}
	set( aInitial );
	mProgressIndicator->setMinimumSize( mProgressIndicator->size() );
	setsize(mProgressIndicator->size());

} // HuiQt::ProgressIndicatorWidget::ProgressIndicatorWidget

//======================================================================

void HuiQt::ProgressIndicatorWidget::set( float aValue )
{
	if ( mProgressIndicator ) {
		if ( aValue < mMin )
			aValue = mMin;
		else if ( aValue > mMax )
			aValue = mMax;
		const int step = int( aValue / ( mMax - mMin ) * mProgressIndicator->totalSteps() );
		mProgressIndicator->setProgress( step );
		mHui.processPendingEvents();
	}

} // HuiQt::ProgressIndicatorWidget::set

//======================================================================

float HuiQt::ProgressIndicatorWidget::set() const
{
	if ( mProgressIndicator ) {
		return(float( mProgressIndicator->progress() ) / float( mProgressIndicator->totalSteps() ) * ( mMax - mMin ) + mMin);
	}
	else
		return -1;

} // HuiQt::ProgressIndicatorWidget::set

//======================================================================

QWidget* HuiQt::ProgressIndicatorWidget::qwidget()
{
	return(mProgressIndicator);

} // HuiQt::ProgressIndicatorWidget::qwidget

//======================================================================

const QWidget* HuiQt::ProgressIndicatorWidget::qwidget() const
{
	return(mProgressIndicator);

} // HuiQt::ProgressIndicatorWidget::qwidget
