/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiQt_RadioMenuWidget.cc,v 1.28 2001/06/20 10:24:47 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiQt.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiQt_hh
#include "HuiQt.hh"
#endif

#ifndef QBTTNGRP_H
#include <qbttngrp.h>
#endif

//======================================================================

HuiQt::RadioMenuWidget::RadioMenuWidget(Hui& aHui,
					Pointer<GroupWidget> aParent,
					const string& aName,
					const vector<string>& aTexts,
					int aInitialValue,
					const string& aTitle,
					const vector<string>& aComments)
	: Hui::Widget(aHui, aName),
	  Hui::EmbeddedWidget(aHui, aName),
	  Hui::ButtonsGroupWidget(aHui, aName),
	  HuiQt::ButtonsGroupWidget(aHui, aParent, aName, aTitle, aComments),
	  Hui::RadioMenuWidget(aHui, aName)
{
	parent(aParent);

	boxes()->setExclusive( true );
	QPoint p = translate( Position( 1, 1 ), Size() ).topLeft();
	QSize size( 0, 0 );
	for ( vector<string>::const_iterator item = aTexts.begin(); item != aTexts.end(); ++item ) {
		QRadioButtonEnterLeave* b = new QRadioButtonEnterLeave( translateAccel( *item ).c_str(), boxes() );
		b->move( p );
		b->resize( b->sizeHint() );
		if ( b->size().width() > size.width() )
			size.setWidth( b->size().width() );
		size.setHeight( size.height() + b->size().height() + 4 );
		b->connect( b, SIGNAL( enter_signal() ), &huiQtEnterLeave(), SLOT( enter() ) );
		b->connect( b, SIGNAL( leave_signal() ), &huiQtEnterLeave(), SLOT( leave() ) );
		p += QPoint( 0, b->sizeHint().height() + 4 );
	}
	boxes()->setButton( aInitialValue );

	boxes()->resize( size + translate( Size( 2, 1 ) ) );
	boxes()->setMinimumSize( boxes()->size() );
	setsize(boxes()->size());

	boxes()->connect( boxes(), SIGNAL( clicked( int ) ), &click(), SLOT( click( int ) ) );

} // HuiQt::RadioMenuWidget::RadioMenuWidget

//======================================================================

void HuiQt::RadioMenuWidget::set( int itemno )
{
	if ( boxes() )
		boxes()->setButton( itemno );

} // HuiQt::RadioMenuWidget::set

//======================================================================

int HuiQt::RadioMenuWidget::set() const
{
	if ( boxes() ) {
		QRadioButton* b = 0;
		for ( int current_itemno = 0; ( b = dynamic_cast<QRadioButton*>( boxes()->find( current_itemno ) ) ) != 0; ++current_itemno ) {
			if ( b->isChecked() )
				return(current_itemno);
		}
	}
	return -1;

} // HuiQt::RadioMenuWidget::set

//======================================================================
