/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiQt_TextEditWidget.cc,v 1.28 2001/06/20 10:24:48 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiQt.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiQt_hh
#include "HuiQt.hh"
#endif

#ifndef QLABEL_H
#include <qlabel.h>
#endif

//======================================================================

HuiQt::TextEditWidget::TextEditWidget(Hui& aHui,
				      Pointer<GroupWidget> aParent,
				      const string& aName,
				      const string& aInitialText,
				      bool aAllowEdit,
				      const string& aTitle,
				      const string& aComment)
	: Hui::Widget(aHui, aName),
	  Hui::EmbeddedWidget(aHui, aName),
	  TitledWidget(aHui, aParent, aName, aTitle),
	  Hui::TextEditWidget(aHui, aName),
	  mComment(aComment)
{
	parent(aParent);

	ContainerWidget *cp = dynamic_cast<ContainerWidget*>((Hui::Widget*) parent());
	GroupWidget *gp = dynamic_cast<GroupWidget*>((Hui::Widget*) parent());
	
	if (cp != NULL)
		mTextEdit = new QMultiLineEditEnterLeave(const_cast<QWidget*>(cp->qwidget()),
							    aName.c_str());
	else if (gp != NULL)
		mTextEdit = new QMultiLineEditEnterLeave(const_cast<QWidget*>(gp->qwidget()),
							 aName.c_str());

	if ( qtitle() )
		qtitle()->setBuddy( mTextEdit );
	text( aInitialText );
	mTextEdit->setReadOnly( !aAllowEdit );
	const size_t lines = count( aInitialText.begin(), aInitialText.end(), '\n' ) + 1;
	mTextEdit->resize( translate( Size( aInitialText.find( '\n' ) == string::npos ? 20 : aInitialText.find( '\n' ) + 2, lines + 3 ) ) );
	mTextEdit->setMinimumSize( translate( Size( 5, 5 ) ) );
	setsize(mTextEdit->size());

	mTextEdit->connect( mTextEdit, SIGNAL( enter_signal() ), &huiQtEnterLeave(), SLOT( enter() ) );
	mTextEdit->connect( mTextEdit, SIGNAL( leave_signal() ), &huiQtEnterLeave(), SLOT( leave() ) );

} // HuiQt::TextEditWidget::TextEditWidget

//======================================================================

void HuiQt::TextEditWidget::text( const string& aText )
{
	if ( mTextEdit )
		mTextEdit->setText( aText.c_str() );

} // HuiQt::TextEditWidget::text

//======================================================================

string HuiQt::TextEditWidget::text() const
{
	if ( mTextEdit )
		return (const char*)mTextEdit->text();
	else
		return string();

} // HuiQt::TextEditWidget::text

//======================================================================

QWidget* HuiQt::TextEditWidget::qwidget()
{
	return(mTextEdit);

} // HuiQt::TextEditWidget::qwidget

//======================================================================

const QWidget* HuiQt::TextEditWidget::qwidget() const
{
	return(mTextEdit);

} // HuiQt::TextEditWidget::qwidget

//======================================================================
