/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiQt_TitledWidget.cc,v 1.29 2001/06/20 10:24:48 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiQt.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiQt_hh
#include "HuiQt.hh"
#endif

#ifndef QLABEL_H
#include <qlabel.h>
#endif

//======================================================================

HuiQt::TitledWidget::TitledWidget( Hui& aHui, Pointer<GroupWidget> aParent, const string& aName, const string& aTitle )
	: Hui::Widget::Widget( aHui, aName ), Widget( aHui, aName )
{
	parent(aParent);

	ContainerWidget *cp = dynamic_cast<ContainerWidget*>((Hui::Widget*) parent());
	GroupWidget *gp = dynamic_cast<GroupWidget*>((Hui::Widget*) parent());
	
	if (aTitle.empty())
		mTitle = 0;
	else {
		if (cp != NULL)
			mTitle = new QLabel(translateAccel(aTitle).c_str(),
					    const_cast<QWidget*>(cp->qwidget()));
		else if (gp != NULL)
			mTitle = new QLabel(translateAccel(aTitle).c_str(),
					    const_cast<QWidget*>(gp->qwidget()));
		
		mTitle->setMinimumSize( mTitle->sizeHint() );
		mTitle->resize( mTitle->sizeHint() );
		mTitle->show();
	}

} // HuiQt::TitledWidget::TitledWidget

//======================================================================

void HuiQt::TitledWidget::title( const string& aTitle )
{
	if ( qtitle() )
		qtitle()->setText( aTitle.c_str() );

} // HuiQt::TitledWidget::title

//======================================================================

string HuiQt::TitledWidget::title() const
{
	if ( qtitle() )
		return static_cast<const char*>(qtitle()->text());
	else
		return string();

} // HuiQt::TitledWidget::title

//======================================================================

void HuiQt::TitledWidget::move( const QPoint& to )
{
	ContainerWidget *cp;
	QPoint newpos;

	setposition(to);
	
	if (parent())
		newpos = to + parent()->real_position();
	else
		newpos = to;

	if ( qtitle() ) {
		qtitle()->move(newpos);
		qwidget()->move(newpos + QPoint(0,
						qtitle()->sizeHint().height()
						+ 2));
	} else if ( qwidget() )
		qwidget()->move(newpos);

} // HuiQt::TitledWidget::move

//======================================================================

void HuiQt::TitledWidget::resize(const QSize& to, int is_arranging = 0)
{
	if (to.width() == -1 && to.height() == -1)
		return;
	if (to.width() >= 0 && to.height() >= 0) {
		/* real resize */
		if (qtitle()) {
			/* Substract title */
			qwidget()->resize(to - QSize(0,
						     qtitle()->sizeHint().height()
						     + 2 ));
		} else {
			qwidget()->resize(to);
		}
		setsize(to);
		return;
	}
	if ((to.width() >= 0 || to.height() >= 0) && !is_arranging) {
		ContainerWidget *cp = dynamic_cast<ContainerWidget*>( (Hui::Widget*) parent());
		GroupWidget *gp = dynamic_cast<GroupWidget*>( (Hui::Widget*) parent());

		setFixedSize((to.width() < 0) ? to.height() : to.width());
		/* arrange parent again since we changed the fixed size */
		if (cp != NULL)
			cp->arrange();
		if (gp != NULL)
			gp->arrange();
	}

} // HuiQt::TitledWidget::resize

//======================================================================
