/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiQt_WindowWidget.cc,v 1.27 2001/06/20 10:24:48 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiQt.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiQt_hh
#include "HuiQt.hh"
#endif

#ifndef QMENUBAR_H
#include <qmenubar.h>
#endif

#ifndef QKEYCODE_H
#include <qkeycode.h>
#endif

#ifndef String_hh
#include "String.hh"
#endif

//======================================================================

HuiQt::WindowWidget::WindowWidget( HuiQt& aHui, const string& aName, const string& aTitle, const Position& aPosition, const Size& aSize )
	: Hui::Widget( aHui, aName ), Hui::GroupWidget( aHui, aName ), HuiQt::Widget( aHui, aName ), HuiQt::ToplevelWidget( aHui, aName, aPosition, aSize ),
	Hui::WindowWidget( aHui, aName ), Hui::ToplevelWidget( aHui, aName ),
	mHui( aHui ), mMainWindow( new QResizableMainWindow( aName ) )
{
	mMainWindow->setCentralWidget( new QWidget( mMainWindow ) );
	mMainWindow->setCaption( aTitle.c_str() );
	//mMainWindow->setGeometry( translate( aPosition, aSize ) );
	resize(aSize);

	QObject::connect( mMainWindow, SIGNAL( resized( const QSize& ) ), this, SLOT( resized( const QSize& ) ) );
	QObject::connect( mMainWindow, SIGNAL( enteredLeft( bool ) ), this, SLOT( enteredLeft( bool ) ) );

} // HuiQt::WindowWidget::WindowWidget

//======================================================================

HuiQt::WindowWidget::~WindowWidget()
{
	delete mMainWindow;

} // HuiQt::WindowWidget::~WindowWidget

//======================================================================

struct KeyCode
{
	const char* accelText;
	int accel;
};

static KeyCode keyCode[] = {
	{ "CTRL-A", Qt::CTRL + Qt::Key_A }, { "CTRL-B", Qt::CTRL + Qt::Key_B }, { "CTRL-C", Qt::CTRL + Qt::Key_C }, { "CTRL-D", Qt::CTRL + Qt::Key_D }, { "CTRL-E", Qt::CTRL + Qt::Key_E },
	{ "CTRL-F", Qt::CTRL + Qt::Key_F }, { "CTRL-G", Qt::CTRL + Qt::Key_G }, { "CTRL-H", Qt::CTRL + Qt::Key_H }, { "CTRL-I", Qt::CTRL + Qt::Key_I }, { "CTRL-J", Qt::CTRL + Qt::Key_J },
	{ "CTRL-K", Qt::CTRL + Qt::Key_K }, { "CTRL-L", Qt::CTRL + Qt::Key_L }, { "CTRL-M", Qt::CTRL + Qt::Key_M }, { "CTRL-N", Qt::CTRL + Qt::Key_N }, { "CTRL-O", Qt::CTRL + Qt::Key_O },
	{ "CTRL-P", Qt::CTRL + Qt::Key_P }, { "CTRL-Q", Qt::CTRL + Qt::Key_Q }, { "CTRL-R", Qt::CTRL + Qt::Key_R }, { "CTRL-S", Qt::CTRL + Qt::Key_S }, { "CTRL-T", Qt::CTRL + Qt::Key_T },
	{ "CTRL-U", Qt::CTRL + Qt::Key_U }, { "CTRL-V", Qt::CTRL + Qt::Key_V }, { "CTRL-W", Qt::CTRL + Qt::Key_W }, { "CTRL-X", Qt::CTRL + Qt::Key_X }, { "CTRL-Y", Qt::CTRL + Qt::Key_Y },
	{ "CTRL-Z", Qt::CTRL + Qt::Key_Z }, { "ESC", Qt::Key_Escape }, { "ALT-SPACE", Qt::ALT+ Qt::Key_Space }, { "CTRL-INS", Qt::CTRL + Qt::Key_Insert }, { "SHIFT-INS", Qt::SHIFT + Qt::Key_Insert },
	{ "CTRL-DEL", Qt::CTRL + Qt::Key_Delete }, { "SHIFT-DEL", Qt::SHIFT + Qt::Key_Delete }, { "BACKSPACE", Qt::Key_Backspace }, { "CTRL-BACKSPACE", Qt::CTRL + Qt::Key_Backspace }, { "SHIFT-TAB", Qt::SHIFT + Qt::Key_Tab },
	{ "TAB", Qt::Key_Tab }, { "ALT-Q", Qt::ALT+ Qt::Key_Q }, { "ALT-W", Qt::ALT+ Qt::Key_W }, { "ALT-E", Qt::ALT+ Qt::Key_E }, { "ALT-R", Qt::ALT+ Qt::Key_R },
	{ "ALT-T", Qt::ALT+ Qt::Key_T }, { "ALT-Y", Qt::ALT+ Qt::Key_Y }, { "ALT-U", Qt::ALT+ Qt::Key_U }, { "ALT-I", Qt::ALT+ Qt::Key_I }, { "ALT-O", Qt::ALT+ Qt::Key_O },
	{ "ALT-P", Qt::ALT+ Qt::Key_P }, { "CTRL-ENTER", Qt::CTRL + Qt::Key_Enter }, { "ENTER", Qt::Key_Enter }, { "ALT-A", Qt::ALT+ Qt::Key_A }, { "ALT-S", Qt::ALT+ Qt::Key_S },
	{ "ALT-D", Qt::ALT+ Qt::Key_D }, { "ALT-F", Qt::ALT+ Qt::Key_F }, { "ALT-G", Qt::ALT+ Qt::Key_G }, { "ALT-H", Qt::ALT+ Qt::Key_H }, { "ALT-J", Qt::ALT+ Qt::Key_J },
	{ "ALT-K", Qt::ALT+ Qt::Key_K }, { "ALT-L", Qt::ALT+ Qt::Key_L }, { "ALT-Z", Qt::ALT+ Qt::Key_Z }, { "ALT-X", Qt::ALT+ Qt::Key_X }, { "ALT-C", Qt::ALT+ Qt::Key_C },
	{ "ALT-V", Qt::ALT+ Qt::Key_V }, { "ALT-B", Qt::ALT+ Qt::Key_B }, { "ALT-N", Qt::ALT+ Qt::Key_N }, { "ALT-M", Qt::ALT+ Qt::Key_M }, { "F1", Qt::Key_F1 },
	{ "F2", Qt::Key_F2 }, { "F3", Qt::Key_F3 }, { "F4", Qt::Key_F4 }, { "F5", Qt::Key_F5 }, { "F6", Qt::Key_F6 }, { "F7", Qt::Key_F7 }, { "F8", Qt::Key_F8 },
	{ "F9", Qt::Key_F9 }, { "F10", Qt::Key_F10 }, { "HOME", Qt::Key_Home }, { "UP", Qt::Key_Up }, { "PGUP", Qt::Key_PageUp }, { "GRAYMINUS", Qt::Key_Minus },
	{ "LEFT", Qt::Key_Left }, { "RIGHT", Qt::Key_Right }, { "GRAYPLUS", Qt::Key_Plus }, { "END", Qt::Key_End }, { "DOWN", Qt::Key_Down },
	{ "PGDN", Qt::Key_PageDown }, { "INS", Qt::Key_Insert }, { "DEL", Qt::Key_Delete }, { "SHIFT-F1", Qt::SHIFT + Qt::Key_F1 }, { "SHIFT-F2", Qt::SHIFT + Qt::Key_F2 },
	{ "SHIFT-F3", Qt::SHIFT + Qt::Key_F3 }, { "SHIFT-F4", Qt::SHIFT + Qt::Key_F4 }, { "SHIFT-F5", Qt::SHIFT + Qt::Key_F5 }, { "SHIFT-F6", Qt::SHIFT + Qt::Key_F6 }, { "SHIFT-F7", Qt::SHIFT + Qt::Key_F7 },
	{ "SHIFT-F8", Qt::SHIFT + Qt::Key_F8 }, { "SHIFT-F9", Qt::SHIFT + Qt::Key_F9 }, { "SHIFT-F10", Qt::SHIFT + Qt::Key_F10 }, { "CTRL-F1", Qt::CTRL + Qt::Key_F1 }, { "CTRL-F2", Qt::CTRL + Qt::Key_F2 },
	{ "CTRL-F3", Qt::CTRL + Qt::Key_F3 }, { "CTRL-F4", Qt::CTRL + Qt::Key_F4 }, { "CTRL-F5", Qt::CTRL + Qt::Key_F5 }, { "CTRL-F6", Qt::CTRL + Qt::Key_F6 }, { "CTRL-F7", Qt::CTRL + Qt::Key_F7 },
	{ "CTRL-F8", Qt::CTRL + Qt::Key_F8 }, { "CTRL-F9", Qt::CTRL + Qt::Key_F9 }, { "CTRL-F10", Qt::CTRL + Qt::Key_F10 }, { "ALT-F1", Qt::ALT+ Qt::Key_F1 }, { "ALT-F2", Qt::ALT+ Qt::Key_F2 },
	{ "ALT-F3", Qt::ALT+ Qt::Key_F3 }, { "ALT-F4", Qt::ALT+ Qt::Key_F4 }, { "ALT-F5", Qt::ALT+ Qt::Key_F5 }, { "ALT-F6", Qt::ALT+ Qt::Key_F6 }, { "ALT-F7", Qt::ALT+ Qt::Key_F7 },
	{ "ALT-F8", Qt::ALT+ Qt::Key_F8 }, { "ALT-F9", Qt::ALT+ Qt::Key_F9 }, { "ALT-F10", Qt::ALT+ Qt::Key_F10 }, { "CTRL-PRTSC", Qt::CTRL + Qt::Key_SysReq }, { "CTRL-LEFT", Qt::CTRL + Qt::Key_Left },
	{ "CTRL-RIGHT", Qt::CTRL + Qt::Key_Right }, { "CTRL-END", Qt::CTRL + Qt::Key_End }, { "CTRL-PGDN", Qt::CTRL + Qt::Key_PageDown }, { "CTRL-HOME", Qt::CTRL + Qt::Key_Home }, { "ALT-1", Qt::ALT+ Qt::Key_1 },
	{ "ALT-2", Qt::ALT+ Qt::Key_2 }, { "ALT-3", Qt::ALT+ Qt::Key_3 }, { "ALT-4", Qt::ALT+ Qt::Key_4 }, { "ALT-5", Qt::ALT+ Qt::Key_5 }, { "ALT-6", Qt::ALT+ Qt::Key_6 },
	{ "ALT-7", Qt::ALT+ Qt::Key_7 }, { "ALT-8", Qt::ALT+ Qt::Key_8 }, { "ALT-9", Qt::ALT+ Qt::Key_9 }, { "ALT-0", Qt::ALT+ Qt::Key_0 }, { "ALT-MINUS", Qt::ALT+ Qt::Key_Minus },
	{ "ALT-EQUAL", Qt::ALT+ Qt::Key_Equal }, { "CTRL-PGUP", Qt::CTRL + Qt::Key_PageUp }, { "ALT-BACKSPACE", Qt::ALT+ Qt::Key_Backspace },
	{ 0, 0 }
};

inline int key2keyCode( const string& aKey )
{
	if ( aKey.length() > 0 ) {
		string k = to_upper( aKey );
		for ( const KeyCode* kc = keyCode; kc->accelText; ++kc )
			if ( !strcmp( kc->accelText, k.c_str() ) )
				return(kc->accel);
		return 0; //tolower( k[0] );
	}
	return 0;

} // key2keyCode

//======================================================================

void HuiQt::WindowWidget::recreateMenu( Hui::MenuSubmenuWidget& aMenuData )
{
	QMenuBar* menuBar = mMainWindow->menuBar();
	menuBar->clear();
	mMenuClicks.clear();
	recreateMenu( menuBar, aMenuData );

} // HuiQt::WindowWidget::recreateMenu

//======================================================================

void HuiQt::WindowWidget::recreateMenu( QMenuData* popup, Hui::MenuSubmenuWidget& aMenuData )
{
	int id = 0;
	for ( Hui::MenuSubmenuWidget::Items::iterator i = aMenuData.items().begin(); i != aMenuData.items().end(); ++i, ++id ) {
		if ( dynamic_cast<Hui::MenuSubmenuWidget*>( (Hui::MenuItemWidget*)*i ) ) {
			QPopupMenu* submenu = new QPopupMenu();
			recreateMenu( submenu, *dynamic_cast<Hui::MenuSubmenuWidget*>( (Hui::MenuItemWidget*)*i ) );
			id = popup->insertItem( translateAccel( (*i)->text() ).c_str(), submenu, key2keyCode( (*i)->key() ) );
		}
		else if ( dynamic_cast<Hui::MenuSeparatorWidget*>( (Hui::MenuItemWidget*)*i ) ) {
			popup->insertSeparator();
		}
		else {
			HuiQtMenuClick* menuClick = new HuiQtMenuClick( **i );
			mMenuClicks.push_back( menuClick );
			id = popup->insertItem( translateAccel( (*i)->text() ).c_str(), menuClick, SLOT( click() ), key2keyCode( (*i)->key() ) );
		}
		popup->setItemEnabled( id, !(*i)->disabled() );
	}

} // HuiQt::WindowWidget::recreateMenu

//======================================================================

void HuiQt::WindowWidget::show()
{
	mHui.subwidgets().add( this );
	mMainWindow->show();
	HuiQt::Widget::show();
// XXX bug in Qt 1.42?
//	  if ( mHui.subwidgets().size() == 1 )
//		  mHui.setMainWidget( mMainWindow );

} // HuiQt::WindowWidget::show

//======================================================================

void HuiQt::WindowWidget::resize( const Hui::Size& aSize )
{
	resize( HuiQt::Widget::translate( aSize ) );

} // HuiQt::WindowWidget::resize

//======================================================================

void HuiQt::WindowWidget::resize( const QSize& to )
{
	mMainWindow->resize(to - QSize(17, 13));
	if ( qwidget() ) {
		qwidget()->resize(to - QSize(17, 13));
	}
	arrange();

} // HuiQt::WindowWidget::resize

//======================================================================

void HuiQt::WindowWidget::hide()
{
	mHui.subwidgets().remove( this );
	mMainWindow->hide();

} // HuiQt::WindowWidget::hide

//======================================================================

QWidget* HuiQt::WindowWidget::qwidget()
{
	return mMainWindow->centralWidget();

} // HuiQt::WindowWidget::qwidget

//======================================================================

const QWidget* HuiQt::WindowWidget::qwidget() const
{
	return const_cast<QResizableMainWindow*>( mMainWindow )->centralWidget();

} // HuiQt::WindowWidget::qwidget

//======================================================================
