/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiTVision.cc,v 1.30 2001/06/20 10:24:48 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiTVision.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiTVision_hh
#include "HuiTVision.hh"
#endif

#define Uses_TDialog
#define Uses_TDeskTop
#define Uses_TLabel
#define Uses_TButton
#define Uses_TInputLine
#define Uses_TRangeValidator
#define Uses_TPXPictureValidator
#define Uses_MsgBox
#define Uses_TSItem
#define Uses_TRadioButtons
#define Uses_TCheckBoxes
#define Uses_TListBox
#define Uses_TScrollBar
#define Uses_TCollection
#define Uses_TStringCollection
#define Uses_TFileDialog
#define Uses_TMenuItem
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuBar
#define Uses_TKeys
#define Uses_TScroller
#define Uses_TEditor
#define Uses_TIndicator

#include <tvision/tv.h>

#ifndef String_hh
#include "String.hh"
#endif

#include <cmath>

#define MAX(a,b) ((a) > (b) ? (a) : (b))

//======================================================================

HuiTVision::HuiTVision()
	: TProgInit(&HuiTVision::initStatusLine, 
		    &HuiTVision::initMenuBar,
		    &HuiTVision::initDeskTop)
	//, mCommands( 0 ), mButtonCommands( 0 ), mWidgets( 0 )
{
} // HuiTVision::HuiTVision

//======================================================================

HuiTVision::~HuiTVision()
{
} // HuiTVision::~HuiTVision

//======================================================================

/* Run the Application */
void HuiTVision::run()
{
	TApplication::run();

} // HuiTVision::run

//======================================================================

/* Exit Application */
void HuiTVision::exit( int status )
{
	/* Send message to TApplication */
	message( this, evCommand, cmQuit, 0 );

} // HuiTVision::exit

//======================================================================

/* Process events */
void HuiTVision::processPendingEvents()
{
	for (;;) {
		TEvent e;
		getEvent(e);
		if (e.what == evNothing)
			break;
		handleEvent(e);
	}
	
} // HuiTVision::processPendingEvents

//======================================================================

/* Return the size of the TApplication, usually the Screenwidth/height */
Hui::Size HuiTVision::size() const
{
	const TPoint& tp = TView::size;
	return(Hui::Size( tp.x /*screenWidth*/, tp.y /*screenHeight*/ ));

} // HuiTVision::size

//======================================================================

/* Accelerators into required format */
string HuiTVision::translateAccel( const string& aText )
{
	string result = aText;
	const string::size_type p = result.find( '&' );
	if ( p != string::npos && result.length() > p + 1 ) {
		result.replace( p, 1, 1, '~' );
		result.insert( p + 2, 1, '~' );
	}
	return(result);

} // HuiTVision::translateAccel

//======================================================================

/* Find a item from the list */
Pointer<Hui::Widget> HuiTVision::Subwidgets::find( TView* p ) const
{
	Pointer<Hui::Widget> result;
	/* go through the list */
	for ( const_iterator w = begin(); !result && w != end(); ++w ) {
		const HuiTVision::Widget* tw = dynamic_cast<const HuiTVision::Widget*>( (const Hui::Widget*)*w );
		if ( tw && tw->tview() == p ) {
			/* Found correct item, use this */
			result = *w;
		} else {
			const HuiTVision::GroupWidget* gw = dynamic_cast<const HuiTVision::GroupWidget*>( (const Hui::Widget*)*w );
			/* If this one is a group widget, search
			   trough the childs of this widget */
			if ( gw )
				result = gw->subwidgets().find( p );
		}
	}
	return(result);

} // HuiTVision::Subwidgets::find

//======================================================================

/* Add widget to end of lists of subwidgets */
void HuiTVision::Subwidgets::add( Pointer<Hui::Widget> w )
{
	/* search end */
	iterator i = ::find( begin(), end(), w );
	/* if not already in list, add it */
	if ( i == end() )
		push_back( w );

} // HuiTVision::Subwidgets::add

//======================================================================

/* remove item from list */
void HuiTVision::Subwidgets::remove( Pointer<Hui::Widget> w )
{
	iterator i = ::find(begin(), end(), w);

	/* if in list, remove it */
	if ( i != end() )
		erase( i );

} // HuiTVision::Subwidgets::remove

//======================================================================

/* Find Widget by name */
Pointer<Hui::Widget> HuiTVision::byName( const string& aName )
{
	Pointer<Hui::Widget> result = NULL;
	/* search through list */
	for ( Subwidgets::const_iterator w = mWidgets.begin(); !result && w != mWidgets.end(); ++w )
		result = (*w)->byName( aName );
	return(result);

} // HuiTVision::byName

//======================================================================

Pointer<Hui::DialogWidget>
HuiTVision::dialog(Pointer<Hui::Widget> aParent,
		   const string& aName,
		   const string& aTitle,
		   const Position& aPosition,
		   const Size& aSize,
		   const string& aAlignment)
{
	Pointer<DialogWidget> dialog = new DialogWidget(*this,
							aName,
							aTitle,
							aPosition,
							aSize);
	dialog->arrange(aAlignment, FALSE);

	return(dialog);

} // HuiTVision::dialog

//======================================================================

/* Window constructor */
/* XXX comment in no_menu? Qt has it */
Pointer<Hui::WindowWidget> HuiTVision::window( const string& aName, const string& aTitle, const Position& aPosition, const Size& aSize, const string& aAlignment, bool no_menu )
{
Pointer<WindowWidget> window = new WindowWidget( *this, aName, aTitle, aPosition, aSize );
	// if ( !no_menu && mMenuBar )
	//        window->recreateMenu( *mMenuBar );

	window->arrange(aAlignment, FALSE);

	return window;

} // HuiTVision::window

//======================================================================

/* fileselectionDialog contructor */
/* returns the filename */
string HuiTVision::fileSelectionDialog( const string& aTitle, const string& aWildCard )
{
	/* XXX Qt uses a different way of the initial path */
	TFileDialog* dialog = new TFileDialog( aWildCard.c_str(), aTitle.c_str(), "", fdOKButton, 0 );
	const ushort status = deskTop->execView( dialog );
	char filename[1024] = "";
	if ( status != cmCancel )
		dialog->getFileName( filename );
	delete dialog;
	return(filename);

} // HuiTVision::fileSelectionDialog

//======================================================================

/* Return a pointer to the menubar */
Pointer<Hui::MenuSubmenuWidget> HuiTVision::menuBar()
{
	if ( !mMenuBar ) {
		/* create a new menubar if it doesn't exist yet */
		mMenuBar = new MenuBarWidget( *this );
	}
	return(mMenuBar);

} // HuiTVision::menuBar

//======================================================================

const ushort HuiTVision::CommandsToClick::sCommandsBase = 0x7FFF;

//======================================================================

bool HuiTVision::CommandsToClick::click( ushort aCommand )
{
	Table::iterator t = mTable.find( aCommand );
	if ( t != mTable.end() ) {
		t->second->click();
		return(true);
	}
	else
		return(false);

} // HuiTVision::CommandsToClick::click

//======================================================================

ushort HuiTVision::CommandsToClick::newCommand( Clickable& aClickable )
{
	ushort command = mNewCommand++;
	mTable.insert( make_pair( command, &aClickable ) );
	return(command);

} // HuiTVision::CommandsToClick::newCommand

//======================================================================

void HuiTVision::CommandsToClick::remove( ushort aCommand )
{
	Table::iterator t = mTable.find( aCommand );
	if ( t != mTable.end() )
		mTable.erase( t );

} // HuiTVision::CommandsToClick::remove

//======================================================================

void HuiTVision::handleEvent( TEvent& event)
{
	bool handled = false;

	if ( event.what == evCommand ) {
		try {
			handled = commandsToClick().click( event.message.command );
		}
		catch ( exception& err ) {
			cerr << "Error handling command: " << err.what() << endl;
		}
	}
	if ( !handled )
		TApplication::handleEvent( event );
	else
		clearEvent( event ); // clear event after handling

} // HuiTVision::handleEvent

//======================================================================
