/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiTVision.hh,v 1.24 2001/06/20 10:24:49 alex Exp $
 */

// -*- C++ -*-  Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiTVision.hh,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiTVision_hh
#define HuiTVision_hh

#include "Hui.hh"

#include <map>

//======================================================================

#define Uses_TApplication
#if defined( H_BUILD_SYSTEMS ) || defined( H_LIB_TCL_INTERFACE )
#define cstrlen tvision_cstrlen // hack to allow both TVision and Qt headers to be included together into the same module
#endif

#define Uses_TDialog
#define Uses_TLabel
#define Uses_TSItem

#include <tvision/tv.h>
#if defined( H_BUILD_SYSTEMS ) || defined( H_LIB_TCL_INTERFACE )
#undef cstrlen
#endif

#define	TRUE	1
#define	FALSE	0

//======================================================================

class HuiTWindow;
class HuiTDialog;
class HuiTLabel;
class HuiTStaticText;
class HuiTButton;
class HuiTInputLine;
class HuiTCheckBoxes;
class TCluster;
class HuiTRadioMenu;
class HuiTListBox;
class HuiTTextEdit;
class HuiTProgressIndicator;
class TMenuBar;
class TSubMenu;
class TMenuItem;

//======================================================================

class HuiTVision : public TApplication, public Hui
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

     static inline size_t accel_text_length( const string& aText )
          {
               return aText.length() - count( aText.begin(), aText.end(), '~' ) - count( aText.begin(), aText.end(), '&' );
          }

  public:
     HuiTVision();
     virtual ~HuiTVision();

     virtual void run(); // Run the application
     virtual void exit( int status ); // exit the application
     virtual void processPendingEvents();
     virtual const char* type() const { return "text"; }

     virtual Size size() const;

       // --------------------------------------------------

     virtual Pointer<Hui::Widget> byName( const string& aName );

     virtual Pointer<Hui::WindowWidget> window( const string& aName, const string& aTitle, const Position& aPosition, const Size& aSize, const string& aAlignment, bool no_menu = false );
     virtual Pointer<Hui::DialogWidget> dialog( Pointer<Hui::Widget> aParent, const string& aName, const string& aTitle, const Position& aPosition, const Size& aSize, const string& aAlignment );
     virtual Pointer<Hui::MenuSubmenuWidget> menuBar();
     virtual string fileSelectionDialog( const string& aTitle, const string& aWildCard );

       // --------------------------------------------------

  protected:
     class Widget;
     class GroupWidget;

     class Subwidgets : public list< Pointer<Hui::Widget> >
     {
       public:
          Subwidgets() {}
          virtual ~Subwidgets() {}

          virtual Pointer<Hui::Widget> find( TView* p ) const;
          virtual void add( Pointer<Hui::Widget> w );
          virtual void remove( Pointer<Hui::Widget> w );

     }; // class Subwidgets

       // --------------------------------------------------

     class Widget : virtual public Hui::Widget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
          virtual ~Widget();

          virtual void show();
          virtual void hide();
          virtual bool hidden() const;
          virtual void disable();
          virtual void enable();
          virtual bool disabled() const;

          virtual void move( const Position& aPosition );
          virtual Position position() const;
          virtual void resize(const Size& aSize);
          virtual Size size() const;

          virtual TView* tview() = 0;
          virtual const TView* tview() const = 0;

          virtual void move( const TPoint& to );
          virtual void resize(const TPoint& to, int is_arranging = 0);

          virtual Size minSize() const = 0;
          virtual Size maxSize() const = 0;

          void parent(Pointer<Hui::Widget> aParent) { mParent = aParent; };
	  Pointer<Hui::Widget> parent() { return mParent ; }
	  const Pointer<Hui::Widget> parent() const { return mParent ; }

	     int getFixedSize() { return(mFixedSize); }
	     void setFixedSize(int to) { mFixedSize = to; }
       protected:
          Widget( Hui& aHui, const string& aName ) : Hui::Widget( aHui, aName ) {
	  setFixedSize(-1);
	  }

          virtual TRect translate( const Hui::Position& aPosition, const Hui::Size& aSize ) const { return TRect( aPosition.x(), aPosition.y(), aPosition.x() + aSize.width(), aPosition.y() + aSize.height() ); }
          virtual TPoint translate( const Hui::Position& aPosition ) const { TPoint p; p.x = aPosition.x(); p.y = aPosition.y(); return p; }
          virtual TPoint translate(  const Hui::Size& aSize ) const { TPoint p; p.x = aSize.width(); p.y = aSize.height(); return p; }
          virtual Hui::Size translate( const TPoint& aSize, int ) const { return Hui::Size( aSize.x, aSize.y ); }
          virtual Hui::Position translate( const TPoint& aPosition ) const { return Hui::Position( aPosition.x - 1, aPosition.y - 1 ); }

       private:
		Pointer<Hui::Widget> mParent;
		TPoint mPosition;
	     int mFixedSize;

     }; // class Widget

       // --------------------------------------------------

     class GroupWidget : virtual public Hui::GroupWidget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
          virtual ~GroupWidget();

          virtual Pointer<Hui::LabelWidget> label( const string& aName, const string& aText, const Hui::Position& aPosition );
          virtual Pointer<Hui::ButtonWidget> button( const string& aName, const string& aText, Pointer<LanguageInterface::CallbackData> aCallbackData, const string& aButtonType, const Hui::Position& aPosition, const string& aComment );
          virtual Pointer<Hui::InputLineWidget> inputLine( const string& aName, const string& aInitialText, int aMaxLength, const string& aValidator, const string& aTitle, const Hui::Position& aPosition, const string& aComment );
          virtual Pointer<Hui::CheckBoxesWidget> checkBoxes( const string& aName, const vector<string>& aTexts, const vector<bool>& aInitialValues, const string& aTitle, const Hui::Position& aPosition, const vector<string>& aComments );
          virtual Pointer<Hui::RadioMenuWidget> radioMenu( const string& aName, const vector<string>& aTexts, const int aInitialValue, const string& aTitle, const Hui::Position& aPosition, const vector<string>& aComments );
          virtual Pointer<Hui::ListBoxWidget> listBox( const string& aName, const vector<string>& aTexts, const int aInitiallySelected, const string& aTitle, const Hui::Position& aPosition, const string& aComment );
          virtual Pointer<Hui::TextEditWidget> textEdit( const string& aName, const string& aInitialText, bool aAllowEdit, const string& aTitle, const Hui::Position& aPosition, const string& aComment );
          virtual Pointer<Hui::ProgressIndicatorWidget> progressIndicator( const string& aName, float aMin, float aMax, float aInitial, const string& aTitle, const Hui::Position& aPosition, const string& aComment );

	     virtual Pointer<Hui::ContainerWidget> container(const string& aName,
                        const Hui::Position& aPosition,
                        const string& aAlignment,
                        const string& aComment);

	 void arrange();
	 void arrange(const string& aAlignment, bool arrange_new);
	 
	 virtual void current( Pointer<Hui::EmbeddedWidget> aWidget );
	 virtual Pointer<Hui::Widget> current() const;
	 virtual void aboutToChangeCurrent( TView* old, TView* current );
	 
	 virtual TGroup* tgroup() = 0;
	 virtual const TGroup* tgroup() const = 0;

	 virtual Pointer<Hui::Widget> byName( const string& aName ) const;
	 virtual void insert( Pointer<Hui::Widget> aWidget );
	 virtual void erase();

	 const Subwidgets& subwidgets() const { return mWidgets; }
	 Subwidgets& subwidgets() { return mWidgets; }

	 virtual HuiTVision& hui() const {return mHui;};

       protected:
          GroupWidget( HuiTVision& aHui, const string& aName, const Position& aPosition, const Size& aSize );

          HuiTVision& hui() { return mHui; }

       private:
          HuiTVision& mHui;
          Subwidgets mWidgets;
	  int mArrangement;

     }; // class GroupWidget

       // --------------------------------------------------

    class ContainerWidget : virtual public Hui::ContainerWidget,
			    public GroupWidget
    {
	CLASS_METHODS_DESCRIPTIONS_DECLARATION;
	
    public:
	ContainerWidget(HuiTVision& aHui,
			Pointer<Hui::Widget> aParent,
			const string& aName);
	
	virtual ~ContainerWidget();

	// Functions, that need to be virtualized
	void show();
	void hide();
	bool hidden() const;
	
	void disable();
	void enable();
	bool disabled() const;
	
	void move(const Hui::Position &);
	void move(const TPoint &);
	class Hui::Position position() const;
	
	void resize(const Hui::Size &);
	void resize(const TPoint &, int is_arranging = 0);
	class Hui::Size size() const;
	
	void parent(Pointer<Hui::Widget> aParent) { mParent = aParent; };
	const Pointer<Hui::Widget> parent() const { return mParent ; }
	Pointer<Hui::Widget> parent() { return mParent ; }

	// Refer to parent's tgroup()
	virtual TGroup* tgroup();
	const TGroup* tgroup() const;
	
	int getFixedSize() { return(mFixedSize); }
	void setFixedSize(int to) { mFixedSize = to; }
	
    protected:
	virtual TRect translate(const Hui::Position& aPosition,
				const Hui::Size& aSize ) const {
	    return TRect(aPosition.x(), aPosition.y(),
			 aPosition.x() + aSize.width(),
			 aPosition.y() + aSize.height());
	}
	virtual TPoint translate(const Hui::Position& aPosition) const {
	    TPoint p;
	    
	    p.x = aPosition.x();
	    p.y = aPosition.y();
	    return p;
	}
	virtual TPoint translate(const Hui::Size& aSize) const {
	    TPoint p;
	    
	    p.x = aSize.width();
	    p.y = aSize.height();
	    return p;
	}
	virtual Hui::Size translate(const TPoint& aSize, int) const {
	    return Hui::Size(aSize.x, aSize.y);
	}
	virtual Hui::Position translate(const TPoint& aPosition) const {
	    return Hui::Position(aPosition.x,
				 aPosition.y);
	}
	
    private:
	TPoint mPosition;
	TPoint mSize;
	Pointer<Hui::Widget> mParent;
	bool mHidden;
	bool mDisabled;
	int mFixedSize;
     };

       // --------------------------------------------------

     class ToplevelWidget : virtual public Hui::ToplevelWidget, public GroupWidget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
          virtual ~ToplevelWidget();

          virtual void title( const string& aTitle );
          virtual string title() const;

          virtual TGroup* tgroup();
          virtual const TGroup* tgroup() const;
          virtual TWindow* twindow() = 0;
          virtual const TWindow* twindow() const = 0;

          virtual void resized();

       protected:
          ToplevelWidget( HuiTVision& aHui, const string& aName, const Position& aPosition, const Size& aSize );

     }; // class ToplevelWidget

       // --------------------------------------------------

     class WindowWidget : public Widget, public ToplevelWidget, public Hui::WindowWidget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
          WindowWidget( HuiTVision& aHui, const string& aName, const string& aTitle, const Position& aPosition, const Size& aSize );
          virtual ~WindowWidget();

          virtual void show();
          virtual void hide();

          virtual TView* tview();
          virtual const TView* tview() const;
          virtual TWindow* twindow();
          virtual const TWindow* twindow() const;

          virtual Size size() const { return HuiTVision::Widget::size() - Size( 2, 2 ); };
          virtual Size minSize() const { return size(); }
          virtual Size maxSize() const { return size(); }

          virtual void close(); // user clicked close icon

       private:
          HuiTWindow* mTWindow;

     }; // class WindowWidget

       // --------------------------------------------------

     class DialogWidget : public Widget, public ToplevelWidget, public Hui::DialogWidget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
          DialogWidget( HuiTVision& aHui, const string& aName, const string& aTitle, const Position& aPosition, const Size& aSize );
          virtual ~DialogWidget();

          virtual const string& result() const;
          virtual const string& exec();

          virtual bool running() const { return mRunning; }
          virtual void endModal( const string& aResult );

          virtual TView* tview();
          virtual const TView* tview() const;
          virtual TWindow* twindow();
          virtual const TWindow* twindow() const;

          virtual Size size() const { return HuiTVision::Widget::size() - Size( 2, 2 ); };
          virtual Size minSize() const { return size(); }
          virtual Size maxSize() const { return size(); }

       private:
          HuiTDialog* mTDialog;
          string mResult;
          bool mRunning;

     }; // class DialogWidget

       // --------------------------------------------------

     class LabelWidget : public Widget, public Hui::LabelWidget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
          LabelWidget(Hui& aHui,
		      Pointer<Hui::Widget> aParent,
		      const string& aName,
		      const string& aText);
          virtual ~LabelWidget();

          virtual void text( const string& aText );
          virtual string text() const;

          virtual TView* tview();
          virtual const TView* tview() const;
          virtual Size minSize() const { return Size( accel_text_length( text() ), 1 ); }
          virtual Size maxSize() const { return Size( accel_text_length( text() ), 1 ); }

       private:
          HuiTStaticText* mLabel;

     }; // class LabelWidget

       // --------------------------------------------------

     class ButtonWidget : public Widget, public Hui::ButtonWidget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
          ButtonWidget( HuiTVision& aHui, Pointer<Hui::Widget> aParent, const string& aName, const string& aText, Pointer<LanguageInterface::CallbackData> aCallbackData, const string& aButtonType, const string& aComment );
          virtual ~ButtonWidget();

          virtual void text( const string& aText );
          virtual string text() const;

          virtual TView* tview();
          virtual const TView* tview() const;
          virtual Size minSize() const { return Size( accel_text_length( text() ) + 4, 2 ); }
          virtual Size maxSize() const { return Size( Size::maxWidth(), 2 ); }

          virtual void callback_completed( const WidgetCallbackData& aCallbackData );

       private:
          HuiTButton* mButton;
          string mComment;

     }; // class ButtonWidget

       // --------------------------------------------------

     class TitledWidget : public Widget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
          TitledWidget( Hui& aHui, Pointer<Hui::Widget> aParent, const string& aName, const string& aTitle );
          virtual ~TitledWidget();

          virtual void title( const string& aTitle );
          virtual string title() const;
          virtual void move( const TPoint& to );
          virtual void resize(const TPoint& to, int is_arranging = 0);
          virtual void move(const Hui::Position& to) {
		  Widget::move( to );
	  }
          virtual void resize(const Hui::Size& to) {
		  Widget::resize(to);
	  }
	     virtual Position position() const;
	     virtual Size size() const;

          HuiTLabel* ttitle() { return mTitle; }
          const HuiTLabel* ttitle() const { return mTitle; }

       private:
          HuiTLabel* mTitle;

     }; // class TitledWidget

       // --------------------------------------------------

     class InputLineWidget : public TitledWidget, public Hui::InputLineWidget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
          InputLineWidget(Hui& aHui,
			  Pointer<Hui::Widget> aParent,
			  const string& aName,
			  const string& aInitialText,
			  int aMaxLength,
			  const string& aValidator,
			  const string& aTitle,
			  const string& aComment);

          virtual void text( const string& aText );
          virtual string text() const;

          virtual void title( const string& aTitle ) { TitledWidget::title( aTitle ); }
          virtual string title() const { return TitledWidget::title(); }

          virtual TView* tview();
          virtual const TView* tview() const;
          virtual Size minSize() const { return Size( 3, ttitle() ? 2 : 1 ); }
          virtual Size maxSize() const { return Size( Size::maxWidth(), ttitle() ? 2 : 1 ); }

       private:
          HuiTInputLine* mInputLine;
          string mComment;

     }; // class InputLineWidget

       // --------------------------------------------------

     class ButtonsGroupWidget : public TitledWidget, virtual public Hui::ButtonsGroupWidget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

     public:
	     ButtonsGroupWidget(Hui& aHui,
				Pointer<Hui::Widget> aParent,
				const string& aName,
				const string& aTitle,
				const vector<string>& aComments);

          virtual void disable();
          virtual void enable();
          virtual void disable_item( int boxno );
          virtual void enable_item( int boxno );
          virtual void title( const string& aTitle ) { TitledWidget::title( aTitle ); }
          virtual string title() const { return TitledWidget::title(); }

          virtual TCluster* tcluster() = 0;
          virtual const TCluster* tcluster() const = 0;
          virtual Size minSize() const;
          virtual Size maxSize() const { return Size( Size::maxWidth(), numberOfItems() + ( ttitle() ? 1 : 0 ) ); }

       private:
          vector<string> mComments;

     }; // class ButtonsGroupWidget

       // --------------------------------------------------

     class CheckBoxesWidget : public ButtonsGroupWidget, public Hui::CheckBoxesWidget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
	     CheckBoxesWidget(Hui& aHui,
			      Pointer<Hui::Widget> aParent,
			      const string& aName,
			      const vector<string>& aTexts,
			      const vector<bool>& aInitialValues,
			      const string& aTitle,
			      const vector<string>& aComments);
	     virtual ~CheckBoxesWidget();
	     
	     virtual bool state( int boxno, int aNewState );
          virtual bool state( int boxno ) const;
          virtual bool toggle( int boxno );
          virtual void text( int boxno, const string& aText );
          virtual string text( int boxno ) const;
          virtual int numberOfItems() const;
          virtual bool item_disabled( int boxno ) const;

          virtual TCluster* tcluster();
          virtual const TCluster* tcluster() const;
          virtual TView* tview();
          virtual const TView* tview() const;

       private:
          HuiTCheckBoxes* mCheckBoxes;

     }; // class CheckBoxesWidget

       // --------------------------------------------------

     class RadioMenuWidget : public ButtonsGroupWidget, public Hui::RadioMenuWidget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

     public:
	     RadioMenuWidget(Hui& aHui,
			     Pointer<Hui::Widget> aParent,
			     const string& aName,
			     const vector<string>& aTexts,
			     int aInitialValue,
			     const string& aTitle,
			     const vector<string>& aComments);
          virtual ~RadioMenuWidget();

          virtual void set( int itemno );
          virtual int set() const;
          virtual void text( int boxno, const string& aText );
          virtual string text( int boxno ) const;
          virtual int numberOfItems() const;
          virtual bool item_disabled( int boxno ) const;

          virtual TCluster* tcluster();
          virtual const TCluster* tcluster() const;
          virtual TView* tview();
          virtual const TView* tview() const;

       private:
          HuiTRadioMenu* mRadioMenu;

     }; // class RadioMenuWidget

       // --------------------------------------------------

     class ListBoxWidget : public TitledWidget, public Hui::ListBoxWidget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

     public:
	     ListBoxWidget(Hui& aHui,
			   Pointer<Hui::Widget> aParent,
			   const string& aName,
			   const vector<string>& aTexts,
			   int aInitiallySelected,
			   const string& aTitle,
			   const string& aComment);
          virtual ~ListBoxWidget();

          virtual int numberOfItems() const;
          virtual string text( int aItemNo ) const;
          virtual void text( int aItemNo, const string& aText );
          virtual int select() const;
          virtual void select( int aItemNo );
          virtual void add( int aBefore, const string& aText );
          virtual void remove( int aItemNo );
          virtual void removeAll();

	  virtual void set_title(const string& aTitle);
	  virtual void title(const string& aTitle);
	  virtual string title() const;

          virtual TView* tview();
          virtual const TView* tview() const;
          virtual Size minSize() const { return Size( 5, 3 + ( ttitle() ? 1 : 0 ) ); }
          virtual Size maxSize() const { return Size( Size::maxWidth(), Size::maxHeight() ); }

	  vector<string> mTexts;
	  string mTitleText;

       private:
          HuiTListBox* mListBox;
          string mComment;

     }; // class ListBoxWidget

       // --------------------------------------------------

     class TextEditWidget : public TitledWidget, public Hui::TextEditWidget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
	     TextEditWidget(Hui& aHui,
			    Pointer<Hui::Widget> aParent,
			    const string& aName,
			    const string& aInitialText,
			    bool aAllowEdit,
			    const string& aTitle,
			    const string& aComment );
          virtual ~TextEditWidget();

          virtual void text( const string& aText );
          virtual string text() const;

          virtual void title( const string& aTitle ) { TitledWidget::title( aTitle ); }
          virtual string title() const { return TitledWidget::title(); }

          virtual TView* tview();
          virtual const TView* tview() const;
          virtual Size minSize() const { return Size( 5, 3 + ( ttitle() ? 1 : 0 ) ); }
          virtual Size maxSize() const { return Size( Size::maxWidth(), Size::maxHeight() ); }

       private:
          HuiTTextEdit* mTextEdit;
          string mComment;

     }; // class TextEditWidget

       // --------------------------------------------------

     class ProgressIndicatorWidget : public TitledWidget, public Hui::ProgressIndicatorWidget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
	     ProgressIndicatorWidget(Hui& aHui,
				     Pointer<Hui::Widget> aParent,
				     const string& aName,
				     float aMin,
				     float aMax,
				     float aInitial,
				     const string& aTitle,
				     const string& aComment);
          virtual ~ProgressIndicatorWidget();

          virtual void set( float aValue );
          virtual float set() const;

          virtual void title( const string& aTitle ) { TitledWidget::title( aTitle ); }
          virtual string title() const { return TitledWidget::title(); }

          virtual TView* tview();
          virtual const TView* tview() const;
          virtual Size minSize() const { return Size( 4, 1 + ( ttitle() ? 1 : 0 ) ); }
          virtual Size maxSize() const { return Size( Size::maxWidth(), 1 + ( ttitle() ? 1 : 0 ) ); }

       private:
          Hui& mHui;
          HuiTProgressIndicator* mProgressIndicator;
          string mComment;

     }; // class ProgressIndicatorWidget

       // --------------------------------------------------

     class Clickable
     {
       public:
          virtual ~Clickable() {}
          virtual void click() = 0;

     }; // class Clickable

       // --------------------------------------------------

     class MenuItemWidget : public Widget, virtual public Hui::MenuItemWidget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
          MenuItemWidget( Hui& aHui, const string& aName, Pointer<LanguageInterface::CallbackData> aCallbackData, MenuItemWidget* aParent );

          virtual void move( const Position& aPosition ) { Hui::MenuItemWidget::move( aPosition ); }
          virtual Position position() const { return Hui::MenuItemWidget::position(); }
          virtual void resize( const Size& aSize ) { Hui::MenuItemWidget::resize( aSize ); }
          virtual Size size() const { return Hui::MenuItemWidget::size(); }
          virtual Size minSize() const { return Size(); }
          virtual Size maxSize() const { return Size(); }
          virtual TView* tview() { return 0; }
          virtual const TView* tview() const { return 0; }
          virtual void refresh();
          virtual void disable();
          virtual void enable();
          virtual bool disabled() const;

          virtual TMenuItem* tmenuitem() = 0;
          virtual const TMenuItem* tmenuitem() const = 0;

          HuiTVision& hui() { return mHui; }

       protected:
          MenuItemWidget* parent() { return mParent; }

          struct KeyCode
          {
               const char* key;
               unsigned short keyCode;
          };

          static KeyCode sKeyCode[];
          unsigned short key2keyCode( const string& aKey );

       private:
          HuiTVision& mHui;
          MenuItemWidget* mParent;

     }; // class MenuItemWidget

       // --------------------------------------------------

     class MenuSeparatorWidget : public MenuItemWidget, public Hui::MenuSeparatorWidget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
          MenuSeparatorWidget( Hui& aHui, const string& aName, MenuItemWidget* aParent );

          virtual TMenuItem* tmenuitem() { return mSeparator; }
          virtual const TMenuItem* tmenuitem() const { return mSeparator; }

       private:
          TMenuItem* mSeparator;

     }; // class MenuSeparatorWidget

       // --------------------------------------------------

     class MenuButtonWidget : public MenuItemWidget, public Hui::MenuButtonWidget, public Clickable
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
          MenuButtonWidget( Hui& aHui, const string& aName, const string& aText, Pointer<LanguageInterface::CallbackData> aCallbackData, const string& aKey, const string& aComment, MenuItemWidget* aParent );
          virtual ~MenuButtonWidget();

          virtual void key( const string& aKey );
          virtual void text( const string& aText );
          virtual string text() const;

          virtual TMenuItem* tmenuitem() { return mButton; }
          virtual const TMenuItem* tmenuitem() const { return mButton; }
          virtual void click() { Hui::MenuButtonWidget::click(); }

       private:
          TMenuItem* mButton;

     }; // class MenuButtonWidget

       // --------------------------------------------------

     class MenuSubmenuWidget : public MenuItemWidget, public Hui::MenuSubmenuWidget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
          MenuSubmenuWidget( Hui& aHui, const string& aName, const string& aText, const string& aComment, MenuItemWidget* aParent );
          virtual Pointer<Hui::MenuItemWidget> addButton( const string& aName, const string& aText, Pointer<LanguageInterface::CallbackData> aCallbackData, const string& aKey, const string& aComment, Pointer<Hui::MenuItemWidget> aBefore );
          virtual Pointer<Hui::MenuItemWidget> addSeparator( const string& aName, Pointer<Hui::MenuItemWidget> aBefore );
          virtual Pointer<Hui::MenuItemWidget> addSubmenu( const string& aName, const string& aText, const string& aComment, Pointer<Hui::MenuItemWidget> aBefore );
          virtual void add( Pointer<Hui::MenuItemWidget> aBefore, Pointer<Hui::MenuItemWidget> aItem );
          virtual void remove( Pointer<Hui::MenuItemWidget> aItem );
          virtual void text( const string& aText );
          virtual string text() const;

          virtual TMenuItem* tmenuitem();
          virtual const TMenuItem* tmenuitem() const;
          TSubMenu*& submenu() { return mSubMenu; }

       private:
          TSubMenu* mSubMenu;

     }; // class MenuSubmenuWidget

       // --------------------------------------------------

     class MenuBarWidget : public MenuSubmenuWidget
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
          MenuBarWidget( Hui& aHui );
          virtual Pointer<Hui::MenuItemWidget> addSubmenu( const string& aName, const string& aText, const string& aComment, Pointer<Hui::MenuItemWidget> aBefore );
          virtual void refresh();

       private:
          TMenuBar* mMenuBar;

     }; // class MenuBarWidget

       // --------------------------------------------------

  public:
     const Subwidgets& subwidgets() const { return mWidgets; }
     Subwidgets& subwidgets() { return mWidgets; }

     static string translateAccel( const string& aText );

     class CommandsToClick
     {
       public:
          CommandsToClick() : mNewCommand( sCommandsBase ) {}

          bool click( ushort aCommand );
          ushort newCommand( Clickable& aClickable );
          void remove( ushort aCommand );

       private:
          typedef map<ushort,Clickable*> Table;
          Table mTable;
          ushort mNewCommand;

          static const ushort sCommandsBase;

     }; // class CommandsToClick

     CommandsToClick& commandsToClick() { return mCommandsToClick; }

     virtual void handleEvent( TEvent& event);

  private:
     Subwidgets mWidgets;
     Pointer<MenuSubmenuWidget> mMenuBar;

     CommandsToClick mCommandsToClick;

}; // class HuiTVision

//======================================================================

class HuiTLabel : public TLabel
{
public:
	HuiTLabel(const string& aText);
	virtual string text() const;
	virtual void text(const string& aText);
	virtual void link(TView* aLink);

}; // class HuiTLabel

//======================================================================

inline TPoint owner_frame_offset(TView* owner)
{
	TPoint offset;
	if (owner && dynamic_cast<TWindow*>(owner) &&
	    dynamic_cast<TWindow*>(owner)->frame)
		offset.x = offset.y = 1;
	else
		offset.x = offset.y = 0;

	return(offset);
}

//======================================================================

static TSItem* strings2tsitem(const vector<string>& aTexts)
{
	TSItem* result = 0;
	for (vector<string>::const_reverse_iterator s = aTexts.rbegin();
	     s != aTexts.rend();
	     ++s )
		result = new TSItem( newStr( HuiTVision::translateAccel( *s ).c_str() ), result );

	return(result);

} // vector_string2tsitem

//======================================================================

#endif // HuiTVision_hh
