/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiTVision_ButtonWidget.cc,v 1.30 2001/06/20 10:24:49 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiTVision.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiTVision_hh
#include "HuiTVision.hh"
#endif

#define Uses_TDialog
#define Uses_TDeskTop
#define Uses_TLabel
#define Uses_TButton
#define Uses_TInputLine
#define Uses_TRangeValidator
#define Uses_TPXPictureValidator
#define Uses_MsgBox
#define Uses_TSItem
#define Uses_TRadioButtons
#define Uses_TCheckBoxes
#define Uses_TListBox
#define Uses_TScrollBar
#define Uses_TCollection
#define Uses_TStringCollection
#define Uses_TFileDialog
#define Uses_TMenuItem
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuBar
#define Uses_TKeys
#define Uses_TScroller
#define Uses_TEditor
#define Uses_TIndicator

#include <tvision/tv.h>

#ifndef String_hh
#include "String.hh"
#endif

#include <cmath>

#define MAX(a,b) ((a) > (b) ? (a) : (b))

//======================================================================

class HuiTButton : public TButton
{
public:
	HuiTButton(HuiTVision::ButtonWidget& aButtonWidget,
		   const string& aText,
		   const string& aButtonType)
		: TButton(TRect(1, 1, 1 + aText.length(), 2),
			  aText.c_str(), 
			  cmDefault, 
			  aButtonType == "default" ? bfDefault : bfNormal),
		  mButtonWidget( aButtonWidget ) {
	}
	
	virtual string text() const {
		return(title);
	}
	
	virtual void text( const string& aText ) {
		delete title;
		title = newStr( aText.c_str() );
	}
	
	virtual void press() {
		TButton::press();
		mButtonWidget.click();
	}
	
private:
	HuiTVision::ButtonWidget& mButtonWidget;
	
}; // class HuiTButton

//======================================================================

HuiTVision::ButtonWidget::ButtonWidget(HuiTVision& aHui,
				       Pointer<Hui::Widget> aParent,
				       const string& aName,
				       const string& aText,
				       Pointer<LanguageInterface::CallbackData> aCallbackData,
				       const string& aButtonType,
				       const string& aComment)
	: Hui::Widget( aHui, aName ),
	  Hui::EmbeddedWidget( aHui, aName ),
	  HuiTVision::Widget( aHui, aName ),
	  Hui::ButtonWidget( aHui, aName, aCallbackData ),
	  mComment(aComment), 
	  mButton( new HuiTButton(*this, translateAccel( aText ).c_str(),
				  aButtonType ) )
{

	parent(aParent);
	
	resize(minSize());

} // HuiTVision::ButtonWidget::ButtonWidget

//======================================================================

HuiTVision::ButtonWidget::~ButtonWidget()
{
	delete mButton;

} // HuiTVision::ButtonWidget::~ButtonWidget

//======================================================================

void HuiTVision::ButtonWidget::text( const string& aText )
{
	if ( mButton )
		mButton->text( aText );

} // HuiTVision::ButtonWidget::text

//======================================================================

string HuiTVision::ButtonWidget::text() const
{
	return mButton->text();

} // HuiTVision::ButtonWidget::text

//======================================================================

TView* HuiTVision::ButtonWidget::tview()
{
	return(mButton);

} // HuiTVision::ButtonWidget::tview

//======================================================================

const TView* HuiTVision::ButtonWidget::tview() const
{
	return(mButton);

} // HuiTVision::ButtonWidget::tview

//======================================================================

void
HuiTVision::ButtonWidget::callback_completed(const WidgetCallbackData& aCallbackData)
{
	const string& result = aCallbackData.result();

	if (!result.empty()) {
		HuiTVision::DialogWidget *dp = dynamic_cast<DialogWidget*>((Hui::Widget *) parent());
		HuiTVision::ContainerWidget *cp = dynamic_cast<ContainerWidget*>((Hui::Widget *) parent());
		if (cp != NULL) {
			/* we need to find the TopLevelWidget */
			HuiTVision::ContainerWidget *next;
			do {
				next = dynamic_cast<ContainerWidget*>((Hui::Widget *) cp->parent());
				if (next != NULL)
					cp = next;
			} while (next != NULL);

			/* cp now contains toplevel container */
			dp = dynamic_cast<DialogWidget*>((Hui::Widget *) cp->parent());
		}
		if (dp != NULL)
			dp->endModal(result);
	}

} // HuiTVision::ButtonWidget::callback_completed

//======================================================================
