/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiTVision_CheckBoxesWidget.cc,v 1.30 2001/06/20 10:24:49 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiTVision.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiTVision_hh
#include "HuiTVision.hh"
#endif

#define Uses_TDialog
#define Uses_TDeskTop
#define Uses_TLabel
#define Uses_TButton
#define Uses_TInputLine
#define Uses_TRangeValidator
#define Uses_TPXPictureValidator
#define Uses_MsgBox
#define Uses_TSItem
#define Uses_TRadioButtons
#define Uses_TCheckBoxes
#define Uses_TListBox
#define Uses_TScrollBar
#define Uses_TCollection
#define Uses_TStringCollection
#define Uses_TFileDialog
#define Uses_TMenuItem
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuBar
#define Uses_TKeys
#define Uses_TScroller
#define Uses_TEditor
#define Uses_TIndicator

#include <tvision/tv.h>

#ifndef String_hh
#include "String.hh"
#endif

#include <cmath>

#define MAX(a,b) ((a) > (b) ? (a) : (b))

//======================================================================

class HuiTCheckBoxes : public TCheckBoxes
{
public:
	HuiTCheckBoxes( HuiTVision::CheckBoxesWidget& aCheckBoxesWidget, const vector<string>& aTexts )
		: TCheckBoxes( TRect( 1, 1, 11, aTexts.size() ), strings2tsitem( aTexts ) ),
		mCheckBoxesWidget( aCheckBoxesWidget )
		{
		}

	virtual bool state( int aItem ) const
		{
			return const_cast<HuiTCheckBoxes*>( this )->mark( aItem );
		}

	virtual void toggle( int aItem )
		{
			press( aItem );
		}

	virtual void state( int aItem, bool aNewState )
		{
			if ( mark( aItem ) != aNewState )
				press( aItem );
		}

	virtual void press( int aItem )
		{
			TCheckBoxes::press( aItem );
			mCheckBoxesWidget.click( aItem );
		}

	virtual string text( int aItem ) const
		{
			if ( strings && aItem >= 0 && aItem < strings->getCount() )
				return (char*)strings->at( aItem );
			else
				return string();
		}

	virtual void text( int aItem, const string& aNewText )
		{
			if ( strings && aItem >= 0 && aItem < strings->getCount() ) {
				delete strings->at( aItem );
				strings->atPut( aItem, const_cast<char*>( aNewText.c_str() ) );
			}
		}

	virtual int size() const
		{
			if ( strings )
				return strings->getCount();
			else
				return 0;
		}

	virtual bool item_disabled( int boxno ) const
		{
			return enableMask & ( 1L << boxno );
		}

private:
	HuiTVision::CheckBoxesWidget& mCheckBoxesWidget;

}; // class HuiTCheckBoxes

//======================================================================

HuiTVision::CheckBoxesWidget::CheckBoxesWidget(Hui& aHui,
					       Pointer<Hui::Widget> aParent,
					       const string& aName,
					       const vector<string>& aTexts,
					       const vector<bool>& aInitialValues,
					       const string& aTitle,
					       const vector<string>& aComments)
	: Hui::Widget(aHui, aName),
	  Hui::EmbeddedWidget(aHui, aName),
	  Hui::ButtonsGroupWidget(aHui, aName),
	  HuiTVision::ButtonsGroupWidget(aHui,
					 aParent, 
					 aName,
					 aTitle,
					 aComments),
	  Hui::CheckBoxesWidget(aHui, aName),
	  mCheckBoxes(new HuiTCheckBoxes(*this, aTexts))
{
	parent(aParent);

	if (ttitle())
		ttitle()->link(mCheckBoxes);
	
	for (vector<bool>::const_iterator i = aInitialValues.begin();
	     i != aInitialValues.end();
	     ++i)
		state(i - aInitialValues.begin(), *i);

	resize(minSize());

} // HuiTVision::CheckBoxesWidget::CheckBoxesWidget

//======================================================================

HuiTVision::CheckBoxesWidget::~CheckBoxesWidget()
{
	delete mCheckBoxes;

} // HuiTVision::CheckBoxesWidget::~CheckBoxesWidget

//======================================================================

bool HuiTVision::CheckBoxesWidget::state( int boxno, int aNewState )
{
	if ( mCheckBoxes && boxno >= 0 && boxno < numberOfItems() ) {
		if ( aNewState == -1967 ) {
			return mCheckBoxes->state( boxno );
		}
		else {
			const bool oldState = mCheckBoxes->state( boxno );
			mCheckBoxes->state( boxno, aNewState );
			return(oldState);
		}
	}
	else
		return(false);

} // HuiTVision::CheckBoxesWidget::state

//======================================================================

bool HuiTVision::CheckBoxesWidget::state( int boxno ) const
{
	if ( mCheckBoxes && boxno >= 0 && boxno < numberOfItems() )
		return mCheckBoxes->state( boxno );
	else
		return(false);

} // HuiTVision::CheckBoxesWidget::state

//======================================================================

bool HuiTVision::CheckBoxesWidget::toggle( int boxno )
{
	if ( mCheckBoxes && boxno >= 0 && boxno < numberOfItems() )
		mCheckBoxes->toggle( boxno );
	return state( boxno );

} // HuiTVision::CheckBoxesWidget::toggle

//======================================================================

void HuiTVision::CheckBoxesWidget::text( int boxno, const string& aText )
{
	if ( mCheckBoxes && boxno >= 0 && boxno < numberOfItems() )
		mCheckBoxes->text( boxno, aText );

} // HuiTVision::CheckBoxesWidget::text

//======================================================================

string HuiTVision::CheckBoxesWidget::text( int boxno ) const
{
	if ( mCheckBoxes )
		return mCheckBoxes->text( boxno );
	else
		return string();

} // HuiTVision::CheckBoxesWidget::text

//======================================================================

int HuiTVision::CheckBoxesWidget::numberOfItems() const
{
	if ( mCheckBoxes )
		return mCheckBoxes->size();
	else
		return 0;

} // HuiTVision::CheckBoxesWidget::numberOfItems

//======================================================================

bool HuiTVision::CheckBoxesWidget::item_disabled( int boxno ) const
{
	if ( mCheckBoxes )
		return mCheckBoxes->item_disabled( boxno );
	else
		return(true);

} // HuiTVision::CheckBoxesWidget::item_disabled

//======================================================================

TCluster* HuiTVision::CheckBoxesWidget::tcluster()
{
	return(mCheckBoxes);

} // HuiTVision::CheckBoxesWidget::tcluster

//======================================================================

const TCluster* HuiTVision::CheckBoxesWidget::tcluster() const
{
	return(mCheckBoxes);

} // HuiTVision::CheckBoxesWidget::tcluster

//======================================================================

TView* HuiTVision::CheckBoxesWidget::tview()
{
	return(mCheckBoxes);

} // HuiTVision::CheckBoxesWidget::tview

//======================================================================

const TView* HuiTVision::CheckBoxesWidget::tview() const
{
	return(mCheckBoxes);

} // HuiTVision::CheckBoxesWidget::tview

//======================================================================
