/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiTVision_ContainerWidget.cc,v 1.30 2001/06/20 10:24:49 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiTVision.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiTVision_hh
#include "HuiTVision.hh"
#endif

#define Uses_TDialog
#define Uses_TDeskTop
#define Uses_TLabel
#define Uses_TButton
#define Uses_TInputLine
#define Uses_TRangeValidator
#define Uses_TPXPictureValidator
#define Uses_MsgBox
#define Uses_TSItem
#define Uses_TRadioButtons
#define Uses_TCheckBoxes
#define Uses_TListBox
#define Uses_TScrollBar
#define Uses_TCollection
#define Uses_TStringCollection
#define Uses_TFileDialog
#define Uses_TMenuItem
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuBar
#define Uses_TKeys
#define Uses_TScroller
#define Uses_TEditor
#define Uses_TIndicator

#include <tvision/tv.h>

#ifndef String_hh
#include "String.hh"
#endif

#include <cmath>

#define MAX(a,b) ((a) > (b) ? (a) : (b))

//======================================================================

HuiTVision::ContainerWidget::ContainerWidget(HuiTVision& aHui,
					     Pointer<Hui::Widget> aParent,
					     const string& aName)
    : Hui::Widget(aHui, aName),
      Hui::ContainerWidget(aHui, aName),
      Hui::GroupWidget(aHui, aName),
      HuiTVision::GroupWidget(aHui, aName, Position(0, 0), Size(0, 0))
//      HuiTVision::Widget(aHui, aName)
{
	parent(aParent);
	setFixedSize(-1);
	
} // HuiTVision::ContainerWidget::ContainerWidget

//======================================================================

HuiTVision::ContainerWidget::~ContainerWidget()
{

} // HuiTVision::ContainerWidget::~ContainerWidget

//======================================================================

void HuiTVision::ContainerWidget::show()
{
	Subwidgets::iterator sw;

	for (sw = subwidgets().begin();
	     sw != subwidgets().end();
	     sw++)
		(*sw)->show();

	mHidden = FALSE;

} // HuiTVision::ContainerWidget::show
//======================================================================

void HuiTVision::ContainerWidget::hide()
{
	Subwidgets::iterator sw;

	for (sw = subwidgets().begin();
	     sw != subwidgets().end();
	     sw++)
		(*sw)->hide();

	mHidden = TRUE;

} // HuiTVision::ContainerWidget::hide

//======================================================================

bool HuiTVision::ContainerWidget::hidden() const
{
	return(mHidden);

} // HuiTVision::ContainerWidget::hidden()

//======================================================================

void HuiTVision::ContainerWidget::disable()
{
	Subwidgets::iterator sw;

	for (sw = subwidgets().begin();
	     sw != subwidgets().end();
	     sw++)
	     (*sw)->disable();

	mDisabled = TRUE;

} // HuiTVision::ContainerWidget::disable

//======================================================================

void HuiTVision::ContainerWidget::enable()
{
	Subwidgets::iterator sw;

	for (sw = subwidgets().begin();
	     sw != subwidgets().end();
	     sw++)
		(*sw)->enable();

	mDisabled = FALSE;

} // HuiTVision::ContainerWidget::enable

//======================================================================

bool HuiTVision::ContainerWidget::disabled() const
{
	return(mDisabled);

} // HuiTVision::ContainerWidget::disabled

//======================================================================

void HuiTVision::ContainerWidget::move(const Hui::Position &aPosition)
{
	move(translate(aPosition));

} // HuiTVision::ContainerWidget::move


//======================================================================

void HuiTVision::ContainerWidget::move(const TPoint &aPosition)
{
	Subwidgets::iterator sw;
	TPoint offset;
	TPoint newpos;
	ContainerWidget *cp;

	offset.x = offset.y = 0;

	/* If parent is a ContainerWidget, too, we must add its position
	   This is, because we move() on the container's parent itself
	*/
	if ((cp = dynamic_cast<HuiTVision::ContainerWidget*>((Hui::Widget*) parent())) != NULL) {
		offset.x = cp->position().x();
		offset.y = cp->position().y();
	}

	/* Move Container */
	mPosition = aPosition + offset;

	/* Move all childs to its old position relative to the container */
	for (sw = subwidgets().begin();
	     sw != subwidgets().end();
	     sw++)
		(*sw)->move((*sw)->position());

} // HuiTVision::ContainerWidget::move

//======================================================================

class Hui::Position
HuiTVision::ContainerWidget::position() const
{
	return(translate(mPosition));

} // HuiTVision::ContainerWidget::position

//======================================================================

void
HuiTVision::ContainerWidget::resize(const Hui::Size & aSize)
{
	resize(translate(aSize));

} // HuiTVision::ContainerWidget::resize

//======================================================================

void HuiTVision::ContainerWidget::resize(const TPoint & aSize, int is_arranging = 0)
{
	if (aSize.x == -1 && aSize.y == -1)
		return;
	if (aSize.x >= 0 && aSize.y >= 0) {
		mSize = aSize;
		arrange();
		return;
	}
	if (!is_arranging && (aSize.x >= 0 || aSize.y >= 0)) {
		ContainerWidget *cp = dynamic_cast<ContainerWidget*>( (Hui::Widget*) parent());
		GroupWidget *gp = dynamic_cast<GroupWidget*>( (Hui::Widget*) parent());

		setFixedSize((aSize.x < 0) ? aSize.y : aSize.x);

		/* arrange parent again since we changed the fixed size */
		if (cp != NULL)
			cp->arrange();
		if (gp != NULL)
			gp->arrange();
		return;
	}

} // HuiTVision::ContainerWidget::resize

//======================================================================

class Hui::Size HuiTVision::ContainerWidget::size() const
{
	Hui::Size tmp;

	tmp.width(mSize.x);
	tmp.height(mSize.y);

	return(tmp);

} // HuiTVision::ContainerWidget::size

//======================================================================

TGroup *
HuiTVision::ContainerWidget::tgroup()
{
    GroupWidget *gp = dynamic_cast<GroupWidget*> ((Hui::Widget*) parent());
    
    if (gp != NULL )
	return(gp->tgroup());
    else
	return (NULL);
    
} // HuiTVision::ContainerWidget::tgroup()

//======================================================================

const TGroup*
HuiTVision::ContainerWidget::tgroup() const
{
    return const_cast<const TGroup*>((TGroup*) tgroup());

} // HuiTVision::ContainerWidget::tgroup()

//======================================================================
