/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiTVision_GroupWidget.cc,v 1.30 2001/06/20 10:24:50 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiTVision.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiTVision_hh
#include "HuiTVision.hh"
#endif

#define Uses_TDialog
#define Uses_TDeskTop
#define Uses_TLabel
#define Uses_TButton
#define Uses_TInputLine
#define Uses_TRangeValidator
#define Uses_TPXPictureValidator
#define Uses_MsgBox
#define Uses_TSItem
#define Uses_TRadioButtons
#define Uses_TCheckBoxes
#define Uses_TListBox
#define Uses_TScrollBar
#define Uses_TCollection
#define Uses_TStringCollection
#define Uses_TFileDialog
#define Uses_TMenuItem
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuBar
#define Uses_TKeys
#define Uses_TScroller
#define Uses_TEditor
#define Uses_TIndicator

#include <tvision/tv.h>

#ifndef String_hh
#include "String.hh"
#endif

#include <cmath>

#define MAX(a,b) ((a) > (b) ? (a) : (b))

//======================================================================

/* GroupWidget constructor */
/* XXX comment in commented-out stuff? */
HuiTVision::GroupWidget::GroupWidget( HuiTVision& aHui, const string& aName, const Position& /*aPosition*/, const Size& /*aSize*/ )
	: Hui::Widget( aHui, aName ), Hui::GroupWidget( aHui, aName ), mHui( aHui )
{
} // HuiTVision::GroupWidget::GroupWidget

//======================================================================

HuiTVision::GroupWidget::~GroupWidget()
{
} // HuiTVision::GroupWidget::~GroupWidget

//======================================================================

Pointer<Hui::LabelWidget>
HuiTVision::GroupWidget::label(const string& aName,
			       const string& aText,
			       const Hui::Position& aPosition)
{
	Pointer<LabelWidget> label = new LabelWidget(hui(),
						     this,
						     aName,
						     aText);

	insert(label);
	label->move(aPosition);
	arrange();
	label->show();

	return(label);

} // HuiTVision::GroupWidget::label

//======================================================================

Pointer<Hui::ButtonWidget>
HuiTVision::GroupWidget::button(const string& aName,
				const string& aText,
				Pointer<LanguageInterface::CallbackData> aCallbackData,
				const string& aButtonType,
				const Hui::Position& aPosition,
				const string& aComment )
{
	Pointer<ButtonWidget> button = new ButtonWidget(hui(), 
							this,
							aName,
							aText,
							aCallbackData,
							aButtonType,
							aComment);

	insert(button);
	button->move(aPosition);
	arrange();
	button->show();

	return(button);

} // HuiTVision::GroupWidget::button

//======================================================================

Pointer<Hui::ContainerWidget> HuiTVision::GroupWidget::container( const string& aName, const Hui::Position& aPosition, const string& aAlignment, const string& aComment)
{
	Pointer<ContainerWidget> container = new ContainerWidget(hui(), this, aName);

	insert(container);

	container->move(aPosition);
	/* keep this order: */
	dynamic_cast<GroupWidget*>((Hui::Widget*) container)->arrange(aAlignment,
								      FALSE);
	arrange();
	container->show();

	return(container);

} // HuiTVision::GroupWidget::container

//======================================================================

Pointer<Hui::InputLineWidget>
HuiTVision::GroupWidget::inputLine(const string& aName,
				   const string& aInitialText,
				   int aMaxLength,
				   const string& aValidator,
				   const string& aTitle,
				   const Hui::Position& aPosition,
				   const string& aComment)
{
	Pointer<InputLineWidget> inputline = new InputLineWidget(hui(),
								 this,
								 aName,
								 aInitialText,
								 aMaxLength,
								 aValidator,
								 aTitle,
								 aComment);
	insert(inputline);
	inputline->move( aPosition );
	arrange();
	inputline->show();
	return(inputline);

} // HuiTVision::GroupWidget::inputLine

//======================================================================

Pointer<Hui::CheckBoxesWidget> HuiTVision::GroupWidget::checkBoxes( const string& aName, const vector<string>& aTexts, const vector<bool>& aInitialValues, const string& aTitle, const Hui::Position& aPosition, const vector<string>& aComments )
{
	Pointer<CheckBoxesWidget> checkbox = new CheckBoxesWidget(hui(),
								  this,
								  aName,
								  aTexts,
								  aInitialValues,
								  aTitle,
								  aComments);
	insert(checkbox);
	checkbox->move(aPosition);
	arrange();
	checkbox->show();

	return(checkbox);

} // HuiTVision::GroupWidget::checkBoxes

//======================================================================

Pointer<Hui::RadioMenuWidget>
HuiTVision::GroupWidget::radioMenu(const string& aName,
				   const vector<string>& aTexts,
				   const int aInitialValue,
				   const string& aTitle,
				   const Hui::Position& aPosition,
				   const vector<string>& aComments)
{
	Pointer<RadioMenuWidget> radiomenu = new RadioMenuWidget(hui(),
								 this,
								 aName,
								 aTexts,
								 aInitialValue,
								 aTitle,
								 aComments);
	insert(radiomenu);
	radiomenu->move(aPosition);
	arrange();
	radiomenu->show();

	return(radiomenu);

} // HuiTVision::GroupWidget::radioMenu

//======================================================================

Pointer<Hui::ListBoxWidget>
HuiTVision::GroupWidget::listBox(const string& aName,
				 const vector<string>& aTexts,
				 const int aInitiallySelected,
				 const string& aTitle,
				 const Hui::Position& aPosition,
				 const string& aComment)
{
	Pointer<ListBoxWidget> listbox = new ListBoxWidget(hui(),
							   this,
							   aName,
							   aTexts,
							   aInitiallySelected,
							   aTitle,
							   aComment);
	insert(listbox);
	listbox->move(aPosition);
	arrange();
	listbox->show();
	return(listbox);

} // HuiTVision::GroupWidget::listBox

//======================================================================

Pointer<Hui::TextEditWidget>
HuiTVision::GroupWidget::textEdit(const string& aName,
				  const string& aInitialText,
				  bool aAllowEdit,
				  const string& aTitle,
				  const Hui::Position& aPosition,
				  const string& aComment)
{
	Pointer<TextEditWidget> textedit = new TextEditWidget(hui(),
							      this,
							      aName,
							      aInitialText,
							      aAllowEdit,
							      aTitle,
							      aComment);
	insert(textedit);
	textedit->move(aPosition);
	arrange();
	textedit->show();

	return(textedit);

} // HuiTVision::GroupWidget::textEdit

//======================================================================

Pointer<Hui::ProgressIndicatorWidget>
HuiTVision::GroupWidget::progressIndicator(const string& aName,
					   float aMin, 
					   float aMax,
					   float aInitial,
					   const string& aTitle,
					   const Hui::Position& aPosition,
					   const string& aComment)
{
	Pointer<ProgressIndicatorWidget> progress = new ProgressIndicatorWidget(hui(),
										this,
										aName,
										aMin,
										aMax,
										aInitial,
										aTitle,
										aComment);
	insert(progress);
	progress->move(aPosition);
	arrange();
	progress->show();

	return(progress);

} // HuiTVision::GroupWidget::progressIndicator

//======================================================================

void HuiTVision::GroupWidget::arrange()
{
	Subwidgets::iterator sw;
	TPoint newsize, mysize;
	TPoint newpos;
	Hui::Size tmpsize;

	if (mArrangement == ARRANGE_FREE)
		return;

	tmpsize = size();
	mysize.x = tmpsize.width();
	mysize.y = tmpsize.height();

	/* First, calculate how much of the given space has
	   to be shared by widgets w/o fixed size */
	int sharedsize = (mArrangement == ARRANGE_VERTICAL) ? mysize.y : mysize.x;
	int numwidgets = 0;
	
	for (sw = subwidgets().begin();
	     sw != subwidgets().end();
	     sw++) {
		ContainerWidget *cp = dynamic_cast<ContainerWidget*>((Hui::Widget*)*sw);
		HuiTVision::Widget *gp = dynamic_cast<HuiTVision::Widget*>((Hui::Widget*)*sw);
		int widgetfixedsize = -1;
		
		if (cp != NULL)
			widgetfixedsize = cp->getFixedSize();
		if (gp != NULL)
			widgetfixedsize = gp->getFixedSize();
		if (widgetfixedsize != -1) {
			sharedsize -= widgetfixedsize;
		} else
			numwidgets++;
	}
	
	if (mArrangement == ARRANGE_HORIZONTAL) {
		int ipos = 0, n = 0;
		
		for (sw = subwidgets().begin();
		     sw != subwidgets().end();
		     sw++) {
			int widgetfixedsize = -1;
			ContainerWidget *cp = dynamic_cast<ContainerWidget*>((Hui::Widget*) *sw);
			HuiTVision::Widget *gp = dynamic_cast<HuiTVision::Widget*>((Hui::Widget*) *sw);
			
			/* Check, if the current widget wants a fixed size */
			if (cp != NULL)
				widgetfixedsize = cp->getFixedSize();
			if (gp != NULL)
				widgetfixedsize = gp->getFixedSize();
			
			newpos.x = ipos;
			newpos.y = 0;

			if (widgetfixedsize != -1) {
				/* If so, just move and resize it */
				newsize.x = widgetfixedsize;
				newsize.y = mysize.y;
				if (cp != NULL) {
					cp->move(newpos);
					cp->resize(newsize, TRUE);
				}
				if (gp != NULL) {
					gp->move(newpos);
					gp->resize(newsize, TRUE);
				}
				ipos += widgetfixedsize;
			} else {
				/* Calculate new size */
				int newwidth = sharedsize / numwidgets;
				newsize.x = newwidth;
				newsize.y = mysize.y;
				if (cp != NULL) {
					cp->move(newpos);
					cp->resize(newsize, TRUE);
				}
				if (gp != NULL) {
					gp->move(newpos);
					gp->resize(newsize, TRUE);
				}
				ipos += newwidth;
				numwidgets--;
				sharedsize -= newwidth;
			}
			
		}
	}
	if (mArrangement == ARRANGE_VERTICAL) {
		int ipos = 0, n = 0;

		for (sw = subwidgets().begin();
		    sw != subwidgets().end();
		    sw++) {
			int widgetfixedsize = -1;
			ContainerWidget *cp = dynamic_cast<ContainerWidget*>((Hui::Widget*)*sw);
			HuiTVision::Widget *gp = dynamic_cast<HuiTVision::Widget*>((Hui::Widget*)*sw);

			/* Check, if the current widget wants a fixed size */
			if (cp != NULL)
				widgetfixedsize = cp->getFixedSize();
			if (gp != NULL)
				widgetfixedsize = gp->getFixedSize();
			
			newpos.x = 0;
			newpos.y = ipos;

			if (widgetfixedsize != -1) {

				/* If so, just move and resize it */
				newsize.x = mysize.x;
				newsize.y = widgetfixedsize;
				if (cp != NULL) {
					cp->move(newpos);
					cp->resize(newsize, TRUE);
				}
				if (gp != NULL) {
					gp->move(newpos);
					gp->resize(newsize, TRUE);
				}
				ipos += widgetfixedsize;
			} else {
				/* Calculate new size */
				int newheight = sharedsize / numwidgets;
				newsize.x = mysize.x;
				newsize.y = newheight;
				if (cp != NULL) {
					cp->move(newpos);
					cp->resize(newsize, TRUE);
				}
				if (gp != NULL) {
					gp->move(newpos);
					gp->resize(newsize, TRUE);
				}
				ipos += newheight;
				numwidgets--;
				sharedsize -= newheight;
			}
			
		}
	}

} // HuiTVision::GroupWidget::arrange

//======================================================================

void HuiTVision::GroupWidget::arrange(const string& aAlignment, bool arrange_new = TRUE)
{
	if ( aAlignment == "horizontal" )
		mArrangement = ARRANGE_HORIZONTAL;
	else if ( aAlignment == "vertical" )
		mArrangement = ARRANGE_VERTICAL;
	else if ( aAlignment == "free" )
		mArrangement = ARRANGE_FREE;
	else if ( aAlignment != "" )
		throw Error( string( "Invalid alignment  `" ) + aAlignment + "' (`horizontal', `vertical' or `free' expected)" );

	if (arrange_new)
		arrange();

} // HuiTVision::GroupWidget::arrange

//======================================================================

/* Get groupwidget by name */
Pointer<Hui::Widget> HuiTVision::GroupWidget::byName( const string& aName ) const
{
	Pointer<Hui::Widget> result = Hui::Widget::byName( aName );
	/* search through list */
	/* XXX strange, somehow. compare to Qt */
	for (Subwidgets::const_iterator w = mWidgets.begin();
	     !result && w != mWidgets.end(); ++w )
		/* XXX why is this assigned in every case? */
		result = (*w)->byName( aName );
	return(result);

} // HuiTVision::GroupWidget::byName

//======================================================================

/* set current Widget (focus) */
void HuiTVision::GroupWidget::current( Pointer<Hui::EmbeddedWidget> aWidget )
{
	TView* old_current = tgroup()->current;
	Pointer<HuiTVision::Widget> w = dynamic_cast<HuiTVision::Widget*>( (Hui::EmbeddedWidget*)aWidget );
	if ( w && w->tview() )
		tgroup()->setCurrent( w->tview(), TView::normalSelect );
	aboutToChangeCurrent( old_current, tgroup()->current );

} // HuiTVision::GroupWidget::current

//======================================================================

/* get current widget (focused) */
Pointer<Hui::Widget> HuiTVision::GroupWidget::current() const
{
	/* Call TVision function to get the current, then search for this
one in the group of subwidgets */
	return subwidgets().find( tgroup()->current );

} // HuiTVision::GroupWidget::current

//======================================================================

/* change focus between two widgets */
void HuiTVision::GroupWidget::aboutToChangeCurrent( TView* old, TView* current )
{
	if ( old != current ) {
		Hui::Widget* cur = subwidgets().find( old );
		Hui::Widget* ncur = subwidgets().find( current );
		if ( ncur ) {
			if (cur)
				cur->leave();
			ncur->enter();
		}
	}

} // HuiTVision::GroupWidget::aboutToChangeCurrent

//======================================================================

/* insert GroupWidget */
void HuiTVision::GroupWidget::insert( Pointer<Hui::Widget> aWidget )
{
	HuiTVision::Widget *wp;

	/* Add wdiget into list */
	subwidgets().add(aWidget);

	if ((wp = dynamic_cast<HuiTVision::Widget *>((Hui::Widget *)aWidget)) != NULL) {

		TView* old_current = tgroup()->current;

		/* Also TVision add it */
		if (wp->tview() != NULL)
			tgroup()->insert(wp->tview());
		
		/* Give focus to new widget */
		aboutToChangeCurrent( old_current, tgroup()->current );
	}

} // HuiTVision::GroupWidget::insert

//======================================================================

/* Remvoe widget */
void HuiTVision::GroupWidget::erase()
{
	Subwidgets::iterator sw;

	for (sw = subwidgets().begin();
	     sw != subwidgets().end();
	     sw++) {
		ContainerWidget *cp = dynamic_cast<HuiTVision::ContainerWidget*>((Hui::Widget*)*sw);
		HuiTVision::Widget *gp = dynamic_cast<HuiTVision::Widget*>((Hui::Widget*)*sw);
		HuiTVision::TitledWidget *tp = dynamic_cast<HuiTVision::TitledWidget*>((Hui::Widget*)*sw);

#if 1
		(*sw)->hide();
#else
		if (cp != NULL) {
			cp->erase();
		} else if (tp != NULL) {
			tgroup()->remove((TLabel *) tp->ttitle());
			tgroup()->remove(tp->tview());
		} else if (gp != NULL) {
			tgroup()->remove(gp->tview());
		}
		/* XXX this should also remove eventual scrollbars */
#endif
	}

	subwidgets().clear();

} // HuiTVision::GroupWidget::erase

//======================================================================
