/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiTVision_ListBoxWidget.cc,v 1.30 2001/06/20 10:24:50 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiTVision.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiTVision_hh
#include "HuiTVision.hh"
#endif

#define Uses_TDialog
#define Uses_TDeskTop
#define Uses_TLabel
#define Uses_TButton
#define Uses_TInputLine
#define Uses_TRangeValidator
#define Uses_TPXPictureValidator
#define Uses_MsgBox
#define Uses_TSItem
#define Uses_TRadioButtons
#define Uses_TCheckBoxes
#define Uses_TListBox
#define Uses_TScrollBar
#define Uses_TCollection
#define Uses_TStringCollection
#define Uses_TFileDialog
#define Uses_TMenuItem
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuBar
#define Uses_TKeys
#define Uses_TScroller
#define Uses_TEditor
#define Uses_TIndicator

#include <tvision/tv.h>

#ifndef String_hh
#include "String.hh"
#endif

#include <cmath>

#define MAX(a,b) ((a) > (b) ? (a) : (b))

//======================================================================

class HuiTListBox : public TView
{
public:
	HuiTListBox( const TRect& bounds, const vector<string>& aTexts, HuiTVision::ListBoxWidget& aListBoxWidget );

	virtual void changeBounds( const TRect& bounds );
	virtual void draw();
	virtual void focus( size_t item );
	virtual size_t focus() const { return(mFocused); }
	virtual TPalette& getPalette() const;
	virtual void handleEvent( TEvent& event );
	virtual void setState( ushort aState, Boolean enable );
	virtual void shutDown();

	TScrollBar* hScrollBar;
	TScrollBar* vScrollBar;

	virtual void save_texts(const vector<string>& aTexts,
				const string& aTitle);
protected:
	virtual int widest_column(const vector<string>& aTexts,
				  const string& aTitle);
	virtual string columns_titletext_magic(const vector<string>& aTexts,
					       const string& aTitle);
	virtual vector<string> columns_text_magic(const vector<string>& aTexts,
						  const string& aTitle);

private:
	vector<string> mData;
	size_t mFocused;
	size_t mTopItem;
	HuiTVision::ListBoxWidget& mListBoxWidget;

	virtual void itemsChanged();
	virtual const char *streamableName() const { return("HuiTListBox"); }

}; // class HuiTListBox

//======================================================================

void
HuiTListBox::save_texts(const vector<string>& aTexts,
			const string& aTitle)
{
	mListBoxWidget.mTexts = aTexts;
	mListBoxWidget.mTitleText = aTitle;

	mData = columns_text_magic(mListBoxWidget.mTexts,
				   mListBoxWidget.mTitleText);
	itemsChanged();
	mListBoxWidget.set_title(columns_titletext_magic(mListBoxWidget.mTexts,
							 mListBoxWidget.mTitleText));	
}

int
HuiTListBox::widest_column(const vector<string>& aTexts,
			   const string& aTitle)
{
	int max = 0;
	bool found = 0;
	string::size_type tmp;
	size_t item;

	for (item = 0; item < aTexts.size(); item++) {
		tmp = aTexts[item].find("\t", 0);
		if (tmp != aTexts[item].npos) {
			max = MAX(max, tmp);
			found = 1;
		}
	}

	tmp = aTitle.find("\t", 0);
	if (tmp != aTitle.npos) {
		max = MAX(max, tmp);
		found = 1;
		}

	if (!found)
		return -1;
	else
		return max;
}

//======================================================================

vector<string>
HuiTListBox::columns_text_magic(const vector<string>& aTexts,
				const string& aTitle)
{
	vector<string> newTexts = aTexts;
	string newTitle = aTitle;
	size_t item;
	int max;
	
	do {
		max = widest_column(newTexts, newTitle);
		if (max != -1) {
			/* Replace each \t with 2 at least one tab up to widest column */
			string::size_type pos;
			for (item = 0; item < newTexts.size(); item++) {
				pos = newTexts[item].find("\t", 0);
				
				if (pos != newTexts[item].npos) {
					while (pos != max + 1)
						newTexts[item].insert(pos++, 1, ' ');
					newTexts[item].replace(pos, 1, ' ');
				}
			}
			pos = newTitle.find("\t", 0);
			if (pos != newTitle.npos) {
				while(pos != max + 1)
					newTitle.insert(pos++, 1, ' ');
				newTitle.replace(pos, 1, ' ');
			}
		}
	} while (max != -1);

	return(newTexts);
}

//======================================================================

string
HuiTListBox::columns_titletext_magic(const vector<string>& aTexts,
				     const string& aTitle)
{
	vector<string> newTexts = aTexts;
	string newTitle = aTitle;
	size_t item;
	int max;
	
	do {
		max = widest_column(newTexts, newTitle);
		if (max != -1) {
			/* Replace each \t with 2 at least one tab up to widest column */
			string::size_type pos;
			for (item = 0; item < newTexts.size(); item++) {
				pos = newTexts[item].find("\t", 0);
				
				if (pos != newTexts[item].npos) {
					while (pos != max + 1)
						newTexts[item].insert(pos++, 1, ' ');
					newTexts[item].replace(pos, 1, ' ');
				}
			}
			pos = newTitle.find("\t", 0);
			if (pos != newTitle.npos) {
				while(pos != max + 1)
					newTitle.insert(pos++, 1, ' ');
				newTitle.replace(pos, 1, ' ');
			}
		}
	} while (max != -1);

	return(newTitle);
}

//======================================================================

HuiTListBox::HuiTListBox( const TRect& bounds, const vector<string>& aTexts, HuiTVision::ListBoxWidget& aListBoxWidget )
	: TView( bounds ),
	hScrollBar( new TScrollBar( TRect( bounds.a.x, bounds.b.y + 1, bounds.b.x, bounds.b.y + 1 ) ) ),
	vScrollBar( new TScrollBar( TRect( bounds.a.x - 1, bounds.a.y, bounds.a.x - 1, bounds.b.y ) ) ),
	mData( aTexts ), mFocused( 0 ), mTopItem( 0 ),
	mListBoxWidget( aListBoxWidget )
{
	options |= ofFirstClick | ofSelectable;
	eventMask |= evBroadcast;

	if ( vScrollBar != 0 )
		vScrollBar->setStep( size.y - 1, 1 );

	if ( hScrollBar != 0 )
		hScrollBar->setStep( size.x / 2, 1 );

} // HuiTListBox::HuiTListBox

//======================================================================

void HuiTListBox::changeBounds( const TRect& bounds )
{
	TView::changeBounds( TRect( bounds.a.x, bounds.a.y, bounds.b.x, bounds.b.y ) );

	if ( hScrollBar != 0 ) {
		hScrollBar->setBounds( TRect( bounds.a.x + 1, bounds.b.y - 1, bounds.b.x, bounds.b.y ) );
		hScrollBar->setStep( size.x / 2, hScrollBar->arStep );
	}
	if ( vScrollBar != 0 ) {
		vScrollBar->setBounds( TRect( bounds.a.x, bounds.a.y, bounds.a.x + 1, bounds.b.y - 1 ) );
		vScrollBar->setStep( size.y, vScrollBar->arStep );
	}

	itemsChanged();

} // HuiTListBox::changeBounds

//======================================================================

void HuiTListBox::draw()
{
	ushort normalColor, selectedColor, focusedColor = 0;
	if ( ( state & ( sfSelected | sfActive ) ) == ( sfSelected | sfActive ) ) {
		normalColor = getColor(1);
		focusedColor = getColor(3);
		selectedColor = getColor(4);
	}
	else {
		normalColor = getColor(2);
		selectedColor = getColor(4);
	}

	const short indent = hScrollBar != 0 ? hScrollBar->value : 0;

	TDrawBuffer b;

	for ( short i = 0; i < size.y - 1; i++ ) {
		const size_t item =  i + mTopItem;
		ushort color;
		uchar scOff;
		if ( ( state & ( sfSelected | sfActive ) ) == ( sfSelected | sfActive ) && mFocused == item && item < mData.size() ) {
			color = focusedColor;
			setCursor( 0 + 1, i );
			scOff = 0;
		}
		else if ( item == mFocused && item < mData.size() ) {
			color = selectedColor;
			scOff = 2;
		}
		else {
			color = normalColor;
			scOff = 4;
		}

		b.moveChar( 0, ' ', color, size.x + 1 );
		if ( item < mData.size() ) {
			const string text = mData[item].substr( indent, size.x - 2 );
			b.moveStr( 1, text.c_str(), color );
			if ( indent > 0 )
				b.moveChar( 0, '<', color, 1 );
			if ( mData[item].length() > size_t( indent + size.x - 2 ) )
				b.moveChar( size.x - 1, '>', color, 1 );
		}
		else if ( i == 0 )
			b.moveStr( 1, _("<empty>"), getColor(1) );

		writeLine( 1, i, size.x - 1, 1, b );
	}

} // HuiTListBox::draw

//======================================================================

void HuiTListBox::focus( size_t item )
{
	if ( item >= mData.size() )
		item = mData.size() - 1;

	const bool changed = mFocused != item;

	mFocused = item;

	if ( vScrollBar != 0 )
		vScrollBar->setValue( item );
	else
		drawView();

	if ( item < mTopItem )
		mTopItem = item;
	else if ( item > mTopItem + size.y - 2 && size.y > 0 )
		mTopItem = item - size.y + 2;

	if ( changed )
		mListBoxWidget.highlighted( mFocused );

} // HuiTListBox::focus

//======================================================================

#define cpListViewer "\032\032\033\034\035"

TPalette& HuiTListBox::getPalette() const
{
	static TPalette palette( cpListViewer, sizeof( cpListViewer ) - 1 );
	return(palette);

} // HuiTListBox::getPalette

//======================================================================

void HuiTListBox::handleEvent( TEvent& event )
{
	TView::handleEvent( event );

	if ( event.what == evMouseDown ) {
		const int mouseAutosToSkip = 4;
		const ushort colWidth = size.x + 1;
		size_t oldItem = mFocused;
		TPoint mouse = makeLocal( event.mouse.where );
		size_t newItem;
		if ( mouseInView( event.mouse.where ) ) {
			newItem = mouse.y + (size.y * (mouse.x / colWidth)) + mTopItem;
		}
		else
			newItem = oldItem;
		ushort count = 0;
		do {
			if ( newItem != oldItem ) {
				focus( newItem );
				drawView();
			}
			oldItem = newItem;
			mouse = makeLocal( event.mouse.where );
			if ( mouseInView( event.mouse.where ) )
				newItem = mouse.y + (size.y * (mouse.x / colWidth)) + mTopItem;
			else {
				if ( event.what == evMouseAuto )
					count++;
				if ( count == mouseAutosToSkip ) {
					count = 0;
					if ( mouse.y < 0 )
						newItem = mFocused - 1;
					else if ( mouse.y >= size.y )
						newItem = mFocused + 1;
				}
			}
			if ( event.mouse.eventFlags & meDoubleClick )
				break;
		} while ( mouseEvent( event, evMouseMove | evMouseAuto ) );
		if ( newItem != oldItem ) {
			focus( newItem );
			drawView();
		}
		clearEvent( event );
	}
	else if ( event.what == evKeyDown ) {
		size_t newItem = mFocused;
		switch ( ctrlToArrow( event.keyDown.keyCode ) ) {
		case kbUp:
			if ( mFocused > 0 )
				newItem = mFocused - 1;
			break;
		case kbDown:
			if ( mFocused < mData.size() - 1 )
				newItem = mFocused + 1;
			break;
		case kbRight:
			if ( hScrollBar != 0 )
				hScrollBar->setValue( hScrollBar->value + 1 );
			break;
		case kbLeft:
			if ( hScrollBar != 0 )
				hScrollBar->setValue( hScrollBar->value - 1 );
			break;
		case kbPgDn:
			newItem = mFocused + size.y - 2;
			if ( newItem >= mData.size() )
				newItem = mData.size() - 1;
			break;
		case  kbPgUp:
			if ( mFocused > size_t( size.y ) )
				newItem = mFocused - size.y + 2;
			else
				newItem  = 0;
			break;
		case kbHome:
			newItem = 0;
			break;
		case kbEnd:
			newItem = mData.size() - 1;
			break;
		case kbCtrlPgDn:
			newItem = mTopItem + size.y - 1;
			if ( newItem >= mData.size() )
				newItem = mData.size() - 1;
			break;
		case kbCtrlPgUp:
			newItem = mTopItem;
			break;
		default:
			return;
		}
		focus( newItem );
		drawView();
		clearEvent( event );
	}
	else if ( event.what == evBroadcast ) {
		if ( ( options & ofSelectable ) != 0 ) {
			if ( event.message.command == cmScrollBarClicked &&
				( event.message.infoPtr == hScrollBar ||
				event.message.infoPtr == vScrollBar ))
				focus();
			else if ( event.message.command == cmScrollBarChanged ) {
				if ( vScrollBar == event.message.infoPtr ) {
					focus( vScrollBar->value );
					drawView();
				}
				else if ( hScrollBar == event.message.infoPtr )
					drawView();
			}
		}
	}

} // HuiTListBox::handleEvent

//======================================================================

void HuiTListBox::setState( ushort aState, Boolean enable )
{
	TView::setState( aState, enable );
	if ( ( aState & ( sfSelected | sfActive | sfVisible ) ) != 0 ) {
		if ( hScrollBar != 0 )
			if ( getState( sfActive ) && getState( sfVisible ) )
				hScrollBar->show();
			else
				hScrollBar->hide();
		if ( vScrollBar != 0 )
			if ( getState( sfActive ) && getState( sfVisible ) )
				vScrollBar->show();
			else
				vScrollBar->hide();
		drawView();
	}

} // HuiTListBox::setState

//======================================================================

void HuiTListBox::shutDown()
{
	delete hScrollBar;
	delete vScrollBar;
	TView::shutDown();

} // HuiTListBox::shutDown

//======================================================================

void HuiTListBox::itemsChanged()
{
	if ( mFocused >= mData.size() )
		mFocused = 0;
	if ( mTopItem >= mData.size() )
		mTopItem = 0;

	if ( vScrollBar != 0 && mData.size() > 0 )
		vScrollBar->setParams( mFocused, 0, mData.size() > 0 ? mData.size() - 1 : 0, vScrollBar->pgStep, vScrollBar->arStep );

	if ( hScrollBar != 0 ) {
		size_t maxTextLength = 0;
		for ( size_t i = 0; i < mData.size(); ++i )
			if ( mData[i].length() > maxTextLength )
				maxTextLength = mData[i].length();
		int maxValue = int( maxTextLength ) - int( size.x ) + 2;
		if ( maxValue < 0 )
			maxValue = 0;
		hScrollBar->setParams( hScrollBar->value, 0, maxValue, hScrollBar->pgStep, hScrollBar->arStep );
	}

	drawView();

} // HuiTListBox::itemsChanged

//======================================================================

HuiTVision::ListBoxWidget::ListBoxWidget(Hui& aHui,
					 Pointer<Hui::Widget> aParent,
					 const string& aName,
					 const vector<string>& aTexts,
					 int aInitiallySelected,
					 const string& aTitle,
					 const string& aComment)
	: Hui::Widget(aHui, aName),
	  Hui::EmbeddedWidget(aHui, aName),
	  TitledWidget(aHui, aParent, aName, aTitle),
	  Hui::ListBoxWidget(aHui, aName),
	  mListBox(new HuiTListBox(TRect(1, 10, 1, 10),
				   aTexts, *this)),
	  mComment(aComment)
{
	mListBox->save_texts(aTexts, aTitle);

	parent(aParent);
	
	if (ttitle())
		ttitle()->link(mListBox);

/*
  XXX enable!!!
	aParent->tgroup()->insert( mListBox->hScrollBar );
	aParent->tgroup()->insert( mListBox->vScrollBar );
*/
	select(aInitiallySelected);
	resize( minSize() );

} // HuiTVision::ListBoxWidget::ListBoxWidget

//======================================================================

HuiTVision::ListBoxWidget::~ListBoxWidget()
{
	delete mListBox;

} // HuiTVision::ListBoxWidget::~ListBoxWidget

//======================================================================

int HuiTVision::ListBoxWidget::numberOfItems() const
{
	return mTexts.size();

} // HuiTVision::ListBoxWidget::numberOfItems

//======================================================================

string HuiTVision::ListBoxWidget::text( int aItemNo ) const
{
	if (aItemNo >= numberOfItems())
		throw runtime_error(string("ListBox itemno ")
				    + to_string(aItemNo)
				    + " is out of range 0.."
				    + to_string(numberOfItems() - 1));

	return mTexts[aItemNo];

} // HuiTVision::ListBoxWidget::text

//======================================================================

void HuiTVision::ListBoxWidget::text( int aItemNo, const string& aText )
{
	if (aItemNo >= 0) {
		if (aItemNo >= numberOfItems())
			throw runtime_error(string("ListBox itemno ")
					    + to_string(aItemNo)
					    + " is out of range 0.."
					    + to_string(numberOfItems() - 1));
		mTexts[aItemNo] = aText;
		if (mListBox)
			mListBox->save_texts(mTexts, mTitleText);
	}

} // HuiTVision::ListBoxWidget::text

//======================================================================

int HuiTVision::ListBoxWidget::select() const
{
	if ( mListBox )
		return mListBox->focus();
	else
		return -1;

} // HuiTVision::ListBoxWidget::select

//======================================================================

void HuiTVision::ListBoxWidget::select( int aItemNo )
{
	if ( mListBox ) {
		if ( aItemNo >= numberOfItems() ) {
			throw runtime_error( string( "ListBox itemno " ) + to_string( aItemNo ) + " is out of range 0.." + to_string( numberOfItems() - 1 ) );
		}
		mListBox->focus( aItemNo );
	}

} // HuiTVision::ListBoxWidget::select

//======================================================================

void HuiTVision::ListBoxWidget::add( int aBefore, const string& aText )
{
	mTexts.insert(mTexts.begin() + aBefore, aText);
	if (mListBox)
		mListBox->save_texts(mTexts, mTitleText);

} // HuiTVision::ListBoxWidget::add

//======================================================================

void HuiTVision::ListBoxWidget::remove( int aItemNo )
{
	if (aItemNo >= numberOfItems())
	    throw runtime_error(string("ListBox itemno ")
				+ to_string(aItemNo)
				+ " is out of range 0.."
				+ to_string(numberOfItems() - 1));

	mTexts.erase(mTexts.begin() + aItemNo);
	if (mListBox)
		mListBox->save_texts(mTexts, mTitleText);

} // HuiTVision::ListBoxWidget::remove


//======================================================================

void
HuiTVision::ListBoxWidget::title(const string& aTitle) {
	if (mListBox)
		mListBox->save_texts(mTexts, aTitle);
}

//======================================================================

string
HuiTVision::ListBoxWidget::title() const {
	return mTitleText;
}

//======================================================================

void
HuiTVision::ListBoxWidget::set_title(const string& aTitle)
{
	TitledWidget::title(string(" ") + aTitle);
}

//======================================================================

void HuiTVision::ListBoxWidget::removeAll()
{
	mTexts.clear();
	if (mListBox)
		mListBox->save_texts(mTexts, mTitleText);

} // HuiTVision::ListBoxWidget::removeAll

//======================================================================

TView* HuiTVision::ListBoxWidget::tview()
{
	return(mListBox);

} // HuiTVision::ListBoxWidget::tview

//======================================================================

const TView* HuiTVision::ListBoxWidget::tview() const
{
	return(mListBox);

} // HuiTVision::ListBoxWidget::tview

//======================================================================
