/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiTVision_MenuBarWidget.cc,v 1.30 2001/06/20 10:24:50 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiTVision.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiTVision_hh
#include "HuiTVision.hh"
#endif

#define Uses_TDialog
#define Uses_TDeskTop
#define Uses_TLabel
#define Uses_TButton
#define Uses_TInputLine
#define Uses_TRangeValidator
#define Uses_TPXPictureValidator
#define Uses_MsgBox
#define Uses_TSItem
#define Uses_TRadioButtons
#define Uses_TCheckBoxes
#define Uses_TListBox
#define Uses_TScrollBar
#define Uses_TCollection
#define Uses_TStringCollection
#define Uses_TFileDialog
#define Uses_TMenuItem
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuBar
#define Uses_TKeys
#define Uses_TScroller
#define Uses_TEditor
#define Uses_TIndicator

#include <tvision/tv.h>

#ifndef String_hh
#include "String.hh"
#endif

#include <cmath>

#define MAX(a,b) ((a) > (b) ? (a) : (b))

//======================================================================

class HuiTMenuBar : public TMenuBar
{
public:
	HuiTMenuBar( HuiTVision::MenuBarWidget& aMenuBarWidget )
		: TMenuBar(TRect( 0, 0, aMenuBarWidget.hui().size().width() - 1, 1 ), *aMenuBarWidget.submenu() ),
		mMenuBarWidget( aMenuBarWidget ) {}

private:
	HuiTVision::MenuBarWidget& mMenuBarWidget;

}; // class HuiTMenuBar

//======================================================================

HuiTVision::MenuBarWidget::MenuBarWidget( Hui& aHui )
	: Hui::Widget( aHui, "MenuBar" ), Hui::EmbeddedWidget( aHui, "MenuBar" ), Hui::MenuItemWidget( aHui, "MenuBar", 0 ),
	MenuSubmenuWidget( aHui, "MenuBar", "", "", 0 ),
	mMenuBar( new HuiTMenuBar( *this ) )
{
	TProgram::menuBar = mMenuBar;

} // HuiTVision::MenuBarWidget::MenuBarWidget

//======================================================================

void HuiTVision::MenuBarWidget::refresh()
{
	dynamic_cast<HuiTVision&>( hui() ).TApplication::remove( TProgram::menuBar );
	// NO delete TProgram::menuBar;
	TProgram::menuBar = new HuiTMenuBar( *this );
	dynamic_cast<HuiTVision&>( hui() ).TApplication::insert( TProgram::menuBar );

} // HuiTVision::MenuBarWidget::refresh

//======================================================================

Pointer<Hui::MenuItemWidget> HuiTVision::MenuBarWidget::addSubmenu( const string& aName, const string& aText, const string& aComment, Pointer<Hui::MenuItemWidget> aBefore )
{
	Pointer<HuiTVision::MenuSubmenuWidget> item = new MenuSubmenuWidget( hui(), aName, aText, aComment, this );
	add( aBefore, item );
	if ( submenu() )
		submenu() = &( *submenu() + *item->submenu() );
	else
		submenu() = item->submenu();
	refresh();
	return(item);

} // HuiTVision::MenuBarWidget::addSubmenu

//======================================================================
