/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiTVision_MenuItemWidget.cc,v 1.30 2001/06/20 10:24:51 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiTVision.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiTVision_hh
#include "HuiTVision.hh"
#endif

#define Uses_TDialog
#define Uses_TDeskTop
#define Uses_TLabel
#define Uses_TButton
#define Uses_TInputLine
#define Uses_TRangeValidator
#define Uses_TPXPictureValidator
#define Uses_MsgBox
#define Uses_TSItem
#define Uses_TRadioButtons
#define Uses_TCheckBoxes
#define Uses_TListBox
#define Uses_TScrollBar
#define Uses_TCollection
#define Uses_TStringCollection
#define Uses_TFileDialog
#define Uses_TMenuItem
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuBar
#define Uses_TKeys
#define Uses_TScroller
#define Uses_TEditor
#define Uses_TIndicator

#include <tvision/tv.h>

#ifndef String_hh
#include "String.hh"
#endif

#include <cmath>

#define MAX(a,b) ((a) > (b) ? (a) : (b))

//======================================================================

HuiTVision::MenuItemWidget::KeyCode HuiTVision::MenuItemWidget::sKeyCode[] = {
	{ "CTRL-A", kbCtrlA }, { "CTRL-B", kbCtrlB }, { "CTRL-C", kbCtrlC }, { "CTRL-D", kbCtrlD }, { "CTRL-E", kbCtrlE },
	{ "CTRL-F", kbCtrlF }, { "CTRL-G", kbCtrlG }, { "CTRL-H", kbCtrlH }, { "CTRL-I", kbCtrlI }, { "CTRL-J", kbCtrlJ },
	{ "CTRL-K", kbCtrlK }, { "CTRL-L", kbCtrlL }, { "CTRL-M", kbCtrlM }, { "CTRL-N", kbCtrlN }, { "CTRL-O", kbCtrlO },
	{ "CTRL-P", kbCtrlP }, { "CTRL-Q", kbCtrlQ }, { "CTRL-R", kbCtrlR }, { "CTRL-S", kbCtrlS }, { "CTRL-T", kbCtrlT },
	{ "CTRL-U", kbCtrlU }, { "CTRL-V", kbCtrlV }, { "CTRL-W", kbCtrlW }, { "CTRL-X", kbCtrlX }, { "CTRL-Y", kbCtrlY },
	{ "CTRL-Z", kbCtrlZ }, { "ESC", kbEsc }, { "ALT-SPACE", kbAltSpace }, { "CTRL-INS", kbCtrlIns }, { "SHIFT-INS", kbShiftIns },
	{ "CTRL-DEL", kbCtrlDel }, { "SHIFT-DEL", kbShiftDel }, { "BACKSPACE", kbBack }, { "CTRL-BACKSPACE", kbCtrlBack }, { "SHIFT-TAB", kbShiftTab },
	{ "TAB", kbTab }, { "ALT-Q", kbAltQ }, { "ALT-W", kbAltW }, { "ALT-E", kbAltE }, { "ALT-R", kbAltR },
	{ "ALT-T", kbAltT }, { "ALT-Y", kbAltY }, { "ALT-U", kbAltU }, { "ALT-I", kbAltI }, { "ALT-O", kbAltO },
	{ "ALT-P", kbAltP }, { "CTRL-ENTER", kbCtrlEnter }, { "ENTER", kbEnter }, { "ALT-A", kbAltA }, { "ALT-S", kbAltS },
	{ "ALT-D", kbAltD }, { "ALT-F", kbAltF }, { "ALT-G", kbAltG }, { "ALT-H", kbAltH }, { "ALT-J", kbAltJ },
	{ "ALT-K", kbAltK }, { "ALT-L", kbAltL }, { "ALT-Z", kbAltZ }, { "ALT-X", kbAltX }, { "ALT-C", kbAltC },
	{ "ALT-V", kbAltV }, { "ALT-B", kbAltB }, { "ALT-N", kbAltN }, { "ALT-M", kbAltM }, { "F1", kbF1 },
	{ "F2", kbF2 }, { "F3", kbF3 }, { "F4", kbF4 }, { "F5", kbF5 }, { "F6", kbF6 }, { "F7", kbF7 }, { "F8", kbF8 },
	{ "F9", kbF9 }, { "F10", kbF10 }, { "HOME", kbHome }, { "UP", kbUp }, { "PGUP", kbPgUp }, { "GRAYMINUS", kbGrayMinus },
	{ "LEFT", kbLeft }, { "RIGHT", kbRight }, { "GRAYPLUS", kbGrayPlus }, { "END", kbEnd }, { "DOWN", kbDown },
	{ "PGDN", kbPgDn }, { "INS", kbIns }, { "DEL", kbDel }, { "SHIFT-F1", kbShiftF1 }, { "SHIFT-F2", kbShiftF2 },
	{ "SHIFT-F3", kbShiftF3 }, { "SHIFT-F4", kbShiftF4 }, { "SHIFT-F5", kbShiftF5 }, { "SHIFT-F6", kbShiftF6 }, { "SHIFT-F7", kbShiftF7 },
	{ "SHIFT-F8", kbShiftF8 }, { "SHIFT-F9", kbShiftF9 }, { "SHIFT-F10", kbShiftF10 }, { "CTRL-F1", kbCtrlF1 }, { "CTRL-F2", kbCtrlF2 },
	{ "CTRL-F3", kbCtrlF3 }, { "CTRL-F4", kbCtrlF4 }, { "CTRL-F5", kbCtrlF5 }, { "CTRL-F6", kbCtrlF6 }, { "CTRL-F7", kbCtrlF7 },
	{ "CTRL-F8", kbCtrlF8 }, { "CTRL-F9", kbCtrlF9 }, { "CTRL-F10", kbCtrlF10 }, { "ALT-F1", kbAltF1 }, { "ALT-F2", kbAltF2 },
	{ "ALT-F3", kbAltF3 }, { "ALT-F4", kbAltF4 }, { "ALT-F5", kbAltF5 }, { "ALT-F6", kbAltF6 }, { "ALT-F7", kbAltF7 },
	{ "ALT-F8", kbAltF8 }, { "ALT-F9", kbAltF9 }, { "ALT-F10", kbAltF10 }, { "CTRL-PRTSC", kbCtrlPrtSc }, { "CTRL-LEFT", kbCtrlLeft },
	{ "CTRL-RIGHT", kbCtrlRight }, { "CTRL-END", kbCtrlEnd }, { "CTRL-PGDN", kbCtrlPgDn }, { "CTRL-HOME", kbCtrlHome }, { "ALT-1", kbAlt1 },
	{ "ALT-2", kbAlt2 }, { "ALT-3", kbAlt3 }, { "ALT-4", kbAlt4 }, { "ALT-5", kbAlt5 }, { "ALT-6", kbAlt6 },
	{ "ALT-7", kbAlt7 }, { "ALT-8", kbAlt8 }, { "ALT-9", kbAlt9 }, { "ALT-0", kbAlt0 }, { "ALT-MINUS", kbAltMinus },
	{ "ALT-EQUAL", kbAltEqual }, { "CTRL-PGUP", kbCtrlPgUp }, { "ALT-BACKSPACE", kbAltBack },
	{ 0, 0 }
};

//======================================================================

inline unsigned short HuiTVision::MenuItemWidget::key2keyCode( const string& aKey )
{
	if ( aKey.length() > 0 ) {
		string k = to_upper( aKey );
		for ( const KeyCode* kc = sKeyCode; kc->key; ++kc )
			if ( !strcmp( kc->key, k.c_str() ) )
				return(kc->keyCode);
		return tolower( k[0] );
	}
	return 0;

} // HuiTVision::key2keyCode

//======================================================================

HuiTVision::MenuItemWidget::MenuItemWidget( Hui& aHui, const string& aName, Pointer<LanguageInterface::CallbackData> aCallbackData, MenuItemWidget* aParent )
	: Hui::Widget( aHui, aName ), Hui::EmbeddedWidget( aHui, aName ), HuiTVision::Widget( aHui, aName ), Hui::MenuItemWidget( aHui, aName, aCallbackData ),
	mHui( dynamic_cast<HuiTVision&>( aHui ) ), mParent( aParent )
{
} // HuiTVision::MenuItemWidget::MenuItemWidget

//======================================================================

void HuiTVision::MenuItemWidget::refresh()
{
//	  if ( parent() )
//		  parent()->refresh();

} // HuiTVision::MenuItemWidget::refresh

//======================================================================

void HuiTVision::MenuItemWidget::disable()
{
	tmenuitem()->disabled = True;

} // HuiTVision::MenuItemWidget::disable

//======================================================================

void HuiTVision::MenuItemWidget::enable()
{
	tmenuitem()->disabled = False;

} // HuiTVision::MenuItemWidget::enable

//======================================================================

bool HuiTVision::MenuItemWidget::disabled() const
{
	return(tmenuitem()->disabled == True);

} // HuiTVision::MenuItemWidget::disabled

//======================================================================
