/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiTVision_MenuSubmenuWidget.cc,v 1.30 2001/06/20 10:24:51 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiTVision.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiTVision_hh
#include "HuiTVision.hh"
#endif

#define Uses_TDialog
#define Uses_TDeskTop
#define Uses_TLabel
#define Uses_TButton
#define Uses_TInputLine
#define Uses_TRangeValidator
#define Uses_TPXPictureValidator
#define Uses_MsgBox
#define Uses_TSItem
#define Uses_TRadioButtons
#define Uses_TCheckBoxes
#define Uses_TListBox
#define Uses_TScrollBar
#define Uses_TCollection
#define Uses_TStringCollection
#define Uses_TFileDialog
#define Uses_TMenuItem
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuBar
#define Uses_TKeys
#define Uses_TScroller
#define Uses_TEditor
#define Uses_TIndicator

#include <tvision/tv.h>

#ifndef String_hh
#include "String.hh"
#endif

#include <cmath>

#define MAX(a,b) ((a) > (b) ? (a) : (b))

//======================================================================

HuiTVision::MenuSubmenuWidget::MenuSubmenuWidget( Hui& aHui, const string& aName, const string& aText, const string& aComment, HuiTVision::MenuItemWidget* aParent )
	: Hui::Widget( aHui, aName ), Hui::EmbeddedWidget( aHui, aName ), Hui::MenuItemWidget( aHui, aName, 0 ), HuiTVision::MenuItemWidget( aHui, aName, 0, aParent ),
	Hui::MenuSubmenuWidget( aHui, aName, aText, aComment ),
	mSubMenu( aText.empty() ? 0 : new TSubMenu( translateAccel( aText ).c_str(), 0 ) )
{
} // HuiTVision::MenuSubmenuWidget::MenuSubmenuWidget

//======================================================================

Pointer<Hui::MenuItemWidget> HuiTVision::MenuSubmenuWidget::addButton( const string& aName, const string& aText, Pointer<LanguageInterface::CallbackData> aCallbackData, const string& aKey, const string& aComment, Pointer<Hui::MenuItemWidget> aBefore )
{
	if ( parent() ) {
		Pointer<HuiTVision::MenuItemWidget> item = new MenuButtonWidget( hui(), aName, aText, aCallbackData, aKey, aComment, this );
		add( aBefore, item );
		mSubMenu = &( *mSubMenu + *item->tmenuitem() );
		refresh();
		return(item);
	}
	else
		return 0;

} // HuiTVision::MenuSubmenuWidget::addButton

//======================================================================

Pointer<Hui::MenuItemWidget> HuiTVision::MenuSubmenuWidget::addSeparator( const string& aName, Pointer<Hui::MenuItemWidget> aBefore )
{
	if ( parent() ) {
		Pointer<HuiTVision::MenuItemWidget> item = new MenuSeparatorWidget( hui(), aName, this );
		add( aBefore, item );
		mSubMenu = &( *mSubMenu + *item->tmenuitem() );
		refresh();
		return(item);
	}
	else
		return 0;

} // HuiTVision::MenuSubmenuWidget::addSeparator

//======================================================================

Pointer<Hui::MenuItemWidget> HuiTVision::MenuSubmenuWidget::addSubmenu( const string& aName, const string& aText, const string& aComment, Pointer<Hui::MenuItemWidget> aBefore )
{
	Pointer<HuiTVision::MenuItemWidget> item = new MenuSubmenuWidget( hui(), aName, aText, aComment, this );
	add( aBefore, item );
	mSubMenu = &( *mSubMenu + *item->tmenuitem() );
	refresh();
	return(item);

} // HuiTVision::MenuSubmenuWidget::addSubmenu

//======================================================================

void HuiTVision::MenuSubmenuWidget::add( Pointer<Hui::MenuItemWidget> /*aBefore*/, Pointer<Hui::MenuItemWidget> aItem )
{
	addItem( 0, aItem ); // TVision does not support insertion of menu items

} // HuiTVision::MenuSubmenuWidget::add

//======================================================================

void HuiTVision::MenuSubmenuWidget::remove( Pointer<Hui::MenuItemWidget> /*aItem*/ )
{

} // HuiTVision::MenuSubmenuWidget::remove

//======================================================================

void HuiTVision::MenuSubmenuWidget::text( const string& aText )
{
	if ( mSubMenu ) {
		delete mSubMenu->name;
		mSubMenu->name = newStr( translateAccel( aText ).c_str() );
		setText( aText );
	}

} // HuiTVision::MenuSubmenuWidget::text

//======================================================================

string HuiTVision::MenuSubmenuWidget::text() const
{
	if ( mSubMenu )
		return(mSubMenu->name);
	else
		return string();

} // HuiTVision::MenuSubmenuWidget::text

//======================================================================

TMenuItem* HuiTVision::MenuSubmenuWidget::tmenuitem()
{
	return(mSubMenu);

} // HuiTVision::MenuSubmenuWidget::tmenuitem

//======================================================================

const TMenuItem* HuiTVision::MenuSubmenuWidget::tmenuitem() const
{
	return(mSubMenu);

} // HuiTVision::MenuSubmenuWidget::tmenuitem

//======================================================================
