/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiTVision_RadioMenuWidget.cc,v 1.30 2001/06/20 10:24:51 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiTVision.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiTVision_hh
#include "HuiTVision.hh"
#endif

#define Uses_TDialog
#define Uses_TDeskTop
#define Uses_TLabel
#define Uses_TButton
#define Uses_TInputLine
#define Uses_TRangeValidator
#define Uses_TPXPictureValidator
#define Uses_MsgBox
#define Uses_TSItem
#define Uses_TRadioButtons
#define Uses_TCheckBoxes
#define Uses_TListBox
#define Uses_TScrollBar
#define Uses_TCollection
#define Uses_TStringCollection
#define Uses_TFileDialog
#define Uses_TMenuItem
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuBar
#define Uses_TKeys
#define Uses_TScroller
#define Uses_TEditor
#define Uses_TIndicator

#include <tvision/tv.h>

#ifndef String_hh
#include "String.hh"
#endif

#include <cmath>

#define MAX(a,b) ((a) > (b) ? (a) : (b))

//======================================================================

class HuiTRadioMenu : public TRadioButtons
{
public:
	HuiTRadioMenu( HuiTVision::RadioMenuWidget& aRadioMenuWidget, const vector<string>& aTexts )
		: TRadioButtons( TRect( 1, 1, 11, aTexts.size() ), strings2tsitem( aTexts ) ),
		mRadioMenuWidget( aRadioMenuWidget )
		{
		}

	virtual int set() const
		{
			ushort val;
			const_cast<HuiTRadioMenu*>( this )->getData( &val );
			return(val);
		}

	virtual void toggle( int aItem )
		{
			press( aItem );
		}

	virtual void set( int aItem )
		{
			if ( !mark( aItem ) )
				press( aItem );
		}

	virtual void press( int aItem )
		{
			TRadioButtons::press( aItem );
			mRadioMenuWidget.click( aItem );
		}

	virtual string text( int aItem ) const
		{
			if ( strings && aItem >= 0 && aItem < strings->getCount() )
				return (char*)strings->at( aItem );
			else
				return string();
		}

	virtual void text( int aItem, const string& aNewText )
		{
			if ( strings && aItem >= 0 && aItem < strings->getCount() ) {
				delete strings->at( aItem );
				strings->atPut( aItem, const_cast<char*>( aNewText.c_str() ) );
			}
		}

	virtual int size() const
		{
			if ( strings )
				return strings->getCount();
			else
				return 0;
		}

	virtual bool item_disabled( int boxno ) const
		{
			return enableMask & ( 1L << boxno );
		}

private:
	HuiTVision::RadioMenuWidget& mRadioMenuWidget;

}; // class HuiTRadioMenu

//======================================================================

HuiTVision::RadioMenuWidget::RadioMenuWidget(Hui& aHui,
					     Pointer<Hui::Widget> aParent,
					     const string& aName,
					     const vector<string>& aTexts,
					     int aInitialValue,
					     const string& aTitle,
					     const vector<string>& aComments)
	: Hui::Widget(aHui, aName),
	  Hui::EmbeddedWidget(aHui, aName),
	  Hui::ButtonsGroupWidget(aHui, aName),
	  HuiTVision::ButtonsGroupWidget(aHui,
					 aParent,
					 aName,
					 aTitle,
					 aComments),
	  Hui::RadioMenuWidget(aHui, aName),
	  mRadioMenu(new HuiTRadioMenu(*this, aTexts))
{
	parent(aParent);
	if (ttitle())
		ttitle()->link(mRadioMenu);

	set(aInitialValue);

	resize(minSize());

} // HuiTVision::RadioMenuWidget::RadioMenuWidget

//======================================================================

HuiTVision::RadioMenuWidget::~RadioMenuWidget()
{
	delete mRadioMenu;

} // HuiTVision::RadioMenuWidget::~RadioMenuWidget

//======================================================================

void HuiTVision::RadioMenuWidget::set( int itemno )
{
	if ( mRadioMenu )
		mRadioMenu->set( itemno );

} // HuiTVision::RadioMenuWidget::set

//======================================================================

int HuiTVision::RadioMenuWidget::set() const
{
	if ( mRadioMenu )
		return mRadioMenu->set();
	else
		return -1;

} // HuiTVision::RadioMenuWidget::set

//======================================================================

void HuiTVision::RadioMenuWidget::text( int boxno, const string& aText )
{
	if ( mRadioMenu && boxno >= 0 && boxno < numberOfItems() )
		mRadioMenu->text( boxno, aText );

} // HuiTVision::RadioMenuWidget::text

//======================================================================

string HuiTVision::RadioMenuWidget::text( int boxno ) const
{
	if ( mRadioMenu )
		return mRadioMenu->text( boxno );
	else
		return string();

} // HuiTVision::RadioMenuWidget::text

//======================================================================

int HuiTVision::RadioMenuWidget::numberOfItems() const
{
	if ( mRadioMenu )
		return mRadioMenu->size();
	else
		return 0;

} // HuiTVision::RadioMenuWidget::numberOfItems

//======================================================================

bool HuiTVision::RadioMenuWidget::item_disabled( int boxno ) const
{
	if ( mRadioMenu )
		return mRadioMenu->item_disabled( boxno );
	else
		return(true);

} // HuiTVision::RadioMenuWidget::item_disabled

//======================================================================

TCluster* HuiTVision::RadioMenuWidget::tcluster()
{
	return(mRadioMenu);

} // HuiTVision::RadioMenuWidget::tcluster

//======================================================================

const TCluster* HuiTVision::RadioMenuWidget::tcluster() const
{
	return(mRadioMenu);

} // HuiTVision::RadioMenuWidget::tcluster

//======================================================================

TView* HuiTVision::RadioMenuWidget::tview()
{
	return(mRadioMenu);

} // HuiTVision::RadioMenuWidget::tview

//======================================================================

const TView* HuiTVision::RadioMenuWidget::tview() const
{
	return(mRadioMenu);

} // HuiTVision::RadioMenuWidget::tview

//======================================================================
