/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiTVision_TextEditWidget.cc,v 1.30 2001/06/20 10:24:51 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiTVision.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiTVision_hh
#include "HuiTVision.hh"
#endif

#define Uses_TDialog
#define Uses_TDeskTop
#define Uses_TLabel
#define Uses_TButton
#define Uses_TInputLine
#define Uses_TRangeValidator
#define Uses_TPXPictureValidator
#define Uses_MsgBox
#define Uses_TSItem
#define Uses_TRadioButtons
#define Uses_TCheckBoxes
#define Uses_TListBox
#define Uses_TScrollBar
#define Uses_TCollection
#define Uses_TStringCollection
#define Uses_TFileDialog
#define Uses_TMenuItem
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuBar
#define Uses_TKeys
#define Uses_TScroller
#define Uses_TEditor
#define Uses_TIndicator

#include <tvision/tv.h>

#ifndef String_hh
#include "String.hh"
#endif

#include <cmath>

#define MAX(a,b) ((a) > (b) ? (a) : (b))

//======================================================================

class HuiTTextEdit : public TEditor
{
public:
	HuiTTextEdit( size_t aBufSize, bool aAllowEdit ) : TEditor( TRect( 1, 1, 11, 11 ), new TScrollBar( TRect( 6, 11, 11, 12 ) ), new TScrollBar( TRect( 11, 1, 12, 11 ) ), 0, aBufSize ),
			mAllowEdit( aAllowEdit )
		{
		}

	virtual void erase()
		{
			deleteRange( 0, bufLen, Boolean( false ) );
		}

	virtual string text() const
		{
			return string( buffer, bufLen );
		}

#define cpHuiTTextEdit "\x1A\x19"
	virtual TPalette& getPalette() const
		{
			static TPalette palette( cpHuiTTextEdit, sizeof( cpHuiTTextEdit ) - 1 );
			return(palette);
		}

	virtual void changeBounds( const TRect& bounds )
		{
			TEditor::changeBounds( TRect( bounds.a.x, bounds.a.y, bounds.b.x - 1, bounds.b.y - 1 ) );

			if ( hScrollBar ) {
				hScrollBar->setBounds( TRect( bounds.a.x, bounds.b.y - 1, bounds.b.x, bounds.b.y ) );
				hScrollBar->setStep( size.x / 2, hScrollBar->arStep );
			}
			if ( vScrollBar ) {
				vScrollBar->setBounds( TRect( bounds.b.x - 1, bounds.a.y, bounds.b.x, bounds.b.y - 1 ) );
				vScrollBar->setStep( size.y, vScrollBar->arStep );
			}
		}

	virtual void text( const string& aText )
		{
			erase();
			insertBuffer( const_cast<char*>( aText.c_str() ), 0, aText.length(), Boolean( false ), Boolean( false ) );
			setCurPtr( 0, 0 );
			drawView();
		}

	virtual void handleEvent( TEvent& event )
		{
			convertEvent( event );
			if ( mAllowEdit || event.what != evKeyDown )
				TEditor::handleEvent( event );
			else
				TView::handleEvent( event );
		}

private:
	bool mAllowEdit;

}; // class HuiTTextEdit

//======================================================================

HuiTVision::TextEditWidget::TextEditWidget(Hui& aHui,
					   Pointer<Hui::Widget> aParent,
					   const string& aName,
					   const string& aInitialText,
					   bool aAllowEdit,
					   const string& aTitle,
					   const string& aComment)
	: Hui::Widget(aHui, aName),
	  Hui::EmbeddedWidget(aHui, aName),
	  TitledWidget(aHui, aParent, aName, aTitle),
	  Hui::TextEditWidget(aHui, aName),
	  mTextEdit(new HuiTTextEdit(1024 * 100, aAllowEdit) ), mComment(aComment)
{
	HuiTVision::GroupWidget *gp = dynamic_cast<HuiTVision::GroupWidget *> ((Hui::Widget *) aParent);

	parent(aParent);

	if (ttitle())
		ttitle()->link(mTextEdit);

	gp->tgroup()->insert(mTextEdit->hScrollBar);
	gp->tgroup()->insert(mTextEdit->vScrollBar);

	text(aInitialText);
	mTextEdit->setCurPtr( 0, Boolean( false ) );
	resize(minSize());

} // HuiTVision::TextEditWidget::TextEditWidget

//======================================================================

HuiTVision::TextEditWidget::~TextEditWidget()
{
	delete mTextEdit;

} // HuiTVision::TextEditWidget::~TextEditWidget

//======================================================================

void HuiTVision::TextEditWidget::text( const string& aText )
{
	if ( mTextEdit )
		mTextEdit->text( aText );

} // HuiTVision::TextEditWidget::text

//======================================================================

string HuiTVision::TextEditWidget::text() const
{
	if ( mTextEdit )
		return mTextEdit->text();
	else
		return string();

} // HuiTVision::TextEditWidget::text

//======================================================================

TView* HuiTVision::TextEditWidget::tview()
{
	return(mTextEdit);

} // HuiTVision::TextEditWidget::tview

//======================================================================

const TView* HuiTVision::TextEditWidget::tview() const
{
	return(mTextEdit);

} // HuiTVision::TextEditWidget::tview

//======================================================================
