/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiTVision_TitledWidget.cc,v 1.30 2001/06/20 10:24:51 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiTVision.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiTVision_hh
#include "HuiTVision.hh"
#endif

#define Uses_TDialog
#define Uses_TDeskTop
#define Uses_TLabel
#define Uses_TButton
#define Uses_TInputLine
#define Uses_TRangeValidator
#define Uses_TPXPictureValidator
#define Uses_MsgBox
#define Uses_TSItem
#define Uses_TRadioButtons
#define Uses_TCheckBoxes
#define Uses_TListBox
#define Uses_TScrollBar
#define Uses_TCollection
#define Uses_TStringCollection
#define Uses_TFileDialog
#define Uses_TMenuItem
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuBar
#define Uses_TKeys
#define Uses_TScroller
#define Uses_TEditor
#define Uses_TIndicator

#include <tvision/tv.h>

#ifndef String_hh
#include "String.hh"
#endif

#include <cmath>

#define MAX(a,b) ((a) > (b) ? (a) : (b))

//======================================================================

HuiTVision::TitledWidget::TitledWidget(Hui& aHui,
				       Pointer<Hui::Widget> aParent,
				       const string& aName,
				       const string& aTitle)
	: Hui::Widget::Widget(aHui, aName),
	  Widget(aHui, aName),
	  mTitle(aTitle.empty() ? NULL : 
		 new HuiTLabel(translateAccel(aTitle).c_str()))
{
	HuiTVision::GroupWidget *gp;
	HuiTVision::ContainerWidget *cp; /* XXX still need this? */
	
	if (mTitle) {
		if ((gp = dynamic_cast<HuiTVision::GroupWidget*>((Hui::Widget*) aParent)) != NULL)
			gp->tgroup()->insert(mTitle);
		else if ((cp = dynamic_cast<HuiTVision::ContainerWidget*>((Hui::Widget*) aParent)) != NULL)
			(const_cast<TGroup*> (cp->tgroup()))->insert(mTitle);

	}

} // HuiTVision::TitledWidget::TitledWidget

//======================================================================

HuiTVision::TitledWidget::~TitledWidget()
{
	delete mTitle;

} // HuiTVision::TitledWidget::~TitledWidget

//======================================================================

void HuiTVision::TitledWidget::title( const string& aTitle )
{
	if ( mTitle )
		mTitle->text( aTitle );

} // HuiTVision::TitledWidget::title

//======================================================================

string HuiTVision::TitledWidget::title() const
{
	if ( mTitle )
		return mTitle->text();
	else
		return string();

} // HuiTVision::TitledWidget::title

//======================================================================

void HuiTVision::TitledWidget::move( const TPoint& to )
{
	TPoint tow = to;
	if ( ttitle() ) {
		const TPoint offset = owner_frame_offset( ttitle()->owner );
		ttitle()->moveTo( offset.x + to.x, offset.y + to.y );
		tow.y += 1;
	}
	Widget::move( tow );

} // HuiTVision::TitledWidget::move

//======================================================================

void HuiTVision::TitledWidget::resize(const TPoint& to, int is_arranging = 0)
{
	TPoint tow = to;
	if (ttitle())
		tow.y -= 1;
	Widget::resize(tow, is_arranging);

} // HuiTVision::TitledWidget::resize

//======================================================================

Hui::Position HuiTVision::TitledWidget::position() const
{
	Hui::Position pos = Widget::position();
	if ( pos.y() > 0 && ttitle() )
		pos.y( pos.y() - 1 );
	return(pos);

} // HuiTVision::TitledWidget::position

//======================================================================

Hui::Size HuiTVision::TitledWidget::size() const
{
	Hui::Size sz = Widget::size();
	if ( ttitle() )
		sz.height( sz.height() + 1 );
	return sz;

} // HuiTVision::TitledWidget::size

//======================================================================
