/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiTVision_ToplevelWidget.cc,v 1.30 2001/06/20 10:24:52 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiTVision.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiTVision_hh
#include "HuiTVision.hh"
#endif

#define Uses_TDialog
#define Uses_TDeskTop
#define Uses_TLabel
#define Uses_TButton
#define Uses_TInputLine
#define Uses_TRangeValidator
#define Uses_TPXPictureValidator
#define Uses_MsgBox
#define Uses_TSItem
#define Uses_TRadioButtons
#define Uses_TCheckBoxes
#define Uses_TListBox
#define Uses_TScrollBar
#define Uses_TCollection
#define Uses_TStringCollection
#define Uses_TFileDialog
#define Uses_TMenuItem
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuBar
#define Uses_TKeys
#define Uses_TScroller
#define Uses_TEditor
#define Uses_TIndicator

#include <tvision/tv.h>

#ifndef String_hh
#include "String.hh"
#endif

#include <cmath>

#define MAX(a,b) ((a) > (b) ? (a) : (b))

//======================================================================

/* Toplevel Widget contructor */
HuiTVision::ToplevelWidget::ToplevelWidget(HuiTVision& aHui, const string& aName, const Position& aPosition, const Size& aSize )
	: Hui::Widget( aHui, aName ), Hui::GroupWidget( aHui, aName ), Hui::ToplevelWidget( aHui, aName ), GroupWidget( aHui, aName, aPosition, aSize )
{
} // HuiTVision::ToplevelWidget::ToplevelWidget

//======================================================================

HuiTVision::ToplevelWidget::~ToplevelWidget()
{
} // HuiTVision::ToplevelWidget::~ToplevelWidget

//======================================================================

/* The Toplevel-Widget is the TVision window */
TGroup* HuiTVision::ToplevelWidget::tgroup() { return twindow(); }
const TGroup* HuiTVision::ToplevelWidget::tgroup() const { return twindow(); }

//======================================================================

/* Set the title of the toplevel widget, which is the window-title */
void HuiTVision::ToplevelWidget::title( const string& aTitle )
{
	if ( twindow() ) {
		delete twindow()->title;
		twindow()->title = newStr( aTitle.c_str() );
	}

} // HuiTVision::ToplevelWidget::title

//======================================================================

/* Get the toplevel widget title, which is the window title */
string HuiTVision::ToplevelWidget::title() const
{
	if ( twindow() )
		/* If there is a window-title, return that one */
		return(twindow()->title);
	else
		/* XXX otherwise, returns something - WHAT is this? */
		return string();

} // HuiTVision::ToplevelWidget::title

//======================================================================

/* XXX strange function, see Qt: Is empty, too, but has commented out stuff */
void HuiTVision::ToplevelWidget::resized()
{
	arrange();

} // HuiTVision::ToplevelWidget::resized

//======================================================================
