/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiTVision_Widget.cc,v 1.30 2001/06/20 10:24:52 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiTVision.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiTVision_hh
#include "HuiTVision.hh"
#endif

#define Uses_TDialog
#define Uses_TDeskTop
#define Uses_TLabel
#define Uses_TButton
#define Uses_TInputLine
#define Uses_TRangeValidator
#define Uses_TPXPictureValidator
#define Uses_MsgBox
#define Uses_TSItem
#define Uses_TRadioButtons
#define Uses_TCheckBoxes
#define Uses_TListBox
#define Uses_TScrollBar
#define Uses_TCollection
#define Uses_TStringCollection
#define Uses_TFileDialog
#define Uses_TMenuItem
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuBar
#define Uses_TKeys
#define Uses_TScroller
#define Uses_TEditor
#define Uses_TIndicator

#include <tvision/tv.h>

#ifndef String_hh
#include "String.hh"
#endif

#include <cmath>

#define MAX(a,b) ((a) > (b) ? (a) : (b))

//======================================================================

HuiTVision::Widget::~Widget()
{
} // HuiTVision::Widget::~Widget

//======================================================================

/* Show widget */
void HuiTVision::Widget::show()
{
	/* If TVision object, call its show() function */
	if ( tview() )
		tview()->show();

} // HuiTVision::Widget::show

//======================================================================

/* Hide widget */
void HuiTVision::Widget::hide()
{
	/* If TVision object exists, call its hide() function */
	if ( tview() )
		tview()->hide();

} // HuiTVision::Widget::hide

//======================================================================

/* Check if a widget is hidden */
bool HuiTVision::Widget::hidden() const
{
	if ( tview() )
		/* If TVision object, call the implemented function */
		return ! ( tview()->state & sfVisible );
	else
		/* otherwise, default to true (where there is no TVision
		   object, there is nothing seen */
		return(true);

} // HuiTVision::Widget::hidden

//======================================================================

/* Disable a widget */
void HuiTVision::Widget::disable()
{
	if ( tview() )
		/* If TVision object, set disabled-flag */
		tview()->setState( sfDisabled, Boolean( true ) );

} // HuiTVision::Widget::disable

//======================================================================

/* Endable a widget */
void HuiTVision::Widget::enable()
{
	if ( tview() )
		/* If TVision object, clear disabled bit */
		tview()->setState( sfDisabled, Boolean( false ) );

} // HuiTVision::Widget::enable

//======================================================================

/* Check if a Widget is disabled */
bool HuiTVision::Widget::disabled() const
{
	if ( tview() )
		/* Call the TVision function */
		/* XXX can we call getState here easier? tried it, got error.
probably my fault */
		return(tview()->state & sfDisabled);
	else
		/* Otherwise, if no TVision object exists, default to
		   disabled */
		return(true);

} // HuiTVision::Widget::disabled

//======================================================================

/* Move a widget */
void HuiTVision::Widget::move( const Position& aPosition )
{
  /* Translate position, call correct function */
  move( translate( aPosition ) );

} // HuiTVision::Widget::move

//======================================================================

/* Get position of Widget */
Hui::Position HuiTVision::Widget::position() const
{

	return(translate(mPosition));

} // HuiTVision::Widget::position

//======================================================================

/* Resize a widget */
void HuiTVision::Widget::resize(const Hui::Size& aSize)
{
	/* translate the size and call the correct function */
	resize(translate(aSize));

} // HuiTVision::Widget::resize

//======================================================================

/* Get size of a Widget */
Hui::Size HuiTVision::Widget::size() const
{
	if ( tview() )
		/* If TVision object, translate TVision size */
		return translate( tview()->size, 0 );
	else
		/* Otherwise, just return size */
		return Hui::Size();

} // HuiTVision::Widget::size

//======================================================================

/* Move Widget */
void HuiTVision::Widget::move( const TPoint& to )
{
	ContainerWidget *cp;

	mPosition = to;

	/* If the parent is a ContainerWidget, add the offset of the parent */
	if ((cp = dynamic_cast<ContainerWidget*>( (Hui::Widget*) parent() )) != NULL) {
		
		TPoint offset = translate(cp->position());
		TPoint newpos;

		newpos.x = offset.x + to.x;
		newpos.y = offset.y + to.y;

		if ( tview() ) {
			/* Call TVision function */
			const TPoint offset = owner_frame_offset( tview()->owner );
			/* Add frame size to new position! */
			tview()->moveTo( offset.x + newpos.x, offset.y + newpos.y );
		}
	} else {
		if ( tview() ) {
			/* Call TVision function */
			const TPoint offset = owner_frame_offset( tview()->owner );
			/* Add frame size to new position! */
			tview()->moveTo( offset.x + to.x, offset.y + to.y );
		}
	}

} // HuiTVision::Widget::move

//======================================================================

/* Resize Widget */
void HuiTVision::Widget::resize(const TPoint& to, int is_arranging = 0)
{
	if (to.x == -1 && to.y == -1)
		return;
	if (to.x >= 0 && to.y >= 0) {
		if (tview())
			tview()->growTo(to.x, to.y);
		return;
	}
	if ((to.x >= 0 || to.y >= 0) && !is_arranging) {
		GroupWidget *gp = dynamic_cast<GroupWidget*>( (Hui::Widget*) parent());
		
		setFixedSize((to.x < 0) ? to.y : to.x);
		/* arrange parent again since we changed the fixed size */
		if (gp != NULL)
			gp->arrange();
	}
	
} // HuiTVision::Widget::resize

//======================================================================
