/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/hui/HuiTVision_WindowWidget.cc,v 1.30 2001/06/20 10:24:52 alex Exp $
 */

// -*- C++ -*-   Wed Dec  2 1998
// Eugene Skepner <eu@qub.com>
// $Id: HuiTVision.cc,v 1.1.1.1 2000/02/15 16:30:35 alex Exp $

#ifndef HuiTVision_hh
#include "HuiTVision.hh"
#endif

#define Uses_TDialog
#define Uses_TDeskTop
#define Uses_TLabel
#define Uses_TButton
#define Uses_TInputLine
#define Uses_TRangeValidator
#define Uses_TPXPictureValidator
#define Uses_MsgBox
#define Uses_TSItem
#define Uses_TRadioButtons
#define Uses_TCheckBoxes
#define Uses_TListBox
#define Uses_TScrollBar
#define Uses_TCollection
#define Uses_TStringCollection
#define Uses_TFileDialog
#define Uses_TMenuItem
#define Uses_TMenu
#define Uses_TSubMenu
#define Uses_TMenuBar
#define Uses_TKeys
#define Uses_TScroller
#define Uses_TEditor
#define Uses_TIndicator

#include <tvision/tv.h>

#ifndef String_hh
#include "String.hh"
#endif

#include <cmath>

#define MAX(a,b) ((a) > (b) ? (a) : (b))

//======================================================================

class HuiTWindow : public TWindow
{
public:
	HuiTWindow( HuiTVision& aHui, HuiTVision::WindowWidget& aWindowWidget, const TRect& bounds, const char *aTitle )
		: TWindow( bounds, aTitle, wnNoNumber ), TWindowInit( &TWindow::initFrame ),
		mHui( aHui ), mWindowWidget( aWindowWidget ), mCurrent( 0 )
		{
			//flags = wfClose;
			setState( sfShadow, False );
		}

	virtual void handleEvent( TEvent& event )
		{
			if ( event.what== evCommand && event.message.command == cmClose ) {
		clearEvent( event );
				mWindowWidget.close();
				return;
			}

			const bool resize_event = event.what== evCommand && ( event.message.command == cmResize || event.message.command == cmZoom );

			TWindow::handleEvent( event );

			if ( mCurrent != current ) {
				mWindowWidget.aboutToChangeCurrent( mCurrent, current );
				mCurrent = current;
			}
			if ( resize_event )
				mWindowWidget.resized();
		}

	virtual TPalette& getPalette() const
		{
			static TPalette paletteCyan( cpCyanDialog, sizeof( cpCyanDialog )-1 );
			return(paletteCyan);
		}

private:
	HuiTVision& mHui;
	HuiTVision::WindowWidget& mWindowWidget;
	TView* mCurrent;
};

//======================================================================

HuiTVision::WindowWidget::WindowWidget(HuiTVision& aHui,
				       const string& aName,
				       const string& aTitle,
				       const Position& aPosition,
				       const Size& aSize)
  : Hui::Widget(aHui, aName),
    Hui::GroupWidget(aHui, aName ),
    HuiTVision::Widget(aHui, aName),
    HuiTVision::ToplevelWidget(aHui, aName, aPosition, aSize),
    Hui::ToplevelWidget( aHui, aName ),
    Hui::WindowWidget( aHui, aName ),
    mTWindow( new HuiTWindow(aHui, *this, translate(aPosition, aSize.width()),
			     aTitle.c_str()))
{
  move( aPosition );
  resize( aSize );
  
} // HuiTVision::WindowWidget::WindowWidget

//======================================================================

HuiTVision::WindowWidget::~WindowWidget()
{
	aboutToChangeCurrent( twindow()->current, 0 );
	delete mTWindow;

} // HuiTVision::WindowWidget::~WindowWidget

//======================================================================

TView* HuiTVision::WindowWidget::tview() { return tgroup(); }
const TView* HuiTVision::WindowWidget::tview() const { return tgroup(); }

//======================================================================

void HuiTVision::WindowWidget::close()
{
	hide();
	if ( hui().subwidgets().size() == 0 )
		hui().exit( 0 );

} // HuiTVision::WindowWidget::close

//======================================================================

void HuiTVision::WindowWidget::show()
{
	ContainerWidget *cp;

	hui().subwidgets().add( this );
	hui().deskTop->insert( tview() );
	HuiTVision::Widget::show();

	Subwidgets::iterator sw;

	for (sw = subwidgets().begin();
	     sw != subwidgets().end();
	     sw++) {
		if ((cp = dynamic_cast<ContainerWidget *>((Hui::Widget *) (*sw))) != NULL)
			cp->show();
	}

} // HuiTVision::WindowWidget::show

//======================================================================

void HuiTVision::WindowWidget::hide()
{
	hui().subwidgets().remove( this );
	HuiTVision::Widget::hide();
	hui().deskTop->remove( tview() );

} // HuiTVision::WindowWidget::hide

//======================================================================

TWindow* HuiTVision::WindowWidget::twindow()
{
	return(mTWindow);

} // HuiTVision::WindowWidget::twindow

//======================================================================

const TWindow* HuiTVision::WindowWidget::twindow() const
{
	return(mTWindow);

} // HuiTVision::WindowWidget::twindow

//======================================================================
