/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/Callback.cc,v 1.2 2001/06/22 15:05:14 alex Exp $
 */

// -*- C++ -*-   Thu Aug 20 1998
// $Id: Callback.cc,v 1.7 1999/01/14 21:48:34 eugene Exp $

#ifndef Callback_hh
#include "Callback.hh"
#endif

#ifndef FeaturesProvided_hh
#include "FeaturesProvided.hh"
#endif

#ifndef Configuration_hh
#include "Configuration.hh"
#endif

#ifndef Log_hh
#include "Log.hh"
#endif

#ifndef PackageFile_hh
#include "PackageFile.hh"
#endif

#ifndef DirectoriesAccess_hh
#include "DirectoriesAccess.hh"
#endif

#ifndef String_hh
#include "String.hh"
#endif

//======================================================================

string Callback::wrapper_package_requirements_not_met( Pointer<Database> database, Pointer<PackageStream> packageStream, const FeatureList& requirements, const FeatureList& conflicts, const list<string>& users, const list<string>& groups, const FeatureList& suggestions )
{
     string result;
     try {
          if ( requirements.size() > 0 && FeaturesProvided::instance()->find_and_install_features( database, requirements, packageStream->progress_indicator_stack() ) == 0 ) {
               for ( FeatureList::const_iterator i = requirements.begin(); i != requirements.end(); ++i ) {
                    if ( i == requirements.begin() )
                         result += " Features required but not provided by the system:\n";
                    result += "\t";
                    result += (*i)->name() + " " + (*i)->version() + " (serial: " + to_string( (*i)->serial() ) + ", snapshot: " + to_string( (*i)->snapshot() ) +  ")\n";
               }
          }
     }
     catch ( runtime_error& err ) {
          result += err.what();
          result += "\n";
     }

     for ( FeatureList::const_iterator i = conflicts.begin(); i != conflicts.end(); ++i ) {
          if ( i == conflicts.begin() )
               result += " Conflicting features provided by the system:\n";
          result += "\t";
          result += (*i)->name() + " " + (*i)->version() + " (serial: " + to_string( (*i)->serial() ) + ", snapshot: " + to_string( (*i)->snapshot() ) +  ")\n";
     }

     for ( list<string>::const_iterator i = users.begin(); i != users.end(); ++i ) {
          if ( i == users.begin() )
               result += " Users not present in the system:\n";
          result += "\t";
          result += *i + "\n";
     }

     for ( list<string>::const_iterator i = groups.begin(); i != groups.end(); ++i ) {
          if ( i == groups.begin() )
               result += " Groups not present in the system:\n";
          result += "\t";
          result += *i + "\n";
     }

     if ( result.length() > 0 ) {
          for ( FeatureList::const_iterator i = suggestions.begin(); i != suggestions.end(); ++i ) {
               if ( i == suggestions.begin() )
                    result += " Suggested features not provided by the system:\n";
               result += "\t";
               result += (*i)->name() + " " + (*i)->version() + " (serial: " + to_string( (*i)->serial() ) + ", snapshot: " + to_string( (*i)->snapshot() ) +  ")\n";
          }
     }

     if ( Configuration::instance()->no_action() && result.length() == 0 )
          result = "ignore";

     return result;

} // Callback::wrapper_package_requirements_not_met

//======================================================================

string Callback::wrapper_installation_over_installed( Pointer<Database> database, Pointer<PackageStream> packageStream, const FeatureList& duplicates )
{
       // Collect packages providing duplicated features
     PackageList old_packages;
     for ( FeatureList::const_iterator f = duplicates.begin(); f != duplicates.end(); ++f ) {
          Pointer<Package> package = database->package( *f );
          if ( package ) {
               bool exists = false;
               for ( PackageList::iterator p = old_packages.begin(); p != old_packages.end() && !exists; ++p )
                    exists = **p == *package;
               if ( !exists )
                    old_packages.push_back( package );
          }
     }

     bool installation_over_installed = false;
     if ( old_packages.size() == 1 ) {
            // Check if the only package providing duplicated features
            // provides exactly the same set of features as package being installed
          Pointer<FeatureList> installed_features = (*old_packages.begin())->features();
          Pointer<FeatureList> new_features = packageStream->features();
          installation_over_installed = installed_features->size() == new_features->size();
          for ( FeatureList::const_iterator f = new_features->begin(); installation_over_installed && f != new_features->end(); ++f ) {
               installation_over_installed = false;
               for ( FeatureList::const_iterator fi = installed_features->begin(); fi != installed_features->end() && !installation_over_installed; ++fi )
                    installation_over_installed = **fi == **f;
          }
     }

     if ( installation_over_installed ) {
            // Uninstall previous installation of this package
          (*old_packages.begin())->uninstall();
          return string();
     }
     else {
          string packages_names;
          for ( PackageList::const_iterator p = old_packages.begin(); p != old_packages.end(); ++p ) {
               if ( packages_names.length() > 0 )
                    packages_names += ", ";
               packages_names += (*p)->name();
          }
          return string( "Package(s) " ) + packages_names + " already provide(s) features provided by package " + packageStream->name() + " being installed";
     }

} // Callback::wrapper_installation_over_installed

//======================================================================

string Callback::wrapper_file_to_be_installed_exists_and_owned_by_other_packages( Pointer<Database> /*database*/, Pointer<PackageStream> packageStream, Pointer<PackageStreamFile> file, PackageList& owners )
{
     string message;
     Pointer<FeatureList> this_provides = packageStream->features();
     for ( PackageList::const_iterator owner = owners.begin(); owner != owners.end(); ++owner ) {
          Pointer<FeatureList> owner_provides = (*owner)->features();
          bool common_features = false;
          for ( FeatureList::const_iterator tf = this_provides->begin(); tf != this_provides->end() && !common_features; ++tf ) {
               for ( FeatureList::const_iterator of = owner_provides->begin(); of != owner_provides->end() && !common_features; ++of ) {
                    common_features |= (*tf)->name() == (*of)->name();
               }
          }
          if ( !common_features )
               message += string( " " ) + (*owner)->name();
     }
     if ( message.length() > 0 )
          message = string( "File " ) + file->pathname() + " is owned by" + message;
     return message;

} // Callback::wrapper_file_to_be_installed_exists_and_owned_by_other_packages

//======================================================================

string Callback::wrapper_reverse_conflict( Pointer<Database> /*database*/, Pointer<PackageStream> /*packageStream*/, const PackageList& /*conflicts*/ )
{
     return "reverse_conflict: abort";

} // Callback::wrapper_reverse_conflict

//======================================================================

Pointer<PackageStreamSubpackageList> Callback::wrapper_install_subpackages( Pointer<PackageStreamStructure> structure )
{
     return structure->subpackages();

} // Callback::wrapper_install_subpackages

//======================================================================

Pointer<Container<list<string> > > Callback::wrapper_directory_write_access( Pointer<Database> /*database*/, Pointer<PackageStream> /*packageStream*/, const string& dirpathname )
{
     if ( DirectoriesAccess::instance()->accessAllowed( dirpathname ) == false ) {
          Pointer<Container<list<string> > > result = new Container<list<string> >;
          result->push_back( "abort" );
          result->push_back( string( "Write access to " ) + dirpathname + " forbidden by the configuration" );
          return result;
     }
     else {
          return new Container<list<string> >;
     }

} // Callback::wrapper_directory_write_access

//======================================================================

Pointer<Container<list<string> > > Callback::wrapper_file_overwrite( Pointer<Database> /*database*/, Pointer<PackageStream> /*packageStream*/, const string& /*filepathname*/ )
{
     return new Container<list<string> >;

} // Callback::wrapper_file_overwrite

//======================================================================

Pointer<Container<list<string> > > Callback::wrapper_file_append( Pointer<Database> /*database*/, Pointer<PackageStream> /*packageStream*/, const string& /*filepathname*/ )
{
     return new Container<list<string> >;

} // Callback::wrapper_file_append

//======================================================================

Pointer<Container<list<string> > > Callback::wrapper_file_patch( Pointer<Database> /*database*/, Pointer<PackageStream> /*packageStream*/, const string& /*filepathname*/ )
{
     return new Container<list<string> >;

} // Callback::wrapper_file_patch

//======================================================================

string Callback::wrapper_owned_file_does_not_exist_for_uninstall( Pointer<Package> /*package*/, Pointer<PackageFile> /*file*/ )
{
       // ignore the problem
     return string();

} // Callback::wrapper_owned_file_does_not_exist_for_uninstall

//======================================================================

string Callback::wrapper_install_upgrade( Pointer<Database> /*database*/, const string& aUrl )
{
     LOG( INFO, "wrapper_Install_upgrade " << aUrl << " providing necessary features" );
     return "install";

} // Callback::wrapper_install_upgrade

//======================================================================

string Callback::wrapper_uninstall_required_features( Pointer<Database> /*database*/, Pointer<Package> /*toUninstall*/, const PackageList& requirers )
{
     string result = "Packages";
     for ( PackageList::const_iterator package = requirers.begin(); package != requirers.end(); ++package ) {
          result += ' ';
          result += (*package)->name();
     }
     result += " require the features it provides";
     return result;

} // Callback::wrapper_uninstall_required_features

//======================================================================

string Callback::wrapper_exec_wanted( const string& aCommand )
{
     return aCommand;

} // Callback::wrapper_exec_wanted

//======================================================================

string Callback::package_requirements_not_met( Pointer<Database> database, Pointer<PackageStream> packageStream, const FeatureList& requirements, const FeatureList& conflicts, const list<string>& users, const list<string>& groups, const FeatureList& suggestions )
{
     Call_package_requirements_not_met call( database, packageStream, requirements, conflicts, users, groups, suggestions, new TclLanguageInterface::CallbackData( tclLanguageInterface, "Callback::package_requirements_not_met" ) );
     call.activate();
     return call.result();

} // Callback::package_requirements_not_met

//======================================================================

string Callback::installation_over_installed( Pointer<Database> database, Pointer<PackageStream> packageStream, const FeatureList& duplicates )
{
     Call_installation_over_installed call( database, packageStream, duplicates, new TclLanguageInterface::CallbackData( tclLanguageInterface, "Callback::installation_over_installed" ) );
     call.activate();
     return call.result();

} // Callback::installation_over_installed

//======================================================================

string Callback::file_to_be_installed_exists_and_owned_by_other_packages( Pointer<Database> database, Pointer<PackageStream> packageStream, Pointer<PackageStreamFile> file, PackageList& owners )
{
     Call_file_to_be_installed_exists_and_owned_by_other_packages call( database, packageStream, file, owners, new TclLanguageInterface::CallbackData( tclLanguageInterface, "Callback::file_to_be_installed_exists_and_owned_by_other_packages" ) );
     call.activate();
     return call.result();

} // Callback::file_to_be_installed_exists_and_owned_by_other_packages

//======================================================================

string Callback::reverse_conflict( Pointer<Database> database, Pointer<PackageStream> packageStream, const PackageList& conflicts )
{
     Call_reverse_conflict call( database, packageStream, conflicts, new TclLanguageInterface::CallbackData( tclLanguageInterface, "Callback::reverse_conflict" ) );
     call.activate();
     return call.result();

} // Callback::reverse_conflict

//======================================================================

Pointer<PackageStreamSubpackageList> Callback::install_subpackages( Pointer<PackageStreamStructure> structure )
{
     Call_install_subpackages call( structure, new TclLanguageInterface::CallbackData( tclLanguageInterface, "Callback::install_subpackages" ) );
     call.activate();
     return call.result();

} // Callback::install_subpackages

//======================================================================

Pointer<Container<list<string> > > Callback::directory_write_access( Pointer<Database> database, Pointer<PackageStream> packageStream, const string& dirpathname )
{
     Call_fileaccess call( database, packageStream, dirpathname, new TclLanguageInterface::CallbackData( tclLanguageInterface, "Callback::directory_write_access" ) );
     call.activate();
     return call.result();

} // Callback::directory_write_access

//======================================================================

Pointer<Container<list<string> > > Callback::file_overwrite( Pointer<Database> database, Pointer<PackageStream> packageStream, const string& filepathname )
{
     Call_fileaccess call( database, packageStream, filepathname, new TclLanguageInterface::CallbackData( tclLanguageInterface, "Callback::file_overwrite" ) );
     call.activate();
     return call.result();

} // Callback::file_overwrite

//======================================================================

Pointer<Container<list<string> > > Callback::file_append( Pointer<Database> database, Pointer<PackageStream> packageStream, const string& filepathname )
{
     Call_fileaccess call( database, packageStream, filepathname, new TclLanguageInterface::CallbackData( tclLanguageInterface, "Callback::file_append" ) );
     call.activate();
     return call.result();

} // Callback::file_append

//======================================================================

Pointer<Container<list<string> > > Callback::file_patch( Pointer<Database> database, Pointer<PackageStream> packageStream, const string& filepathname )
{
     Call_fileaccess call( database, packageStream, filepathname, new TclLanguageInterface::CallbackData( tclLanguageInterface, "Callback::file_patch" ) );
     call.activate();
     return call.result();

} // Callback::file_patch

//======================================================================

string Callback::owned_file_does_not_exist_for_uninstall( Pointer<Package> package, Pointer<PackageFile> file )
{
     Call_owned_file_does_not_exist_for_uninstall call( package, file, new TclLanguageInterface::CallbackData( tclLanguageInterface, "Callback::owned_file_does_not_exist_for_uninstall" ) );
     call.activate();
     return call.result();

} // Callback::owned_file_does_not_exist_for_uninstall

//======================================================================

string Callback::install_upgrade( Pointer<Database> database, const string& aUrl )
{
     Call_install_upgrade call( database, aUrl, new TclLanguageInterface::CallbackData( tclLanguageInterface, "Callback::install_upgrade" ) );
     call.activate();
     return call.result();

} // Callback::install_upgrade

//======================================================================

string Callback::uninstall_required_features( Pointer<Database> database, Pointer<Package> toUninstall, const PackageList& requirers )
{
     Call_uninstall_required_features call( database, toUninstall, requirers, new TclLanguageInterface::CallbackData( tclLanguageInterface, "Callback::install_upgrade" ) );
     call.activate();
     return call.result();

} // Callback::uninstall_required_features

//======================================================================

string Callback::exec_wanted( const string& aCommand )
{
     Call_exec_wanted call( aCommand, new TclLanguageInterface::CallbackData( tclLanguageInterface, "Callback::exec_wanted" ) );
     call.activate();
     return call.result();

} // Callback::exec_wanted

//======================================================================
