/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/Callback.cd.cc,v 1.2 2001/06/22 15:05:14 alex Exp $
 */

// -*- C++ -*-   Sat Jan  2 1999
// Eugene Skepner <eu@qub.com>
// $Id: Callback.cd.cc,v 1.3 1999/01/14 15:10:53 eugene Exp $

#ifndef Callback_hh
#include "Callback.hh"
#endif

//======================================================================

LanguageInterface::Object::MethodDescription Callback::sMethods[] =
{
     MethodDescription( "package_requirements_not_met", "wrapper_package_requirements_not_met",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "",
                        MethodDescription::vtString,
                        MethodDescription::Value( MethodDescription::vtObject, "Database", 0, "database" ), MethodDescription::Value( MethodDescription::vtObject, "PackageStream", 0, "package_stream" ), MethodDescription::Value( MethodDescription::vtListRef, "FeatureList", 0, "requirements" ), MethodDescription::Value( MethodDescription::vtListRef, "FeatureList", 0, "conflicts" ), MethodDescription::Value( MethodDescription::vtListRef, "list<string>", 0, "users" ), MethodDescription::Value( MethodDescription::vtListRef, "list<string>", 0, "groups" ), MethodDescription::Value( MethodDescription::vtListRef, "FeatureList", 0, "suggestions" ) ),
     MethodDescription( "installation_over_installed", "wrapper_installation_over_installed",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "",
                        MethodDescription::vtString,
                        MethodDescription::Value( MethodDescription::vtObject, "Database", 0, "database" ), MethodDescription::Value( MethodDescription::vtObject, "PackageStream", 0, "package_stream" ), MethodDescription::Value( MethodDescription::vtListRef, "FeatureList", 0, "duplicates" ) ),
     MethodDescription( "file_to_be_installed_exists_and_owned_by_other_packages", "wrapper_file_to_be_installed_exists_and_owned_by_other_packages",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "",
                        MethodDescription::vtString,
                        MethodDescription::Value( MethodDescription::vtObject, "Database", 0, "database" ), MethodDescription::Value( MethodDescription::vtObject, "PackageStream", 0, "package_stream" ), MethodDescription::Value( MethodDescription::vtObject, "PackageStreamFile", 0, "file" ), MethodDescription::Value( MethodDescription::vtListRef, "PackageList", 0, "owners" ) ),
     MethodDescription( "reverse_conflict", "wrapper_reverse_conflict",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "",
                        MethodDescription::vtString,
                        MethodDescription::Value( MethodDescription::vtObject, "Database", 0, "database" ), MethodDescription::Value( MethodDescription::vtObject, "PackageStream", 0, "package_stream" ), MethodDescription::Value( MethodDescription::vtListRef, "PackageList", 0, "conflicts" ) ),
     MethodDescription( "install_subpackages", "wrapper_install_subpackages",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "",
                        MethodDescription::Value( MethodDescription::vtListPtr, "PackageStreamSubpackageList" ),
                        MethodDescription::Value( MethodDescription::vtObject, "PackageStreamStructure", 0, "structure" ) ),
     MethodDescription( "directory_write_access", "wrapper_directory_write_access",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "",
                        MethodDescription::Value( MethodDescription::vtListPtr, "Container<list<string> >" ),
                        MethodDescription::Value( MethodDescription::vtObject, "Database", 0, "database" ), MethodDescription::Value( MethodDescription::vtObject, "PackageStream", 0, "package_stream" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "dirpathname" ) ),
     MethodDescription( "file_overwrite", "wrapper_file_overwrite",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "",
                        MethodDescription::Value( MethodDescription::vtListPtr, "Container<list<string> >" ),
                        MethodDescription::Value( MethodDescription::vtObject, "Database", 0, "database" ), MethodDescription::Value( MethodDescription::vtObject, "PackageStream", 0, "package_stream" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "filepathname" ) ),
     MethodDescription( "file_append", "wrapper_file_append",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "",
                        MethodDescription::Value( MethodDescription::vtListPtr, "Container<list<string> >" ),
                        MethodDescription::Value( MethodDescription::vtObject, "Database", 0, "database" ), MethodDescription::Value( MethodDescription::vtObject, "PackageStream", 0, "package_stream" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "filepathname" ) ),
     MethodDescription( "file_patch", "wrapper_file_patch",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "",
                        MethodDescription::Value( MethodDescription::vtListPtr, "Container<list<string> >" ),
                        MethodDescription::Value( MethodDescription::vtObject, "Database", 0, "database" ), MethodDescription::Value( MethodDescription::vtObject, "PackageStream", 0, "package_stream" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "filepathname" ) ),
     MethodDescription( "owned_file_does_not_exist_for_uninstall", "wrapper_owned_file_does_not_exist_for_uninstall",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "",
                        MethodDescription::vtString,
                        MethodDescription::Value( MethodDescription::vtObject, "Package", 0, "package" ), MethodDescription::Value( MethodDescription::vtObject, "PackageFile", 0, "file" ) ),
     MethodDescription( "install_upgrade", "wrapper_install_upgrade",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "",
                        MethodDescription::vtString,
                        MethodDescription::Value( MethodDescription::vtObject, "Database", 0, "" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "url" ) ),
     MethodDescription( "uninstall_required_features", "wrapper_uninstall_required_features",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "",
                        MethodDescription::vtString,
                        MethodDescription::Value( MethodDescription::vtObject, "Database", 0, "database" ), MethodDescription::Value( MethodDescription::vtObject, "Package", 0, "toUninstall" ), MethodDescription::Value( MethodDescription::vtListRef, "PackageList", 0, "requirers" ) ),
     MethodDescription( "exec_wanted", "wrapper_exec_wanted",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "",
                        MethodDescription::vtString,
                        MethodDescription::Value( MethodDescription::vtString, 0, 0, "command" ) ),
     MethodDescription()
};

LanguageInterface::Object::CallbackDescription Callback::sCallbacks[] =
{
     CallbackDescription( "Callback::Call_package_requirements_not_met", "database()", "packageStream()", "requirements()", "conflicts()", "users()", "groups()", "suggestions()" ),
     CallbackDescription( "Callback::Call_installation_over_installed", "database()", "packageStream()", "duplicates()" ),
     CallbackDescription( "Callback::Call_file_to_be_installed_exists_and_owned_by_other_packages", "database()", "packageStream()", "file()", "owners()" ),
     CallbackDescription( "Callback::Call_reverse_conflict", "database()", "packageStream()", "conflicts()" ),
     CallbackDescription( "Callback::Call_install_subpackages", "structure()" ),
     CallbackDescription( "Callback::Call_fileaccess", "database()", "packageStream()", "path()" ),
     CallbackDescription( "Callback::Call_owned_file_does_not_exist_for_uninstall", "package()", "file()" ),
     CallbackDescription( "Callback::Call_install_upgrade", "database()", "url()" ),
     CallbackDescription( "Callback::Call_exec_wanted", "command()" ),
     CallbackDescription( "Callback::Call_uninstall_required_features", "database()", "toUninstall()", "requirers()" ),
     CallbackDescription()
};

LanguageInterface::Object::ClassDescription Callback::sClassDescription(
     "Callback", "Callback",
     "Callback interface",
     sMethods,
     sCallbacks
);

//======================================================================
