/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/Callback.hh,v 1.2 2001/06/22 15:05:15 alex Exp $
 */

// -*- C++ -*-  Thu Aug 20 1998
// $Id: Callback.hh,v 1.3 1999/01/14 15:10:53 eugene Exp $

#ifndef Callback_hh
#define Callback_hh

#ifndef Feature_hh
#include "Feature.hh"
#endif

#ifndef PackageStreamSubpackage_hh
#include "PackageStreamSubpackage.hh"
#endif

#ifndef Package_hh
#include "Package.hh"
#endif

#ifndef PackageStream_hh
#include "PackageStream.hh"
#endif

#ifndef Database_hh
#include "Database.hh"
#endif

#ifndef PackageStreamStructure_hh
#include "PackageStreamStructure.hh"
#endif

#ifndef TclLanguageInterface_hh
#include "TclLanguageInterface.hh"
#endif

//======================================================================

class Callback : virtual public LanguageInterface::Object
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     static string package_requirements_not_met( Pointer<Database> database, Pointer<PackageStream> packageStream, const FeatureList& requirements, const FeatureList& conflicts, const list<string>& users, const list<string>& groups, const FeatureList& suggestions );
     static string installation_over_installed( Pointer<Database> database, Pointer<PackageStream> packageStream, const FeatureList& duplicates );
     static string file_to_be_installed_exists_and_owned_by_other_packages( Pointer<Database> database, Pointer<PackageStream> packageStream, Pointer<PackageStreamFile> file, PackageList& owners );
     static string reverse_conflict( Pointer<Database> database, Pointer<PackageStream> packageStream, const PackageList& conflicts );
     static Pointer<PackageStreamSubpackageList> install_subpackages( Pointer<PackageStreamStructure> structure );
     static Pointer<Container<list<string> > > directory_write_access( Pointer<Database> database, Pointer<PackageStream> packageStream, const string& dirpathname );
     static Pointer<Container<list<string> > > file_overwrite( Pointer<Database> database, Pointer<PackageStream> packageStream, const string& filepathname );
     static Pointer<Container<list<string> > > file_append( Pointer<Database> database, Pointer<PackageStream> packageStream, const string& filepathname );
     static Pointer<Container<list<string> > > file_patch( Pointer<Database> database, Pointer<PackageStream> packageStream, const string& filepathname );
     static string owned_file_does_not_exist_for_uninstall( Pointer<Package> package, Pointer<PackageFile> file );
     static string install_upgrade( Pointer<Database> database, const string& aUrl );
     static string uninstall_required_features( Pointer<Database> database, Pointer<Package> toUninstall, const PackageList& requirers );
     static string exec_wanted( const string& aCommand );

       // --------------------------------------------------

     static string wrapper_package_requirements_not_met( Pointer<Database> database, Pointer<PackageStream> packageStream, const FeatureList& requirements, const FeatureList& conflicts, const list<string>& users, const list<string>& groups, const FeatureList& suggestions );
     static string wrapper_installation_over_installed( Pointer<Database> database, Pointer<PackageStream> packageStream, const FeatureList& duplicates );
     static string wrapper_file_to_be_installed_exists_and_owned_by_other_packages( Pointer<Database> database, Pointer<PackageStream> packageStream, Pointer<PackageStreamFile> file, PackageList& owners );
     static string wrapper_reverse_conflict( Pointer<Database> database, Pointer<PackageStream> packageStream, const PackageList& conflicts );
     static Pointer<PackageStreamSubpackageList> wrapper_install_subpackages( Pointer<PackageStreamStructure> structure );
     static Pointer<Container<list<string> > > wrapper_directory_write_access( Pointer<Database> database, Pointer<PackageStream> packageStream, const string& dirpathname );
     static Pointer<Container<list<string> > > wrapper_file_overwrite( Pointer<Database> database, Pointer<PackageStream> packageStream, const string& filepathname );
     static Pointer<Container<list<string> > > wrapper_file_append( Pointer<Database> database, Pointer<PackageStream> packageStream, const string& filepathname );
     static Pointer<Container<list<string> > > wrapper_file_patch( Pointer<Database> database, Pointer<PackageStream> packageStream, const string& filepathname );
     static string wrapper_owned_file_does_not_exist_for_uninstall( Pointer<Package> package, Pointer<PackageFile> file );
     static string wrapper_install_upgrade( Pointer<Database> database, const string& aUrl );
     static string wrapper_uninstall_required_features( Pointer<Database> database, Pointer<Package> toUninstall, const PackageList& requirers );
     static string wrapper_exec_wanted( const string& aCommand );

       // --------------------------------------------------

     class Call : public LanguageInterface::SpecificCallbackData
     {
       public:
          Call( Pointer<Database> aDatabase, Pointer<PackageStream> aPackageStream, Pointer<TclLanguageInterface::CallbackData> aCallbackData )
               : LanguageInterface::SpecificCallbackData( aCallbackData ), mDatabase( aDatabase ), mPackageStream( aPackageStream ) {}

          virtual const char* typeid_name() const { return typeid( Callback ).name(); }
          virtual void activate() { if ( callbackData() ) callbackData()->activate( *this ); }

          Pointer<Database> database() { return mDatabase; }
          Pointer<PackageStream> packageStream() { return mPackageStream; }

       private:
          Pointer<Database> mDatabase;
          Pointer<PackageStream> mPackageStream;

     }; // class Call

     class Call_package_requirements_not_met : public Call
     {
       public:
          Call_package_requirements_not_met( Pointer<Database> aDatabase, Pointer<PackageStream> aPackageStream, const FeatureList& requirements, const FeatureList& conflicts, const list<string>& users, const list<string>& groups, const FeatureList& suggestions, Pointer<TclLanguageInterface::CallbackData> aCallbackData )
               : Call( aDatabase, aPackageStream, aCallbackData ), mRequirements( requirements ), mConflicts( conflicts ), mUsers( users ), mGroups( groups ), mSuggestions( suggestions ) {}

          string& result() { return mResult; }
          const string& result() const { return mResult; }

          const FeatureList& requirements() { return mRequirements; }
          const FeatureList& conflicts() { return mConflicts; }
          const list<string>& users() { return mUsers; }
          const list<string>& groups() { return mGroups; }
          const FeatureList& suggestions() { return mSuggestions; }

       private:
          string mResult;
          const FeatureList& mRequirements;
          const FeatureList& mConflicts;
          const list<string>& mUsers;
          const list<string>& mGroups;
          const FeatureList& mSuggestions;

     }; // Call_package_requirements_not_met

     class Call_installation_over_installed : public Call
     {
       public:
          Call_installation_over_installed( Pointer<Database> aDatabase, Pointer<PackageStream> aPackageStream, const FeatureList& duplicates, Pointer<TclLanguageInterface::CallbackData> aCallbackData )
               : Call( aDatabase, aPackageStream, aCallbackData ), mDuplicates( duplicates ) {}

          string& result() { return mResult; }
          const string& result() const { return mResult; }

          const FeatureList& duplicates() { return mDuplicates; }

       private:
          string mResult;
          const FeatureList& mDuplicates;

     }; // Call_installation_over_installed

     class Call_file_to_be_installed_exists_and_owned_by_other_packages : public Call
     {
       public:
          Call_file_to_be_installed_exists_and_owned_by_other_packages( Pointer<Database> aDatabase, Pointer<PackageStream> aPackageStream, Pointer<PackageStreamFile> file, PackageList& owners, Pointer<TclLanguageInterface::CallbackData> aCallbackData )
               : Call( aDatabase, aPackageStream, aCallbackData ), mFile( file ), mOwners( owners ) {}

          string& result() { return mResult; }
          const string& result() const { return mResult; }

          Pointer<PackageStreamFile> file() { return mFile; }
          const PackageList& owners() { return mOwners; }

       private:
          string mResult;
          Pointer<PackageStreamFile> mFile;
          PackageList& mOwners;

     }; // Call_file_to_be_installed_exists_and_owned_by_other_packages

     class Call_reverse_conflict : public Call
     {
       public:
          Call_reverse_conflict( Pointer<Database> aDatabase, Pointer<PackageStream> aPackageStream, const PackageList& conflicts, Pointer<TclLanguageInterface::CallbackData> aCallbackData )
               : Call( aDatabase, aPackageStream, aCallbackData ), mConflicts( conflicts ) {}

          string& result() { return mResult; }
          const string& result() const { return mResult; }

          const PackageList& conflicts() { return mConflicts; }

       private:
          string mResult;
          const PackageList& mConflicts;

     }; // Call_reverse_conflict

     class Call_install_subpackages : public LanguageInterface::SpecificCallbackData
     {
       public:
          Call_install_subpackages( Pointer<PackageStreamStructure> structure, Pointer<TclLanguageInterface::CallbackData> aCallbackData )
               : LanguageInterface::SpecificCallbackData( aCallbackData ), mStructure( structure ) {}

          virtual const char* typeid_name() const { return typeid( Callback ).name(); }
          virtual void activate() { if ( callbackData() ) callbackData()->activate( *this ); }

          Pointer<PackageStreamSubpackageList>& result() { return mResult; }
          Pointer<PackageStreamSubpackageList> result() const { return mResult; }

          Pointer<PackageStreamStructure> structure() { return mStructure; }

       private:
          Pointer<PackageStreamSubpackageList> mResult;
          Pointer<PackageStreamStructure> mStructure;

     }; // Call_install_subpackages

     class Call_fileaccess : public Call
     {
       public:
          Call_fileaccess( Pointer<Database> aDatabase, Pointer<PackageStream> aPackageStream, const string& aPath, Pointer<TclLanguageInterface::CallbackData> aCallbackData )
               : Call( aDatabase, aPackageStream, aCallbackData ), mPath( aPath ) {}

          Pointer<Container<list<string> > >& result() { return mResult; }
          Pointer<Container<list<string> > > result() const { return mResult; }

          const string& path() { return mPath; }

       private:
          Pointer<Container<list<string> > > mResult;
          string mPath;

     }; // Call_fileaccess

     class Call_owned_file_does_not_exist_for_uninstall : public LanguageInterface::SpecificCallbackData
     {
       public:
          Call_owned_file_does_not_exist_for_uninstall( Pointer<Package> aPackage, Pointer<PackageFile> aFile, Pointer<TclLanguageInterface::CallbackData> aCallbackData )
               : LanguageInterface::SpecificCallbackData( aCallbackData ), mPackage( aPackage ), mFile( aFile ) {}

          virtual const char* typeid_name() const { return typeid( Callback ).name(); }
          virtual void activate() { if ( callbackData() ) callbackData()->activate( *this ); }

          string& result() { return mResult; }
          const string& result() const { return mResult; }

          Pointer<Package> package() { return mPackage; }
          Pointer<PackageFile> file() { return mFile; }

       private:
          Pointer<Package> mPackage;
          Pointer<PackageFile> mFile;
          string mResult;

     }; // Call_owned_file_does_not_exist_for_uninstall

     class Call_install_upgrade : public LanguageInterface::SpecificCallbackData
     {
       public:
          Call_install_upgrade( Pointer<Database> aDatabase, const string& aUrl, Pointer<TclLanguageInterface::CallbackData> aCallbackData )
               : LanguageInterface::SpecificCallbackData( aCallbackData ), mDatabase( aDatabase ), mUrl( aUrl ) {}

          virtual const char* typeid_name() const { return typeid( Callback ).name(); }
          virtual void activate() { if ( callbackData() ) callbackData()->activate( *this ); }

          string& result() { return mResult; }
          const string& result() const { return mResult; }

          Pointer<Database> database() { return mDatabase; }
          const string& url() { return mUrl; }

       private:
          Pointer<Database> mDatabase;
          const string& mUrl;
          string mResult;

     }; // Call_install_upgrade

     class Call_exec_wanted : public LanguageInterface::SpecificCallbackData
     {
       public:
          Call_exec_wanted( const string& aCommand, Pointer<TclLanguageInterface::CallbackData> aCallbackData )
               : LanguageInterface::SpecificCallbackData( aCallbackData ), mCommand( aCommand ) {}

          virtual const char* typeid_name() const { return typeid( Callback ).name(); }
          virtual void activate() { if ( callbackData() ) callbackData()->activate( *this ); }

          string& result() { return mResult; }
          const string& result() const { return mResult; }

          const string& command() { return mCommand; }

       private:
          const string& mCommand;
          string mResult;

     }; // Call_exec_wanted

     class Call_uninstall_required_features : public LanguageInterface::SpecificCallbackData
     {
       public:
          Call_uninstall_required_features( Pointer<Database> aDatabase, Pointer<Package> toUninstall, const PackageList& requirers, Pointer<TclLanguageInterface::CallbackData> aCallbackData )
               : LanguageInterface::SpecificCallbackData( aCallbackData ), mDatabase( aDatabase ), mToUninstall( toUninstall ), mRequirers( requirers ) {}

          virtual const char* typeid_name() const { return typeid( Callback ).name(); }
          virtual void activate() { if ( callbackData() ) callbackData()->activate( *this ); }

          string& result() { return mResult; }
          const string& result() const { return mResult; }

          Pointer<Database> database() { return mDatabase; }
          Pointer<Package> toUninstall() { return mToUninstall; }
          const PackageList& requirers() const { return mRequirers; }

       private:
          Pointer<Database> mDatabase;
          Pointer<Package> mToUninstall;
          const PackageList& mRequirers;
          string mResult;

     }; // Call_uninstall_required_features

}; // class Callback

//======================================================================

#endif // Callback_hh
