/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/Database.cc,v 1.2 2001/06/22 15:05:15 alex Exp $
 */

// -*- C++ -*-   Tue Aug 11 1998
// $Id: Database.cc,v 1.19 1999/02/03 14:43:05 eugene Exp $

#ifndef Database_hh
#include "Database.hh"
#endif

#ifndef DatabaseEngineDb185_hh
#include "DatabaseEngineDb185.hh"
#endif

#ifndef Configuration_hh
#include "Configuration.hh"
#endif

#ifndef DirectoriesAccess_hh
#include "DirectoriesAccess.hh"
#endif

#ifndef PackageDescription_hh
#include "PackageDescription.hh"
#endif

#ifndef Log_hh
#include "Log.hh"
#endif

#ifndef File_hh
#include "File.hh"
#endif

#ifndef History_hh
#include "History.hh"
#endif

#include <sys/utsname.h> // uname()
#include <cerrno>
#include <fstream>
#include <limits>

//======================================================================

Database::Database( const string& aBasedir, OpenFlag aOpenFlag, bool addSYSTEMpackage )
     : mBaseDir( aBasedir ), mOpenFlag( aOpenFlag ),
       mEngine( new DatabaseEngineDb185( aBasedir, mOpenFlag == ofCreate ) )
{
     try {
          addSYSTEMpackage &= mOpenFlag == ofCreate;
          if ( mOpenFlag == ofCreate ) {
               mOpenFlag = ofReadWrite;
               create();
          }

          engine().lock();

            // Open tables
          for ( int i = 0; i < tiNone; ++i )
               mTable[i] = engine().table( sTables[i] );

          if ( addSYSTEMpackage ) {
               insert_SYSTEM_package();
               insert_denied_allowed_directories();
          }
          else {
               readDirectoriesAccess();
          }
     }
     catch ( ... ) {
          engine().unlock();
          throw;
     }

} // Database::Database

//======================================================================

Database::~Database()
{
     for ( int i = 0; i < tiNone; ++i )
          delete mTable[i];

     engine().unlock();

} // Database::~Database

//======================================================================

const DatabaseEngine::FieldDescription Database::sTablePackages[] = {
     { "Name", DatabaseEngine::tString, DatabaseEngine::kDuplicate, false },
     { "Installation_Time", DatabaseEngine::tTime, DatabaseEngine::kNone, false },
     { 0, DatabaseEngine::tString, DatabaseEngine::kNone, false }
};

const DatabaseEngine::FieldDescription Database::sTableFeatures[] = {
     { "Package_Id", DatabaseEngine::tId, DatabaseEngine::kDuplicate, false },
     { "Type", DatabaseEngine::tString, DatabaseEngine::kDuplicate, false },
     { "Name", DatabaseEngine::tString, DatabaseEngine::kDuplicate, false },
     { "Version", DatabaseEngine::tString, DatabaseEngine::kDuplicate, false },
     { "Serial", DatabaseEngine::tInteger, DatabaseEngine::kNone, true },
     { "Snapshot", DatabaseEngine::tTime, DatabaseEngine::kNone, true },
     { 0, DatabaseEngine::tString, DatabaseEngine::kNone, false }
};

const DatabaseEngine::FieldDescription Database::sTableScripts[] = {
     { "Package_Id", DatabaseEngine::tId, DatabaseEngine::kDuplicate, false },
     { "Type", DatabaseEngine::tString, DatabaseEngine::kDuplicate, false },
     { "Data", DatabaseEngine::tString, DatabaseEngine::kNone, false },
     { 0, DatabaseEngine::tString, DatabaseEngine::kNone, false }
};

const DatabaseEngine::FieldDescription Database::sTableDescription[] = {
     { "Package_Id", DatabaseEngine::tId, DatabaseEngine::kDuplicate, false },
     { "Summary", DatabaseEngine::tString, DatabaseEngine::kNone, true },
     { "Description", DatabaseEngine::tString, DatabaseEngine::kNone, true },
     { "Author", DatabaseEngine::tString, DatabaseEngine::kNone, true },
     { "Canonical_Name", DatabaseEngine::tString, DatabaseEngine::kNone, true },
     { 0, DatabaseEngine::tString, DatabaseEngine::kNone, false }
};

const DatabaseEngine::FieldDescription Database::sTableReferences[] = {
     { "Package_Id", DatabaseEngine::tId, DatabaseEngine::kDuplicate, false },
     { "Reference", DatabaseEngine::tString, DatabaseEngine::kNone, false },
     { 0, DatabaseEngine::tString, DatabaseEngine::kNone, false }
};

const DatabaseEngine::FieldDescription Database::sTableFiles[] = {
     { "Package_Id", DatabaseEngine::tId, DatabaseEngine::kDuplicate, false },
     { "FileInfo_Id", DatabaseEngine::tId, DatabaseEngine::kDuplicate, false },
     { 0, DatabaseEngine::tString, DatabaseEngine::kNone, false }
};

const DatabaseEngine::FieldDescription Database::sTableFileInfo[] = {
     { "Pathname", DatabaseEngine::tString, DatabaseEngine::kDuplicate, false },
     { "Filesystem", DatabaseEngine::tString, DatabaseEngine::kDuplicate, false },
     { "Inode", DatabaseEngine::tString, DatabaseEngine::kDuplicate, false },
     { "MD5", DatabaseEngine::tString, DatabaseEngine::kNone, true },
     { "Length", DatabaseEngine::tLongLong, DatabaseEngine::kNone, false },
     { "MTime", DatabaseEngine::tTime, DatabaseEngine::kNone, false },
     { "User", DatabaseEngine::tString, DatabaseEngine::kNone, false },
     { "Group", DatabaseEngine::tString, DatabaseEngine::kNone, false },
     { "Permissions", DatabaseEngine::tInteger, DatabaseEngine::kNone, false },
     { "Symlink", DatabaseEngine::tString, DatabaseEngine::kNone, true },
     { "Device", DatabaseEngine::tLongLong, DatabaseEngine::kNone, true },
     { 0, DatabaseEngine::tString, DatabaseEngine::kNone, false }
};

const DatabaseEngine::FieldDescription Database::sTableUndo[] = {
     { "Package_Id", DatabaseEngine::tId, DatabaseEngine::kUnique, false },
     { "Script", DatabaseEngine::tString, DatabaseEngine::kNone, false },
     { 0, DatabaseEngine::tString, DatabaseEngine::kNone, false }
};

const DatabaseEngine::FieldDescription Database::sTableFilesUndo[] = {
     { "Package_Id", DatabaseEngine::tId, DatabaseEngine::kDuplicate, false },
     { "SavedPathname", DatabaseEngine::tString, DatabaseEngine::kDuplicate, false },
     { "Type", DatabaseEngine::tString, DatabaseEngine::kNone, false },
     { "Compression", DatabaseEngine::tString, DatabaseEngine::kNone, true },
     { "Package_Id", DatabaseEngine::tString, DatabaseEngine::kDuplicate, false },
     { "FileInfo_Id", DatabaseEngine::tId, DatabaseEngine::kDuplicate, false },
     { 0, DatabaseEngine::tString, DatabaseEngine::kNone, false }
};

const DatabaseEngine::FieldDescription Database::sTableConfiguration[] = {
     { "Key", DatabaseEngine::tString, DatabaseEngine::kDuplicate, false },
     { "Value", DatabaseEngine::tString, DatabaseEngine::kNone, true },
     { 0, DatabaseEngine::tString, DatabaseEngine::kNone, false }
};

const DatabaseEngine::FieldDescription Database::sTableHistory[] = {
     { "Time", DatabaseEngine::tTime, DatabaseEngine::kNone, false },
     { "Type", DatabaseEngine::tString, DatabaseEngine::kDuplicate, false },
     { "Name", DatabaseEngine::tString, DatabaseEngine::kDuplicate, false },
     { 0, DatabaseEngine::tString, DatabaseEngine::kNone, false }
};

const DatabaseEngine::TableDescription Database::sTables[] = {
     { "Packages", Database::sTablePackages },
     { "Features", Database::sTableFeatures },
     { "Scripts", Database::sTableScripts },
     { "Description", Database::sTableDescription },
     { "References", Database::sTableReferences },
     { "Files", Database::sTableFiles },
     { "FileInfo", Database::sTableFileInfo },
     { "Undo", Database::sTableUndo },
     { "FilesUndo", Database::sTableFilesUndo },
     { "Configuration", Database::sTableConfiguration },
     { "History", Database::sTableHistory },
     { 0, 0 }
};

//======================================================================

void Database::create()
{
     if ( Configuration::instance()->no_action() ) {
          LOG( NO_ACTION, "Create database in " << mBaseDir );
     }
     else {
          try {
               File::make_path( mBaseDir );
          }
          catch ( exception& err ) {
               throw runtime_error( string( "Cannot use " ) + mBaseDir + " as database dir: " + err.what() );
          }

          engine().lock();
          for ( const DatabaseEngine::TableDescription* table = sTables; table->name; ++table )
               engine().create( *table );
     }

} // Database::create

//======================================================================

struct TypeNotEqual
{
     TypeNotEqual( const string& aType ) : mType( aType ) {}
     const string& mType;
     bool operator () ( Pointer<DatabaseEngine::Record> record ) const { return record->str( "Type" ) != mType; }
};

Pointer<FeatureList> Database::feature( const string& aFeatureName, const string& aType )
{
     Pointer<DatabaseEngine::RecordList> records = table( Database::tiFeatures ).select( "Name", DatabaseEngine::FieldValue( aFeatureName ) );
     DatabaseEngine::RecordList::iterator last = records->end();
     if ( !aType.empty() )
          last = remove_if( records->begin(), last, TypeNotEqual( aType ) ); // remove features records with different type
     return records2features( records->begin(), last );

} // Database::feature

//======================================================================

Pointer<FeatureList> Database::all_features( const string& aType )
{
     Pointer<DatabaseEngine::RecordList> records = table( Database::tiFeatures ).selectAll();
     DatabaseEngine::RecordList::iterator last = records->end();
     if ( !aType.empty() )
          last = remove_if( records->begin(), last, TypeNotEqual( aType ) ); // remove features records with different type
     return records2features( records->begin(), last );

} // Database::all_features

//======================================================================

struct VersionEqual
{
     VersionEqual( const string& aVersion ) : mVersion( aVersion ) {}
     const string& mVersion;
     bool operator () ( Pointer<DatabaseEngine::Record> record ) const { return record->str( "Version" ) == mVersion; }
};

Pointer<FeatureList> Database::feature_other_versions( const Feature& aFeature, const string& aType )
{
     Pointer<DatabaseEngine::RecordList> records = table( Database::tiFeatures ).select( "Name", DatabaseEngine::FieldValue( aFeature.name() ) );
     DatabaseEngine::RecordList::iterator last = remove_if( records->begin(), records->end(), VersionEqual( aFeature.version() ) ); // remove features records with the same version
     if ( !aType.empty() )
          last = remove_if( records->begin(), last, TypeNotEqual( aType ) ); // remove features records with different type
     return records2features( records->begin(), last );

} // Database::feature_other_versions

//======================================================================

static inline bool serial_equal( Pointer<DatabaseEngine::Record> record, int aSerial )
{
     return record->integer( "Serial" ) == aSerial;
}

static inline bool snapshot_equal( Pointer<DatabaseEngine::Record> record, time_t aSnapshot )
{
     return record->time( "Snapshot" ) == aSnapshot;
}

Pointer<Package> Database::package( const Feature& aFeature )
{
     Pointer<DatabaseEngine::RecordList> records = table( Database::tiFeatures ).select( "Name", DatabaseEngine::FieldValue( aFeature.name() ), "Version", DatabaseEngine::FieldValue( aFeature.version() ) );
     DatabaseEngine::RecordList::iterator rec;
     if ( aFeature.serial() != 0 )
          rec = find_if( records->begin(), records->end(), bind2nd( ptr_fun( &serial_equal ), aFeature.serial() ) );
     else if ( aFeature.snapshot() != 0 )
          rec = find_if( records->begin(), records->end(), bind2nd( ptr_fun( &snapshot_equal ), aFeature.snapshot() ) );
     else
          rec = records->begin();

     Pointer<Package> package = rec != records->end() ? new Package( *this, (*rec)->id( "Package_Id" ) ) : 0;
     return package;

} // Database::package

//======================================================================

struct Record2package
{
     Record2package( Database& aDatabase ) : mDatabase( aDatabase ) {}
     Database& mDatabase;
     Pointer<Package> operator () ( Pointer<DatabaseEngine::Record> record ) { return new Package( mDatabase, record->id() ); }
};

Pointer<PackageList> Database::packages_by_name( const string& aPackageName )
{
     Pointer<DatabaseEngine::RecordList> records = table( Database::tiPackages ).select( "Name", DatabaseEngine::FieldValue( aPackageName ) );
     Pointer<PackageList> packages = new PackageList;
     transform( records->begin(), records->end(), back_inserter( *packages ), Record2package( *this ) );
     return packages;

} // Database::packages_by_name

//======================================================================

struct Id2package
{
     Id2package( Database& aDatabase ) : mDatabase( aDatabase ) {}
     Database& mDatabase;
     Pointer<Package> operator () ( DatabaseEngine::Id aId ) { return new Package( mDatabase, aId ); }
};

Pointer<PackageList> Database::all_packages()
{
     Pointer<DatabaseEngine::IdList> ids = table( Database::tiPackages ).select();
     Pointer<PackageList> packages = new PackageList;
     transform( ids->begin(), ids->end(), back_inserter( *packages ), Id2package( *this ) );
     return packages;

} // Database::all_packages

//======================================================================

struct Record2packageByPackage_Id
{
     Record2packageByPackage_Id( Database& aDatabase ) : mDatabase( aDatabase ) {}
     Database& mDatabase;
     Pointer<Package> operator () ( Pointer<DatabaseEngine::Record> record ) { return new Package( mDatabase, record->id( "Package_Id" ) ); }
};

Pointer<PackageList> Database::packages_owning( const string& aPathname )
{
     Pointer<DatabaseEngine::RecordList> records_fileinfo = table( Database::tiFileInfo ).select( "Pathname", DatabaseEngine::FieldValue( aPathname ) );
     Pointer<PackageList> packages = new PackageList;
     for ( DatabaseEngine::RecordList::const_iterator record_fileinfo = records_fileinfo->begin(); record_fileinfo != records_fileinfo->end(); ++record_fileinfo ) {
          Pointer<DatabaseEngine::RecordList> records_files = table( Database::tiFiles ).select( "FileInfo_Id", DatabaseEngine::FieldValue( (*record_fileinfo)->id() ) );
          transform( records_files->begin(), records_files->end(), back_inserter( *packages ), Record2packageByPackage_Id( *this ) );
     }
     return packages;

} // Database::packages_owning

//======================================================================

Pointer<PackageList> Database::packages_owning( Pointer<PackageFile> aFile )
{
       // TODO: use filesystem, inode
     return packages_owning( aFile->pathname() );

} // Database::packages_owning

//======================================================================

static inline bool equal_by_id( Pointer<Package> a, Pointer<Package> b )
{
     return a->id() == b->id();
}

Pointer<PackageList> Database::unique_packages( const DatabaseEngine::RecordList& records )
{
     PackageList vpackages;
     transform( records.begin(), records.end(), back_inserter( vpackages ), Record2packageByPackage_Id( *this ) );
     vpackages.sort( &Package::lessById );
     Pointer<PackageList> packages = new PackageList;
     unique_copy( vpackages.begin(), vpackages.end(), back_inserter( *packages ) );
     return packages;

} // Database::unique_packages

//======================================================================

Pointer<PackageList> Database::packages_requiring( const FeatureList& aFeatures )
{
     DatabaseEngine::RecordList all_records;
     for ( FeatureList::const_iterator feature = aFeatures.begin(); feature != aFeatures.end(); ++feature ) {
          Pointer<DatabaseEngine::RecordList> records = table( Database::tiFeatures ).select( "Name", DatabaseEngine::FieldValue( (*feature)->name() ), "Type", DatabaseEngine::FieldValue( "require" ) );
          copy( records->begin(), records->end(), back_inserter( all_records ) );
     }
     Pointer<PackageList> packages = unique_packages( all_records );
     return packages;

} // Database::packages_requiring

//======================================================================

Pointer<PackageList> Database::packages_conflicting( const Feature& aFeature )
{
     return unique_packages( *table( Database::tiFeatures ).select( "Name", DatabaseEngine::FieldValue( aFeature.name() ), "Type", DatabaseEngine::FieldValue( "conflict" ) ) );

} // Database::packages_conflicting

//======================================================================

//  Pointer<Undo> Database::package_undo( const Package& aPackage )
//  {

//  } // Database::package_undo

//  //======================================================================

//  void Database::package_undo_store( const Package& aPackage, const Undo& aUndo )
//  {

//  } // Database::package_undo_store

//  //======================================================================

//  void Database::package_undo_erase( const Package& aPackage )
//  {

//  } // Database::package_undo_erase

//  //======================================================================

Pointer<Package> Database::new_package()
{
     if ( Configuration::instance()->no_action() ) {
          LOG( NO_ACTION, "New package in the database" );
          return new Package( *this, 0 );
     }
     else {
          Pointer<DatabaseEngine::Record> record = table( tiPackages ).newRecord();
          record->str( "Name", "NoName" );
          record->time( "Installation_Time", ::time( 0 ) );
          table( tiPackages ).insert( *record );
          return new Package( *this, record->id() );
     }

} // Database::new_package

//======================================================================

#ifndef SYSINSTALL_FLOPPY
Pointer<PackageList> Database::hardware_upgrade()
{
     Pointer<PackageList> conflicting_packages = new PackageList();
     if ( Configuration::instance()->no_action() ) {
          LOG( NO_ACTION, "hardware_upgrade" );
     }
     else {
          Pointer<Package> system_package = *packages_by_name( "SYSTEM" )->begin();
          string architecture_feature;
          Pointer<FeatureList> system_features = make_architecture_features( architecture_feature );
          system_package->features( *system_features );
          system_package->installation_time( ::time( 0 ) );

            // Find packages having the features provided by the
            // SYSTEM package in the conflicts lists
          for ( FeatureList::const_iterator system_feature = system_features->begin(); system_feature != system_features->end(); ++system_feature ) {
               Pointer<PackageList> cpackages = packages_conflicting( **system_feature );
               ::copy( cpackages->begin(), cpackages->end(), back_inserter( *conflicting_packages ) );
          }

            // Call proc could_be_installed for each package with features provided by
            // the SYSTEM package.
          Pointer<PackageList> packages = all_packages();
          for ( PackageList::iterator package = packages->begin(); package != packages->end(); ++package ) {
               try {
                    if ( ! ( (**package) == (*system_package) ) ) {
                         for ( FeatureList::const_iterator system_feature = system_features->begin(); system_feature != system_features->end(); ++system_feature ) {
                              if ( !(*package)->could_be_installed( **system_feature ) ) {
                                   conflicting_packages->push_back( *package );
                                   packages->erase( package );
                              }
                         }
                    }
               }
               catch ( exception& err ) {
                    LOG( ERROR, "Problem of calling proc could_be_installed for package " << (*package)->name() << ": " << err.what() );
               }
          }

          conflicting_packages->sort( &Package::lessById );
          conflicting_packages->erase( unique( conflicting_packages->begin(), conflicting_packages->end() ), conflicting_packages->end() );
     }
     return conflicting_packages;

} // Database::hardware_upgrade
#endif

//======================================================================

#ifndef SYSINSTALL_FLOPPY
class ProblematicPackage
{
     Database& mDatabase;

  public:
     typedef Pointer<Package> argument_type;
     ProblematicPackage( Database& aDatabase ) : mDatabase( aDatabase ) {}

     bool operator() ( Pointer<Package> aPackage ) const
          {
               Pointer<FeatureList> required_features = aPackage->requires();
               FeatureList::const_iterator required_feature = required_features->begin();
               for ( ; required_feature != required_features->end(); ++required_feature ) {
                    Pointer<FeatureList> provided_features = mDatabase.feature( (*required_feature)->name(), "provide" );
                    bool problematic = true;
                    for ( FeatureList::const_iterator provided_feature = provided_features->begin(); problematic && provided_feature != provided_features->end(); ++provided_feature ) {
                         if ( (*required_feature)->compare( **provided_feature ) <= 0 )
                              problematic = false;
                    }
                    if ( problematic )
                         break;
               }

               bool problematic = required_feature != required_features->end();

               if ( problematic ) {
                    LOG( INFO, "Package " << aPackage->name() << " requires feature " << (*required_feature)->print() << " not provided by the system" );
               }
               else {
                    Pointer<FeatureList> conflicting_features = aPackage->conflicts();
                    for ( FeatureList::const_iterator conflicting_feature = conflicting_features->begin(); !problematic && conflicting_feature != conflicting_features->end(); ++conflicting_feature ) {
                         Pointer<FeatureList> provided_features = mDatabase.feature( (*conflicting_feature)->name(), "provide" );
                         for ( FeatureList::const_iterator provided_feature = provided_features->begin(); !problematic && provided_feature != provided_features->end(); ++provided_feature ) {
                              if ( (*conflicting_feature)->compare( **provided_feature ) <= 0 ) {
                                   problematic = true;
                                   LOG( INFO, "Package " << aPackage->name() << " conflicts with feature " << (*conflicting_feature)->print() << " provided by the system" );
                              }
                         }
                    }
               }

               return problematic;
          }
};

Pointer<PackageList> Database::check_conflicts()
{
     Pointer<PackageList> problematic_packages = new PackageList();

     Pointer<PackageList> packages = all_packages();
     remove_copy_if( packages->begin(), packages->end(), back_inserter( *problematic_packages ), not1( ProblematicPackage( *this ) ) );

     return problematic_packages;

} // Database::check_conflicts
#endif

//======================================================================

#ifndef SYSINSTALL_FLOPPY
Pointer<FeatureList> Database::check_features_duplicates()
{
     Pointer<FeatureList> features = all_features( "provide" );
     features->sort( &Feature::lessByName );

     for ( FeatureList::iterator first_for_find = features->begin(); first_for_find != features->end(); ) {
          FeatureList::iterator first_adjacent = adjacent_find( first_for_find, features->end(), &Feature::equalByName );
          features->erase( first_for_find, first_adjacent );
          for ( first_for_find = first_adjacent; first_for_find != features->end() && Feature::equalByName( *first_adjacent, *first_for_find ); ++first_for_find );
     }

     return features;

} // Database::check_features_duplicates
#endif

//======================================================================

void Database::sync()
{
     for ( int i = 0; i < tiNone; ++i )
          mTable[i]->sync();

} // Database::sync

//======================================================================

string Database::select_string( TableId tid, DatabaseEngine::Id id, const string& aField )
{
     return table( tid ).select( id, aField )->str( aField.c_str() );

} // Database::select_string

//======================================================================

void Database::update( TableId tid, DatabaseEngine::Id id, const string& aField, const string& aValue )
{
     if ( Configuration::instance()->no_action() ) {
          LOG( NO_ACTION, "Update database table " << table( tid ).name() << " set " << aField << " to " << aValue );
     }
     else {
          Pointer<DatabaseEngine::Record> record = table( tid ).newRecord();
          record->id( id );
          record->str( aField.c_str(), aValue );
          table( tid ).update( *record );
     }

} // Database::update

//======================================================================

time_t Database::select_time( TableId tid, DatabaseEngine::Id id, const string& aField )
{
     return table( tid ).select( id, aField )->time( aField.c_str() );

} // Database::select_time

//======================================================================

void Database::update( TableId tid, DatabaseEngine::Id id, const string& aField, time_t aValue )
{
     if ( Configuration::instance()->no_action() ) {
          LOG( NO_ACTION, "Update database table " << table( tid ).name() << " set " << aField );
     }
     else {
          Pointer<DatabaseEngine::Record> record = table( tid ).newRecord();
          record->id( id );
          record->time( aField.c_str(), aValue );
          table( tid ).update( *record );
     }

} // Database::update

//======================================================================

Pointer<FeatureList> Database::records2features( DatabaseEngine::RecordList::const_iterator first, DatabaseEngine::RecordList::const_iterator last )
{
     Pointer<FeatureList> features = new FeatureList;
     for ( DatabaseEngine::RecordList::const_iterator record = first; record != last; ++record ) {
          unsigned serial = 0;
          if ( ! (*record)->null( "Serial" ) )
               serial = (*record)->integer( "Serial" );
          time_t snapshot = 0;
          if ( ! (*record)->null( "Snapshot" ) )
               snapshot = (*record)->time( "Snapshot" );
          features->push_back( new Feature( (*record)->str( "Name" ), (*record)->str( "Version" ), serial, snapshot ) );
     }
     return features;

} // Database::records2features

//======================================================================

void Database::insert_SYSTEM_package()
{
     if ( Configuration::instance()->no_action() ) {
          LOG( NO_ACTION, "Create database entry for SYSTEM package" );
     }
     else {
          string architecture_feature;
          Pointer<FeatureList> features = make_architecture_features( architecture_feature );
          const size_t number_of_architecture_features = feature( architecture_feature, "provide" )->size();
          if ( number_of_architecture_features > 1 )
               LOG( WARNING, "Several architecture features provided by the system" );
          else if ( number_of_architecture_features == 0 ) {
               Pointer<Package> package = new_package();
               package->name( "SYSTEM" );
               package->installation_time( ::time( 0 ) );
               package->features( *features );
               PackageDescription description( "SYSTEM package provides architecture feature", "SYSTEM package provides architecture feature", "FreeBSD Inc.", "SYSTEM", 0 );
               package->description( description );
          }
     }

} // Database::insert_SYSTEM_package

//======================================================================

Pointer<FeatureList> Database::make_architecture_features( string& architecture_feature )
{
     Pointer<FeatureList> features = new FeatureList();

     utsname buf;
     uname( &buf );
     string architecture_feature_version;
     size_t architecture_feature_serial;
     if ( !strcmp( buf.machine, "i386" ) ) {
          architecture_feature = "i386";
          architecture_feature_version = "3.0";
          architecture_feature_serial = 386;
     }
     else if ( !strcmp( buf.machine, "i486" ) ) {
          architecture_feature = "i386";
          architecture_feature_version = "4.0";
          architecture_feature_serial = 486;
     }
     else if ( !strcmp( buf.machine, "i586" ) ) {
          architecture_feature = "i386";
          architecture_feature_version = "5.0";
          architecture_feature_serial = 586;
     }
     else if ( !strcmp( buf.machine, "i686" ) ) {
          architecture_feature = "i386";
          architecture_feature_version = "6.0";
          architecture_feature_serial = 686;
     }
     else {
          architecture_feature = buf.machine;
          architecture_feature_version = "0.0";
          architecture_feature_serial = 0;
     }

     features->push_back( new Feature( architecture_feature, architecture_feature_version, architecture_feature_serial, 0 ) );
     features->push_back( new Feature( buf.sysname, buf.release, 0, 0 ) );

     return features;

} // Database::make_architecture_features

//======================================================================

void Database::insert_denied_allowed_directories()
{
     if ( DirectoriesAccess::instance()->deny().empty() ) {
          list<string> deny( 0 );
          deny.push_back( "/" );
          DirectoriesAccess::instance()->deny( deny );
     }

     if ( DirectoriesAccess::instance()->allow().empty() ) {
          list<string> allow( 0 );
          allow.push_back( "/usr/local" );
          DirectoriesAccess::instance()->allow( allow );
     }

     saveDirectoriesAccess();

} // Database::insert_denied_allowed_directories

//======================================================================

void Database::saveDirectoriesAccess()
{
     writeConfiguration( "directories_denied", DirectoriesAccess::instance()->deny() );
     writeConfiguration( "directories_allowed", DirectoriesAccess::instance()->allow() );
     sync();

} // Database::saveDirectoriesAccess

//======================================================================

void Database::readDirectoriesAccess()
{
     Pointer<Container<list<string> > > deny = readConfiguration( "directories_denied" );
     DirectoriesAccess::instance()->deny( *deny );

     Pointer<Container<list<string> > > allow = readConfiguration( "directories_allowed" );
     DirectoriesAccess::instance()->allow( *allow );

} // Database::readDirectoriesAccess

//======================================================================

static inline string record2value( Pointer<DatabaseEngine::Record> record )
{
     return record->null( "Value" ) ? string() : record->str( "Value" );
}

Pointer<Container<list<string> > > Database::readConfiguration( const string& aKey )
{
     Pointer<DatabaseEngine::RecordList> records = table( Database::tiConfiguration ).select( "Key", DatabaseEngine::FieldValue( aKey ) );
     Pointer<Container<list<string> > > result = new Container<list<string> >;
     transform( records->begin(), records->end(), back_inserter( *result ), &record2value );
     return result;

} // Database::readConfiguration

//======================================================================

void Database::writeConfiguration( const string& aKey, const string& aValue )
{
     if ( Configuration::instance()->no_action() ) {
          LOG( NO_ACTION, "Update database configuration entry for key " << aKey );
     }
     else {
            // remove old records corresponding to this key
          table( Database::tiConfiguration ).remove( "Key", DatabaseEngine::FieldValue( aKey ) );

            // insert new record
          Pointer<DatabaseEngine::Record> record = table( Database::tiConfiguration ).newRecord();
          record->str( "Key", aKey );
          record->str( "Value", aValue );
          table( Database::tiConfiguration ).insert( *record );
     }

} // Database::writeConfiguration

//======================================================================

void Database::writeConfiguration( const string& aKey, const list<string>& aValues )
{
     if ( Configuration::instance()->no_action() ) {
          LOG( NO_ACTION, "Update database configuration entries for key " << aKey );
     }
     else {
            // remove old records corresponding to this key
          table( Database::tiConfiguration ).remove( "Key", DatabaseEngine::FieldValue( aKey ) );

            // insert new records
          for ( list<string>::const_iterator value = aValues.begin(); value != aValues.end(); ++value ) {
               Pointer<DatabaseEngine::Record> record = table( Database::tiConfiguration ).newRecord();
               record->str( "Key", aKey );
               record->str( "Value", *value );
               table( Database::tiConfiguration ).insert( *record );
          }
     }

} // Database::writeConfiguration

//======================================================================

Pointer<PackageFile> Record_to_PackageFile::operator () ( Pointer<DatabaseEngine::Record> record )
{
     Pointer<DatabaseEngine::Record> record_f = mDatabase.table( Database::tiFileInfo ).select( record->id( "FileInfo_Id" ) );
     const string symlink = record_f->null( "Symlink" ) ? string() : record_f->str( "Symlink" );
     const unsigned long long device = record_f->null( "Device" ) ? 0 : record_f->longlong( "Device" );
     const string pathname = record_f->str( "Pathname" );
     if ( pathname.empty() || pathname[0] != '/' )
          throw runtime_error( string( "Internal error: Invalid package file pathname in the database: " ) + pathname );
     FileInfo file( pathname, record_f->longlong( "Length" ), record_f->time( "MTime" ), record_f->str( "User" ), record_f->str( "Group" ), record_f->integer( "Permissions" ), symlink, device );
     file.md5( record_f->str( "MD5" ) );
     return new PackageFile( file );

} // Record_to_PackageFile::operator

//======================================================================

Pointer<PackageFileList> Database::all_files()
{
     Pointer<DatabaseEngine::RecordList> records = table( Database::tiFiles ).selectAll();
     Pointer<PackageFileList> files = new PackageFileList;
     transform( records->begin(), records->end(), back_inserter( *files ), Record_to_PackageFile( *this ) );
     return files;

} // Database::all_files

//======================================================================

Pointer<PackageFileList> Database::file( const string& aPathname )
{
     Pointer<DatabaseEngine::RecordList> records_fileinfo = table( Database::tiFileInfo ).select( "Pathname", DatabaseEngine::FieldValue( aPathname ) );
     Pointer<PackageFileList> files = new PackageFileList;
     for ( DatabaseEngine::RecordList::const_iterator record_fileinfo = records_fileinfo->begin(); record_fileinfo != records_fileinfo->end(); ++record_fileinfo ) {
          const string symlink = (*record_fileinfo)->null( "Symlink" ) ? string() : (*record_fileinfo)->str( "Symlink" );
          const unsigned long long device = (*record_fileinfo)->null( "Device" ) ? 0 : (*record_fileinfo)->longlong( "Device" );
          FileInfo file( (*record_fileinfo)->str( "Pathname" ), (*record_fileinfo)->longlong( "Length" ), (*record_fileinfo)->time( "MTime" ), (*record_fileinfo)->str( "User" ), (*record_fileinfo)->str( "Group" ), (*record_fileinfo)->integer( "Permissions" ), symlink, device );
          file.md5( (*record_fileinfo)->str( "MD5" ) );
          files->push_back( new PackageFile( file ) );
     }
     return files;

} // Database::file

//======================================================================

#ifndef SYSINSTALL_FLOPPY
void Database::export_db( ostream& os )
{
     const time_t t = time( 0 );
     os << "# This file is automatically created " << ctime( &t );

     for ( int table_id = 0; table_id != tiNone; ++table_id ) {
          os << "Table " << sTables[table_id].name << endl;
          DatabaseEngine::Id id0 = table( TableId( table_id ) ).internalId();
          if ( id0 != 0 )
               os << "Id0 " << id0 << endl;
          Pointer<DatabaseEngine::RecordList> records = table( TableId( table_id ) ).selectAll();
          for ( DatabaseEngine::RecordList::const_iterator record = records->begin(); record != records->end(); ++record ) {
               os << "Id " << (*record)->id() << ' ';
               for ( int field = 0; sTables[table_id].fields[field].name; ++field ) {
                    os << sTables[table_id].fields[field].name << ' ';
                    switch ( sTables[table_id].fields[field].type ) {
                      case DatabaseEngine::tId:
                           os << (*record)->id( sTables[table_id].fields[field].name );
                           break;
                      case DatabaseEngine::tString:
                      {
                           const string s = (*record)->str( sTables[table_id].fields[field].name );
                           os << s.length() << ' ' << s;
                      }
                      break;
                      case DatabaseEngine::tInteger:
                           os << (*record)->integer( sTables[table_id].fields[field].name );
                           break;
                      case DatabaseEngine::tLongLong:
                           os << (*record)->longlong( sTables[table_id].fields[field].name );
                           break;
                      case DatabaseEngine::tTime:
                           os << (*record)->time( sTables[table_id].fields[field].name );
                           break;
                    }
                    os << ' ';
               }
               os << endl;
          }
          os << endl;
     }

} // Database::export_db
#endif

//======================================================================

#ifndef SYSINSTALL_FLOPPY
void Database::export_db( const string& aFilename )
{
     if ( aFilename.empty() || aFilename == "-" ) {
          export_db( cout );
     }
     else {
          ofstream out( aFilename.c_str() );
          if ( !out )
               throw runtime_error( string( "Unable to open/create " ) + aFilename + ": " + ::strerror( errno ) );
          export_db( out );
     }

} // Database::export_db
#endif

//======================================================================

#ifndef SYSINSTALL_FLOPPY
void Database::import_db( istream& is )
{
     for ( int table_id = 0; table_id != tiNone; ++table_id ) {
          size_t records = table( TableId( table_id ) ).number_of_records();
          if ( records > 0 )
               throw runtime_error( string( "Unable to import database: table " ) + sTables[table_id].name + " is not empty" );
     }

     const ios::fmtflags old_flags = is.flags();
     is.unsetf( ios::skipws );

     try {
          size_t lineno = 1;
          while ( is ) {
               skip_ws_comments( is, lineno );
               if ( is.eof() )
                    break;

               string s;
               is >> s;
               if ( !is || s != "Table" )
                    throw runtime_error( string( "Invalid import file format, line " ) + ::to_string( lineno ) );

               string table_name;
               char endOfLine;
               is >> ws >> table_name >> endOfLine;
               if ( !is || table_name.empty() || endOfLine != '\n' )
                    throw runtime_error( string( "Invalid import file format, line " ) + ::to_string( lineno ) );

               const TableId table_id = tableByName( table_name );
               import_table( table_id, is, lineno );
          }
     }
     catch ( ... ) {
          is.flags( old_flags );
          throw;
     }

     is.flags( old_flags );

} // Database::import_db
#endif

//======================================================================

#ifndef SYSINSTALL_FLOPPY
void Database::import_db( const string& aFilename )
{
     if ( aFilename.empty() || aFilename == "-" ) {
          import_db( cin );
     }
     else {
          ifstream in( aFilename.c_str() );
          if ( !in )
               throw runtime_error( string( "Unable to open " ) + aFilename + ": " + ::strerror( errno ) );
          import_db( in );
     }

} // Database::import_db
#endif

//======================================================================

#ifndef SYSINSTALL_FLOPPY
void Database::import_table( Database::TableId table_id, istream& is, size_t& lineno )
{
     while ( is && is.peek() != '\n' ) {
          Pointer<DatabaseEngine::Record> record = table( table_id ).newRecord();
          bool end_record = false;
          bool record_fields_set = false;
          while ( is && !end_record ) {
               switch ( is.peek() ) {
                 case ' ':
                 case '\t':
                      is.ignore();
                      break;
                 case '\n':
                      is.ignore();
                      ++lineno;
                      end_record = true;
                      break;
                 default:
                 {
                      string field_name;
                      char c;
                      is >> field_name >> c;
                      if ( !is || field_name.empty() || c != ' ' )
                           throw runtime_error( string( "Invalid import file format, line " ) + ::to_string( lineno ) );
                      if ( field_name == "Id" ) {
                           DatabaseEngine::Id id;
                           is >> id;
                           record->id( id );
                           record_fields_set = true;
                      }
                      else if ( field_name == "Id0" ) {
                           DatabaseEngine::Id id0;
                           is >> id0;
                           table( table_id ).internalId( id0 );
                      }
                      else {
                           const size_t field = sTables[table_id].fieldByName( field_name.c_str() );
                           switch ( sTables[table_id].fields[field].type ) {
                             case DatabaseEngine::tId:
                             {
                                  DatabaseEngine::Id id;
                                  is >> id;
                                  record->id( sTables[table_id].fields[field].name, id );
                                  record_fields_set = true;
                             }
                             break;
                             case DatabaseEngine::tString:
                             {
                                  size_t length;
                                  is >> length >> c;
                                  if ( !is || c != ' ' )
                                       throw runtime_error( string( "Invalid import file format, line " ) + ::to_string( lineno ) );
                                  string s;
                                  if ( length > 0 ) {
                                       char ss[length];
                                       is.read( ss, length );
                                       s.assign( ss, length );
                                  }
                                  record->str( sTables[table_id].fields[field].name, s );
                                  record_fields_set = true;
                             }
                             break;
                             case DatabaseEngine::tInteger:
                             {
                                  int i;
                                  is >> i;
                                  record->integer( sTables[table_id].fields[field].name, i );
                                  record_fields_set = true;
                             }
                             break;
                             case DatabaseEngine::tLongLong:
                             {
                                  long long ll;
                                  is >> ll;
                                  record->longlong( sTables[table_id].fields[field].name, ll );
                                  record_fields_set = true;
                             }
                             break;
                             case DatabaseEngine::tTime:
                             {
                                  time_t t;
                                  is >> t;
                                  record->time( sTables[table_id].fields[field].name, t );
                                  record_fields_set = true;
                             }
                             break;
                           }
                      }
                 }
                      break;
               }
          }

          if ( record_fields_set )
               table( table_id ).insert( *record, true );
     }

} // Database::import_table
#endif

//======================================================================

#ifndef SYSINSTALL_FLOPPY
void Database::skip_ws_comments( istream& is, size_t& lineno ) const
{
     bool end = false;
     while ( !end ) {
          char c = is.peek();
          if ( is.eof() )
               break;
          else if ( !is )
               throw runtime_error( string( "Invalid import file format, line " ) + ::to_string( lineno ) );
          switch ( c ) {
            case ' ':
            case '\t':
                 is.ignore();
                 break;
            case '\n':
                 is.ignore();
                 ++lineno;
                 break;
            case '#':  // comment to the end of line
                 cin.ignore( numeric_limits<int>::max(), '\n' );
                 ++lineno;
                 break;
            default:
                 end = true;
                 break;
          }
     }

} // Database::skip_ws_comments
#endif

//======================================================================

Database::TableId Database::tableByName( const string& aTableName ) const
{
     for ( int table = 0; table < tiNone; ++table ) {
          if ( aTableName == sTables[table].name )
               return Database::TableId( table );
     }
     throw runtime_error( string( "Unknown table name: " ) + aTableName );

} // Database::tableByName

//======================================================================

#ifndef SYSINSTALL_FLOPPY
void Database::repair()
{
     remove_not_referenced_files();
     remove_not_referenced_features();
     remove_not_referenced_scripts();
     remove_not_referenced_description();
     remove_not_referenced_reference();
     remove_not_referenced_undo();
     sync();

} // Database::repair
#endif

//======================================================================

#ifndef SYSINSTALL_FLOPPY
void Database::remove_not_referenced_files()
{
       // Check unreferenced tiFiles entries
     Pointer<DatabaseEngine::RecordList> records_file = table( tiFiles ).selectAll();
     for ( DatabaseEngine::RecordList::const_iterator rec_file = records_file->begin(); rec_file != records_file->end(); ++rec_file ) {
          if ( !table( Database::tiFileInfo ).present( (*rec_file)->id( "FileInfo_Id" ) ) || !table( Database::tiPackages ).present( (*rec_file)->id( "Package_Id" ) ) ) {
               if ( Configuration::instance()->no_action() ) {
                    LOG( NO_ACTION, "Remove unreferenced Files entry" );
               }
               else {
                    LOG( INFO, "Remove unreferenced Files entry" );
                    table( tiFiles ).remove( (*rec_file)->id() );
               }
          }
     }

       // TODO: use filesystem, inode
     Pointer<DatabaseEngine::RecordList> records_fileinfo = table( tiFileInfo ).selectAll();
     for ( DatabaseEngine::RecordList::const_iterator rec_fileinfo = records_fileinfo->begin(); rec_fileinfo != records_fileinfo->end(); ++rec_fileinfo ) {
          Pointer<DatabaseEngine::RecordList> records_file = table( tiFiles ).select( "FileInfo_Id", DatabaseEngine::FieldValue( (*rec_fileinfo)->id() ) );
          if ( records_file->empty() ) {
               if ( Configuration::instance()->no_action() ) {
                    LOG( NO_ACTION, "Remove unreferenced FileInfo entry for " << (*rec_fileinfo)->str( "Pathname" ) );
               }
               else {
                    LOG( INFO, "Remove unreferenced FileInfo entry for " << (*rec_fileinfo)->str( "Pathname" ) );
                    table( tiFileInfo ).remove( (*rec_fileinfo)->id() );
                      // remove FileUndo entry for this file
                    table( tiFilesUndo ).remove( "FileInfo_Id", (*rec_fileinfo)->id() );
               }
          }
     }

} // Database::remove_not_referenced_files
#endif

//======================================================================

#ifndef SYSINSTALL_FLOPPY
void Database::remove_not_referenced_features()
{
     Pointer<DatabaseEngine::RecordList> records_feature = table( tiFeatures ).selectAll();
     for ( DatabaseEngine::RecordList::const_iterator rec_feature = records_feature->begin(); rec_feature != records_feature->end(); ++rec_feature ) {
          if ( !table( Database::tiPackages ).present( (*rec_feature)->id( "Package_Id" ) ) ) {
               if ( Configuration::instance()->no_action() ) {
                    LOG( NO_ACTION, "Remove unreferenced Features entry" );
               }
               else {
                    LOG( INFO, "Remove unreferenced Features entry" );
                    table( tiFeatures ).remove( (*rec_feature)->id() );
               }
          }
     }

} // Database::remove_not_referenced_features
#endif

//======================================================================

#ifndef SYSINSTALL_FLOPPY
void Database::remove_not_referenced_scripts()
{
     Pointer<DatabaseEngine::RecordList> records_script = table( tiScripts ).selectAll();
     for ( DatabaseEngine::RecordList::const_iterator rec_script = records_script->begin(); rec_script != records_script->end(); ++rec_script ) {
          if ( !table( Database::tiPackages ).present( (*rec_script)->id( "Package_Id" ) ) ) {
               if ( Configuration::instance()->no_action() ) {
                    LOG( NO_ACTION, "Remove unreferenced Scripts entry" );
               }
               else {
                    LOG( INFO, "Remove unreferenced Scripts entry" );
                    table( tiScripts ).remove( (*rec_script)->id() );
               }
          }
     }

} // Database::remove_not_referenced_scripts
#endif

//======================================================================

#ifndef SYSINSTALL_FLOPPY
void Database::remove_not_referenced_description()
{
     Pointer<DatabaseEngine::RecordList> records_description = table( tiDescription ).selectAll();
     for ( DatabaseEngine::RecordList::const_iterator rec_description = records_description->begin(); rec_description != records_description->end(); ++rec_description ) {
          if ( !table( Database::tiPackages ).present( (*rec_description)->id( "Package_Id" ) ) ) {
               if ( Configuration::instance()->no_action() ) {
                    LOG( NO_ACTION, "Remove unreferenced Descriptions entry" );
               }
               else {
                    LOG( INFO, "Remove unreferenced Descriptions entry" );
                    table( tiDescription ).remove( (*rec_description)->id() );
               }
          }
     }

} // Database::remove_not_referenced_description
#endif

//======================================================================

#ifndef SYSINSTALL_FLOPPY
void Database::remove_not_referenced_reference()
{
     Pointer<DatabaseEngine::RecordList> records_reference = table( tiReferences ).selectAll();
     for ( DatabaseEngine::RecordList::const_iterator rec_reference = records_reference->begin(); rec_reference != records_reference->end(); ++rec_reference ) {
          if ( !table( Database::tiPackages ).present( (*rec_reference)->id( "Package_Id" ) ) ) {
               if ( Configuration::instance()->no_action() ) {
                    LOG( NO_ACTION, "Remove unreferenced References entry" );
               }
               else {
                    LOG( INFO, "Remove unreferenced References entry" );
                    table( tiReferences ).remove( (*rec_reference)->id() );
               }
          }
     }

} // Database::remove_not_referenced_reference
#endif

//======================================================================

#ifndef SYSINSTALL_FLOPPY
void Database::remove_not_referenced_undo()
{
     Pointer<DatabaseEngine::RecordList> records_undo = table( tiUndo ).selectAll();
     for ( DatabaseEngine::RecordList::const_iterator rec_undo = records_undo->begin(); rec_undo != records_undo->end(); ++rec_undo ) {
          if ( !table( Database::tiPackages ).present( (*rec_undo)->id( "Package_Id" ) ) ) {
               if ( Configuration::instance()->no_action() ) {
                    LOG( NO_ACTION, "Remove unreferenced Undos entry" );
               }
               else {
                    LOG( INFO, "Remove unreferenced Undos entry" );
                    table( tiUndo ).remove( (*rec_undo)->id() );
               }
          }
     }

} // Database::remove_not_referenced_undo
#endif

//======================================================================

void Database::updateHistory( const History& aEntry )
{
     if ( Configuration::instance()->no_action() ) {
          LOG( NO_ACTION, "Update history: package " << aEntry.name() << " was " << aEntry.type() );
     }
     else {
          Pointer<DatabaseEngine::Record> record = table( tiHistory ).newRecord();
          record->time( "Time", aEntry.time() );
          record->str( "Type", aEntry.type() );
          record->str( "Name", aEntry.name() );
          table( tiHistory ).insert( *record );
     }

} // Database::updateHistory

//======================================================================

#ifndef SYSINSTALL_FLOPPY
struct Record2HistoryEntry
{
     Record2HistoryEntry() {}
     Pointer<History> operator () ( Pointer<DatabaseEngine::Record> record )
          {
               return new History( record->str( "Name" ), record->str( "Type" ), record->time( "Time" ) );
          }
};

Pointer<HistoryList> Database::history()
{
     Pointer<DatabaseEngine::RecordList> records = table( Database::tiHistory ).selectAll();
     Pointer<HistoryList> history = new HistoryList;
     ::transform( records->begin(), records->end(), back_inserter( *history ), Record2HistoryEntry() );
     history->sort( &History::lessByTime );
     return history;

} // Database::history
#endif

//======================================================================
