/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/Database.cd.cc,v 1.2 2001/06/22 15:05:15 alex Exp $
 */

// -*- C++ -*-   Thu Dec 24 1998
// Eugene Skepner <eu@qub.com>
// $Id: Database.cd.cc,v 1.13 1999/02/03 14:43:05 eugene Exp $

#ifndef Database_hh
#include "Database.hh"
#endif

//======================================================================

LanguageInterface::Object::MethodDescription Database::sMethods[] =
{
     MethodDescription( "open", "open",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "Opens existing Database",
                        MethodDescription::Value( MethodDescription::vtObject, "Database" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "base_dir" ) ),
     MethodDescription( "Database", "open",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "Opens existing Database",
                        MethodDescription::Value( MethodDescription::vtObject, "Database" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "base_dir" ) ),
     MethodDescription( "create", "create",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "Opens existing Database or creates new database",
                        MethodDescription::Value( MethodDescription::vtObject, "Database" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "base_dir" ) ),
#ifndef SYSINSTALL_FLOPPY
     MethodDescription( "repair", "repair",
                        LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fSafeExecution,
                        "Repairs existing Database",
                        MethodDescription::vtVoid, MethodDescription::vtVoid ),
#endif
     MethodDescription( "feature", "feature",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns list of features matching feature_name (and optionally with the specified type: provide, require, conflict). If there are no installed features matching the specified name returns an empty list.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "FeatureList" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "feature_name" ), MethodDescription::Value( MethodDescription::vtString, 0, "string()", "type" ) ),
     MethodDescription( "all_features", "all_features",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns list of all features (optionally with the specified type: provide, require, conflict).",
                        MethodDescription::Value( MethodDescription::vtListPtr, "FeatureList" ), MethodDescription::Value( MethodDescription::vtString, 0, "string()", "type" ) ),
     MethodDescription( "feature_other_versions", "feature_other_versions",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns list of features with the same name (and optionally with the specified type: provide, require, conflict) as specified feature but of other versions or an empty list.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "FeatureList" ), MethodDescription::Value( MethodDescription::vtObject, "Feature", 0, "feature" ), MethodDescription::Value( MethodDescription::vtString, 0, "string()", "type" ) ),
     MethodDescription( "package", "package",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns package providing the specified feature. If there is no package providing the feature returns 0.",
                        MethodDescription::Value( MethodDescription::vtObject, "Package" ), MethodDescription::Value( MethodDescription::vtObject, "Feature", 0, "feature" ) ),
     MethodDescription( "packages_by_name", "packages_by_name",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns packages with the specified name. If there is no packages with this name returns 0.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "PackageList" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "name" ) ),
     MethodDescription( "all_packages", "all_packages",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns all packages installed.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "PackageList" ), MethodDescription::vtVoid ),
     MethodDescription( "packages_owning", "packages_owning",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns list of packages owning the specified file.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "PackageList" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "pathname" ) ),
     MethodDescription( "packages_requiring", "packages_requiring",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns list of packages requiring at least one of the specified features. Versions of the features are ignored.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "PackageList" ), MethodDescription::Value( MethodDescription::vtListRef, "FeatureList", 0, "features" ) ),
     MethodDescription( "packages_conflicting", "packages_conflicting",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns list of packages conflicting the specified feature. Version of the feature is ignored.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "PackageList" ), MethodDescription::Value( MethodDescription::vtObject, "Feature", 0, "feature" ) ),
     MethodDescription( "new_package", "new_package",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns new package handle to store in the database. If Configuration::no_action returns non-zero does not do anything just reports what is to be done and returns dummy package handle.",
                        MethodDescription::Value( MethodDescription::vtObject, "Package" ), MethodDescription::vtVoid ),
#ifndef SYSINSTALL_FLOPPY
     MethodDescription( "hardware_upgrade", "hardware_upgrade",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Updates records of the SYSTEM package and checks _all_ the packages for fitness into new hardware system. Returns list of package having conflicts with the SYSTEM package. If Configuration::no_action returns an emtpy list does not do anything just reports what is to be done.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "PackageList" ), MethodDescription::vtVoid ),
#endif
     MethodDescription( "check_conflicts", "check_conflicts",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns list of package having problems, e.g. requiring features not provided by the system, conflicting features provided by the system.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "PackageList" ), MethodDescription::vtVoid ),
     MethodDescription( "check_features_duplicates", "check_features_duplicates",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns list of features provided multiple times (different versions).",
                        MethodDescription::Value( MethodDescription::vtListPtr, "FeatureList" ), MethodDescription::vtVoid ),
     MethodDescription( "save_directories_access", "saveDirectoriesAccess",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Saves directories access lists in the database.",
                        MethodDescription::vtVoid, MethodDescription::vtVoid ),
     MethodDescription( "sync", "sync",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Flushes database to disk.",
                        MethodDescription::vtVoid, MethodDescription::vtVoid ),
     MethodDescription( "flush", "sync",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Flushes database to disk.",
                        MethodDescription::vtVoid, MethodDescription::vtVoid ),
     MethodDescription( "all_files", "all_files",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns list of all files owned by any package.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "PackageFileList" ), MethodDescription::vtVoid ),
     MethodDescription( "file", "file",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns records in the database corresponding to the specified file.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "PackageFileList" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "pathname" ) ),
#ifndef SYSINSTALL_FLOPPY
     MethodDescription( "export", "export_db",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Exports the database into the specified file (`-' for stdout).",
                        MethodDescription::vtVoid, MethodDescription::Value( MethodDescription::vtString, 0, 0, "filename" ) ),
     MethodDescription( "import", "import_db",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "Creates and imports the database from the specified file (`-' for stdin).",
                        MethodDescription::Value( MethodDescription::vtObject, "Database" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "base_dir" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "filename" ) ),
#endif
#ifndef SYSINSTALL_FLOPPY
     MethodDescription( "history", "history",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns list of history records sorted by time.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "HistoryList" ), MethodDescription::vtVoid ),
#endif
     MethodDescription()
};

LanguageInterface::Object::CallbackDescription Database::sCallbacks[] =
{
     CallbackDescription()
};

LanguageInterface::Object::ClassDescription Database::sClassDescription(
     "Database", "Database",
     "Installed packages database interface",
     sMethods,
     sCallbacks
);

//======================================================================
