/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/Feature.cd.cc,v 1.2 2001/06/22 15:05:15 alex Exp $
 */

// -*- C++ -*-   Fri Dec 25 1998
// Eugene Skepner <eu@qub.com>
// $Id: Feature.cd.cc,v 1.5 1999/01/14 15:10:53 eugene Exp $

#ifndef Feature_hh
#include "Feature.hh"
#endif

//======================================================================

LanguageInterface::Object::MethodDescription Feature::sMethods[] =
{
     MethodDescription( "name", "name",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns feature name",
                        MethodDescription::vtString, MethodDescription::vtVoid ),
     MethodDescription( "version", "version",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns feature version",
                        MethodDescription::vtString, MethodDescription::vtVoid ),
     MethodDescription( "serial", "serial",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns feature serial",
                        MethodDescription::vtUInt, MethodDescription::vtVoid ),
     MethodDescription( "snapshot", "snapshot",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns feature snapshot",
                        MethodDescription::vtULong, MethodDescription::vtVoid ),
     MethodDescription( "print", "print",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns string representation of the feature",
                        MethodDescription::vtString, MethodDescription::vtVoid ),
     MethodDescription( "check_integrity", "check_integrity",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns if integrity of the feature correct",
                        MethodDescription::vtBool, MethodDescription::vtVoid ),
     MethodDescription( "compare", "compare",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns 0 if this feature and the argument are the same feature, >0 if this feature is newer, <0 if this feature is older",
                        MethodDescription::vtInt, MethodDescription::Value( MethodDescription::vtObject, "Feature", 0, "to_compare" ) ),
     MethodDescription( "equal", "operator ==",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns if this feature and the argument are the same feature",
                        MethodDescription::vtBool, MethodDescription::Value( MethodDescription::vtObject, "Feature", 0, "to_compare" ) ),
     MethodDescription( "Feature", "make",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "Returns newly created Feature object",
                        MethodDescription::Value( MethodDescription::vtObject, "Feature" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "name" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "version" ), MethodDescription::Value( MethodDescription::vtUInt, 0, "0", "serial" ), MethodDescription::Value( MethodDescription::vtULong, 0, "0", "snapshot" ) ),
     MethodDescription()
};

LanguageInterface::Object::CallbackDescription Feature::sCallbacks[] =
{
     CallbackDescription()
};

LanguageInterface::Object::ClassDescription Feature::sClassDescription(
     "Feature", "Feature",
     "Feature interface",
     sMethods,
     sCallbacks
);

//======================================================================
