/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/FeaturesProvided.hh,v 1.2 2001/06/22 15:05:16 alex Exp $
 */

// -*- C++ -*-  Mon Sep 14 1998
// $Id: FeaturesProvided.hh,v 1.3 1999/01/14 21:48:34 eugene Exp $

#ifndef FeaturesProvided_hh
#define FeaturesProvided_hh

#ifndef Feature_hh
#include "Feature.hh"
#endif

class Database;
class ProgressIndicatorStack;

//======================================================================

// Keeps track of which packages provide what features

class FeaturesProvided : virtual public LanguageInterface::Object
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     FeaturesProvided();
     virtual ~FeaturesProvided();

     static Pointer<FeaturesProvided> instance();

     virtual void load_dir( const string& aDirUrl, bool search_subdirs ); //scans all archives in dir
     virtual void save( const string& aDirUrl ); // saves entries corresponding to the specified url only

       // returns package url providing the specified feature or an
       // empty string if no required packages found
     virtual string package( const Feature& aFeature );

       // Builds list of packages (no duplicates) providing specified
       // features. Returns number of necessary features satisfied.
     virtual size_t packages( const FeatureList& aFeatures, list<string>& aPackages );

     virtual void scan_archive( const string& aPathname );

       // Tries to find packages providing features specified and
       // install them. Returns number of packages installed.
     virtual size_t find_and_install_features( Pointer<Database> database, const FeatureList& features, Pointer<ProgressIndicatorStack> aProgressIndicatorStack );

  private:
     struct Entry
     {
          Feature feature;
          string url; // scanned url
     };

     list<Entry> mEntries;

     void add_feature( const Feature& aFeature, const string& aPathname );

       // Returns entry with the specified feature of 0 if not found.
       // If exact looks for entry with excatly the same feature (compare versions etc.)
       // otherwise find entry with most recent feature.
     list<Entry>::const_iterator entry_by_feature( const Feature& aFeature, bool exact );

       // loads database from file if it exists
       // returns true if file exists and loaded
     bool load_file( const string& aUrl );

     static const char* sDatabaseFile;

}; // class FeaturesProvided

//======================================================================

#endif // FeaturesProvided_hh
