/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/History.hh,v 1.2 2001/06/22 15:05:16 alex Exp $
 */

// -*- C++ -*-  Wed Feb  3 1999
// Eugene Skepner <eu@qub.com>
// $Id: History.hh,v 1.1 1999/02/03 14:43:05 eugene Exp $

#ifndef History_hh
#define History_hh

#ifndef LanguageInterface_hh
#include "LanguageInterface.hh"
#endif

#ifndef Container_hh
#include "Container.hh"
#endif

#include <ctime>

//======================================================================

class History : virtual public LanguageInterface::Object
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     History( const string& aPackageName, const string& aUpdateType )
          : mPackageName( aPackageName ), mUpdateType( aUpdateType ), mTime( ::time( 0 ) ) {}
     History( const string& aPackageName, const string& aUpdateType, time_t aTime )
          : mPackageName( aPackageName ), mUpdateType( aUpdateType ), mTime( aTime ) {}

     const string& name() const { return mPackageName; }
     const string& type() const { return mUpdateType; }
     time_t time() const { return mTime; }

     static inline bool lessByTime( Pointer<History> a, Pointer<History> b )
          {
               return a->time() < b->time();
          }

  private:
     string mPackageName;
     string mUpdateType;
     time_t mTime;

}; // class History

//======================================================================

typedef Container<list<Pointer<History> > > HistoryList;

//======================================================================

#endif // History_hh
