/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/Package.cd.cc,v 1.2 2001/06/22 15:05:16 alex Exp $
 */

// -*- C++ -*-   Fri Dec 25 1998
// Eugene Skepner <eu@qub.com>
// $Id: Package.cd.cc,v 1.5 1999/01/14 15:10:53 eugene Exp $

#ifndef Package_hh
#include "Package.hh"
#endif

//======================================================================

LanguageInterface::Object::MethodDescription Package::sMethods[] =
{
     MethodDescription( "name", "name",
                        LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
                        "Gets or Sets the package name",
                        MethodDescription::vtString, MethodDescription::Value( MethodDescription::vtString, 0, "optional-get-or-set" ) ),
     MethodDescription( "installation_time", "installation_time",
                        LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
                        "Gets or Sets the package installation time",
                        MethodDescription::vtUInt, MethodDescription::Value( MethodDescription::vtUInt, 0, "optional-get-or-set" ) ),
     MethodDescription( "features", "features",
                        LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
                        "Sets or gets list of features provided by the package.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "FeatureList" ), MethodDescription::Value( MethodDescription::vtListRef, "FeatureList", "optional-get-or-set", "features" ) ),
     MethodDescription( "requires", "requires",
                        LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
                        "Sets or gets list of features required by the package.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "FeatureList" ), MethodDescription::Value( MethodDescription::vtListRef, "FeatureList", "optional-get-or-set", "features" ) ),
     MethodDescription( "conflicts", "conflicts",
                        LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
                        "Sets or gets list of conflicting features.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "FeatureList" ), MethodDescription::Value( MethodDescription::vtListRef, "FeatureList", "optional-get-or-set", "features" ) ),
     MethodDescription( "could_be_uninstalled", "could_be_uninstalled",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns if a package providing the specified features could be uninstalled, i.e this package does not require neither of the specified features.",
                        MethodDescription::vtBool, MethodDescription::Value( MethodDescription::vtListRef, "FeatureList", 0, "features" ) ),
     MethodDescription( "could_be_uninstalled_proc", "could_be_uninstalled_proc",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Sets could_be_uninstalled_proc code.",
                        MethodDescription::vtVoid, MethodDescription::Value( MethodDescription::vtString, 0, 0, "proc" ) ),
     MethodDescription( "could_be_installed", "could_be_installed",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns if a package providing the specified feature could be installed, i.e this package does not conflict with either of the specified features.",
                        MethodDescription::vtBool, MethodDescription::Value( MethodDescription::vtObject, "Feature", 0, "feature" ) ),
     MethodDescription( "could_be_installed_proc", "could_be_installed_proc",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Sets could_be_installed_proc code.",
                        MethodDescription::vtVoid, MethodDescription::Value( MethodDescription::vtString, 0, 0, "proc" ) ),
     MethodDescription( "description", "description",
                        LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet,
                        "Sets or gets package description.",
                        MethodDescription::Value( MethodDescription::vtObject, "PackageDescription" ), MethodDescription::Value( MethodDescription::vtObject, "PackageDescription", "optional-get-or-set", "description" ) ),
     MethodDescription( "files", "files",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns list of files owned by the package.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "PackageFileList" ), MethodDescription::vtVoid ),
     MethodDescription( "number_of_files", "number_of_files",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns number of files owned by the package.",
                        MethodDescription::vtUInt, MethodDescription::vtVoid ),
     MethodDescription( "disown", "disown",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Make the package to not own the specified file.",
                        MethodDescription::vtVoid, MethodDescription::Value( MethodDescription::vtString, 0, 0, "pathname" ) ),
     MethodDescription( "own", "own",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Make the package to own the specified file.",
                        MethodDescription::vtVoid, MethodDescription::Value( MethodDescription::vtString, 0, 0, "pathname" ) ),
     MethodDescription( "owns", "owns",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns PackageFile object if the specified file is owned by the package.",
                        MethodDescription::Value( MethodDescription::vtObject, "PackageFile" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "pathname" ) ),
     MethodDescription( "has_undo_info", "has_undo_info",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns if undo info is saved for the package.",
                        MethodDescription::vtBool, MethodDescription::vtVoid ),
     MethodDescription( "undo", "undo",
                        LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fSafeExecution,
                        "Undoes package installation.",
                        MethodDescription::vtVoid, MethodDescription::vtVoid ),
     MethodDescription( "uninstall_proc", "uninstall_proc",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Sets uninstall proc code.",
                        MethodDescription::vtVoid, MethodDescription::Value( MethodDescription::vtString, 0, 0, "proc" ) ),
     MethodDescription( "has_uninstall_proc", "has_uninstall_proc",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns if proc uninstall exists for the package.",
                        MethodDescription::vtBool, MethodDescription::vtVoid ),
     MethodDescription( "uninstall", "uninstall",
                        LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fSafeExecution,
                        "Uninstalls the package.",
                        MethodDescription::vtVoid, MethodDescription::Value( MethodDescription::vtBool, 0, "false", "no_log" ) ),
     MethodDescription( "uninstall_file", "uninstall_file",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Uninstalls the file.",
                        MethodDescription::vtVoid, MethodDescription::Value( MethodDescription::vtObject, "PackageFile", 0, "file" ) ),
     MethodDescription( "remove", "remove",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Removes from the database all information regarding package.",
                        MethodDescription::vtVoid, MethodDescription::vtVoid ),
     MethodDescription( "exists", "exists",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns if this package records are still exist in the database.",
                        MethodDescription::vtBool, MethodDescription::vtVoid ),
     MethodDescription( "progress_indicator_stack", "progress_indicator_stack",
                        LanguageInterface::Object::MethodDescription::fDynamic | LanguageInterface::Object::MethodDescription::fGetOrSet | LanguageInterface::Object::MethodDescription::fSafeExecution,
                        "Sets or gets progress_indicator_stack.",
                        MethodDescription::Value( MethodDescription::vtObject, "ProgressIndicatorStack" ), MethodDescription::Value( MethodDescription::vtObject, "ProgressIndicatorStack", "optional-get-or-set", "progress_indicator_stack" ) ),
     MethodDescription( "consistency_check", "consistency_check",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns list of differences between database record for the package files and these files on disk.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "PackageFile::ConsistencyList" ),
                        MethodDescription::vtVoid ),
     MethodDescription()
};

LanguageInterface::Object::CallbackDescription Package::sCallbacks[] =
{
     CallbackDescription( "Package::Call_could_be_uninstalled", "database()", "package()", "features()" ),
     CallbackDescription( "Package::Call_could_be_installed", "database()", "package()", "feature()" ),
     CallbackDescription( "Package::Call_uninstall", "database()", "package()" ),
     CallbackDescription()
};

LanguageInterface::Object::ClassDescription Package::sClassDescription(
     "Package", "Package",
     "Package interface",
     sMethods,
     sCallbacks
);

//======================================================================
