/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/PackageCreator.hh,v 1.3 2001/07/17 18:29:21 alex Exp $
 */

// -*- C++ -*-  Mon Aug  3 1998
// $Id: PackageCreator.hh,v 1.4 1998/12/30 08:40:15 eugene Exp $

#ifndef PackageCreator_hh
#define PackageCreator_hh

#ifndef PackageStreamFile_hh
#include "PackageStreamFile.hh"
#endif

#ifndef TclLanguageInterface_hh
#include "TclLanguageInterface.hh"
#endif

//======================================================================

class PackageCreator : virtual public LanguageInterface::Object
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     PackageCreator( const string& aPackageName, const string& aPackageDir, const string& aPackageFilesDir, Pointer<Container<list<string> > > aIgnore );

     virtual void generate();
     virtual void write_package_file();

     const string& name() const { return mPackageName; }
     const string& packagedir() const { return mPackageDir; }
     const string& dir() const { return mPackageFilesDir; }

     static Pointer<PackageCreator> make( const string& aPackageName, const string& aPackageDir, const string& aPackageFilesDir, Pointer<Container<list<string> > > aIgnore )
          { return new PackageCreator( aPackageName, aPackageDir, aPackageFilesDir, aIgnore ); }

  protected:
     const PackageStreamFileList& files() const { return mFiles; }

     Pointer<TclLanguageInterface::SlaveInterpreter> interpreter() { return mSlaveInterp; }

  private:
     string mPackageName;
     string mPackageDir;
     string mPackageFilesDir;
     Pointer<Container<list<string> > > mIgnore;
     PackageStreamFileList mFiles;
     Pointer<TclLanguageInterface::SlaveInterpreter> mSlaveInterp;

     void scan_files( const string& name, const string& basename );
     void scan_scripts();

     void add_file( Pointer<RealFile> aFile, const string& name, const string& basename );

     PackageCreator( const PackageCreator& ) {}
     PackageCreator& operator = ( const PackageCreator& ) { return *this; }

     static const char* SYSINSTALL_DIR;

     struct SysinstallScript : virtual public Shared
     {
          typedef string (*Generate)( const string& aName, PackageCreator& packageCreator );
          typedef void (*Check)( const string& aProcName, const string& aNamespace, PackageCreator& packageCreator );

          SysinstallScript( const char* filename, const char** procs, const char** vars, const char* helper_code, Generate generate, Check check )
               : filename( filename ), procs( procs ), vars( vars ), helper_code( const_cast<char*>( helper_code ) ), generate( generate ), check( check ) {}

          const char* filename;
          const char** procs;
          const char** vars;
          char* helper_code;
          Generate generate;
          Check check;
     };

     static const SysinstallScript sScripts[];

     struct SysinstallRealScript : public SysinstallScript
     {
          SysinstallRealScript() : SysinstallScript( 0, 0, 0, 0, 0, 0 ), absentProcs( 0 ) { }
          SysinstallRealScript( const SysinstallScript& ss ) : SysinstallScript( ss ), absentProcs( 0 ) { }
//            SysinstallRealScript& operator = ( const SysinstallScript& ss )
//                 {
//                      SysinstallScript::operator=( ss );
//                      return *this;
//                 }

          list<string> absentProcs;

       private:
          SysinstallRealScript& operator = ( const SysinstallScript& ) { return *this; }
     };

     typedef Container<list<Pointer<SysinstallRealScript> > > SysinstallRealScriptList;

     Pointer<SysinstallRealScriptList> sourceExistingScripts( const string& aNamespace, const string& aDir );
     void generateAbsentProcs( const string& aNamespace, SysinstallRealScriptList& scripts );
     void writeAbsentProcs( const string& aDir, SysinstallRealScriptList& scripts );
     void checkProcs( const string& aNamespace );

     static string generate_description( const string& aProcName, PackageCreator& packageCreator );
     static string generate_features( const string& aProcName, PackageCreator& packageCreator );
     static string generate_structure( const string& aProcName, PackageCreator& packageCreator );
     static string generate_install( const string& aProcName, PackageCreator& packageCreator );

     static void check_description( const string& aProcName, const string& aNamespace, PackageCreator& packageCreator );
     static void check_features( const string& aProcName, const string& aNamespace, PackageCreator& packageCreator );
     static void check_structure( const string& aProcName, const string& aNamespace, PackageCreator& packageCreator );
     static void check_install( const string& aProcName, const string& aNamespace, PackageCreator& packageCreator );

}; // class PackageCreator

//======================================================================

#endif // PackageCreator_hh
