/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/PackageDescription.hh,v 1.2 2001/06/22 15:05:17 alex Exp $
 */

// -*- C++ -*-  Wed Aug  5 1998
// $Id: PackageDescription.hh,v 1.1 1998/12/25 18:36:36 eugene Exp $

#ifndef PackageDescription_hh
#define PackageDescription_hh

#ifndef PackageStreamReference_hh
#include "PackageStreamReference.hh"
#endif

//======================================================================

class PackageDescription : virtual public LanguageInterface::Object
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     PackageDescription( const string& aSummary, const string& aDescription, const string& aAuthor, const string& aCanonicalName, Pointer<PackageStreamReferenceList> aReferences )
          : mSummary( aSummary ), mDescription( aDescription ), mAuthor( aAuthor ), mCanonicalName( aCanonicalName ), mReferences( aReferences ) {}

     virtual const string& summary() const { return mSummary; }
     virtual const string& description() const { return mDescription; }
     virtual const string& author() const { return mAuthor; }
     virtual const string& canonical_name() const { return mCanonicalName; }
     virtual const Pointer<PackageStreamReferenceList> references() const { return mReferences; }

     virtual bool check_integrity() const { return false; }

     static Pointer<PackageDescription> make( const string& aSummary, const string& aDescription, const string& aAuthor, const string& aCanonicalName, Pointer<PackageStreamReferenceList> aReferences )
          { return new PackageDescription( aSummary, aDescription, aAuthor, aCanonicalName, aReferences ); }

   private:
     string mSummary;
     string mDescription;
     string mAuthor;
     string mCanonicalName;
     Pointer<PackageStreamReferenceList> mReferences;

}; // class PackageDescription

//======================================================================

#endif // PackageDescription_hh
