/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/PackageFile.cc,v 1.2 2001/06/22 15:05:17 alex Exp $
 */

// -*- C++ -*-   Thu Aug 13 1998
// $Id: PackageFile.cc,v 1.3 1999/01/10 19:02:00 eugene Exp $

#ifndef PackageFile_hh
#include "PackageFile.hh"
#endif

#ifndef File_hh
#include "File.hh"
#endif

#ifndef String_hh
#include "String.hh"
#endif

#include <cerrno>

//======================================================================

Pointer<PackageFile::Consistency> PackageFile::check() const
{
     if ( !File::exist( pathname() ) ) {
          return new Consistency( const_cast<PackageFile*>( this ), errno, 0, true );
     }
     else {
          Pointer<RealFile> rf = new RealFile( pathname() );
          return new Consistency( const_cast<PackageFile*>( this ), 0, rf, differs( *rf ) );
     }

} // PackageFile::check

//======================================================================

bool PackageFile::differs( const RealFile& aRealFile ) const
{
     const bool d = length() != aRealFile.length()
               || mtime() != aRealFile.mtime()
               || user() != aRealFile.user()
               || group() != aRealFile.group()
               || mode() != aRealFile.mode()
               || symlink() != aRealFile.symlink()
               || dev() != aRealFile.dev()
               || md5() != aRealFile.md5();

     return d;

} // PackageFile::differs

//======================================================================

Pointer<PackageFile::DifferenceList> PackageFile::Consistency::differences() const
{
     Pointer<PackageFile::DifferenceList> diffs = new PackageFile::DifferenceList;
     if ( mLstatErrno != 0 ) {
          if ( mLstatErrno == ENOENT )
               diffs->push_back( new Difference( "absent", "", "" ) );
          else if ( mLstatErrno == EACCES )
               diffs->push_back( new Difference( "permissions-denied", "", "" ) );
          else
               diffs->push_back( new Difference( "unknown-problem", "", "" ) );
     }
     else if ( mPackageFile && mRealFile ) {
          if ( mPackageFile->length() != mRealFile->length() )
               diffs->push_back( new Difference( "length", to_string( mPackageFile->length() ), to_string( mRealFile->length() ) ) );

          if ( mPackageFile->mtime() != mRealFile->mtime() )
               diffs->push_back( new Difference( "mtime", to_string( mPackageFile->mtime() ), to_string( mRealFile->mtime() ) ) );

          if ( mPackageFile->user() != mRealFile->user() )
               diffs->push_back( new Difference( "user", mPackageFile->user(), mRealFile->user() ) );

          if ( mPackageFile->group() != mRealFile->group() )
               diffs->push_back( new Difference( "group", mPackageFile->group(), mRealFile->group() ) );

          if ( mPackageFile->mode() != mRealFile->mode() )
               diffs->push_back( new Difference( "mode", to_string_octal( mPackageFile->mode() ), to_string_octal( mRealFile->mode() ) ) );

          if ( mPackageFile->symlink() != mRealFile->symlink() )
               diffs->push_back( new Difference( "symlink", mPackageFile->symlink(), mRealFile->symlink() ) );

          if ( mRealFile->dev() != mPackageFile->dev() )
               diffs->push_back( new Difference( "device", to_string( major( mPackageFile->dev() ), minor( mPackageFile->dev() ) ), to_string( major( mRealFile->dev() ), minor( mRealFile->dev() ) ) ) );

          if ( mPackageFile->md5() != mRealFile->md5() )
               diffs->push_back( new Difference( "md5", mPackageFile->md5(), mRealFile->md5() ) );
     }
     else
          throw runtime_error( "Internal: PackageFile::Consistency::difference" );

     return diffs;

} // PackageFile::Consistency::difference

//======================================================================

Pointer<Container<list<string> > > PackageFile::Difference::description() const
{
     Pointer<Container<list<string> > > desc = new Container<list<string> >;
     desc->push_back( mElementName );
     desc->push_back( mNewValue );
     desc->push_back( mOldValue );
     return desc;

} // PackageFile::Difference::description

//======================================================================
