/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/PackageFile.hh,v 1.2 2001/06/22 15:05:17 alex Exp $
 */

// -*- C++ -*-  Thu Aug 13 1998
// $Id: PackageFile.hh,v 1.3 1999/01/10 19:02:00 eugene Exp $

#ifndef PackageFile_hh
#define PackageFile_hh

#ifndef FileInfo_hh
#include "FileInfo.hh"
#endif

#ifndef RealFile_hh
#include "RealFile.hh"
#endif

//======================================================================

class PackageFile : public FileInfo
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     PackageFile( const FileInfo& aFile ) : FileInfo( aFile ) {}
     PackageFile( const PackageFile& aFile ) : FileInfo( aFile ) {}

     class Consistency;

     class Difference : virtual public LanguageInterface::Object
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
          Pointer<Container<list<string> > > description() const;

       private:
          Difference( const string& aName, const string& aOld, const string& aNew ) : mElementName( aName ), mOldValue( aOld ), mNewValue( aNew ) {}

          string mElementName;
          string mOldValue;
          string mNewValue;

          friend class Consistency;
     };

     typedef Container<list<Pointer<Difference> > > DifferenceList;

     class Consistency : virtual public LanguageInterface::Object
     {
          CLASS_METHODS_DESCRIPTIONS_DECLARATION;

       public:
          Consistency( Pointer<PackageFile> aPackageFile, int aLstatErrno, Pointer<RealFile> aRealFile, bool aDiffers )
               : mPackageFile( aPackageFile ), mLstatErrno( aLstatErrno ), mRealFile( aRealFile ), mDiffers( aDiffers ) {}

          virtual bool differs() const { return mDiffers; }
          virtual Pointer<DifferenceList> differences() const;
          virtual Pointer<PackageFile> file() const { return mPackageFile; }

       private:
          Pointer<PackageFile> mPackageFile;
          int  mLstatErrno;
          Pointer<RealFile> mRealFile;
          bool mDiffers;
     };

     typedef Container<list<Pointer<Consistency> > > ConsistencyList;

     Pointer<Consistency> check() const;

  private:
     virtual bool differs( const RealFile& aRealFile ) const;

}; // class PackageFile

//======================================================================

typedef Container<list<Pointer<PackageFile> > > PackageFileList;

//======================================================================

#endif // PackageFile_hh
