/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/PackageStream.cc,v 1.3 2001/07/17 21:19:41 alex Exp $
 */

// -*- C++ -*-   Wed Aug 19 1998
// $Id: PackageStream.cc,v 1.21 1999/02/03 14:43:05 eugene Exp $

#ifndef PackageStream_hh
#include "PackageStream.hh"
#endif

#ifndef PackageDescription_hh
#include "PackageDescription.hh"
#endif

#ifndef PackageStreamStructure_hh
#include "PackageStreamStructure.hh"
#endif

#ifndef Configuration_hh
#include "Configuration.hh"
#endif

#ifndef PackageFile_hh
#include "PackageFile.hh"
#endif

#ifndef Callback_hh
#include "Callback.hh"
#endif

#ifndef FeaturesProvided_hh
#include "FeaturesProvided.hh"
#endif

#ifndef Log_hh
#include "Log.hh"
#endif

#ifndef Syslog_hh
#include "Syslog.hh"
#endif

#ifndef H_hh
#include "H.hh"
#endif

#ifndef History_hh
#include "History.hh"
#endif

#include <cctype>
#include <cerrno>
#include <unistd.h>

//======================================================================

const char* PackageStream::description_helper_procs =
"\
variable description_name_internal\n\
variable description_summary_internal\n\
variable description_description_internal\n\
variable description_author_internal\n\
\n\
proc name {n} {variable description_name_internal; set description_name_internal $n}\n\
proc summary {v} {variable description_summary_internal; set description_summary_internal $v}\n\
proc description {v} {variable description_description_internal; set description_description_internal $v}\n\
proc author {v} {variable description_author_internal; set description_author_internal $v}\n\
\n\
proc package_description {} {variable description_summary_internal; variable description_description_internal; variable description_author_internal; PackageDescription $description_summary_internal $description_description_internal $description_author_internal}\n\
proc package_name {} {variable description_name_internal; return $description_name_internal}\n\
";

const char* PackageStream::features_helper_procs =
"\
";

const char* PackageStream::structure_helper_procs =
"\
";

const char* PackageStream::install_helper_procs =
"\
";

//======================================================================

PackageStream::PackageStream( Pointer<Archive> aStream, const string& aSlaveInterpreterPrefix, Pointer<ProgressIndicatorStack> aProgressIndicatorStack )
     : mStream( aStream ), mNameSpace( aStream->url() ), mSlaveInterpreterPrefix( aSlaveInterpreterPrefix ),
       mProgressIndicatorStack( aProgressIndicatorStack ? aProgressIndicatorStack : Pointer<ProgressIndicatorStack>( new ProgressIndicatorStack() ) )
{
     replace_if( mNameSpace.begin(), mNameSpace.end(), not1( ptr_fun( &isalnum ) ), '_' );
     mSlaveInterp = tclLanguageInterface->make_slave_interpreter( mSlaveInterpreterPrefix + mNameSpace, true );

} // PackageStream::PackageStream

//======================================================================

PackageStream::~PackageStream()
{
} // PackageStream::~PackageStream

//======================================================================

Pointer<PackageStream> PackageStream::open( Pointer<Archive> aStream, const string& aSlaveInterpreterPrefix, Pointer<ProgressIndicatorStack> aProgressIndicatorStack )
{
     return new PackageStream( aStream, aSlaveInterpreterPrefix, aProgressIndicatorStack );

} // PackageStream::open

//======================================================================

const string& PackageStream::name_space() const
{
     return mNameSpace;

} // PackageStream::name_space

//======================================================================

string PackageStream::name()
{
     source_description();
     PackageStream_CallPackageSuppliedProc<string> call( new TclLanguageInterface::CallbackDataSlaveInterpreter( slave(), string( "::" ) + name_space() + "::package_name" ) );
     call.activate();
     return call.result();

} // PackageStream::name

//======================================================================

Pointer<PackageDescription> PackageStream::description()
{
     source_description();
     PackageStream_CallPackageSuppliedProc<Pointer<PackageDescription> > call( new TclLanguageInterface::CallbackDataSlaveInterpreter( slave(), string( "::" ) + name_space() + "::package_description" ) );
     call.activate();
     return call.result();

} // PackageStream::description

//======================================================================

Pointer<FeatureList> PackageStream::features()
{
     source_features();
     PackageStream_CallPackageSuppliedProc<Pointer<FeatureList> > call( new TclLanguageInterface::CallbackDataSlaveInterpreter( slave(), string( "::" ) + name_space() + "::provides" ) );
     call.activate();
     return call.result();

} // PackageStream::features

//======================================================================

Pointer<FeatureList> PackageStream::requires()
{
     source_features();
     PackageStream_CallPackageSuppliedProc<Pointer<FeatureList> > call( new TclLanguageInterface::CallbackDataSlaveInterpreter( slave(), string( "::" ) + name_space() + "::requires" ) );
     call.activate();
     return call.result();

} // PackageStream::requires

//======================================================================

Pointer<FeatureList> PackageStream::conflicts()
{
     source_features();
     PackageStream_CallPackageSuppliedProc<Pointer<FeatureList> > call( new TclLanguageInterface::CallbackDataSlaveInterpreter( slave(), string( "::" ) + name_space() + "::conflicts" ) );
     call.activate();
     return call.result();

} // PackageStream::conflicts

//======================================================================

string PackageStream::could_be_uninstalled_proc()
{
     source_features();
     PackageStream_CallPackageSuppliedProc<string> call( new TclLanguageInterface::CallbackDataSlaveInterpreter( slave(), string( "::" ) + name_space() + "::could_be_uninstalled_proc" ) );
     call.activate();
     return call.result();

} // PackageStream::could_be_uninstalled_proc

//======================================================================

string PackageStream::could_be_installed_proc()
{
     source_features();
     PackageStream_CallPackageSuppliedProc<string> call( new TclLanguageInterface::CallbackDataSlaveInterpreter( slave(), string( "::" ) + name_space() + "::could_be_installed_proc" ) );
     call.activate();
     return call.result();

} // PackageStream::could_be_installed_proc

//======================================================================

Pointer<PackageStreamStructure> PackageStream::structure()
{
     source_structure();
     PackageStream_CallPackageSuppliedProc<Pointer<PackageStreamStructure> > call( new TclLanguageInterface::CallbackDataSlaveInterpreter( slave(), string( "::" ) + name_space() + "::structure" ) );
     call.activate();
     return call.result();

} // PackageStream::structure

//======================================================================

void PackageStream::source()
{
     source_description();
     source_features();
     source_structure();
     source_install();

} // PackageStream::source

//======================================================================

void PackageStream::install( Pointer<Database> database )
{
     Pointer<PackageStreamStructure> package_structure = structure();
     size_t subpackages_files = 0;
     for ( PackageStreamSubpackageList::const_iterator subpackage = package_structure->subpackages()->begin(); subpackage != package_structure->subpackages()->end(); ++subpackage )
          subpackages_files += (*subpackage)->files()->size();

     ProgressIndicatorPusher progressIndicatorPusher( mProgressIndicatorStack, name(), ProgressIndicator::ProgressInfo( package_structure->files()->size() + subpackages_files, 0, package_structure->has_references(), 4 ) );

     update_features_provided_database();

     progress().progress( ProgressIndicator::itAdditional );

     source();
     mDatabase = database;

     progress().progress( ProgressIndicator::itAdditional );

     Pointer<Package> package = database->new_package();

     progress().progress( ProgressIndicator::itAdditional );

     try {
          PackageStream_CallPackageInstall call( new TclLanguageInterface::CallbackDataSlaveInterpreter( slave(), string( "::" ) + name_space() + "::install" ), database, this, package );
          call.activate();
          if ( !call.result().empty() )
               throw runtime_error( string( "Installation of " ) + stream().url() + " aborted: " + call.result() );
     }
     catch ( runtime_error& err ) {
          mDatabase = 0;
          string message = err.what();
          try {
               package->progress_indicator_stack( mProgressIndicatorStack );
               if ( package->has_undo_info() ) {
                    try {
                         package->undo();
                    }
                    catch ( exception& err2 ) {
                         LOG( ERROR, "Undoing of " << package->name() << " failed: " << err2.what() );
                    }
               }
               else if ( package->has_uninstall_proc() ) {
                    try {
                         package->uninstall( true ); // true means do not log uninstallation completion
                    }
                    catch ( exception& err2 ) {
                         LOG( ERROR, "Uninstallation of " << package->name() << " failed: " << err2.what() );
                    }
               }
               else {
                    try {
                         LOG( INFO, "Neither undo nor uninstall info was created for package " << name() << ", just erasing all information regarding this package from the database" );
                         package->remove();
                    }
                    catch ( exception& err2 ) {
                         LOG( ERROR, "Removing of " << package->name() << " failed: " << err2.what() );
                    }
               }
          }
          catch ( exception& err1 ) {
               message += string( " (" ) + err1.what() + ")";
          }
          throw runtime_error( message );
     }
     mDatabase = 0;

     progress().progress( ProgressIndicator::itAdditional );

     progress().progress( 1.0 );

     Syslog syslog;
     syslog.message( string( "Package " ) + name() + " installed" );

     database->updateHistory( History( name(), "installed" ) );
     LOG( INFO, "Package " << name() << " installed" );

} // PackageStream::install

//======================================================================

void PackageStream::upgrade( Pointer<Database> database )
{
     Pointer<PackageStreamStructure> package_structure = structure();
     size_t subpackages_files = 0;
     for ( PackageStreamSubpackageList::const_iterator subpackage = package_structure->subpackages()->begin(); subpackage != package_structure->subpackages()->end(); ++subpackage )
          subpackages_files += (*subpackage)->files()->size();
     ProgressIndicatorPusher progressIndicatorPusher( mProgressIndicatorStack, name(), ProgressIndicator::ProgressInfo( package_structure->files()->size() + subpackages_files, package_structure->has_references(), 6 ) );

     update_features_provided_database();

     progress().progress( ProgressIndicator::itAdditional );

       // Find packages providing other versions of features provided
       // by the package being installed
     PackageList packages_to_uninstall;
     Pointer<FeatureList> provided_features = features();
     for ( FeatureList::const_iterator f = provided_features->begin(); f != provided_features->end(); ++f ) {
          Pointer<FeatureList> other_versions = database->feature_other_versions( **f, "provide" );
          for ( FeatureList::const_iterator ovf = other_versions->begin(); ovf != other_versions->end(); ++ovf ) {
               Pointer<Package> package = database->package( **ovf );
               if ( package )
                    packages_to_uninstall.push_back( package );
          }
     }

     progress().progress( ProgressIndicator::itAdditional );

       // Run proc upgrade supplied with the package
     source();
     mDatabase = database;

     progress().progress( ProgressIndicator::itAdditional );

     Pointer<Package> package = database->new_package();

     progress().progress( ProgressIndicator::itAdditional );

     try {
          PackageStream_CallPackageInstall call( new TclLanguageInterface::CallbackDataSlaveInterpreter( slave(), string( "::" ) + name_space() + "::upgrade" ), database, this, package );
          call.activate();
          if ( !call.result().empty() )
               throw runtime_error( string( "Upgrade of " ) + stream().url() + " aborted: " + call.result() );
     }
     catch ( runtime_error& err ) {
          mDatabase = 0;
          string message = err.what();
          try {
               package->progress_indicator_stack( mProgressIndicatorStack );
               if ( package->has_undo_info() ) {
                    try {
                         package->undo();
                    }
                    catch ( exception& err2 ) {
                         LOG( ERROR, "Undoing of " << package->name() << " failed: " << err2.what() );
                    }
               }
               else if ( package->has_uninstall_proc() ) {
                    try {
                         package->uninstall();
                    }
                    catch ( exception& err2 ) {
                         LOG( ERROR, "Uninstallation of " << package->name() << " failed: " << err2.what() );
                    }
               }
               else {
                    try {
                         LOG( INFO, "Neither undo nor uninstall info was created for package " << name() << ", just erasing all information regarding this package from the database" );
                         package->remove();
                    }
                    catch ( exception& err2 ) {
                         LOG( ERROR, "Removing of " << package->name() << " failed: " << err2.what() );
                    }
               }
          }
          catch ( runtime_error& err1 ) {
               message += string( " (" ) + err1.what() + ")";
          }
          throw runtime_error( message );
     }
     mDatabase = 0;

     progress().progress( ProgressIndicator::itAdditional );

       // Uninstall upgraded packages
     string old_name;
     for ( PackageList::iterator p = packages_to_uninstall.begin(); p != packages_to_uninstall.end(); ++p ) {
          if ( (*p)->exists() ) {
               if ( old_name.length() > 0 )
                    old_name += ", ";
               old_name += (*p)->name();

               (*p)->progress_indicator_stack( mProgressIndicatorStack );
               (*p)->uninstall();
          }
     }

     progress().progress( ProgressIndicator::itAdditional );

     progress().progress( 1.0 );

     Syslog syslog;
     syslog.message( string( "Package " ) + old_name + " upgraded to " + name() );

     database->updateHistory( History( name(), "upgraded" ) );
     LOG( INFO, "Package " << old_name << " upgraded to " << name() );

} // PackageStream::upgrade

//======================================================================

string PackageStream::prefix()
{
     if ( mPrefix.length() == 0 ) {
          if ( Configuration::instance()->usePackagePrefix() )
               mPrefix = structure()->default_prefix();
          else
               mPrefix = Configuration::instance()->prefix();
          LOG( INFO, "Prefix: " << mPrefix << "\n" );
     }
     return mPrefix;

} // PackageStream::prefix

//======================================================================

void PackageStream::create_leading_dirs( Pointer<PackageStreamFile> aFile )
{
     const string pathname = aFile->pathname();
     size_t pos = pathname.rfind( '/' );
     string dir_to_create = prefix();
     if ( pos != string::npos && pos != 0 ) {
          dir_to_create += "/";
          dir_to_create += pathname.substr( 0, pos );
     }

     if ( Configuration::instance()->no_action() ) {
          LOG( NO_ACTION, "Create directory: " << dir_to_create );
     }
     else {
            //LOG( INFO, string( "About to create directory " ) + dir_to_create + "\n" );
          File::make_path( dir_to_create );
     }

} // PackageStream::create_leading_dirs

//======================================================================

string PackageStream::create_symlink( Pointer<PackageStreamFile> aFile )
{
     LOG( INFO, "About to create symlink " << aFile->pathname() << "\n" );
     create_leading_dirs( aFile );
     directory_write_access( aFile );

     const string filename = file_pathname_for_installation( aFile );
     if ( Configuration::instance()->no_action() ) {
          LOG( NO_ACTION, "Create symlink: " << filename << " --> " << aFile->symlink() );
     }
     else {
          if ( ::symlink( aFile->symlink().c_str(), filename.c_str() ) != 0 )
               throw runtime_error( "Cannot create symlink " + filename + " --> " + aFile->symlink() + ": " + ::strerror( errno ) );

          RealFile rf( filename );
          rf.mtime( aFile->mtime() ); // jopa changes mtime of pointed file
          rf.user( aFile->user() );
          rf.group( aFile->group() );
     }

     progress().progress( ProgressIndicator::itFile );
     return filename;

} // PackageStream::create_symlink

//======================================================================

string PackageStream::create_device( Pointer<PackageStreamFile> aFile )
{
     LOG( INFO, "About to create device " << aFile->pathname() );
     create_leading_dirs( aFile );
     directory_write_access( aFile );

     const string filename = file_pathname_for_installation( aFile );
     if ( Configuration::instance()->no_action() ) {
          LOG( NO_ACTION, "Create device: " << filename );
     }
     else {
          if ( ::mknod( filename.c_str(), aFile->mode(), aFile->dev() ) != 0 )
               throw runtime_error( "Cannot create device " + filename + ": " + ::strerror( errno ) );

          RealFile rf( filename );
          rf.mtime( aFile->mtime() );
          rf.user( aFile->user() );
          rf.group( aFile->group() );
     }

     progress().progress( ProgressIndicator::itFile );
     return filename;

} // PackageStream::create_device

//======================================================================

string PackageStream::create_hardlink( Pointer<PackageStreamFile> aFile )
{
     LOG( INFO, "About to create hardlink " << aFile->pathname() );
     create_leading_dirs( aFile );
     directory_write_access( aFile );

     const string filename = file_pathname_for_installation( aFile );
     const string oldfile = prefix() + "/" + aFile->hardlink();
     if ( Configuration::instance()->no_action() ) {
          LOG( NO_ACTION, "Create hardlink: " << filename << " --> " << oldfile );
     }
     else {
          if ( link( oldfile.c_str(), filename.c_str() ) != 0 )
               throw runtime_error( "Cannot create hardlink " + filename + " --> " + oldfile + ": " + ::strerror( errno ) );
     }

     progress().progress( ProgressIndicator::itFile );
     return filename;

} // PackageStream::create_hardlink

//======================================================================

string PackageStream::install_file( Pointer<PackageStreamFile> aFile )
{
     LOG( INFO, "About to install file " << aFile->pathname() );
     create_leading_dirs( aFile );
     directory_write_access( aFile );

     const string filename = file_pathname_for_installation( aFile );
     if ( Configuration::instance()->no_action() ) {
          LOG( NO_ACTION, "Install file: " << filename );
     }
     else {
          aFile->write( stream(), filename );

          RealFile rf( filename );
          rf.mode( aFile->mode() );
          rf.user( aFile->user() );
          rf.group( aFile->group() );
          rf.mtime( aFile->mtime() );
     }

     progress().progress( ProgressIndicator::itFile );
     return filename;

} // PackageStream::install_file

//======================================================================

string PackageStream::patch_file( Pointer<PackageStreamFile> aFile )
{
     LOG( INFO, "About to patch file " << aFile->pathname() );
     directory_write_access( aFile );

     Pointer<Container<list<string> > > callback_result = Callback::file_patch( mDatabase, this, aFile->pathname() );
     string err;
     if ( callback_result && callback_result->size() > 0 ) {
          list<string>::const_iterator i = callback_result->begin();
          err = *++i;
     }
     if ( err.length() > 0 )
          throw runtime_error( err );

     const string filename = file_pathname_for_installation( aFile );
     unsigned long patch_length = 0;
     void* patch_data = stream().extract( aFile->pathname(), patch_length );
     if ( patch_data == 0 )
          throw runtime_error( string( "Unable to extract " ) + aFile->pathname() + " from package file: unknown error" );
     try {
          if ( patch_length == 0 )
               throw runtime_error( string( "Unable to extract " ) + aFile->pathname() + " from package file: file has zero length" );
          File::patch( filename, patch_data, patch_length );
     }
     catch ( ... ) {
          delete patch_data;
          throw;
     }
     delete patch_data;

     progress().progress( ProgressIndicator::itFile );
     return filename;

} // PackageStream::patch_file

//======================================================================

string PackageStream::append_file( Pointer<PackageStreamFile> aFile, bool create )
{
     LOG( INFO, "About to append to file " << aFile->pathname() );
     directory_write_access( aFile );

     Pointer<Container<list<string> > > callback_result = Callback::file_append( mDatabase, this, aFile->pathname() );
     string err;
     if ( callback_result && callback_result->size() > 0 ) {
          list<string>::const_iterator i = callback_result->begin();
          err = *++i;
     }
     if ( err.length() > 0 )
          throw runtime_error( err );

     const string filename = file_pathname_for_installation( aFile );
     if ( Configuration::instance()->no_action() ) {
          LOG( NO_ACTION, "Append file: " << filename );
     }
     else {
          const bool existed = file_exists( aFile );
          aFile->append( stream(), filename, create );
          if ( create && !existed ) {
               RealFile rf( filename );
               rf.mode( aFile->mode() );
               rf.mtime( aFile->mtime() );
               rf.user( aFile->user() );
               rf.group( aFile->group() );
          }
     }

     progress().progress( ProgressIndicator::itFile );
     return filename;

} // PackageStream::append_file

//======================================================================

void PackageStream::install_references()
{
     progress().progress( ProgressIndicator::itReference );

} // PackageStream::install_references

//======================================================================

void PackageStream::source_description()
{
     if ( !command_exists( "description", name_space() ) ) {
  	  slave()->eval( description_helper_procs, strlen( description_helper_procs ), name_space() );
          source( "SYSINSTALL/description.tcl" );
     }

} // PackageStream::source_description

//======================================================================

void PackageStream::source_features()
{
     if ( !command_exists( "provides", name_space() ) ) {
          source( "SYSINSTALL/features.tcl" );
     }

} // PackageStream::source_features

//======================================================================

void PackageStream::source_structure()
{
     if ( !command_exists( "structure", name_space() ) ) {
          source( "SYSINSTALL/structure.tcl" );
     }

} // PackageStream::source_structure

//======================================================================

void PackageStream::source_install()
{
     if ( !command_exists( "install", name_space() ) ) {
          source( "SYSINSTALL/install.tcl" );
     }

} // PackageStream::source_install

//======================================================================

bool PackageStream::command_exists( const char* cmdName, const string& aNamespace )
{
     return slave()->command_exists( string( "::" ) + aNamespace + "::" + cmdName );

} // PackageStream::command_exists

//======================================================================

void PackageStream::source( const string& file )
{
     unsigned long len;
     void* text = stream().extract( file, len );
     try {
          slave()->eval( text, len, name_space() );
     }
     catch ( ... ) {
          delete text;
          throw;
     }
     delete text;

} // PackageStream::source

//======================================================================

void PackageStream::directory_write_access( Pointer<PackageStreamFile> aFile )
{
     Pointer<Container<list<string> > > result = Callback::directory_write_access( mDatabase, this, File::dirname( file_pathname_for_installation( aFile ) ) );
     string err;
     if ( result && result->size() > 0 ) {
          list<string>::const_iterator i = result->begin();
          err = *++i;
     }
     if ( err.length() > 0 )
          throw runtime_error( err );

} // PackageStream::directory_write_access

//======================================================================

bool PackageStream::file_exists( Pointer<PackageStreamFile> aFile )
{
     return File::exist( file_pathname_for_installation( aFile ) );

} // PackageStream::file_exists

//======================================================================

string PackageStream::file_pathname_for_installation( Pointer<PackageStreamFile> aFile )
{
     return prefix() + "/" + aFile->pathname();

} // PackageStream::file_pathname_for_installation

//======================================================================

void PackageStream::update_features_provided_database()
#warning "update_features_provided_database() does nothing that makes sense"
{
     try {
          FeaturesProvided::instance()->load_dir( File::dirname( stream().url() ), false );
#if 0
          if ( !Configuration::instance()->no_action() )
	       /* XXX This makes absolutely no sense:  It saves
                  a list of the features together with the filename
                  of the just created package in the directory,
                  where the package is stored as well.
                  I assume Eugen wanted to update the DB featuredatabase
                  with this information, and this was just debugging
                  stuff */
               /* XXX I #if 0 this for now, since it makes absolutely
                  no sense to create a dumb file in the directory.
                  One must implement this function later */
               FeaturesProvided::instance()->save( File::dirname( stream().url() ) );
#endif
     }
     catch ( ... ) {
            // Ignore errors
     }

} // PackageStream::update_features_provided_database

//======================================================================
