/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/PackageStream.cd.cc,v 1.2 2001/06/22 15:05:17 alex Exp $
 */

// -*- C++ -*-   Sun Dec 27 1998
// Eugene Skepner <eu@qub.com>
// $Id: PackageStream.cd.cc,v 1.6 1999/01/04 11:13:25 eugene Exp $

#ifndef PackageStream_hh
#include "PackageStream.hh"
#endif

//======================================================================

LanguageInterface::Object::MethodDescription PackageStream::sMethods[] =
{
     MethodDescription( "open", "open",
                        LanguageInterface::Object::MethodDescription::fStatic,
                        "Opens PackageStream for reading",
                        MethodDescription::Value( MethodDescription::vtObject, "PackageStream" ), MethodDescription::Value( MethodDescription::vtObject, "Archive", 0, "archive" ), MethodDescription::Value( MethodDescription::vtString, 0, 0, "slave_interpreter_prefix" ), MethodDescription::Value( MethodDescription::vtObject, "ProgressIndicatorStack", "Pointer<ProgressIndicatorStack>()", "progress_indicator_stack" ) ),
     MethodDescription( "name_space", "name_space",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns name of namespace where all procs supplied with the package file are to be sourced.",
                        MethodDescription::vtString, MethodDescription::vtVoid ),
     MethodDescription( "name", "name",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns package_name. Sources the SYSINSTALL/description.tcl script if not sourced yet and then calls package_namespace::name.",
                        MethodDescription::vtString, MethodDescription::vtVoid ),
     MethodDescription( "description", "description",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns package_description. Sources the SYSINSTALL/description.tcl script if not sourced yet and then calls package_namespace::description.",
                        MethodDescription::Value( MethodDescription::vtObject, "PackageDescription" ), MethodDescription::vtVoid ),
     MethodDescription( "features", "features",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns list of features provided by the package. Sources the SYSINSTALL/features.tcl script if not sourced yet and then calls package_namespace::provides.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "FeatureList" ), MethodDescription::vtVoid ),
     MethodDescription( "requires", "requires",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns list of features required by the package. Sources the SYSINSTALL/features.tcl script if not sourced yet and then calls package_namespace::requires.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "FeatureList" ), MethodDescription::vtVoid ),
     MethodDescription( "conflicts", "conflicts",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns list of conflicting features for the package. Sources the SYSINSTALL/features.tcl script if not sourced yet and then calls package_namespace::conflicts.",
                        MethodDescription::Value( MethodDescription::vtListPtr, "FeatureList" ), MethodDescription::vtVoid ),
     MethodDescription( "could_be_uninstalled_proc", "could_be_uninstalled_proc",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns string with proc code. Sources the SYSINSTALL/features.tcl script if not sourced yet.",
                        MethodDescription::vtString, MethodDescription::vtVoid ),
     MethodDescription( "could_be_installed_proc", "could_be_installed_proc",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns string with proc code. Sources the SYSINSTALL/features.tcl script if not sourced yet.",
                        MethodDescription::vtString, MethodDescription::vtVoid ),
     MethodDescription( "structure", "structure",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns package_structure. Sources the SYSINSTALL/structure.tcl script if not sourced yet and then calls package_namespace::structure.",
                        MethodDescription::Value( MethodDescription::vtObject, "PackageStreamStructure" ), MethodDescription::vtVoid ),
     MethodDescription( "source", "source",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Sources all not yet sourced scripts supplied with the package file.",
                        MethodDescription::vtVoid, MethodDescription::vtVoid ),
     MethodDescription( "install", "install",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Calls proc install supplied with package in the safe interpreter.",
                        MethodDescription::vtVoid,
                        MethodDescription::Value( MethodDescription::vtObject, "Database", 0, "database" ) ),
     MethodDescription( "upgrade", "upgrade",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Calls proc upgrade supplied with package in the safe interpreter.",
                        MethodDescription::vtVoid,
                        MethodDescription::Value( MethodDescription::vtObject, "Database", 0, "database" ) ),
     MethodDescription( "prefix", "prefix",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns installation prefix based on configuration.",
                        MethodDescription::vtString, MethodDescription::vtVoid ),
     MethodDescription( "create_symlink", "create_symlink",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        ".",
                        MethodDescription::vtString,
                        MethodDescription::Value( MethodDescription::vtObject, "PackageStreamFile", 0, "file" ) ),
     MethodDescription( "create_device", "create_device",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        ".",
                        MethodDescription::vtString,
                        MethodDescription::Value( MethodDescription::vtObject, "PackageStreamFile", 0, "file" ) ),
     MethodDescription( "create_hardlink", "create_hardlink",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        ".",
                        MethodDescription::vtString,
                        MethodDescription::Value( MethodDescription::vtObject, "PackageStreamFile", 0, "file" ) ),
     MethodDescription( "install_file", "install_file",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Installs the file attached to the package. Returns file pathname. If file already exists it is replaced.",
                        MethodDescription::vtString,
                        MethodDescription::Value( MethodDescription::vtObject, "PackageStreamFile", 0, "file" ) ),
     MethodDescription( "patch_file", "patch_file",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Patches original file using the patch attached to the package. Returns file pathname. If original file does not exist raises an exception.",
                        MethodDescription::vtString,
                        MethodDescription::Value( MethodDescription::vtObject, "PackageStreamFile", 0, "file" ) ),
     MethodDescription( "append_file", "append_file",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Append to original file using the data attached to the package. Returns file pathname. If original file does not exist and !create raises an exception.",
                        MethodDescription::vtString,
                        MethodDescription::Value( MethodDescription::vtObject, "PackageStreamFile", 0, "file" ), MethodDescription::Value( MethodDescription::vtBool, 0, 0, "create" ) ),
     MethodDescription( "install_references", "install_references",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Installs each reference containing in the package using a separate stream.",
                        MethodDescription::vtVoid, MethodDescription::vtVoid ),
     MethodDescription( "file_exists", "file_exists",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns if file exists.",
                        MethodDescription::vtBool,
                        MethodDescription::Value( MethodDescription::vtObject, "PackageStreamFile", 0, "file" ) ),
     MethodDescription( "file_pathname_for_installation", "file_pathname_for_installation",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        ".",
                        MethodDescription::vtString,
                        MethodDescription::Value( MethodDescription::vtObject, "PackageStreamFile", 0, "file" ) ),
     MethodDescription( "progress_indicator_stack", "progress_indicator_stack",
                        LanguageInterface::Object::MethodDescription::fDynamic,
                        "Returns progress indicator stack object used in this package stream.",
                        MethodDescription::Value( MethodDescription::vtObject, "ProgressIndicatorStack" ), MethodDescription::vtVoid ),
     MethodDescription()
};

LanguageInterface::Object::CallbackDescription PackageStream::sCallbacks[] =
{
     CallbackDescription( "PackageStream_CallPackageSuppliedProc<string>" ),
     CallbackDescription( "PackageStream_CallPackageSuppliedProc<Pointer<PackageDescription> >" ),
     CallbackDescription( "PackageStream_CallPackageSuppliedProc<Pointer<FeatureList> >" ),
     CallbackDescription( "PackageStream_CallPackageSuppliedProc<Pointer<PackageStreamStructure> >" ),
     CallbackDescription( "PackageStream_CallPackageInstall", "database()", "packageStream()", "package()" ),
     CallbackDescription()
};

LanguageInterface::Object::ClassDescription PackageStream::sClassDescription(
     "PackageStream", "PackageStream",
     "",
     sMethods,
     sCallbacks
);

//======================================================================
