/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/PackageStreamFile.cc,v 1.2 2001/06/22 15:05:17 alex Exp $
 */

// -*- C++ -*-   Mon Jul 27 1998
// $Id: PackageStreamFile.cc,v 1.3 1998/12/30 11:17:25 eugene Exp $

#ifndef PackageStreamFile_hh
#include "PackageStreamFile.hh"
#endif

#ifndef Archive_hh
#include "Archive.hh"
#endif

#ifndef String_hh
#include "String.hh"
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

//======================================================================

const PackageStreamFile::AttachmentToString PackageStreamFile::sAttachmentToString[] =
{
     { aGenerated, "" },
     { aFile, "file" },
     { aPatch, "patch" },
     { aAppend, "append" },
     { aUnknown, "unknown" }
};

const PackageStreamFile::ReplacementToString PackageStreamFile::sReplacementToString[] =
{
     { rDefault, "" },
     { rAlwaysReplace, "always_replace" },
     { rNeverReplace, "never_replace" },
     { rPatchIfOld, "patch_if_old" },
     { rUnknown, "unknown" }
};

//======================================================================

PackageStreamFile::PackageStreamFile( const FileInfo& aFile, const string& aAttachment, const string& aReplacement, const string& aHardlink )
     : FileInfo( aFile ), mAttachment( aUnknown ), mReplacement( rUnknown ), mHardlink( aHardlink )
{
     for ( size_t i = 0; sAttachmentToString[i].a != aUnknown; ++i ) {
          if (  aAttachment == sAttachmentToString[i].s ) {
               mAttachment = sAttachmentToString[i].a;
               break;
          }
     }
     if ( mAttachment == aUnknown )
          throw runtime_error( "Invalid PackageStreamFile attachment scheme" );

     for ( size_t i = 0; sReplacementToString[i].r != rUnknown; ++i ) {
          if ( aReplacement == sReplacementToString[i].s ) {
               mReplacement = sReplacementToString[i].r;
               break;
          }
     }
     if ( mReplacement == rUnknown )
          throw runtime_error( "Invalid PackageStreamFile replacement scheme" );

} // PackageStreamFile::PackageStreamFile

//======================================================================

const char* PackageStreamFile::attached() const
{
     size_t i = 0;
     for ( ; sAttachmentToString[i].a != aUnknown; ++i ) {
          if ( sAttachmentToString[i].a == mAttachment )
               break;
     }
     return sAttachmentToString[i].s;

} // PackageStreamFile::attached

//======================================================================

const char* PackageStreamFile::replacement_scheme() const
{
     size_t i = 0;
     for ( ; sReplacementToString[i].r != rUnknown; ++i ) {
          if ( sReplacementToString[i].r == mReplacement )
               break;
     }
     return sReplacementToString[i].s;

} // PackageStreamFile::replacement_scheme

//======================================================================

string PackageStreamFile::tclForCreation( const list<string>& suffixes_to_strip ) const
{
     string stripped_pathname = pathname();
     for ( list<string>::const_iterator suffix = suffixes_to_strip.begin(); suffix != suffixes_to_strip.end(); ++suffix ) {
          if ( stripped_pathname.length() > suffix->length() && !strcmp( stripped_pathname.c_str() + stripped_pathname.length() - suffix->length(), suffix->c_str() ) ) {
               stripped_pathname.erase( stripped_pathname.length() - suffix->length() );
               break;
          }
     }
     return string( "PackageStreamFile [FileInfo {" ) + stripped_pathname + "} " + to_string( length() ) + " " + to_string( mtime() ) + " {" + user() + "} {" + group() + "} " + to_string_octal( mode() ) + " {" + symlink() + "} {" + to_string( major( device() ) ) + " " + to_string( minor( device() ) ) + "}] \"" + attached() + "\" \"" + replacement_scheme() + "\"";

} // PackageStreamFile::tclForCreation

//======================================================================

void PackageStreamFile::write( Archive& archive, const string& filename )
{
     ::unlink( filename.c_str() ); // avoid hardlink problems, ignore errors
     archive.extract( pathname(), filename );

} // PackageStreamFile::write

//======================================================================

void PackageStreamFile::append( Archive& archive, const string& filename, bool create )
{
     archive.extract_append( pathname(), filename, create );

} // PackageStreamFile::append

//======================================================================
