/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/PackageStreamFile.hh,v 1.2 2001/06/22 15:05:17 alex Exp $
 */

// -*- C++ -*-  Mon Jul 27 1998
// $Id: PackageStreamFile.hh,v 1.3 1998/12/30 13:07:59 eugene Exp $

#ifndef PackageStreamFile_hh
#define PackageStreamFile_hh

#ifndef FileInfo_hh
#include "FileInfo.hh"
#endif

//======================================================================

class PackageStream;
class Archive;

//======================================================================

class PackageStreamFile : public FileInfo
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     enum Attachment { aUnknown, aGenerated, aFile, aPatch, aAppend };

     enum Replacement { rUnknown, rDefault, rAlwaysReplace, rNeverReplace, rPatchIfOld };

     PackageStreamFile( const FileInfo& aFile, Attachment aAttachment, Replacement aReplacement, const string& aHardlink )
          : FileInfo( aFile ), mAttachment( aAttachment ), mReplacement( aReplacement ), mHardlink( aHardlink ) {}
     PackageStreamFile( const FileInfo& aFile, const string& aAttachment, const string& aReplacement, const string& aHardlink );

     virtual const char* attached() const;
     virtual Attachment attachment() const { return mAttachment; }

     virtual const char* replacement_scheme() const;

     virtual string hardlink() const { return mHardlink; }

       // Returns tcl code for creation this object.
     virtual string tclForCreation( const list<string>& suffixes_to_strip ) const;

     virtual void write( Archive& archive, const string& filename );
     virtual void append( Archive& archive, const string& filename, bool create );

     static bool present_in_package( Attachment a )
          {
               return a == aFile || a == aPatch || a == aAppend;
          }

     virtual bool present_in_package() const
          {
               return present_in_package( attachment() );
          }

     static Pointer<PackageStreamFile> make( Pointer<FileInfo> aFile, const string& aAttachment, const string& aReplacement, const string& aHardlink ) { return new PackageStreamFile( *aFile, aAttachment, aReplacement, aHardlink ); }

  private:
     Attachment mAttachment;
     Replacement mReplacement;
     string mHardlink;

     struct AttachmentToString
     {
          Attachment a;
          const char* s;
     };

     struct ReplacementToString
     {
          Replacement r;
          const char* s;
     };

     static const AttachmentToString sAttachmentToString[];
     static const ReplacementToString sReplacementToString[];

}; // class PackageStreamFile

//======================================================================

typedef Container<list<Pointer<PackageStreamFile> > > PackageStreamFileList;

//======================================================================

inline bool have_the_same_name( Pointer<PackageStreamFile> a, Pointer<PackageStreamFile> b )
{
     return a->pathname() == b->pathname();
}

//======================================================================

inline bool has_the_name( Pointer<PackageStreamFile> a, const string& aName )
{
     return a->pathname() == aName;
}

//======================================================================

inline PackageStreamFileList::const_iterator find( const PackageStreamFileList& files, const string& name )
{
     PackageStreamFileList::const_iterator file = files.begin();
     while ( file != files.end() && !has_the_name( *file, name ) )
          ++file;
     return file;
}

//======================================================================

#endif // PackageStreamFile_hh
