/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/PackageStreamStructure.cc,v 1.2 2001/06/22 15:05:18 alex Exp $
 */

// -*- C++ -*-   Wed Aug  5 1998
// $Id: PackageStreamStructure.cc,v 1.2 1998/12/30 13:07:59 eugene Exp $

#ifndef PackageStreamStructure_hh
#include "PackageStreamStructure.hh"
#endif

#ifndef Log_hh
#include "Log.hh"
#endif

#ifndef PatchSuffixes_hh
#include "PatchSuffixes.hh"
#endif

//======================================================================

bool PackageStreamStructure::checkPackageCreationData( const PackageStreamFileList& aFiles ) const
{
     LOG( INFO, "Checking package structure described in structure.tcl" );

     bool fatal_problem = false;

     for ( PackageStreamFileList::const_iterator f = aFiles.begin(); f != aFiles.end(); ++f ) {
          Pointer<PackageStreamFile> file = find( (*f)->pathname() );
          for ( const char* const* suffix = PatchSuffixes; !file && *suffix; ++suffix ) {
               if ( (*f)->pathname().length() > strlen( *suffix ) )
                    file = find( (*f)->pathname().substr( 0, (*f)->pathname().length() - strlen( *suffix ) ) );
          }
          for ( const char* const* suffix = AppendSuffixes; !file && *suffix; ++suffix ) {
               if ( (*f)->pathname().length() > strlen( *suffix ) )
                    file = find( (*f)->pathname().substr( 0, (*f)->pathname().length() - strlen( *suffix ) ) );
          }
          if ( !file ) {
               LOG( WARNING, "File " << (*f)->pathname() << " is to be added to the package but not referenced in structure.tcl" );
          }
          else {
                 // if ( file->present_in_package() )
                 //      LOG( WARNING, "File " << (*f)->pathname() << " is to be added to the package but referenced in structure.tcl as not attached" );
               if ( file->length() != (*f)->length() )
                    LOG( WARNING, "File " << (*f)->pathname() << " length differs from the specified in structure.tcl" );
          }
     }

     for ( PackageStreamFileList::const_iterator f = files()->begin(); f != files()->end(); ++f ) {
          PackageStreamFileList::const_iterator file = ::find( aFiles, (*f)->pathname() );
          if ( (*f)->attachment() == PackageStreamFile::aFile && file == aFiles.end() ) {
               fatal_problem = true;
               LOG( ERROR, "File " << (*f)->pathname() << " is referenced in structure.tcl as attached but not present for adding to the package" );
          }
          else if ( (*f)->attachment() == PackageStreamFile::aPatch && file == aFiles.end() ) {
                 // patches could have some suffixes
               for ( const char* const* suffix = PatchSuffixes; *suffix && file == aFiles.end(); ++suffix )
                    file = ::find( aFiles, (*f)->pathname() + *suffix );
               if ( file == aFiles.end() ) {
                    fatal_problem = true;
                    LOG( ERROR, "File " << (*f)->pathname() << " is referenced in structure.tcl as attached patch but not present for adding to the package" );
               }
          }
          else if ( (*f)->attachment() == PackageStreamFile::aAppend && file == aFiles.end() ) {
                 // appendings could have some suffixes
               for ( const char* const* suffix = AppendSuffixes; *suffix && file == aFiles.end(); ++suffix )
                    file = ::find( aFiles, (*f)->pathname() + *suffix );
               if ( file == aFiles.end() ) {
                    fatal_problem = true;
                    LOG( ERROR, "File " << (*f)->pathname() << " is referenced in structure.tcl as attached appending but not present for adding to the package" );
               }
          }
     }

     return fatal_problem;

} // PackageStreamStructure::checkPackageCreationData

//======================================================================

Pointer<PackageStreamFile> PackageStreamStructure::find( const string& name ) const
{
     PackageStreamFileList::const_iterator file = ::find( *files(), name );
     if ( file == files()->end() ) {
          Pointer<PackageStreamFile> result;
          for ( PackageStreamSubpackageList::const_iterator sub = subpackages()->begin(); sub != subpackages()->end() && !result; ++sub ) {
               Pointer<PackageStreamFileList> files = (*sub)->files();
               PackageStreamFileList::const_iterator file = ::find( *files, name );
               if ( file != files->end() )
                    result = *file;
          }
          return result;
     }
     else
          return *file;

} // PackageStreamStructure::find

//======================================================================
