/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/sysinstall/PackageStreamStructure.hh,v 1.2 2001/06/22 15:05:18 alex Exp $
 */

// -*- C++ -*-  Mon Jul 27 1998
// $Id: PackageStreamStructure.hh,v 1.1 1998/12/26 09:26:54 eugene Exp $

#ifndef PackageStreamStructure_hh
#define PackageStreamStructure_hh

#ifndef PackageStreamFile_hh
#include "PackageStreamFile.hh"
#endif

#ifndef PackageStreamSubpackage_hh
#include "PackageStreamSubpackage.hh"
#endif

#ifndef PackageStreamReference_hh
#include "PackageStreamReference.hh"
#endif

//======================================================================

class PackageStreamStructure : virtual public LanguageInterface::Object
{
     CLASS_METHODS_DESCRIPTIONS_DECLARATION;

  public:
     PackageStreamStructure( const string& aPrefix, Pointer<PackageStreamFileList> aFiles, Pointer<PackageStreamSubpackageList> aSubpackages, Pointer<PackageStreamReferenceList> aReferences )
          : mPrefix( aPrefix ), mFiles( aFiles ), mSubpackages( aSubpackages ), mReferences( aReferences )
          {
          }

     virtual const string& default_prefix() const { return mPrefix; }

     virtual const Pointer<PackageStreamFileList> files() const { return mFiles; }
     virtual size_t has_subpackages() const { return mSubpackages ? mSubpackages->size() : 0; }
     virtual const Pointer<PackageStreamSubpackageList> subpackages() const { return mSubpackages; }
     virtual size_t has_references() const { return mReferences ? mReferences->size() : 0; }
     virtual const Pointer<PackageStreamReferenceList> references() const { return mReferences; }

     virtual bool check_integrity() const { return default_prefix()[0] != '/'; }

       // Returns true if this structure and aFiles have fatal differences.
       // All problems are reported using log.
     virtual bool checkPackageCreationData( const PackageStreamFileList& aFiles ) const;

     static Pointer<PackageStreamStructure> make( const string& aPrefix, Pointer<PackageStreamFileList> aFiles, Pointer<PackageStreamSubpackageList> aSubpackages, Pointer<PackageStreamReferenceList> aReferences ) { return new PackageStreamStructure( aPrefix, aFiles, aSubpackages, aReferences ); }

  private:
     string mPrefix;
     Pointer<PackageStreamFileList> mFiles;
     Pointer<PackageStreamSubpackageList> mSubpackages;
     Pointer<PackageStreamReferenceList> mReferences;

       // Looks for the specified file in mFiles and mSubpackages.
       // Returns 0 if file was not found.
     virtual Pointer<PackageStreamFile> find( const string& name ) const;

}; // class PackageStreamStructure

//======================================================================

#endif // PackageStreamStructure_hh
