/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/tcl/LibHTclInit_common.cc,v 1.4 2001/06/20 10:29:59 alex Exp $
 */

// -*- C++ -*-   Thu Nov  5 1998
// Eugene Skepner <eu@qub.com>
// $Id: LibHTclInit_common.cc,v 1.3 1998/12/28 10:18:09 eugene Exp $

#ifndef TclLanguageInterface_hh
#include "TclLanguageInterface.hh"
#endif

#include <tcl.h>
#include <iostream>

//======================================================================

static void register_command_access_tcl(); // generated by tcl_interface_gen in LibHTclInterface.cc

#include "LibTclInterface_common.cc"

//======================================================================

static inline bool is_object( Tcl_Obj* obj )
{
	try {
		tclLanguageInterface->tcl2object( obj, (LanguageInterface::Object*)0 );
		return(true);
	}
	catch ( ... ) {
		return(false);
	}

} // is_object

//======================================================================

static int call_method( ClientData, Tcl_Interp *interp, int objc, Tcl_Obj* const objv[] )
{
	if ( objc < 2 )
		return TclLanguageInterface::CommandAccess::error( interp, "Wrong number of arguments for command \"H::call object method ...\"", 0 );

	int result = TCL_OK;
	try {
		Tcl_Obj* object = objv[2];
		Tcl_Obj* interp_name = objv[1];

		if ( is_object( object ) ) {
			Pointer<LanguageInterface::Object> t = tclLanguageInterface->tcl2object( object, (LanguageInterface::Object*)0 );
			if ( !t )
				throw runtime_error( "Cannot call methods of NULL object" );
			TclLanguageInterface::CommandAccess* commandAccess = tclLanguageInterface->commandAccess( t->objecttypename() );
			const_cast<Tcl_Obj*&>( objv[1] ) = object;
			const_cast<Tcl_Obj*&>( objv[2] ) = interp_name;
			result = (*commandAccess->methodCmd())( static_cast<void*>( (LanguageInterface::Object*)t ), interp, objc - 1, objv + 1 );
		}
		else {
			Tcl_Obj* toEval = Tcl_NewListObj( objc - 2, objv + 2 );
			Tcl_IncrRefCount( toEval );
			result = Tcl_EvalObj( interp, toEval );
			Tcl_DecrRefCount( toEval );
		}
	}
	catch ( exception& err ) {
		return TclLanguageInterface::CommandAccess::error( interp, err.what(), 0 );
	}

	return(result);
}

//======================================================================

static int is_object_method( ClientData, Tcl_Interp *interp, int objc, Tcl_Obj* const objv[] )
{
	if ( objc != 3 )
		return TclLanguageInterface::CommandAccess::error( interp, "Wrong number of arguments for command \"H::is_object arg\"", 0 );

	try {
		Tcl_SetObjResult( interp, Tcl_NewBooleanObj( is_object( objv[2] ) ) );
		return(TCL_OK);
	}
	catch ( exception& err ) {
		return TclLanguageInterface::CommandAccess::error( interp, err.what(), 0 );
	}

} // is_object_method

//======================================================================

extern "C" int Htcl_Init( Tcl_Interp* interp )
{
	if ( interp == 0 )
		return(TCL_ERROR);

	bool enableGC = false;
	int egc;
	if ( Tcl_GlobalEval( interp, const_cast<char*>( "set H_automatic_garbage_collection" ) ) == TCL_OK && Tcl_GetBooleanFromObj( interp, Tcl_GetObjResult( interp ), &egc ) == TCL_OK )
		enableGC = egc;

	if ( enableGC )
		cerr << "Automatic Garbage Collection enabled" << endl;

	tclLanguageInterface = new TclLanguageInterface( interp, enableGC );

	register_command_access_tcl();

	Tcl_CreateObjCommand( interp, (char*)"H::call_master", &call_method, 0, 0 );
	char* argv_call[] = { (char*)"master", 0 };
	Tcl_CreateAlias( interp, (char*)"H::call", interp, (char*)"H::call_master", 1, argv_call );

	Tcl_CreateObjCommand( interp, (char*)"H::is_object_master", &is_object_method, 0, 0 );
	char* argv_is_object[] = { (char*)"master", 0 };
	Tcl_CreateAlias( interp, (char*)"H::is_object", interp, (char*)"H::is_object_master", 1, argv_is_object );

	return(TCL_OK);
}

//======================================================================
