/*-
 * Copyright (c) 1998-2001 The FreeBSD Project
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FREEBSD PROJECT OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation
 * are those of the authors and should not be interpreted as representing
 * official policies, either expressed or implied, of the FreeBSD Project.
 *
 * $FreeBSD: libh/lib/tcl/TclLanguageInterface.cc,v 1.4 2001/06/20 10:29:59 alex Exp $
 */

// -*- C++ -*-   Thu Nov  5 1998
// Eugene Skepner <eu@qub.com>
// $Id: TclLanguageInterface.cc,v 1.23 1999/01/04 23:15:58 eugene Exp $

#ifndef TclLanguageInterface_hh
#include "TclLanguageInterface.hh"
#endif

#ifndef String_hh
#include "String.hh"
#endif

#ifndef Log_hh
#include "Log.hh"
#endif

#include <iostream>
#include <string>
#include <cstdio>
#include <cstdarg>

//======================================================================

Tcl_ObjType TclLanguageInterface::sHtcl = {
	(char*)"htcl",
	&TclLanguageInterface::HTclFreeInternalRepProc,
	&TclLanguageInterface::HTclDupInternalRepProc,
	&TclLanguageInterface::HTclUpdateStringProc,
	&TclLanguageInterface::HTclSetFromAnyProc
};

TclLanguageInterface* tclLanguageInterface = 0;

//======================================================================

TclLanguageInterface::TclLanguageInterface( Tcl_Interp* aInterp, bool enableGC )
	: LanguageInterface( enableGC ), mInterp( aInterp )
{
	if ( gc() )
		registerObjType();

} // TclLanguageInterface::TclLanguageInterface

//======================================================================

TclLanguageInterface::~TclLanguageInterface()
{

} // TclLanguageInterface::~TclLanguageInterface

//======================================================================

Tcl_Obj* TclLanguageInterface::object2tcl( Pointer<Object> ptr )
{
	Tcl_Obj* obj = 0;
	if ( gc() ) {
		obj = Tcl_NewObj();
		obj->bytes = 0;
		obj->length = 0;
		obj->internalRep.otherValuePtr = static_cast<void*>( (Object*)ptr );
		obj->typePtr = &sHtcl;
	}
	else {
		obj = Tcl_NewStringObj( const_cast<char*>( object2string( ptr ).c_str() ), -1 );
	}
	if ( ptr )
		registerObject( ptr );
	return(obj);

} // TclLanguageInterface::object2tcl

//======================================================================

string TclLanguageInterface::object_type( Tcl_Obj* obj )
{
	if ( obj == 0 )
		return("<null>");
	if ( gc() ) {
		if ( obj->typePtr != &sHtcl )
			return("<Not Object>");
		Object* o = reinterpret_cast<Object*>( obj->internalRep.otherValuePtr );
		return type_to_name( typeid( o ).name() );
	}
	else {
		const char* s = Tcl_GetStringFromObj( obj, 0 );
		if ( s != 0 ) {
			void* p;
			char type[256];
			if ( sscanf( s, ObjectStringFormat, &p, &type ) == 2 )
				return type_to_name( type );
			else
				return("<Not Object>");
		}
		else
			return("<empty string>");
	}

} // TclLanguageInterface::object_type

//======================================================================

void TclLanguageInterface::registerObjType()
{
	Tcl_RegisterObjType( &sHtcl );

} // TclLanguageInterface::registerObjType

//======================================================================

void TclLanguageInterface::HTclFreeInternalRepProc( Tcl_Obj* obj )
{
	LanguageInterface::Object* ptr = static_cast< LanguageInterface::Object* >( obj->internalRep.otherValuePtr );
	//cerr << "HTclFreeInternalRepProc: " << " ptr:" << (void*)(Object*)ptr << endl;
	tclLanguageInterface->unregisterObject( ptr );

	obj->typePtr = 0;
	obj->internalRep.otherValuePtr = 0;

} // TclLanguageInterface::HTclFreeInternalRepProc

//======================================================================

void TclLanguageInterface::HTclDupInternalRepProc( Tcl_Obj* src, Tcl_Obj* dest )
{
	//cerr << "HTclDupInternalRepProc: " << (const void*)src << ' ' << (const void*)dest << endl;

	tclLanguageInterface->languageRef( static_cast< LanguageInterface::Object* >( src->internalRep.otherValuePtr ) );
	dest->internalRep.otherValuePtr = src->internalRep.otherValuePtr;
	dest->typePtr = src->typePtr;

} // TclLanguageInterface::HTclDupInternalRepProc

//======================================================================

void TclLanguageInterface::HTclUpdateStringProc( Tcl_Obj* obj )
{
	cerr << "ERROR: Improper use of an object when automatic garbage collection enabled, you should call object's methods via H::call, use H::as_string to get string representation of an object handle, avoid inserting objects into strings" << endl;

	const char* s = "Improper use of an object (Hint: use H::call to call object's methods)";
	obj->length = strlen( s );
	obj->bytes = (char*)Tcl_Alloc( obj->length + 1 );
	strcpy( obj->bytes, s );
	obj->internalRep.otherValuePtr = 0;
	obj->typePtr = &sHtcl;

} // TclLanguageInterface::HTclUpdateStringProc

//======================================================================

int TclLanguageInterface::HTclSetFromAnyProc( Tcl_Interp* aInterp, Tcl_Obj* /*obj*/ )
{
	//cerr << "HTclSetFromAnyProc: " << endl;

	Tcl_ResetResult( aInterp );
	Tcl_AppendResult( aInterp, "htcl object cannot be created from a string", 0 );

	return(TCL_ERROR);

} // TclLanguageInterface::HTclSetFromAnyProc

//======================================================================

void TclLanguageInterface::registerCommandAccess( const string& aName, CommandAccess* aAccess )
{
	mCommandsAccess[aName] = aAccess;

} // TclLanguageInterface::registerCommandAccess

//======================================================================

void TclLanguageInterface::registerObject( const Object* aObject )
{
	if ( aObject ) {
		commandAccess( aObject->objecttypename() )->create( *this, aObject );
		languageRef( aObject );
	}

} // TclLanguageInterface::registerObject

//======================================================================

TclLanguageInterface::CommandAccess* TclLanguageInterface::commandAccess( const string& aName )
{
	CommandsAccess::iterator cai = mCommandsAccess.find( aName );
	if ( cai == mCommandsAccess.end() )
		throw Error( string( "CommandAccess is not registered for " ) + aName );
	else
		return(cai->second);

} // TclLanguageInterface::commandAccess

//======================================================================

void TclLanguageInterface::unregisterObject( const Object* aObject )
{
	if ( aObject ) {
		if ( languageUnref( aObject ) == Object::Reference::NotReferenced || !gc() ) {
			const string cmd = object2string( aObject );
			Tcl_DeleteCommand( interpreter(), const_cast<char*>( cmd.c_str() ) );
			Tcl_DeleteCommand( interpreter(), const_cast<char*>( ( cmd + "_master" ).c_str() ) );
			for ( list<SlaveInterpreter*>::iterator slave = mSlaveInterpreters.begin(); slave != mSlaveInterpreters.end(); ++slave )
				(*slave)->delete_command( cmd );
		}
		if ( !aObject->referenced() )
			delete aObject;
	}

} // TclLanguageInterface::unregisterObject

//======================================================================

void TclLanguageInterface::CommandAccess::create( TclLanguageInterface& aTclLanguageInterface, const Object* aObject )
{
	if ( aObject ) {
		const string cmd = TclLanguageInterface::object2string( aObject );
		aTclLanguageInterface.create_command( cmd, methodCmd(), static_cast<void*>( const_cast<Object*>( aObject ) ) );
	}

} // TclLanguageInterface::CommandAccess::create

//======================================================================

void TclLanguageInterface::create_command( const string& aName, Tcl_ObjCmdProc* aProc, void* aData )
{
	Tcl_CmdInfo dummy;
	if ( !Tcl_GetCommandInfo( interpreter(), const_cast<char*>( aName.c_str() ), &dummy ) ) {
		const string commandName = master_command_name( aName );
		Tcl_CreateObjCommand( interpreter(), const_cast<char*>( commandName.c_str() ), aProc, aData, 0 );
		char* argv[] = { (char*)"master", 0 };
		if ( Tcl_CreateAlias( interpreter(), const_cast<char*>( aName.c_str() ), interpreter(), const_cast<char*>( commandName.c_str() ), 1, argv ) != TCL_OK )
			throw Error( string( "Creating alias for command \"" ) + commandName + "\" failed: " + interpreter()->result );
		for ( list<SlaveInterpreter*>::iterator slave = mSlaveInterpreters.begin(); slave != mSlaveInterpreters.end(); ++slave )
			(*slave)->alias_command( aName );
		if ( aData == 0 )
			mStaticMethods.push_back( aName );
	}

} // TclLanguageInterface::create_command

//======================================================================

int TclLanguageInterface::CommandAccess::error( Tcl_Interp* aInterp, const char* message ... )
{
	Tcl_ResetResult( aInterp );
	Tcl_Obj* result = Tcl_GetObjResult( aInterp );
	Tcl_SetStringObj( result, const_cast<char*>( message ), -1 );

	va_list ap;
	va_start( ap, message );
	const char* s;
	while ( ( s = va_arg( ap, const char* ) ) != 0 )
		Tcl_AppendStringsToObj( result, const_cast<char*>( s ), 0 );
	va_end( ap );
	return(TCL_ERROR);

} // TclLanguageInterface::CommandAccess::error

//======================================================================

void TclLanguageInterface::CommandAccess::activate_callback( Tcl_Interp* /*aInterp*/, Tcl_Obj* /*aCommand*/, Tcl_Obj* /*aUserData*/, Tcl_Obj* /*aArg*/, SpecificCallbackData& /*aSpecificCallbackData*/ )
{
	cerr << "No callbacks invocation data defined for CommandAccess " << typeid( *this ).name() << endl;

} // TclLanguageInterface::CommandAccess::activate_callback

//======================================================================

TclLanguageInterface::CallbackData::CallbackData( TclLanguageInterface* aLanguageInterface, Tcl_Obj* aCommand, Tcl_Obj* aUserData )
	: LanguageInterface::CallbackData( aLanguageInterface ), mCommand( aCommand ), mUserData( aUserData )
{
	Tcl_IncrRefCount( mCommand );
	if ( mUserData )
		Tcl_IncrRefCount( mUserData );

} // TclLanguageInterface::CallbackData::CallbackData

//======================================================================

TclLanguageInterface::CallbackData::CallbackData( TclLanguageInterface* aLanguageInterface, const string& aCommand )
	: LanguageInterface::CallbackData( aLanguageInterface ), mCommand( Tcl_NewStringObj( const_cast<char*>( aCommand.c_str() ), -1 ) ), mUserData( 0 )
{
	Tcl_IncrRefCount( mCommand );

} // TclLanguageInterface::CallbackData::CallbackData

//======================================================================

TclLanguageInterface::CallbackData::~CallbackData()
{
	Tcl_DecrRefCount( mCommand );
	if ( mUserData )
		Tcl_DecrRefCount( mUserData );

} // TclLanguageInterface::CallbackData::~CallbackData

//======================================================================

Pointer<LanguageInterface::CallbackData> TclLanguageInterface::tcl2callbackData( Tcl_Interp* interp, Tcl_Obj* aData )
{
	int size;
	if ( Tcl_ListObjLength( interp, aData, &size ) != TCL_OK )
		throw Error( "Cannot get list length" );
	if ( size == 0 )
		return Pointer<LanguageInterface::CallbackData>(); // reset callback
	if ( size != 2 )
		throw Error( "CallbackData should be list of two elements: command and user_data" );
	Tcl_Obj* command;
	if ( Tcl_ListObjIndex( interp, aData, 0, &command ) != TCL_OK )
		throw Error( "Cannot get command from CallbackData list" );
	Tcl_Obj* user_data;
	if ( Tcl_ListObjIndex( interp, aData, 1, &user_data ) != TCL_OK )
		throw Error( "Cannot get user_data from CallbackData list" );

	Pointer<LanguageInterface::CallbackData> cd = new CallbackData( this, command, user_data );
	return cd;

} // TclLanguageInterface::tcl2callbackData

//======================================================================

Tcl_Obj* TclLanguageInterface::callbackData2tcl( Tcl_Interp* interp, Pointer<LanguageInterface::CallbackData> aCallbackData )
{
	CallbackData* data = dynamic_cast<CallbackData*>( (LanguageInterface::CallbackData*)aCallbackData );
	Tcl_Obj* result = Tcl_NewObj();
	if ( data  ) {
		if ( Tcl_ListObjAppendElement( interp, result, data->command() ) != TCL_OK )
				throw Error( string( "Error creating result list: " ) + Tcl_GetStringResult( interp ) );
		if ( Tcl_ListObjAppendElement( interp, result, data->userData() ) != TCL_OK )
				throw Error( string( "Error creating result list: " ) + Tcl_GetStringResult( interp ) );
	}
	return(result);

} // TclLanguageInterface::callbackData2tcl

//======================================================================

void TclLanguageInterface::CallbackData::activate( LanguageInterface::SpecificCallbackData& aSpecificCallbackData )
{
	TclLanguageInterface::CommandAccess* ca = dynamic_cast<TclLanguageInterface&>( languageInterface() ).commandAccess( aSpecificCallbackData.typeid_name() );
	ca->activate_callback( tclLanguageInterface->interpreter(), command(), userData(), 0, aSpecificCallbackData );

} // TclLanguageInterface::CallbackData::activate

//======================================================================

void TclLanguageInterface::CallbackData::activate( SpecificCallbackData& aSpecificCallbackData, Tcl_Obj* arg )
{
	TclLanguageInterface::CommandAccess* ca = dynamic_cast<TclLanguageInterface&>( languageInterface() ).commandAccess( aSpecificCallbackData.typeid_name() );
	ca->activate_callback( tclLanguageInterface->interpreter(), command(), userData(), arg, aSpecificCallbackData );

} // TclLanguageInterface::CallbackData::activate

//======================================================================

TclLanguageInterface::CallbackDataSlaveInterpreter::CallbackDataSlaveInterpreter( Pointer<SlaveInterpreter> aSlaveInterpreter, const string& aCommand )
	: TclLanguageInterface::CallbackData( &aSlaveInterpreter->tclLanguageInterface(), Tcl_NewStringObj( const_cast<char*>( aCommand.c_str() ), -1 ), 0 ), mSlaveInterpreter( aSlaveInterpreter )
{
} // TclLanguageInterface::CallbackDataSlaveInterpreter::CallbackDataSlaveInterpreter

//======================================================================

TclLanguageInterface::CallbackDataSlaveInterpreter::CallbackDataSlaveInterpreter( Pointer<SlaveInterpreter> aSlaveInterpreter, const string& aCommand, const string& aUserData )
	: TclLanguageInterface::CallbackData( &aSlaveInterpreter->tclLanguageInterface(), Tcl_NewStringObj( const_cast<char*>( aCommand.c_str() ), -1 ), Tcl_NewStringObj( const_cast<char*>( aUserData.c_str() ), -1 ) ), mSlaveInterpreter( aSlaveInterpreter )
{
} // TclLanguageInterface::CallbackDataSlaveInterpreter::CallbackDataSlaveInterpreter

//======================================================================

TclLanguageInterface::CallbackDataSlaveInterpreter::CallbackDataSlaveInterpreter( Pointer<SlaveInterpreter> aSlaveInterpreter, Tcl_Obj* aCommand, Tcl_Obj* aUserData )
	: TclLanguageInterface::CallbackData( &aSlaveInterpreter->tclLanguageInterface(), aCommand, aUserData ), mSlaveInterpreter( aSlaveInterpreter )
{
} // TclLanguageInterface::CallbackDataSlaveInterpreter::CallbackDataSlaveInterpreter

//======================================================================

void TclLanguageInterface::CallbackDataSlaveInterpreter::activate( SpecificCallbackData& aSpecificCallbackData )
{
	TclLanguageInterface::CommandAccess* ca = dynamic_cast<TclLanguageInterface&>( mSlaveInterpreter->tclLanguageInterface() ).commandAccess( aSpecificCallbackData.typeid_name() );
	ca->activate_callback( mSlaveInterpreter->interpreter(), command(), userData(), 0, aSpecificCallbackData );

} // TclLanguageInterface::CallbackDataSlaveInterpreter::activate

//======================================================================

unsigned TclLanguageInterface::SlaveInterpreter::sInterpreterSerial = 0;

Pointer<TclLanguageInterface::SlaveInterpreter> TclLanguageInterface::make_slave_interpreter( const string& aNamePrefix, bool aSafe )
{
	string name = aNamePrefix + ::to_string( SlaveInterpreter::sInterpreterSerial++ );

	Tcl_Interp* slave = Tcl_CreateSlave( interpreter(), const_cast<char*>( name.c_str() ), aSafe );
	if ( slave == 0 )
		throw runtime_error( string( "Creating slave interpreter failed: " ) + interpreter()->result );


	// Don't know how to do this better
	Tcl_Eval( interpreter(), const_cast<char*>( ( string( "interp share {} stdout " ) + name ).c_str() ) );
	Tcl_Eval( interpreter(), const_cast<char*>( ( string( "interp share {} stderr " ) + name ).c_str() ) );

	Pointer<TclLanguageInterface::SlaveInterpreter> si = new SlaveInterpreter( name, slave, *this );
	mSlaveInterpreters.push_back( si );

	// Make alias for built-in commands
	si->alias_command( "H::call" );
	si->alias_command( "H::is_object" );

	// Make aliases for existing objects
	for ( list<const Object*>::const_iterator o = referencedObjects().begin(); o != referencedObjects().end(); ++o )
		si->alias_command( TclLanguageInterface::object2string( *o ) );

	// Make aliases for static methods commands
	for ( list<string>::const_iterator staticMethod = mStaticMethods.begin(); staticMethod != mStaticMethods.end(); ++staticMethod )
		si->alias_command( *staticMethod );
	return si;

} // TclLanguageInterface::make_slave_interpreter

//======================================================================

void TclLanguageInterface::slave_interpreter_destroyed( TclLanguageInterface::SlaveInterpreter* aSlaveInterpreter )
{
	mSlaveInterpreters.erase( find( mSlaveInterpreters.begin(), mSlaveInterpreters.end(), aSlaveInterpreter ) );

} // TclLanguageInterface::slave_interpreter_destroyed

//======================================================================

TclLanguageInterface::SlaveInterpreter::SlaveInterpreter( const string& aInterpName, Tcl_Interp* aInterp, TclLanguageInterface& aParent )
	: mInterpName( aInterpName ), mInterp( aInterp ), mParent( aParent )
{
} // TclLanguageInterface::SlaveInterpreter::SlaveInterpreter

//======================================================================

TclLanguageInterface::SlaveInterpreter::~SlaveInterpreter()
{
	if ( mInterp && !Tcl_InterpDeleted( mInterp ) )
		Tcl_DeleteInterp( mInterp );
	mParent.slave_interpreter_destroyed( this );

} // TclLanguageInterface::SlaveInterpreter::~SlaveInterpreter

//======================================================================

void TclLanguageInterface::SlaveInterpreter::alias_command( const string& aCommand )
{
	char* argv[] = { const_cast<char*>( mInterpName.c_str() ), 0 };
	Tcl_CreateAlias( interpreter(), const_cast<char*>( aCommand.c_str() ), mParent.interpreter(), const_cast<char*>( TclLanguageInterface::master_command_name( aCommand ).c_str() ), 1, argv );

} // TclLanguageInterface::SlaveInterpreter::alias_command

//======================================================================

void TclLanguageInterface::SlaveInterpreter::delete_command( const string& aCommand )
{
	Tcl_DeleteCommand( interpreter(), const_cast<char*>( aCommand.c_str() ) );

} // TclLanguageInterface::SlaveInterpreter::delete_command

//======================================================================

bool TclLanguageInterface::SlaveInterpreter::command_exists( const string& aCommand )
{
	Tcl_CmdInfo info;
	return Tcl_GetCommandInfo( interpreter(), const_cast<char*>( aCommand.c_str() ), &info );

} // TclLanguageInterface::SlaveInterpreter::command_exists

//======================================================================

bool TclLanguageInterface::SlaveInterpreter::variable_exists( const string& aVariable )
{
	return(Tcl_GetVar( interpreter(), const_cast<char*>( aVariable.c_str() ), 0 ) != 0);

} // TclLanguageInterface::SlaveInterpreter::variable_exists

//======================================================================

const char* TclLanguageInterface::SlaveInterpreter::eval( const string& data, const string& aNamespace )
{
	string toEvaluate = string( "namespace eval " ) + aNamespace + " {\n" + data + "\n}";
	//LOG( DEBUG, toEvaluate );
	if ( Tcl_Eval( interpreter(), const_cast<char*>( toEvaluate.c_str() ) ) != TCL_OK ) {
		//throw runtime_error( string( "Evaluating of \"" ) + toEvaluate + "\" failed: " + interp->result );
		throw runtime_error( interpreter()->result );
	}

	return(interpreter()->result);

} // TclLanguageInterface::SlaveInterpreter::eval

//======================================================================

const char* TclLanguageInterface::SlaveInterpreter::eval( const string& data )
{
	if ( Tcl_Eval( interpreter(), const_cast<char*>( data.c_str() ) ) != TCL_OK ) {
		//throw ::error( string( "Evaluating of \"" ) + data + "\" failed: " + interp->result );
		throw runtime_error( interpreter()->result );
	}

	return(interpreter()->result);

} // TclLanguageInterface::SlaveInterpreter::eval

//======================================================================
