proc slice_create_dialog {hui parent slice} {

    set dialog [$hui dialog $parent slice_type_dialog \
		    "Create Slice" "10 6" "57 10"]

    set label [$dialog label "desc label" "Please specify the size for new FreeBSD slice in blocks\nor append a trailing `M' or `G' as sizeunit (e.g. 2G)."]
    set sizetext [$dialog inputLine "sizetext" [$slice size] 30]

    set bootable [$dialog checkBoxes "bootable" {"Bootable"} {FALSE} "Slice Options"]
    set type [$dialog radioMenu "type" {"FreeBSD" "FAT"} {0} "Type"]

    set okbutton [$dialog button "okbutton" "&OK" {button_click_result "OK"}]
    set cancelbutton [$dialog button "cancelbutton" "&Cancel" {button_click_result "CANCEL"}]

    $label move "1 0"
    $label resize "55 2"
    $sizetext move "1 2"
    $sizetext resize "55 1"
    $bootable move "1 4"
    $bootable resize "27 3"
    $type move "28 4"
    $type resize "28 3"

    $okbutton move "1 8"
    $okbutton resize "10 2"
    $cancelbutton move "15 8"
    $cancelbutton resize "10 2"

    $dialog exec
    set result [$dialog result]

    if {[string compare $result "OK"] == 0} {
	set result {}
	# {size, type, bootable}

	# Size
	set size [$sizetext text]
	if {[string match "*\[Mm\]" $size]} {
	    global ONE_MEG
	    set size [expr $ONE_MEG * [string trim $size "Mm"]]
	} elseif {[string match "*\[Gg\]" $size]} {
	    global ONE_GIG
	    set size [expr $ONE_GIG * [string trim $size "Gg"]]
	} else {
	    # XXX errorbox for wrong input
	}
	lappend result $size

	# Type
	if {[$type set] == 0} {
	    # FreeBSD
	    set type 165
	} elseif {[$type set] == 1} {
	    # FAT
	    set type 6
	}
	lappend result $type

	# Bootable
	if {[$bootable state 0]} {
	    lappend result "CHUNK_ACTIVE"
	} else {
	    lappend result ""
	}
    }
    
    return $result
}

proc slice_chunk_flags_letters {slice} {
    
    set result ""
    set flags [$slice flags]
    if {[lsearch $flags "CHUNK_ACTIVE"] > 0} {     set result "${result}A" }
    if {[lsearch $flags "CHUNK_ALIGN"] > 0} {      set result "${result}=" }
    if {[lsearch $flags "CHUNK_IS_ROOT"] > 0} {    set result "${result}R" }
    if {[lsearch $flags "CHUNK_BSD_COMPAT"] > 0} { set result "${result}C" }
    return $result;
}

proc slice_info {slice} {
    return [format "%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s"\
		[$slice offset]\
		[$slice size]\
		[$slice end]\
		[$slice name]\
		[$slice type]\
		[$slice slice_type_name]\
		[$slice subtype]\
		[slice_chunk_flags_letters $slice]]
}

proc slice_of_disk {disk sliceno} {
    
    set slice [[$disk chunks] part]
    for {set i 0} {$i < $sliceno && ![H::is_null $slice]} {incr i} {
	set next_slice [$slice next]
	H::delete $slice
	set slice $next_slice
    }
    return $slice
}


