# File names
MFS_NAME	=	mfsroot
MFSFL_NAME	=	mfsroot.flp
KERN_NAME	=	kern.flp
BOOT_NAME	=	boot.flp

MFS_VNODE	=	/vn0
MFSFL_VNODE	=	/vn1
KERN_VNODE	=	/vn2
BOOT_VNODE	=	/vn3

MFS_MP		=	mmnt
MFSFL_MP	=	fmnt
KERN_MP		=	kmnt
BOOT_MP		=	bmnt

# Information about mfs parameters
# Size of mfs in kilobytes
MFS_SIZE	=	4500

# System source
SRC		=	/usr/src

# Name of init program
INITDIR		=	${.CURDIR}/../init
INIT		=	init

# Tclh
TCLHDIR		=	${.CURDIR}/../../bin/tclh
TCLH		=	tclh.static

# Variables for building kernel
CONF		?=	$(SRC)/sys/i386/conf
CONFFILE	=	LIBH-BOOT
COMPILE		=	$(OBJ)/$(SRC)/sys/$(CONFFILE)
KERNFILE	=	$(COMPILE)/kernel

# Number of bytes per inode in mfs
MFS_INODES	=	4096

# Number of bytes per inode in kern image
FLOPPY_INODES	=	32768

# Suffix to tack on somewhere
suffix		=	-C

# Common targets

vnconfig-mfs:
	@cd ${.CURDIR} && ${MAKE} vnunconfig-mfs 2>/dev/null >&2
	vnconfig -s labels -c ${MFS_VNODE} ${MFS_NAME}
vnconfig-mfsfl:
	@cd ${.CURDIR} && ${MAKE} vnunconfig-mfsfl 2>/dev/null >&2
	vnconfig -s labels -c ${MFSFL_VNODE} ${MFSFL_NAME}
vnconfig-kern:
	@cd ${.CURDIR} && ${MAKE} vnunconfig-kern 2>/dev/null >&2
	vnconfig -s labels -c ${KERN_VNODE} ${KERN_NAME}
vnconfig-boot:
	@cd ${.CURDIR} && ${MAKE} vnunconfig-boot 2>/dev/null >&2
	vnconfig -s labels -c ${BOOT_VNODE} ${BOOT_NAME}

vnunconfig-mfs:
	@cd ${.CURDIR} && ${MAKE} umount-mfs 2>/dev/null >&2
	@-vnconfig -u  ${MFS_VNODE} 2>/dev/null
vnunconfig-mfsfl:
	@cd ${.CURDIR} && ${MAKE} umount-mfsfl 2>/dev/null >&2
	@-vnconfig -u  ${MFSFL_VNODE} 2>/dev/null
vnunconfig-kern:
	@cd ${.CURDIR} && ${MAKE} umount-kern 2>/dev/null >&2
	@-vnconfig -u ${KERN_VNODE} 2>/dev/null
vnunconfig-boot:
	@cd ${.CURDIR} && ${MAKE} umount-boot 2>/dev/null >&2
	@-vnconfig -u ${BOOT_VNODE} 2>/dev/null

mount-mfs: ${MFS_MP}
	@cd ${.CURDIR} && ${MAKE} vnconfig-mfs
	mount ${MFS_VNODE} ${MFS_MP}
mount-mfsfl: ${MFSFL_MP}
	@cd ${.CURDIR} && ${MAKE} vnconfig-mfs
	mount ${MFSFL_VNODE} ${MFSFL_MP}
mount-kern: ${KERN_MP}
	@cd ${.CURDIR} && ${MAKE} vnconfig-kern
	mount ${KERN_VNODE} ${KERN_MP}
mount-boot: ${BOOT_MP}
	@cd ${.CURDIR} && ${MAKE} vnconfig-boot
	mount ${BOOT_VNODE} ${BOOT_MP}

umount-mfs:
	@-umount ${MFS_VNODE} 2>/dev/null >&2
umount-mfsfl:
	@-umount ${MFSFL_VNODE} 2>/dev/null
umount-kern:
	@-umount ${KERN_VNODE} 2>/dev/null
umount-boot:
	@-umount ${BOOT_VNODE} 2>/dev/null

${MFS_MP} ${MFSFL_MP} ${KERN_MP} ${BOOT_MP}:
	@mkdir -p $@
