/*
 * Copyright (c) 1994, Paul Richards.
 * 
 * All rights reserved.
 * 
 * This software may be used, modified, copied, distributed, and sold, in both
 * source and binary form provided that the above copyright and these terms
 * are retained, verbatim, as the first lines of this file.  Under no
 * circumstances is the author responsible for the proper functioning of this
 * software, nor does the author assume any responsibility for damages
 * incurred with its use.
 */

#include "extvars.h"
#include <stdarg.h>
#include <fcntl.h>
#include <sys/errno.h>
#include <sys/ioctl.h>
#include <sys/consio.h>
#include <stdio.h>
#include <unistd.h>

#define VTY_STATUS_LINE    24
#define TTY_STATUS_LINE    23

static void
prompt_term(char **termp, char **termcapp)
{
	char            str[80];
	static struct {
		const char     *term, *termcap;
	} lookup[] = {
			{"ansi", termcap_ansi },
			{"vt100", termcap_vt100},
			{"cons25w", termcap_cons25w},
			{"cons25", termcap_cons25},
			{"cons25-m", termcap_cons25_m},
			{"xterm", termcap_xterm}
	};

	while (1) {
		int             i;

		printf("\nThese are the predefined terminal types available to\n");
		printf("sysinstall when running stand-alone.  Please choose the\n");
		printf("closest match for your particular terminal.\n\n");
		printf("1 ...................... Standard ANSI terminal.\n");
		printf("2 ...................... VT100 or compatible terminal.\n");
		printf("3 ...................... FreeBSD system console (color).\n");
		printf("4 ...................... FreeBSD system console (monochrome).\n\n");
		printf("5 ...................... xterm terminal emulator.\n\n");
		printf("Your choice: (1-5) ");
		fflush(stdout);
		fgets(str, 80, stdin);
		i = str[0] - '0';
		if (i > 0 && i < 6) {
			*termp = (char *)lookup[i - 1].term;
			*termcapp = (char *)lookup[i - 1].termcap;
			break;
		} else
			printf("\007Invalid choice, please try again.\n\n");
	}
}

int
set_termcap(void)
{
	char           *term;
	int             stat;
	struct ttysize  ts;
	int             ColorDisplay;

	term = getenv("TERM");
	stat = ioctl(STDERR_FILENO, GIO_COLOR, &ColorDisplay);

	if (stat < 0) {
		if (!term) {
			char           *term, *termcap;

			prompt_term(&term, &termcap);
			if (setenv("TERM", term, 1) < 0)
				return -1;
			if (setenv("TERMCAP", termcap, 1) < 0)
				return -1;
		}
	} else {
		int             i, on;

		if (ColorDisplay) {
			if (!term) {
				if (setenv("TERM", "cons25", 1) < 0)
					return -1;
				if (setenv("TERMCAP", termcap_cons25, 1) < 0)
					return -1;
			}
		} else {
			if (!term) {
				if (setenv("TERM", "cons25-m", 1) < 0)
					return -1;
				if (setenv("TERMCAP", termcap_cons25_m, 1) < 0)
					return -1;
			}
		}
	}
	if (ioctl(0, TIOCGSIZE, &ts) == -1) {
		fprintf(stderr, "Unable to get terminal size - errno %d\n", errno);
		ts.ts_lines = 0;
	}
	return 0;
}
